/*
 * Licensed under a Creative Commons Attribution 2.5 Slovenia License
 * http://creativecommons.org/licenses/by/2.5/si/
 * 2009 TineL Studio
 */

package net.tinelstudio.gis.model.domain;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;

/**
 * The main entity providing only geographic names.
 * 
 * @author TineL
 */
@Entity
public class GeoName extends BaseEntity {

  @Column(nullable=false)
  private String name;

  @Column(nullable=false)
  @Enumerated(EnumType.STRING)
  private Type type;

  /**
   * The GeoName types.
   * 
   * @author TineL
   */
  public enum Type {

    /** The geographic name at the address level accuracy. */
    ADDRESS,

    /** The geographic name at the street level accuracy. */
    STREET,

    /** The geographic name at the building (premise, property) level accuracy. */
    BUILDING,

    /** The geographic name at the town (city, village) level accuracy. */
    TOWN,

    /**
     * The geographic name at the region (state, province, prefecture, etc.)
     * level accuracy.
     */
    REGION,

    /** The geographic name at the country level accuracy. */
    COUNTRY,

    /** The geographic name at the continent level accuracy. */
    CONTINENT
  }

  // ##################################################

  public String getName() {
    return this.name;
  }

  public void setName(String name) {
    this.name=name;
  }

  public Type getType() {
    return this.type;
  }

  public void setType(Type type) {
    this.type=type;
  }

  @Override
  public String toString() {
    StringBuilder builder=new StringBuilder();
    builder.append("GeoName [id=").append(getId()).append(", name=").append(
      this.name).append(", type=").append(this.type).append("]");
    return builder.toString();
  }
}