/*
 * Licensed under a Creative Commons Attribution 2.5 Slovenia License
 * http://creativecommons.org/licenses/by/2.5/si/
 * 2009 TineL Studio
 */

package net.tinelstudio.gis.routing.dto;

import java.util.List;

import net.tinelstudio.gis.common.dto.StreetDto;

import org.springframework.util.Assert;

import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.operation.linemerge.LineMerger;

/**
 * The default implementation of {@link Route}.
 * 
 * @author TineL
 */
public class DefaultRoute implements Route {

  /** The <code>serialVersionUID</code>. */
  private static final long serialVersionUID=-7839781844467651524L;

  private List<StreetDto> streets;

  private double cost;

  private long timeTaken;

  @Override
  public List<StreetDto> getStreets() {
    return this.streets;
  }

  public void setStreets(List<StreetDto> streets) {
    Assert.notNull(streets, "Streets cannot be null");
    this.streets=streets;
  }

  @Override
  public double getCost() {
    return this.cost;
  }

  public void setCost(double cost) {
    this.cost=cost;
  }

  @Override
  public long getTimeTaken() {
    return this.timeTaken;
  }

  public void setTimeTaken(long timeTaken) {
    this.timeTaken=timeTaken;
  }

  @Override
  @SuppressWarnings("unchecked")
  public LineString mergeStreets() {
    if (getStreets().isEmpty()) return null;

    LineMerger lm=new LineMerger();
    for (StreetDto s : getStreets()) {
      lm.add(s.getLineString());
    }
    List<LineString> lines=(List<LineString>)lm.getMergedLineStrings();
    return lines.get(0);
  }

  @Override
  public String toString() {
    StringBuilder builder=new StringBuilder();
    builder.append("DefaultRoute [cost=");
    builder.append(this.cost);
    builder.append(", timeTaken=");
    builder.append(this.timeTaken);
    builder.append(", mergeStreets()=");
    builder.append(this.mergeStreets());
    builder.append("]");
    return builder.toString();
  }
}
