/*
 * Licensed under a Creative Commons Attribution 2.5 Slovenia License
 * http://creativecommons.org/licenses/by/2.5/si/
 * 2009 TineL Studio
 */

package net.tinelstudio.gis.routing.searchalgorithm.astar;

import net.tinelstudio.gis.routing.searchalgorithm.AbstractSearchAlgorithm;
import net.tinelstudio.gis.routing.searchalgorithm.astar.heuristic.HeuristicFunction;

import com.vividsolutions.jts.geom.Point;

/**
 * The abstract implementation of A* routing search algorithm.
 * 
 * @author TineL
 */
public abstract class AbstractAStarSearchAlgorithm extends
  AbstractSearchAlgorithm {

  private HeuristicFunction heuristicFunction;

  /**
   * Computes heuristic h(n) between the given point and the goal.
   * 
   * @param point the point n (<code>null</code> not permitted)
   * @return a heuristic
   */
  protected double h(Point point) {
    Point goalPoint=getGoalNode().getPoint();
    return getHeuristicFunction().computeHeuristic(point.getCoordinate(),
      goalPoint.getCoordinate());
  }

  public HeuristicFunction getHeuristicFunction() {
    return this.heuristicFunction;
  }

  public void setHeuristicFunction(HeuristicFunction heuristicFunction) {
    this.heuristicFunction=heuristicFunction;
  }
}
