/*
 * Licensed under a Creative Commons Attribution 2.5 Slovenia License
 * http://creativecommons.org/licenses/by/2.5/si/
 * 2009 TineL Studio
 */

package net.tinelstudio.gis.routing.searchalgorithm.astar;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;
import static org.junit.Assert.assertEquals;

import java.util.concurrent.TimeoutException;

import net.tinelstudio.gis.model.domain.StreetNode;
import net.tinelstudio.gis.routing.dto.Route;
import net.tinelstudio.gis.routing.searchalgorithm.AbstractSearchAlgorithmTest;
import net.tinelstudio.gis.routing.searchalgorithm.astar.heuristic.HeuristicFunction;

import org.junit.Test;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Point;

/**
 * @author TineL
 */
public class AbstractAStarSearchAlgorithmTest extends
  AbstractSearchAlgorithmTest {

  @Override
  protected AbstractAStarSearchAlgorithm createInstance() {
    return new AbstractAStarSearchAlgorithm() {
      @Override
      public Route run() throws TimeoutException {
        throw new UnsupportedOperationException();
      }
    };
  }

  @Test
  public void testSetHeuristicFunction() {
    HeuristicFunction heuristicFunction=createMock(HeuristicFunction.class);
    AbstractAStarSearchAlgorithm aStar=createInstance();
    aStar.setHeuristicFunction(heuristicFunction);
    assertEquals(heuristicFunction, aStar.getHeuristicFunction());
  }

  @Test
  public void testH() {
    // Init
    Coordinate location=createMock(Coordinate.class);
    Coordinate goalLocation=createMock(Coordinate.class);

    Point point=createMock(Point.class);
    expect(point.getCoordinate()).andReturn(location);

    Point goalPoint=createMock(Point.class);
    expect(goalPoint.getCoordinate()).andReturn(goalLocation);

    StreetNode goalNode=createMock(StreetNode.class);
    expect(goalNode.getPoint()).andReturn(goalPoint);

    double h=Math.random()*100;

    HeuristicFunction heuristicFunction=createMock(HeuristicFunction.class);
    expect(heuristicFunction.computeHeuristic(location, goalLocation))
      .andReturn(h);

    replay(point, goalPoint, goalNode, heuristicFunction);

    // Run
    AbstractAStarSearchAlgorithm aStar=createInstance();
    aStar.setHeuristicFunction(heuristicFunction);
    aStar.setGoalNode(goalNode);
    double heuristic=aStar.h(point);

    // Verify
    verify(point, goalPoint, goalNode, heuristicFunction);
    assertEquals(h, heuristic, 0.0001);
  }
}
