/*
 * Licensed under a Creative Commons Attribution 2.5 Slovenia License
 * http://creativecommons.org/licenses/by/2.5/si/
 * 2009 TineL Studio
 */

package net.tinelstudio.gis.routing.searchalgorithm.astar;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import java.util.HashMap;
import java.util.Map;

import net.tinelstudio.commons.spring.test.PerformanceTestExecutionListener;
import net.tinelstudio.gis.common.dto.StreetDto.Level;
import net.tinelstudio.gis.model.dao.FindingDao;
import net.tinelstudio.gis.model.dao.StreetNodeDao;
import net.tinelstudio.gis.model.domain.StreetNode;
import net.tinelstudio.gis.routing.SpringContexts;
import net.tinelstudio.gis.routing.dto.Route;
import net.tinelstudio.gis.routing.searchalgorithm.astar.heuristic.GreatCircleDistanceHeuristicFunction;
import net.tinelstudio.gis.routing.searchalgorithm.astar.heuristic.HeuristicFunction;
import net.tinelstudio.gis.routing.searchalgorithm.cost.CostFunction;
import net.tinelstudio.gis.routing.searchalgorithm.cost.LengthCostFunction;
import net.tinelstudio.gis.routing.searchalgorithm.cost.LevelLengthCostFunction;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.TestExecutionListeners;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.support.DependencyInjectionTestExecutionListener;
import org.springframework.test.context.transaction.TransactionalTestExecutionListener;
import org.springframework.transaction.annotation.Transactional;

/**
 * Manual tests.
 * 
 * @author TineL
 */
// Remove @Ignore if you want to manually test
@Ignore
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations={
  net.tinelstudio.gis.model.SpringContexts.DB_CONTEXT,
  SpringContexts.ROUTING_SERVICE_CONTEXT})
@TestExecutionListeners({DependencyInjectionTestExecutionListener.class,
  TransactionalTestExecutionListener.class,
  PerformanceTestExecutionListener.class})
@Transactional(readOnly=true)
public class DefaultAStarSearchAlgorithmManualTest {

  private final Log logger=LogFactory.getLog(getClass());

  @Autowired
  private FindingDao findingDao;

  @Autowired
  private StreetNodeDao streetNodeDao;

  private Route test(long startNodeId, long goalNodeId,
    CostFunction costFunction, HeuristicFunction heuristicFunction)
    throws Exception {
    StreetNode startNode=streetNodeDao.load(startNodeId);
    StreetNode goalNode=streetNodeDao.load(goalNodeId);

    DefaultAStarSearchAlgorithm aStar=new DefaultAStarSearchAlgorithm();
    aStar.setStartNode(startNode);
    aStar.setGoalNode(goalNode);
    aStar.setCostFunction(costFunction);
    aStar.setHeuristicFunction(heuristicFunction);
    aStar.setFindingDao(this.findingDao);

    Route path=aStar.run();

    if (path!=null) {
      logger.info("Found route: "+path.mergeStreets());

    } else {
      logger.info("No route found");
    }

    return path;
  }

  // @Test
  public void test1H1() throws Exception {
    Route route=test(5987, 3934, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(1.0));

    logger.info("Found route="+route);

    String result="LINESTRING (13.405896 46.243709, 13.405982 46.243835, 13.406304 46.244094, 13.406561 46.244007, 13.406861 46.243988, 13.407054 46.24403, 13.407118 46.244178, 13.40714 46.244308, 13.406819 46.244586, 13.406497 46.244716, 13.406517 46.244804, 13.406711 46.244823, 13.406926 46.244739, 13.407097 46.244693, 13.407226 46.244628, 13.407977 46.244438, 13.40817 46.244373, 13.408278 46.244308, 13.408342 46.244117, 13.408642 46.243774, 13.408986 46.243537, 13.409243 46.243301, 13.409329 46.243194, 13.409544 46.24287, 13.410016 46.242034, 13.41023 46.24184, 13.410445 46.241519, 13.410659 46.241413, 13.410981 46.241455, 13.41111 46.24139, 13.411303 46.241153, 13.411561 46.241153, 13.411732 46.241111, 13.411818 46.241004, 13.41199 46.240684, 13.412312 46.240448, 13.412655 46.240081, 13.412805 46.240146, 13.412462 46.240489, 13.412268 46.240745, 13.41214 46.240962, 13.412096 46.241111, 13.412096 46.241348, 13.412033 46.241455, 13.411818 46.241561, 13.41154 46.241775, 13.411324 46.242057, 13.410917 46.242141, 13.410767 46.242378, 13.410702 46.242633, 13.410809 46.242786, 13.410809 46.242912, 13.410681 46.243213, 13.410874 46.243515, 13.410659 46.243644, 13.410552 46.243751, 13.410595 46.243881, 13.410831 46.244136, 13.411046 46.24448, 13.411046 46.244739, 13.41096 46.244911, 13.411067 46.244953, 13.411196 46.244758, 13.411346 46.244396, 13.411217 46.244159, 13.411196 46.244007, 13.411238 46.243881, 13.411496 46.243473, 13.411625 46.243171, 13.411732 46.243, 13.411969 46.242721, 13.412076 46.242721, 13.411797 46.243213, 13.411818 46.243385, 13.411818 46.2439, 13.411904 46.244071, 13.412011 46.244178, 13.412526 46.24435, 13.412633 46.244415, 13.412676 46.244522, 13.412633 46.244651, 13.41257 46.245082, 13.412633 46.245296, 13.412762 46.245426, 13.412848 46.245574, 13.413191 46.24596, 13.413406 46.246067, 13.41362 46.246219, 13.413792 46.246368, 13.413771 46.24654, 13.41347 46.246948, 13.413492 46.247375, 13.413449 46.247848, 13.413298 46.248279, 13.412762 46.248813, 13.412633 46.249137, 13.412783 46.249931, 13.41317 46.250701, 13.413342 46.251197, 13.413706 46.251949, 13.413792 46.252269, 13.414007 46.252868, 13.414156 46.253406, 13.414221 46.253471, 13.414328 46.253471, 13.414544 46.253387, 13.415101 46.252826, 13.41538 46.252719, 13.415723 46.2527, 13.41611 46.252761, 13.416453 46.252868, 13.416817 46.25315, 13.417397 46.253921, 13.417462 46.254287, 13.417483 46.254714, 13.417225 46.255058, 13.416839 46.255382, 13.416689 46.255554, 13.416667 46.255725, 13.416817 46.255897, 13.417011 46.256366, 13.417054 46.256603, 13.417054 46.256839, 13.417011 46.257183, 13.417033 46.257354, 13.41714 46.25774, 13.417011 46.258277, 13.417033 46.258922, 13.417075 46.259349, 13.417225 46.259521, 13.417397 46.259651, 13.418684 46.260295, 13.41935 46.260639, 13.419608 46.260704, 13.420015 46.260681, 13.42038 46.260616, 13.420723 46.260486, 13.421946 46.260124, 13.422461 46.259929, 13.423813 46.259284, 13.424284 46.259006, 13.425465 46.258041, 13.425851 46.257698, 13.426023 46.257614, 13.42656 46.257118, 13.426624 46.257205, 13.426452 46.257442, 13.426217 46.258106, 13.426023 46.258815, 13.425637 46.260551, 13.425486 46.260852, 13.425658 46.261024, 13.425723 46.261196, 13.425787 46.261882, 13.425829 46.262012, 13.426001 46.262268, 13.426045 46.262634, 13.426087 46.262805, 13.426302 46.262977, 13.426881 46.262912, 13.427225 46.262935, 13.427504 46.26287, 13.427825 46.262763, 13.428383 46.262504, 13.428641 46.262355, 13.42907 46.261989, 13.429456 46.261859, 13.429928 46.261669, 13.430464 46.261516, 13.430787 46.26139, 13.431151 46.261325, 13.431645 46.261196, 13.432826 46.26094, 13.433018 46.260875, 13.433297 46.260746, 13.433726 46.260509, 13.43377 46.260337, 13.433662 46.260059, 13.433598 46.259735, 13.433705 46.259521, 13.434155 46.259159, 13.435185 46.258407, 13.435486 46.258342, 13.436259 46.258449, 13.437009 46.258426, 13.437353 46.258384, 13.437502 46.258255, 13.437825 46.25772, 13.437996 46.257549, 13.438297 46.257312, 13.438961 46.257034, 13.439198 46.25682, 13.43952 46.25669, 13.439734 46.256645, 13.439864 46.256519, 13.439928 46.256366, 13.439885 46.256176, 13.439756 46.255916, 13.439562 46.255573, 13.439435 46.255424, 13.439435 46.255252, 13.439562 46.254951, 13.43952 46.254714, 13.43937 46.254352, 13.43937 46.254093, 13.439562 46.253963, 13.439906 46.253944, 13.440293 46.253963, 13.440657 46.254028, 13.441043 46.254051, 13.441644 46.25418, 13.442331 46.254436, 13.442481 46.254566, 13.44261 46.25476, 13.442631 46.254867, 13.442824 46.254974, 13.443596 46.25476, 13.444863 46.254673, 13.44512 46.254714, 13.445249 46.254909, 13.445335 46.25521, 13.445507 46.255489, 13.445678 46.255638, 13.445872 46.255725, 13.446043 46.255832, 13.446236 46.255874, 13.446473 46.255851, 13.4466 46.255638, 13.44673 46.255336, 13.446923 46.25521, 13.447459 46.255058, 13.447824 46.255081, 13.448018 46.255039, 13.448339 46.254928, 13.448489 46.254844, 13.448639 46.254844, 13.448876 46.254737, 13.448962 46.254566, 13.448725 46.254329, 13.448596 46.254158, 13.448554 46.254051, 13.448554 46.253898, 13.44879 46.253837, 13.449111 46.253837, 13.449391 46.253814, 13.450078 46.253749, 13.450442 46.253791, 13.45072 46.254116, 13.451043 46.254394, 13.451235 46.254673, 13.451493 46.254867, 13.451858 46.255016, 13.452052 46.254993, 13.452244 46.254909, 13.452416 46.254779, 13.452673 46.254695, 13.452931 46.25476, 13.453103 46.254886, 13.453209 46.255123, 13.453295 46.255252, 13.45381 46.255531, 13.454197 46.25568, 13.454605 46.255767, 13.455205 46.255832, 13.455441 46.255832, 13.455698 46.255661, 13.455678 46.255447, 13.455463 46.254974, 13.455463 46.25476, 13.455549 46.254608, 13.455892 46.254287, 13.455935 46.253898, 13.45615 46.253749, 13.456643 46.253707, 13.457501 46.253707, 13.457844 46.253707, 13.458445 46.253772, 13.459197 46.253944, 13.459583 46.254074, 13.459883 46.254241, 13.460206 46.254695, 13.460333 46.254779, 13.460742 46.254802, 13.461106 46.254779, 13.461407 46.254886, 13.4616 46.255058, 13.461792 46.255165, 13.462286 46.255336, 13.462609 46.255359, 13.462887 46.255336, 13.463124 46.255229, 13.463509 46.254974, 13.463725 46.254928, 13.464046 46.254821, 13.464453 46.254566, 13.46469 46.254566, 13.465012 46.25465, 13.465162 46.254779, 13.465205 46.254951, 13.465098 46.25521, 13.46514 46.255401, 13.465441 46.255554, 13.46572 46.255851, 13.465956 46.255897, 13.466106 46.255851, 13.466278 46.255615, 13.466535 46.255489, 13.466815 46.255447, 13.467286 46.255294, 13.467436 46.25521, 13.467694 46.254993, 13.467886 46.255058, 13.468423 46.255359, 13.468617 46.255424, 13.46881 46.255401, 13.469153 46.255294, 13.469389 46.255317, 13.469647 46.255424, 13.46999 46.255874, 13.470183 46.256004, 13.470698 46.256217, 13.470763 46.256431, 13.470719 46.256584, 13.470741 46.256862, 13.470827 46.256946, 13.470955 46.256969, 13.471213 46.256946, 13.471749 46.256774, 13.472071 46.256645, 13.472308 46.256477, 13.472543 46.256366, 13.472693 46.25624, 13.472887 46.256134, 13.473166 46.256176, 13.473402 46.256282, 13.473595 46.256366, 13.473767 46.256366, 13.473874 46.256347, 13.474024 46.256259, 13.474368 46.255958, 13.474646 46.255851, 13.47501 46.255939, 13.475461 46.256195, 13.475934 46.256389, 13.476126 46.256431, 13.476191 46.256195, 13.476342 46.255981, 13.476706 46.255897, 13.477458 46.255809, 13.477972 46.255874, 13.478466 46.255874, 13.479044 46.255744, 13.479346 46.255832, 13.479667 46.255958, 13.479968 46.255916, 13.480654 46.255874, 13.481427 46.255725, 13.481813 46.255725, 13.482543 46.255809, 13.482886 46.255981, 13.483078 46.256046, 13.483229 46.256069, 13.483508 46.256069, 13.48413 46.255939, 13.484431 46.255851, 13.484795 46.255851, 13.485332 46.255958, 13.485718 46.256111, 13.485955 46.256176, 13.48619 46.256195, 13.486426 46.256195, 13.487027 46.256111, 13.487306 46.256023, 13.487607 46.256088, 13.487885 46.256176, 13.488101 46.256153, 13.488444 46.255958, 13.488679 46.255744, 13.489388 46.255466, 13.489773 46.255424, 13.490632 46.255508, 13.490911 46.255466, 13.49134 46.255359, 13.491683 46.255294, 13.492777 46.254993, 13.493035 46.254909, 13.493937 46.254673, 13.49413 46.254802, 13.494387 46.255165, 13.49458 46.255229, 13.494751 46.255187, 13.494967 46.25476, 13.495074 46.254631, 13.495911 46.254695, 13.49619 46.254737, 13.496554 46.254737, 13.496897 46.254779, 13.497155 46.254886, 13.497563 46.255229)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void test2H00() throws Exception {
    Route route=test(33869, 34466, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(0.0));

    logger.info("Found route="+route);

    String result="LINESTRING (14.476098 46.075824, 14.477363 46.075565, 14.477492 46.075523, 14.477942 46.074943, 14.478501 46.074256, 14.478629 46.074108, 14.478758 46.074062, 14.47996 46.073829, 14.480196 46.073635, 14.481526 46.073375, 14.481225 46.072605, 14.481268 46.07254, 14.48174 46.07228, 14.481934 46.072433, 14.48247 46.071876, 14.482878 46.071468, 14.483307 46.071189, 14.484509 46.070674, 14.48526 46.070373, 14.486483 46.069858, 14.487062 46.069644, 14.487277 46.069557, 14.487469 46.06945, 14.487706 46.06932, 14.488092 46.069, 14.488629 46.068443, 14.490324 46.066852, 14.490817 46.067111, 14.491032 46.067195, 14.491547 46.067455, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.501803 46.054279)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void test2H10() throws Exception {
    Route route=test(33869, 34466, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(1.0));

    logger.info("Found route="+route);

    String result="LINESTRING (14.476098 46.075824, 14.477363 46.075565, 14.477492 46.075523, 14.477942 46.074943, 14.478501 46.074256, 14.478629 46.074108, 14.478758 46.074062, 14.47996 46.073829, 14.480196 46.073635, 14.481526 46.073375, 14.481225 46.072605, 14.481268 46.07254, 14.48174 46.07228, 14.481934 46.072433, 14.48247 46.071876, 14.482878 46.071468, 14.483307 46.071189, 14.484509 46.070674, 14.48526 46.070373, 14.486483 46.069858, 14.487062 46.069644, 14.487277 46.069557, 14.487469 46.06945, 14.487706 46.06932, 14.488092 46.069, 14.488629 46.068443, 14.490324 46.066852, 14.490817 46.067111, 14.491032 46.067195, 14.491547 46.067455, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.501803 46.054279)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void test2H12() throws Exception {
    Route route=test(33869, 34466, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(1.2));

    logger.info("Found route="+route);

    String result="LINESTRING (14.476098 46.075824, 14.477363 46.075565, 14.477492 46.075523, 14.477942 46.074943, 14.478501 46.074256, 14.478629 46.074108, 14.478758 46.074062, 14.47996 46.073829, 14.480196 46.073635, 14.481526 46.073375, 14.481225 46.072605, 14.481268 46.07254, 14.48174 46.07228, 14.481934 46.072433, 14.48247 46.071876, 14.482878 46.071468, 14.483307 46.071189, 14.484509 46.070674, 14.48526 46.070373, 14.486483 46.069858, 14.487062 46.069644, 14.487277 46.069557, 14.487469 46.06945, 14.487706 46.06932, 14.488092 46.069, 14.488629 46.068443, 14.490324 46.066852, 14.489615 46.066509, 14.48923 46.066402, 14.489851 46.066188, 14.490624 46.06578, 14.491568 46.065223, 14.492383 46.064815, 14.492555 46.064708, 14.492749 46.064514, 14.492856 46.064323, 14.492898 46.064193, 14.493156 46.063934, 14.494036 46.063163, 14.49438 46.063335, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.501803 46.054279)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void test2H20() throws Exception {
    Route route=test(33869, 34466, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(2.0));

    logger.info("Found route="+route);

    String result="LINESTRING (14.476098 46.075824, 14.477363 46.075565, 14.477492 46.075523, 14.477942 46.074943, 14.478501 46.074256, 14.478629 46.074108, 14.478758 46.074062, 14.47996 46.073829, 14.480196 46.073635, 14.481526 46.073375, 14.481225 46.072605, 14.481268 46.07254, 14.48174 46.07228, 14.481934 46.072433, 14.48247 46.071876, 14.482878 46.071468, 14.483307 46.071189, 14.484509 46.070674, 14.48526 46.070373, 14.486483 46.069858, 14.487062 46.069644, 14.487277 46.069557, 14.487469 46.06945, 14.487706 46.06932, 14.488092 46.069, 14.488629 46.068443, 14.490324 46.066852, 14.489615 46.066509, 14.48923 46.066402, 14.489851 46.066188, 14.490624 46.06578, 14.491568 46.065223, 14.492383 46.064815, 14.492555 46.064708, 14.492749 46.064514, 14.492856 46.064323, 14.492898 46.064193, 14.493156 46.063934, 14.494036 46.063163, 14.494529 46.06269, 14.494787 46.0625, 14.495044 46.06224, 14.495259 46.061962, 14.495345 46.061748, 14.49528 46.061382, 14.495194 46.061103, 14.49498 46.060783, 14.494851 46.060672, 14.494637 46.060565, 14.494637 46.060501, 14.495473 46.060695, 14.495709 46.060783, 14.495817 46.060802, 14.496117 46.060825, 14.496439 46.06076, 14.496697 46.060565, 14.497813 46.059577, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.501803 46.054279)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void test2SpeedH12() throws Exception {
    Map<Level, Double> speedMap=new HashMap<Level, Double>();
    speedMap.put(Level.UNKNOWN, 100/70.0);
    speedMap.put(Level.MAJOR, 100/110.0);
    speedMap.put(Level.MEDIUM, 100/65.0);
    speedMap.put(Level.MINOR, 100/55.0);

    Route route=test(33869, 34466, new LevelLengthCostFunction(speedMap),
      new GreatCircleDistanceHeuristicFunction(1.2));

    logger.info("Found route="+route);

    String result="LINESTRING (14.476098 46.075824, 14.477363 46.075565, 14.477492 46.075523, 14.477942 46.074943, 14.478501 46.074256, 14.478629 46.074108, 14.478758 46.074062, 14.47996 46.073829, 14.480196 46.073635, 14.481526 46.073375, 14.481547 46.073463, 14.481826 46.073421, 14.481869 46.073356, 14.482427 46.07286, 14.483006 46.073333, 14.483521 46.07357, 14.485152 46.073635, 14.485796 46.073699, 14.486569 46.072883, 14.487062 46.072368, 14.488607 46.070674, 14.489745 46.069473, 14.491354 46.06784, 14.491633 46.06752, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.501803 46.054279)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testMiddleLong1SpeedH12() throws Exception {
    Map<Level, Double> speedMap=new HashMap<Level, Double>();
    speedMap.put(Level.UNKNOWN, 70/70.0);
    speedMap.put(Level.MAJOR, 70/110.0);
    speedMap.put(Level.MEDIUM, 70/65.0);
    speedMap.put(Level.MINOR, 70/55.0);

    Route route=test(38574, 17774, new LevelLengthCostFunction(speedMap),
      new GreatCircleDistanceHeuristicFunction(1.2));

    logger.info("Found route="+route);

    String result="LINESTRING (15.102147 46.369964, 15.102576 46.369728, 15.102919 46.369491, 15.103606 46.36885, 15.103777 46.368721, 15.103863 46.368614, 15.104763 46.367649, 15.105193 46.367153, 15.105558 46.36679, 15.105666 46.366638, 15.107017 46.365093, 15.107275 46.364921, 15.106802 46.364536, 15.106245 46.364105, 15.106159 46.363807, 15.106181 46.363548, 15.10648 46.362991, 15.106566 46.362819, 15.10661 46.362625, 15.106545 46.362327, 15.106502 46.362155, 15.106545 46.361938, 15.106781 46.361724, 15.107125 46.361511, 15.107296 46.361316, 15.10736 46.361125, 15.107424 46.360801, 15.107575 46.360179, 15.107747 46.359622, 15.107897 46.359214, 15.108047 46.358699, 15.108948 46.358612, 15.109334 46.358528, 15.109678 46.358421, 15.110021 46.358291, 15.110579 46.358036, 15.110879 46.357841, 15.111416 46.357582, 15.112167 46.357284, 15.112746 46.357112, 15.11266 46.356941, 15.112639 46.356811, 15.112832 46.356575, 15.113219 46.356208, 15.113283 46.356082, 15.113305 46.355609, 15.113261 46.355415, 15.112639 46.35445, 15.111716 46.351921, 15.111416 46.350997, 15.111115 46.350246, 15.110879 46.349967, 15.110536 46.349731, 15.110064 46.349536, 15.109077 46.3493, 15.108927 46.349235, 15.108648 46.349063, 15.108626 46.348892, 15.108648 46.348785, 15.108691 46.348613, 15.108454 46.348487, 15.108262 46.348293, 15.10809 46.348056, 15.107897 46.347862, 15.107682 46.34769, 15.107403 46.347606, 15.106717 46.347564, 15.106137 46.347347, 15.106073 46.34692, 15.106116 46.346576, 15.106073 46.346233, 15.106095 46.345954, 15.105751 46.345848, 15.105215 46.34576, 15.104871 46.345741, 15.104614 46.345783, 15.104334 46.345695, 15.104185 46.345676, 15.103755 46.345676, 15.103541 46.345611, 15.103434 46.345523, 15.103369 46.345397, 15.103155 46.345161, 15.102875 46.344837, 15.102704 46.344535, 15.102296 46.34428, 15.102168 46.34415, 15.102125 46.344001, 15.101824 46.343723, 15.101567 46.343616, 15.101352 46.343444, 15.10133 46.343315, 15.101244 46.343162, 15.101288 46.343055, 15.10146 46.342971, 15.101738 46.342929, 15.102662 46.342929, 15.103476 46.342758, 15.103755 46.34267, 15.103819 46.342586, 15.103841 46.342414, 15.103648 46.341426, 15.103692 46.340847, 15.103262 46.340805, 15.102789 46.340805, 15.101632 46.34074, 15.101052 46.34074, 15.100151 46.340675, 15.099378 46.340591, 15.098799 46.340503, 15.098198 46.340374, 15.097983 46.34029, 15.097447 46.339988, 15.097254 46.339775, 15.097039 46.339603, 15.096825 46.339279, 15.096695 46.339218, 15.096589 46.339, 15.096417 46.338764, 15.09646 46.338573, 15.096568 46.338401, 15.09676 46.338165, 15.096911 46.337844, 15.096997 46.337562, 15.097104 46.337112, 15.097275 46.336448, 15.097576 46.335395, 15.097769 46.334987, 15.09852 46.334194, 15.099013 46.333831, 15.099699 46.333251, 15.100258 46.332843, 15.100644 46.3325, 15.100666 46.332412, 15.100644 46.332286, 15.100472 46.332157, 15.099593 46.33179, 15.098799 46.331489, 15.097361 46.330997, 15.096889 46.33076, 15.096695 46.330612, 15.096653 46.330417, 15.096717 46.330245, 15.096825 46.330009, 15.096803 46.329795, 15.096546 46.329322, 15.096503 46.329132, 15.096524 46.328979, 15.09676 46.328422, 15.096739 46.32825, 15.096589 46.327842, 15.096674 46.327369, 15.096159 46.326641, 15.095837 46.326339, 15.095387 46.326084, 15.095194 46.325847, 15.095087 46.325634, 15.095129 46.325332, 15.095366 46.325141, 15.095816 46.324966, 15.095967 46.324752, 15.095881 46.32439, 15.095902 46.324001, 15.095837 46.32381, 15.095473 46.323486, 15.09543 46.323314, 15.095473 46.323081, 15.095665 46.322544, 15.09573 46.322177, 15.095665 46.321922, 15.095602 46.321792, 15.095494 46.321662, 15.095023 46.321254, 15.094807 46.321041, 15.094743 46.320911, 15.094614 46.320739, 15.094357 46.319927, 15.094314 46.319881, 15.0944 46.318962, 15.094336 46.318744, 15.093971 46.318252, 15.093928 46.318058, 15.094014 46.317821, 15.094057 46.317607, 15.093971 46.317371, 15.093713 46.317008, 15.093585 46.316772, 15.093413 46.316577, 15.093134 46.316406, 15.092426 46.316085, 15.092232 46.315979, 15.092168 46.315849, 15.092146 46.315696, 15.092276 46.315399, 15.092232 46.315162, 15.092125 46.314949, 15.091954 46.314712, 15.091675 46.314498, 15.091482 46.314369, 15.091224 46.314239, 15.091074 46.31409, 15.090817 46.313961, 15.090559 46.313854, 15.090237 46.313766, 15.090044 46.313747, 15.089851 46.313659, 15.089615 46.313404, 15.0894 46.313144, 15.089207 46.31306, 15.088478 46.312889, 15.088263 46.312801, 15.088091 46.312671, 15.088006 46.3125, 15.088156 46.312072, 15.087963 46.311599, 15.087877 46.310955, 15.087769 46.310398, 15.087683 46.310119, 15.087683 46.309906, 15.087813 46.309711, 15.088112 46.309368, 15.088306 46.309005, 15.088414 46.308662, 15.088414 46.308597, 15.088606 46.308082, 15.088843 46.307693, 15.089743 46.306556, 15.089894 46.306278, 15.090001 46.306041, 15.090172 46.304283, 15.090086 46.303939, 15.08998 46.303638, 15.089787 46.303253, 15.089722 46.302993, 15.089743 46.302761, 15.089808 46.302524, 15.089808 46.302265, 15.089636 46.30162, 15.089657 46.301406, 15.089873 46.301086, 15.090065 46.300678, 15.090194 46.300483, 15.090216 46.300376, 15.090108 46.300098, 15.089915 46.299778, 15.089679 46.299453, 15.089015 46.298961, 15.088606 46.298316, 15.08837 46.297801, 15.088156 46.297565, 15.087941 46.297374, 15.08747 46.297031, 15.087254 46.296813, 15.086546 46.295848, 15.086461 46.295291, 15.086396 46.295097, 15.085903 46.294754, 15.085495 46.29441, 15.085259 46.294155, 15.08513 46.293899, 15.085108 46.293556, 15.085173 46.293319, 15.085259 46.293125, 15.085989 46.292331, 15.086546 46.291839, 15.086869 46.291496, 15.08704 46.291236, 15.08734 46.290443, 15.087405 46.290077, 15.087212 46.28984, 15.085645 46.28862, 15.084873 46.28804, 15.084229 46.287525, 15.0841 46.28746, 15.083027 46.286643, 15.082727 46.286388, 15.08232 46.286109, 15.081805 46.285678, 15.081225 46.28527, 15.080302 46.284713, 15.07895 46.283962, 15.077469 46.283317, 15.076741 46.282932, 15.076247 46.282783, 15.075882 46.282695, 15.075474 46.282524, 15.075217 46.282394, 15.075066 46.282226, 15.074767 46.281967, 15.074509 46.281795, 15.074101 46.28244, 15.073822 46.28276, 15.073565 46.282955, 15.073222 46.283145, 15.07292 46.283317, 15.072299 46.283706, 15.07189 46.28405, 15.071633 46.284221, 15.071397 46.284263, 15.070904 46.284286, 15.070561 46.284263, 15.070324 46.284286, 15.069617 46.284519, 15.068844 46.284778, 15.068007 46.285121, 15.067234 46.285335, 15.066956 46.285141, 15.066247 46.284542, 15.065904 46.28424, 15.06511 46.283641, 15.064788 46.28347, 15.064553 46.283382, 15.064059 46.283145, 15.062792 46.282611, 15.06187 46.282203, 15.060926 46.281753, 15.061355 46.280487, 15.061463 46.28012, 15.061635 46.279647, 15.061698 46.279479, 15.062042 46.278556, 15.062042 46.278469, 15.06202 46.278297, 15.06187 46.278018, 15.06172 46.27787, 15.061269 46.27761, 15.060797 46.277397, 15.060347 46.277267, 15.059746 46.27703, 15.061141 46.27581, 15.061441 46.275615, 15.062042 46.275272, 15.062535 46.274951, 15.062922 46.274456, 15.063115 46.274112, 15.063286 46.273876, 15.063694 46.273082, 15.064617 46.272052, 15.064852 46.27169, 15.065003 46.27098, 15.065003 46.270767, 15.065068 46.270378, 15.065175 46.270099, 15.065281 46.269844, 15.065475 46.269565, 15.065625 46.269393, 15.06629 46.268836, 15.066333 46.268791, 15.067471 46.267784, 15.067599 46.267631, 15.067729 46.267482, 15.068158 46.266838, 15.068371 46.266368, 15.068651 46.265617, 15.069401 46.263298, 15.069745 46.262355, 15.070024 46.261753, 15.070345 46.261238, 15.070561 46.260894, 15.070647 46.260704, 15.071183 46.25978, 15.07114 46.259758, 15.069445 46.259479, 15.068393 46.25933, 15.067835 46.259201, 15.066699 46.259159, 15.065432 46.259136, 15.061827 46.259006, 15.059918 46.258964, 15.058243 46.258899, 15.05775 46.258899, 15.055604 46.258792, 15.054961 46.25875, 15.053566 46.258621, 15.052944 46.258579, 15.051656 46.258449, 15.050776 46.258342, 15.049059 46.258106, 15.046614 46.25772, 15.045133 46.257507, 15.044146 46.257377, 15.043995 46.257377, 15.042515 46.257205, 15.040584 46.257053, 15.037087 46.256881, 15.036765 46.256881, 15.034147 46.256881, 15.032881 46.256927, 15.029919 46.257076, 15.027838 46.25727, 15.023633 46.257675, 15.021208 46.257934, 15.019556 46.258083, 15.016959 46.258277, 15.013934 46.258384, 15.01241 46.258384, 15.010607 46.258342, 15.009062 46.258277, 15.008698 46.258277, 15.006188 46.258083, 15.004321 46.257892, 15.001595 46.257568, 14.999986 46.257396, 14.998227 46.257141, 14.995866 46.256732, 14.993721 46.256412, 14.991425 46.256134, 14.989022 46.255916, 14.984257 46.255661, 14.983721 46.255638, 14.98312 46.255554, 14.982648 46.255508, 14.981189 46.255271, 14.979601 46.254886, 14.978099 46.254436, 14.976941 46.254009, 14.975461 46.253322, 14.974924 46.253063, 14.974022 46.252525, 14.973572 46.252227, 14.972821 46.251861, 14.972477 46.251754, 14.972199 46.251731, 14.971812 46.251754, 14.971362 46.251754, 14.971147 46.251712, 14.971018 46.251647, 14.970912 46.251625, 14.970761 46.25156, 14.970524 46.251369, 14.970438 46.251262, 14.970397 46.251091, 14.970397 46.250961, 14.970438 46.250747, 14.970524 46.250595, 14.969988 46.250381, 14.969688 46.250232, 14.96958 46.250144, 14.968937 46.249759, 14.968358 46.249458, 14.967307 46.24905, 14.965848 46.248558, 14.965203 46.248321, 14.964409 46.247955, 14.9634 46.247398, 14.962414 46.246864, 14.961728 46.246521, 14.961105 46.246261, 14.960698 46.246067, 14.960225 46.245834, 14.959689 46.245597, 14.957779 46.244716, 14.956942 46.244331, 14.956341 46.24403, 14.954839 46.243343, 14.953294 46.242614, 14.950913 46.24139, 14.949904 46.240684, 14.949604 46.240425, 14.949238 46.240039, 14.948745 46.239437, 14.948424 46.238967, 14.947865 46.237743, 14.947651 46.237075, 14.94735 46.236434, 14.9472 46.236045, 14.946879 46.235359, 14.946213 46.23418, 14.945698 46.233257, 14.94542 46.232593, 14.945226 46.23225, 14.944861 46.231239, 14.944025 46.229053, 14.94351 46.227851, 14.943274 46.227893, 14.942931 46.227893, 14.942544 46.227787, 14.942136 46.227527, 14.941879 46.227401, 14.941535 46.227314, 14.940634 46.227249, 14.939862 46.227272, 14.938853 46.227272, 14.938509 46.227249, 14.93808 46.227184, 14.937523 46.227058, 14.937072 46.226905, 14.936106 46.226627, 14.93542 46.226478, 14.932523 46.22594, 14.931815 46.225791, 14.930506 46.225425, 14.929519 46.225105, 14.928275 46.224716, 14.927395 46.224502, 14.926836 46.224437, 14.925549 46.224395, 14.924927 46.224266, 14.923683 46.223987, 14.923125 46.223922, 14.922287 46.22388, 14.920872 46.223922, 14.918833 46.224029, 14.918297 46.224029, 14.917481 46.22388, 14.916902 46.223686, 14.916559 46.223495, 14.91613 46.223194, 14.915529 46.222679, 14.915271 46.222507, 14.914692 46.222229, 14.91407 46.221969, 14.913083 46.221519, 14.91231 46.221134, 14.912138 46.220985, 14.911924 46.220771, 14.911581 46.220169, 14.911173 46.219654, 14.910508 46.219097, 14.909778 46.218559, 14.90922 46.218109, 14.908898 46.217849, 14.908513 46.217617, 14.908083 46.217399, 14.90761 46.21725, 14.907032 46.217144, 14.904091 46.216693, 14.903534 46.216587, 14.901753 46.216178, 14.901345 46.216114, 14.900057 46.216072, 14.899329 46.215877, 14.899071 46.215877, 14.898792 46.215965, 14.898277 46.216156, 14.898127 46.216178, 14.897097 46.216178, 14.895874 46.216285, 14.895466 46.21635, 14.895037 46.216369, 14.892291 46.216369, 14.891303 46.216415, 14.890895 46.216457, 14.890358 46.216541, 14.889586 46.216606, 14.888621 46.216629, 14.887742 46.21667, 14.887033 46.216758, 14.886904 46.216758, 14.886839 46.216777, 14.88551 46.21693, 14.884909 46.21693, 14.883921 46.216819, 14.883063 46.216606, 14.88184 46.216178, 14.881218 46.216007, 14.880402 46.2159, 14.877442 46.215835, 14.876282 46.215835, 14.875767 46.215877, 14.875403 46.215942, 14.874845 46.216026, 14.874545 46.216049, 14.874201 46.216007, 14.873707 46.2159, 14.873621 46.215854, 14.873408 46.215747, 14.872592 46.215255, 14.871863 46.215103, 14.871498 46.215084, 14.870811 46.214977, 14.868472 46.21429, 14.868001 46.214183, 14.867206 46.21416, 14.865511 46.214031, 14.86476 46.214031, 14.863881 46.214096, 14.863022 46.214118, 14.862034 46.214073, 14.86094 46.214138, 14.860597 46.214054, 14.860082 46.213882, 14.859717 46.213729, 14.859417 46.213668, 14.859074 46.213668, 14.858065 46.213775, 14.856242 46.213752, 14.855662 46.213794, 14.854975 46.213924, 14.854697 46.214054, 14.854374 46.214183, 14.854138 46.214309, 14.853645 46.214504, 14.853216 46.214591, 14.851134 46.214698, 14.849911 46.214698, 14.848881 46.214611, 14.848044 46.214504, 14.847529 46.214504, 14.847144 46.214611, 14.8468 46.21474, 14.846457 46.214954, 14.845877 46.215362, 14.845641 46.215492, 14.844332 46.216156, 14.84386 46.216308, 14.843646 46.21635, 14.843237 46.216392, 14.842916 46.216369, 14.842616 46.216327, 14.842144 46.216156, 14.841435 46.21577, 14.841199 46.215663, 14.840856 46.215576, 14.840491 46.215557, 14.840106 46.215599, 14.839676 46.215705, 14.839203 46.215789, 14.838646 46.215854, 14.837573 46.216007, 14.8368 46.216072, 14.836415 46.216049, 14.835277 46.215877, 14.834719 46.215812, 14.834355 46.215705, 14.833882 46.215557, 14.833475 46.215362, 14.833045 46.21519, 14.832638 46.215019, 14.832209 46.214889, 14.83193 46.214847, 14.831629 46.214847, 14.831265 46.21487, 14.829978 46.215042, 14.82914 46.215126, 14.828668 46.215103, 14.827896 46.214912, 14.827595 46.214805, 14.826844 46.214611, 14.826329 46.214546, 14.826115 46.214527, 14.825728 46.214546, 14.825299 46.214591, 14.824784 46.214675, 14.823926 46.214912, 14.82296 46.215148, 14.822552 46.215232, 14.822188 46.215255, 14.821522 46.215168, 14.8209 46.215148, 14.819977 46.215168, 14.819656 46.215126, 14.818841 46.214977, 14.818154 46.214912, 14.816931 46.21474, 14.816823 46.21474, 14.816094 46.214611, 14.815171 46.214416, 14.814913 46.214374, 14.813605 46.21429, 14.813262 46.214225, 14.811888 46.214031, 14.811459 46.213989, 14.810515 46.214012, 14.809292 46.214073, 14.808905 46.214012, 14.80852 46.213859, 14.808283 46.213729, 14.807875 46.213687, 14.807511 46.213752, 14.807274 46.213882, 14.807189 46.214054, 14.807339 46.214504, 14.807232 46.214675, 14.806931 46.214824, 14.805602 46.215255, 14.805386 46.215297, 14.805001 46.21532, 14.804678 46.215297, 14.802897 46.215255, 14.80221 46.215148, 14.80176 46.215103, 14.801374 46.215126, 14.800967 46.215213, 14.800538 46.215339, 14.80015 46.215492, 14.7997 46.215621, 14.79925 46.215705, 14.798176 46.215812, 14.797984 46.215812, 14.797575 46.215854, 14.797146 46.215877, 14.796868 46.215877, 14.79601 46.215942, 14.795366 46.216007, 14.79498 46.216072, 14.79395 46.216327, 14.793456 46.216541, 14.793242 46.21667, 14.793026 46.216842, 14.792511 46.217636, 14.791354 46.218475, 14.791182 46.218688, 14.790967 46.218902, 14.790645 46.219181, 14.789958 46.219482, 14.789637 46.219589, 14.78925 46.21976, 14.788907 46.21989, 14.788199 46.220169, 14.787447 46.220428, 14.785731 46.22092, 14.785152 46.221199, 14.784701 46.221519, 14.784316 46.221755, 14.783671 46.222099, 14.783371 46.222206, 14.782663 46.222358, 14.782234 46.2224, 14.78129 46.2224, 14.78056 46.222377, 14.78011 46.222335, 14.779894 46.22227, 14.779745 46.222251, 14.77908 46.221862, 14.778479 46.221565, 14.778286 46.221435, 14.777599 46.221069, 14.776762 46.220813, 14.775002 46.220428, 14.774294 46.220298, 14.773565 46.220127, 14.77335 46.220104, 14.772728 46.219974, 14.771784 46.219783, 14.77026 46.219631, 14.769874 46.219589, 14.7676 46.219223, 14.765904 46.219055, 14.765068 46.218944, 14.764575 46.218925, 14.763866 46.218944, 14.763287 46.218925, 14.762235 46.218772, 14.761806 46.218772, 14.760668 46.218902, 14.760347 46.218879, 14.759789 46.218624, 14.759167 46.218368, 14.757643 46.218067, 14.755798 46.217807, 14.755261 46.217788, 14.75479 46.217849, 14.753309 46.218238, 14.752965 46.218303, 14.752129 46.218345, 14.751764 46.218322, 14.749424 46.218238, 14.747 46.218368, 14.74685 46.218387, 14.745991 46.218429, 14.744726 46.218536, 14.744211 46.218559, 14.743481 46.218559, 14.742859 46.218536, 14.742386 46.218494, 14.741057 46.21828, 14.739061 46.217872, 14.738975 46.217872, 14.73831 46.217681, 14.737216 46.21725, 14.736464 46.216735, 14.736292 46.216541, 14.735177 46.215232, 14.734919 46.214954, 14.734555 46.214633, 14.73419 46.214462, 14.733675 46.214355, 14.733139 46.214355, 14.732474 46.214397, 14.731765 46.214439, 14.731293 46.214504, 14.73095 46.214591, 14.730306 46.214847, 14.72962 46.215084, 14.728976 46.215278, 14.727688 46.215599, 14.726552 46.215835, 14.725113 46.216072, 14.724749 46.216072, 14.723719 46.216114, 14.723182 46.216197, 14.723053 46.21622, 14.722432 46.216197, 14.722023 46.216243, 14.721723 46.216262, 14.720757 46.216072, 14.72035 46.216049, 14.720028 46.216091, 14.719256 46.216369, 14.718848 46.216541, 14.718462 46.216629, 14.717753 46.21667, 14.717238 46.216777, 14.716767 46.216842, 14.716465 46.216842, 14.71535 46.216651, 14.714663 46.216564, 14.713827 46.216693, 14.713461 46.216777, 14.712968 46.21693, 14.712689 46.217037, 14.712453 46.217208, 14.712002 46.2177, 14.71181 46.217872, 14.711573 46.217979, 14.71093 46.218151, 14.709836 46.218475, 14.709492 46.218517, 14.70872 46.218429, 14.708591 46.21841, 14.707797 46.218303, 14.705781 46.218086, 14.705329 46.218086, 14.704878 46.218044, 14.703742 46.21783, 14.703527 46.217357, 14.703291 46.217144, 14.703398 46.216972, 14.703398 46.216777, 14.703355 46.216629, 14.703227 46.216564, 14.702755 46.216392, 14.702647 46.216327, 14.702347 46.215984, 14.702176 46.216007, 14.70196 46.216007, 14.701661 46.216114, 14.701081 46.216327, 14.700888 46.216285, 14.700758 46.216178, 14.701146 46.215362, 14.701445 46.215042, 14.701617 46.214805, 14.701725 46.214462, 14.701725 46.214183, 14.701617 46.213108, 14.701531 46.212699, 14.701424 46.212486, 14.701167 46.212165, 14.700866 46.211994, 14.700415 46.211822, 14.699343 46.211479, 14.698291 46.210769, 14.69739 46.210189, 14.697154 46.209999, 14.696703 46.209377, 14.696339 46.208755, 14.696082 46.208152, 14.695845 46.207725, 14.695523 46.207359, 14.694815 46.206478, 14.694579 46.206092, 14.694558 46.205898, 14.694579 46.205726, 14.694644 46.205642, 14.69503 46.205383, 14.695244 46.205276, 14.695459 46.205104, 14.695609 46.20502, 14.695652 46.204891, 14.695523 46.204784, 14.695201 46.204719, 14.694729 46.204505, 14.694558 46.204525, 14.6943 46.204742, 14.694043 46.204826, 14.69372 46.204742, 14.693313 46.204399, 14.692391 46.203948, 14.692197 46.203666, 14.692026 46.203517, 14.691768 46.203388, 14.691511 46.203067, 14.691854 46.202873, 14.692197 46.202743, 14.692412 46.202529, 14.692477 46.202293, 14.69269 46.202056, 14.693549 46.201351, 14.693828 46.201221, 14.694043 46.201091, 14.694214 46.200794, 14.694644 46.199871, 14.694858 46.199569, 14.694901 46.19942, 14.695179 46.199203, 14.695481 46.199012, 14.695716 46.198883, 14.696467 46.198581, 14.696639 46.19841, 14.696618 46.198261, 14.696553 46.198112, 14.696597 46.197895, 14.696597 46.197723, 14.69636 46.197059, 14.696038 46.196971, 14.694836 46.196823, 14.694644 46.196758, 14.694343 46.196693, 14.694193 46.196628, 14.69385 46.196414, 14.693635 46.196178, 14.693549 46.195964, 14.693571 46.195793, 14.69372 46.195514, 14.693871 46.195278, 14.694086 46.195064, 14.694235 46.194934, 14.694815 46.194656, 14.695158 46.194568, 14.695373 46.194442, 14.695459 46.194248, 14.695481 46.194053, 14.695395 46.193862, 14.694966 46.193603, 14.694429 46.193241, 14.694171 46.192787, 14.694022 46.192188, 14.693764 46.192146, 14.693442 46.191802, 14.693184 46.191631, 14.69297 46.191394, 14.692669 46.191288, 14.692412 46.191246, 14.692004 46.191223, 14.691682 46.191181, 14.691082 46.190856, 14.691039 46.19075, 14.690888 46.190559, 14.69063 46.190063, 14.690438 46.189849, 14.690094 46.189529, 14.6896 46.189163, 14.688786 46.188667, 14.687928 46.188217, 14.687562 46.187873, 14.687348 46.187595, 14.687262 46.187423, 14.687133 46.186439, 14.686962 46.186244, 14.686747 46.186096, 14.686469 46.185966, 14.686211 46.185813, 14.685932 46.185577, 14.685566 46.185321, 14.685309 46.185192, 14.684924 46.185066, 14.684473 46.184955, 14.683421 46.184783, 14.682906 46.184741, 14.68252 46.184635, 14.682284 46.184528, 14.681769 46.184333, 14.68134 46.18412, 14.680502 46.183734, 14.679902 46.183368, 14.679623 46.183284, 14.679387 46.183303, 14.679065 46.18341, 14.678636 46.183734, 14.678401 46.18386, 14.678185 46.184013, 14.677907 46.18412, 14.676834 46.184333, 14.676319 46.184463, 14.67589 46.184547, 14.675761 46.18457, 14.674796 46.184806, 14.674494 46.184848, 14.674259 46.184848, 14.674044 46.184806, 14.67383 46.184783, 14.673658 46.184741, 14.673615 46.184741, 14.673572 46.184722, 14.672671 46.184505, 14.672606 46.184268, 14.67252 46.18399, 14.672349 46.183734, 14.672221 46.183456, 14.672478 46.182769, 14.67252 46.182533, 14.672456 46.182296, 14.671919 46.181674, 14.671662 46.181415, 14.670975 46.180366, 14.670632 46.179935, 14.670482 46.179656, 14.670353 46.179336, 14.670311 46.179164, 14.67046 46.178497, 14.670611 46.178005, 14.670676 46.177703, 14.670611 46.177402, 14.67059 46.177253, 14.670418 46.176868, 14.670225 46.176544, 14.670075 46.176139, 14.669989 46.175815, 14.669796 46.175342, 14.669646 46.175064, 14.66956 46.174957, 14.669388 46.174591, 14.66943 46.174591, 14.669732 46.174591, 14.669967 46.174613, 14.670161 46.174678, 14.670825 46.17485, 14.671169 46.174957, 14.67207 46.17517, 14.672843 46.175277, 14.673293 46.1753, 14.673486 46.175258, 14.673658 46.17517, 14.67383 46.175022, 14.673915 46.174915, 14.673958 46.17472, 14.674065 46.174312, 14.674302 46.174057, 14.674559 46.173969, 14.674881 46.17395, 14.675482 46.17382, 14.675696 46.173713, 14.676125 46.173393, 14.676276 46.17324, 14.676684 46.17279, 14.676791 46.172595, 14.676941 46.172382, 14.677134 46.172275, 14.677714 46.171997, 14.678271 46.17176, 14.678442 46.171653, 14.678464 46.171394, 14.678336 46.171245, 14.677928 46.171051, 14.677584 46.170795, 14.677371 46.170581, 14.677285 46.170429, 14.677263 46.17028, 14.677091 46.169357, 14.677091 46.169078, 14.676919 46.16867, 14.676748 46.168476, 14.676468 46.168243, 14.676276 46.167984, 14.676147 46.167747, 14.676125 46.167167, 14.676169 46.16687, 14.676255 46.166526, 14.67619 46.166244, 14.676169 46.165901, 14.675696 46.165409, 14.675503 46.165195, 14.674967 46.16468, 14.674624 46.164249, 14.67458 46.164077, 14.674624 46.163822, 14.674817 46.163093, 14.67486 46.162811, 14.673293 46.162574, 14.672843 46.16249, 14.672049 46.162384, 14.67046 46.162231, 14.669495 46.16217, 14.669323 46.162189, 14.667178 46.162124, 14.665825 46.162017, 14.663873 46.161804, 14.663552 46.161762, 14.663358 46.161739, 14.662478 46.161655, 14.66029 46.16159, 14.658466 46.161609, 14.657757 46.161632, 14.656212 46.16159, 14.655633 46.161567, 14.653208 46.161437, 14.6522 46.161312, 14.650913 46.161075, 14.650827 46.161033, 14.650398 46.160945, 14.649476 46.16069, 14.647136 46.159915, 14.646513 46.15966, 14.644625 46.158798, 14.644304 46.158672, 14.643509 46.158435, 14.642888 46.158264, 14.642544 46.158199, 14.641858 46.158027, 14.639326 46.157382, 14.638617 46.157169, 14.638038 46.15702, 14.637415 46.156826, 14.637051 46.156738, 14.636751 46.156631, 14.63542 46.156227, 14.634562 46.15599, 14.633254 46.155582, 14.629305 46.154399, 14.628597 46.154167, 14.62806 46.153991, 14.627974 46.153972, 14.627138 46.153694, 14.626279 46.153434, 14.626151 46.153392, 14.625335 46.153137, 14.625056 46.153072, 14.624091 46.152748, 14.622954 46.152385, 14.622353 46.152214, 14.62098 46.151763, 14.616452 46.150344, 14.614821 46.149829, 14.61304 46.149272, 14.61289 46.14923, 14.612353 46.149101, 14.611731 46.148906, 14.610938 46.148609, 14.609908 46.148307, 14.608534 46.147941, 14.607397 46.147644, 14.606045 46.147319, 14.605036 46.147106, 14.604844 46.147083, 14.604092 46.146934, 14.601303 46.146503, 14.600487 46.146354, 14.598877 46.146095, 14.59772 46.145862, 14.596776 46.145584, 14.596474 46.145496, 14.595488 46.14513, 14.595273 46.145088, 14.595059 46.145088, 14.594908 46.145111, 14.59463 46.14524, 14.5945 46.145324, 14.594115 46.145366, 14.594093 46.145366, 14.594029 46.145347, 14.593899 46.145259, 14.593771 46.14513, 14.593707 46.144981, 14.593899 46.144489, 14.593985 46.144271, 14.594157 46.143608, 14.594307 46.142837, 14.594414 46.141933, 14.594372 46.141761, 14.594265 46.140861, 14.594115 46.139896, 14.594115 46.139488, 14.594136 46.138736, 14.593921 46.137855, 14.593964 46.137298, 14.594029 46.13702, 14.594157 46.136096, 14.594372 46.135025, 14.59463 46.133911, 14.594672 46.133567, 14.594822 46.133029, 14.595037 46.132148, 14.595101 46.131805, 14.595123 46.13084, 14.595015 46.129554, 14.595015 46.129402, 14.594973 46.127494, 14.594908 46.127063, 14.594887 46.126701, 14.594716 46.125949, 14.5945 46.125305, 14.594286 46.124595, 14.594136 46.123996, 14.593964 46.123458, 14.593835 46.123203, 14.593535 46.122558, 14.59332 46.122257, 14.593063 46.121871, 14.592955 46.121593, 14.592548 46.120262, 14.592548 46.120197, 14.592354 46.119167, 14.59229 46.118522, 14.592141 46.11771, 14.592097 46.117172, 14.592141 46.116313, 14.59229 46.114791, 14.592505 46.11301, 14.592591 46.112514, 14.592698 46.112472, 14.592913 46.11185, 14.592999 46.111484, 14.593171 46.110969, 14.593277 46.111057, 14.593428 46.111377, 14.5936 46.111442, 14.593985 46.111484, 14.594328 46.111595, 14.594779 46.111701, 14.595059 46.111827, 14.595358 46.111938)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testMiddleLong1RelevanceH12() throws Exception {
    Map<Level, Double> relevanceMap=new HashMap<Level, Double>();
    relevanceMap.put(Level.UNKNOWN, 1/1.0);
    relevanceMap.put(Level.MAJOR, 1/2.0);
    relevanceMap.put(Level.MEDIUM, 1/1.3);
    relevanceMap.put(Level.MINOR, 1/0.9);

    Route route=test(38574, 17774, new LevelLengthCostFunction(relevanceMap),
      new GreatCircleDistanceHeuristicFunction(1.2));

    logger.info("Found route="+route);

    String result="LINESTRING (15.102147 46.369964, 15.102576 46.369728, 15.102919 46.369491, 15.103606 46.36885, 15.103777 46.368721, 15.103863 46.368614, 15.104763 46.367649, 15.105193 46.367153, 15.105558 46.36679, 15.105666 46.366638, 15.107017 46.365093, 15.107275 46.364921, 15.106802 46.364536, 15.106245 46.364105, 15.106159 46.363807, 15.106181 46.363548, 15.10648 46.362991, 15.106566 46.362819, 15.10661 46.362625, 15.106545 46.362327, 15.106502 46.362155, 15.106545 46.361938, 15.106781 46.361724, 15.107125 46.361511, 15.107296 46.361316, 15.10736 46.361125, 15.107424 46.360801, 15.107575 46.360179, 15.107747 46.359622, 15.107897 46.359214, 15.108047 46.358699, 15.108948 46.358612, 15.109334 46.358528, 15.109678 46.358421, 15.110021 46.358291, 15.110579 46.358036, 15.110879 46.357841, 15.111416 46.357582, 15.112167 46.357284, 15.112746 46.357112, 15.11266 46.356941, 15.112639 46.356811, 15.112832 46.356575, 15.113219 46.356208, 15.113283 46.356082, 15.113305 46.355609, 15.113261 46.355415, 15.112639 46.35445, 15.111716 46.351921, 15.111416 46.350997, 15.111115 46.350246, 15.110879 46.349967, 15.110536 46.349731, 15.110064 46.349536, 15.109077 46.3493, 15.108927 46.349235, 15.108648 46.349063, 15.108626 46.348892, 15.108648 46.348785, 15.108691 46.348613, 15.108454 46.348487, 15.108262 46.348293, 15.10809 46.348056, 15.107897 46.347862, 15.107682 46.34769, 15.107403 46.347606, 15.106717 46.347564, 15.106137 46.347347, 15.106073 46.34692, 15.106116 46.346576, 15.106073 46.346233, 15.106095 46.345954, 15.105751 46.345848, 15.105215 46.34576, 15.104871 46.345741, 15.104614 46.345783, 15.104334 46.345695, 15.104185 46.345676, 15.103755 46.345676, 15.103541 46.345611, 15.103434 46.345523, 15.103369 46.345397, 15.103155 46.345161, 15.102875 46.344837, 15.102704 46.344535, 15.102296 46.34428, 15.102168 46.34415, 15.102125 46.344001, 15.101824 46.343723, 15.101567 46.343616, 15.101352 46.343444, 15.10133 46.343315, 15.101244 46.343162, 15.101288 46.343055, 15.10146 46.342971, 15.101738 46.342929, 15.102662 46.342929, 15.103476 46.342758, 15.103755 46.34267, 15.103819 46.342586, 15.103841 46.342414, 15.103648 46.341426, 15.103692 46.340847, 15.103262 46.340805, 15.102789 46.340805, 15.101632 46.34074, 15.101052 46.34074, 15.100151 46.340675, 15.099378 46.340591, 15.098799 46.340503, 15.098198 46.340374, 15.097983 46.34029, 15.097447 46.339988, 15.097254 46.339775, 15.097039 46.339603, 15.096825 46.339279, 15.096695 46.339218, 15.096589 46.339, 15.096417 46.338764, 15.09646 46.338573, 15.096568 46.338401, 15.09676 46.338165, 15.096911 46.337844, 15.096997 46.337562, 15.097104 46.337112, 15.097275 46.336448, 15.097576 46.335395, 15.097769 46.334987, 15.09852 46.334194, 15.099013 46.333831, 15.099699 46.333251, 15.100258 46.332843, 15.100644 46.3325, 15.100666 46.332412, 15.100644 46.332286, 15.100472 46.332157, 15.099593 46.33179, 15.098799 46.331489, 15.097361 46.330997, 15.096889 46.33076, 15.096695 46.330612, 15.096653 46.330417, 15.096717 46.330245, 15.096825 46.330009, 15.096803 46.329795, 15.096546 46.329322, 15.096503 46.329132, 15.096524 46.328979, 15.09676 46.328422, 15.096739 46.32825, 15.096589 46.327842, 15.096674 46.327369, 15.096159 46.326641, 15.095837 46.326339, 15.095387 46.326084, 15.095194 46.325847, 15.095087 46.325634, 15.095129 46.325332, 15.095366 46.325141, 15.095816 46.324966, 15.095967 46.324752, 15.095881 46.32439, 15.095902 46.324001, 15.095837 46.32381, 15.095473 46.323486, 15.09543 46.323314, 15.095473 46.323081, 15.095665 46.322544, 15.09573 46.322177, 15.095665 46.321922, 15.095602 46.321792, 15.095494 46.321662, 15.095023 46.321254, 15.094807 46.321041, 15.094743 46.320911, 15.094614 46.320739, 15.094357 46.319927, 15.094314 46.319881, 15.0944 46.318962, 15.094336 46.318744, 15.093971 46.318252, 15.093928 46.318058, 15.094014 46.317821, 15.094057 46.317607, 15.093971 46.317371, 15.093713 46.317008, 15.093585 46.316772, 15.093413 46.316577, 15.093134 46.316406, 15.092426 46.316085, 15.092232 46.315979, 15.092168 46.315849, 15.092146 46.315696, 15.092276 46.315399, 15.092232 46.315162, 15.092125 46.314949, 15.091954 46.314712, 15.091675 46.314498, 15.091482 46.314369, 15.091224 46.314239, 15.091074 46.31409, 15.090817 46.313961, 15.090559 46.313854, 15.090237 46.313766, 15.090044 46.313747, 15.089851 46.313659, 15.089615 46.313404, 15.0894 46.313144, 15.089207 46.31306, 15.088478 46.312889, 15.088263 46.312801, 15.088091 46.312671, 15.088006 46.3125, 15.088156 46.312072, 15.087963 46.311599, 15.087877 46.310955, 15.087769 46.310398, 15.087683 46.310119, 15.087683 46.309906, 15.087813 46.309711, 15.088112 46.309368, 15.088306 46.309005, 15.088414 46.308662, 15.088414 46.308597, 15.088606 46.308082, 15.088843 46.307693, 15.089743 46.306556, 15.089894 46.306278, 15.090001 46.306041, 15.090172 46.304283, 15.090086 46.303939, 15.08998 46.303638, 15.089787 46.303253, 15.089722 46.302993, 15.089743 46.302761, 15.089808 46.302524, 15.089808 46.302265, 15.089636 46.30162, 15.089657 46.301406, 15.089873 46.301086, 15.090065 46.300678, 15.090194 46.300483, 15.090216 46.300376, 15.090108 46.300098, 15.089915 46.299778, 15.089679 46.299453, 15.089015 46.298961, 15.088606 46.298316, 15.08837 46.297801, 15.088156 46.297565, 15.087941 46.297374, 15.08747 46.297031, 15.087254 46.296813, 15.086546 46.295848, 15.086461 46.295291, 15.086396 46.295097, 15.085903 46.294754, 15.085495 46.29441, 15.085259 46.294155, 15.08513 46.293899, 15.085108 46.293556, 15.085173 46.293319, 15.085259 46.293125, 15.085989 46.292331, 15.086546 46.291839, 15.086869 46.291496, 15.08704 46.291236, 15.08734 46.290443, 15.087405 46.290077, 15.087212 46.28984, 15.085645 46.28862, 15.084873 46.28804, 15.084229 46.287525, 15.0841 46.28746, 15.083027 46.286643, 15.082727 46.286388, 15.08232 46.286109, 15.081805 46.285678, 15.081225 46.28527, 15.080302 46.284713, 15.07895 46.283962, 15.077469 46.283317, 15.076741 46.282932, 15.076247 46.282783, 15.075882 46.282695, 15.075474 46.282524, 15.075217 46.282394, 15.075066 46.282226, 15.074767 46.281967, 15.074509 46.281795, 15.074101 46.28244, 15.073822 46.28276, 15.073565 46.282955, 15.073222 46.283145, 15.07292 46.283317, 15.072299 46.283706, 15.07189 46.28405, 15.071633 46.284221, 15.071397 46.284263, 15.070904 46.284286, 15.070561 46.284263, 15.070324 46.284286, 15.069617 46.284519, 15.068844 46.284778, 15.068007 46.285121, 15.067234 46.285335, 15.066956 46.285141, 15.066247 46.284542, 15.065904 46.28424, 15.06511 46.283641, 15.064788 46.28347, 15.064553 46.283382, 15.064059 46.283145, 15.062792 46.282611, 15.06187 46.282203, 15.060926 46.281753, 15.060261 46.281475, 15.059424 46.281196, 15.059038 46.281085, 15.058566 46.281002, 15.057836 46.280723, 15.05745 46.280551, 15.056119 46.280056, 15.056269 46.279823, 15.056527 46.279563, 15.057536 46.278747, 15.058157 46.278297, 15.058694 46.277912, 15.059746 46.27703, 15.061141 46.27581, 15.061441 46.275615, 15.062042 46.275272, 15.062535 46.274951, 15.062922 46.274456, 15.063115 46.274112, 15.063286 46.273876, 15.063694 46.273082, 15.064617 46.272052, 15.064852 46.27169, 15.065003 46.27098, 15.065003 46.270767, 15.065068 46.270378, 15.065175 46.270099, 15.065281 46.269844, 15.065475 46.269565, 15.065625 46.269393, 15.06629 46.268836, 15.066333 46.268791, 15.067471 46.267784, 15.067599 46.267631, 15.067729 46.267482, 15.068158 46.266838, 15.068371 46.266368, 15.068651 46.265617, 15.069401 46.263298, 15.069745 46.262355, 15.070024 46.261753, 15.070345 46.261238, 15.070561 46.260894, 15.070647 46.260704, 15.071183 46.25978, 15.07114 46.259758, 15.069445 46.259479, 15.068393 46.25933, 15.067835 46.259201, 15.066699 46.259159, 15.065432 46.259136, 15.061827 46.259006, 15.059918 46.258964, 15.058243 46.258899, 15.05775 46.258899, 15.055604 46.258792, 15.054961 46.25875, 15.053566 46.258621, 15.052944 46.258579, 15.051656 46.258449, 15.050776 46.258342, 15.049059 46.258106, 15.046614 46.25772, 15.045133 46.257507, 15.044146 46.257377, 15.043995 46.257377, 15.042515 46.257205, 15.040584 46.257053, 15.037087 46.256881, 15.036765 46.256881, 15.034147 46.256881, 15.032881 46.256927, 15.029919 46.257076, 15.027838 46.25727, 15.023633 46.257675, 15.021208 46.257934, 15.019556 46.258083, 15.016959 46.258277, 15.013934 46.258384, 15.01241 46.258384, 15.010607 46.258342, 15.009062 46.258277, 15.008698 46.258277, 15.006188 46.258083, 15.004321 46.257892, 15.001595 46.257568, 14.999986 46.257396, 14.998227 46.257141, 14.995866 46.256732, 14.993721 46.256412, 14.991425 46.256134, 14.989022 46.255916, 14.984257 46.255661, 14.983721 46.255638, 14.98312 46.255554, 14.982648 46.255508, 14.981189 46.255271, 14.979601 46.254886, 14.978099 46.254436, 14.976941 46.254009, 14.975461 46.253322, 14.974924 46.253063, 14.974022 46.252525, 14.973572 46.252227, 14.972821 46.251861, 14.972477 46.251754, 14.972199 46.251731, 14.971812 46.251754, 14.971362 46.251754, 14.971147 46.251712, 14.971018 46.251647, 14.970912 46.251625, 14.970761 46.25156, 14.970524 46.251369, 14.970438 46.251262, 14.970397 46.251091, 14.970397 46.250961, 14.970438 46.250747, 14.970524 46.250595, 14.969988 46.250381, 14.969688 46.250232, 14.96958 46.250144, 14.968937 46.249759, 14.968358 46.249458, 14.967307 46.24905, 14.965848 46.248558, 14.965203 46.248321, 14.964409 46.247955, 14.9634 46.247398, 14.962414 46.246864, 14.961728 46.246521, 14.961105 46.246261, 14.960698 46.246067, 14.960225 46.245834, 14.959689 46.245597, 14.957779 46.244716, 14.956942 46.244331, 14.956341 46.24403, 14.954839 46.243343, 14.953294 46.242614, 14.950913 46.24139, 14.949904 46.240684, 14.949604 46.240425, 14.949238 46.240039, 14.948745 46.239437, 14.948424 46.238967, 14.947865 46.237743, 14.947651 46.237075, 14.94735 46.236434, 14.9472 46.236045, 14.946879 46.235359, 14.946213 46.23418, 14.945698 46.233257, 14.94542 46.232593, 14.945226 46.23225, 14.944861 46.231239, 14.944025 46.229053, 14.94351 46.227851, 14.943274 46.227893, 14.942931 46.227893, 14.942544 46.227787, 14.942136 46.227527, 14.941879 46.227401, 14.941535 46.227314, 14.940634 46.227249, 14.939862 46.227272, 14.938853 46.227272, 14.938509 46.227249, 14.93808 46.227184, 14.937523 46.227058, 14.937072 46.226905, 14.936106 46.226627, 14.93542 46.226478, 14.932523 46.22594, 14.931815 46.225791, 14.930506 46.225425, 14.929519 46.225105, 14.928275 46.224716, 14.927395 46.224502, 14.926836 46.224437, 14.925549 46.224395, 14.924927 46.224266, 14.923683 46.223987, 14.923125 46.223922, 14.922287 46.22388, 14.920872 46.223922, 14.918833 46.224029, 14.918297 46.224029, 14.917481 46.22388, 14.916902 46.223686, 14.916559 46.223495, 14.91613 46.223194, 14.915529 46.222679, 14.915271 46.222507, 14.914692 46.222229, 14.91407 46.221969, 14.913083 46.221519, 14.91231 46.221134, 14.912138 46.220985, 14.911924 46.220771, 14.911581 46.220169, 14.911173 46.219654, 14.910508 46.219097, 14.909778 46.218559, 14.90922 46.218109, 14.908898 46.217849, 14.908513 46.217617, 14.908083 46.217399, 14.90761 46.21725, 14.907032 46.217144, 14.904091 46.216693, 14.903534 46.216587, 14.901753 46.216178, 14.901345 46.216114, 14.900057 46.216072, 14.899329 46.215877, 14.899071 46.215877, 14.898792 46.215965, 14.898277 46.216156, 14.898127 46.216178, 14.897097 46.216178, 14.895874 46.216285, 14.895466 46.21635, 14.895037 46.216369, 14.892291 46.216369, 14.891303 46.216415, 14.890895 46.216457, 14.890358 46.216541, 14.889586 46.216606, 14.888621 46.216629, 14.887742 46.21667, 14.887033 46.216758, 14.886904 46.216758, 14.886839 46.216777, 14.88551 46.21693, 14.884909 46.21693, 14.883921 46.216819, 14.883063 46.216606, 14.88184 46.216178, 14.881218 46.216007, 14.880402 46.2159, 14.877442 46.215835, 14.876282 46.215835, 14.875767 46.215877, 14.875403 46.215942, 14.874845 46.216026, 14.874545 46.216049, 14.874201 46.216007, 14.873707 46.2159, 14.873621 46.215854, 14.873408 46.215747, 14.872592 46.215255, 14.871863 46.215103, 14.871498 46.215084, 14.870811 46.214977, 14.868472 46.21429, 14.868001 46.214183, 14.867206 46.21416, 14.865511 46.214031, 14.86476 46.214031, 14.863881 46.214096, 14.863022 46.214118, 14.862034 46.214073, 14.86094 46.214138, 14.860597 46.214054, 14.860082 46.213882, 14.859717 46.213729, 14.859417 46.213668, 14.859074 46.213668, 14.858065 46.213775, 14.856242 46.213752, 14.855662 46.213794, 14.854975 46.213924, 14.854697 46.214054, 14.854374 46.214183, 14.854138 46.214309, 14.853645 46.214504, 14.853216 46.214591, 14.851134 46.214698, 14.849911 46.214698, 14.848881 46.214611, 14.848044 46.214504, 14.847529 46.214504, 14.847144 46.214611, 14.8468 46.21474, 14.846457 46.214954, 14.845877 46.215362, 14.845641 46.215492, 14.844332 46.216156, 14.84386 46.216308, 14.843646 46.21635, 14.843237 46.216392, 14.842916 46.216369, 14.842616 46.216327, 14.842144 46.216156, 14.841435 46.21577, 14.841199 46.215663, 14.840856 46.215576, 14.840491 46.215557, 14.840106 46.215599, 14.839676 46.215705, 14.839203 46.215789, 14.838646 46.215854, 14.837573 46.216007, 14.8368 46.216072, 14.836415 46.216049, 14.835277 46.215877, 14.834719 46.215812, 14.834355 46.215705, 14.833882 46.215557, 14.833475 46.215362, 14.833045 46.21519, 14.832638 46.215019, 14.832209 46.214889, 14.83193 46.214847, 14.831629 46.214847, 14.831265 46.21487, 14.829978 46.215042, 14.82914 46.215126, 14.828668 46.215103, 14.827896 46.214912, 14.827595 46.214805, 14.826844 46.214611, 14.826329 46.214546, 14.826115 46.214527, 14.825728 46.214546, 14.825299 46.214591, 14.824784 46.214675, 14.823926 46.214912, 14.82296 46.215148, 14.822552 46.215232, 14.822188 46.215255, 14.821522 46.215168, 14.8209 46.215148, 14.819977 46.215168, 14.819656 46.215126, 14.818841 46.214977, 14.818154 46.214912, 14.816931 46.21474, 14.816823 46.21474, 14.816094 46.214611, 14.815171 46.214416, 14.814913 46.214374, 14.813605 46.21429, 14.813262 46.214225, 14.811888 46.214031, 14.811459 46.213989, 14.810515 46.214012, 14.809292 46.214073, 14.808905 46.214012, 14.80852 46.213859, 14.808283 46.213729, 14.807875 46.213687, 14.807511 46.213752, 14.807274 46.213882, 14.807189 46.214054, 14.807339 46.214504, 14.807232 46.214675, 14.806931 46.214824, 14.805602 46.215255, 14.805386 46.215297, 14.805001 46.21532, 14.804678 46.215297, 14.802897 46.215255, 14.80221 46.215148, 14.80176 46.215103, 14.801374 46.215126, 14.800967 46.215213, 14.800538 46.215339, 14.80015 46.215492, 14.7997 46.215621, 14.79925 46.215705, 14.798176 46.215812, 14.797984 46.215812, 14.797575 46.215854, 14.797146 46.215877, 14.796868 46.215877, 14.79601 46.215942, 14.795366 46.216007, 14.79498 46.216072, 14.79395 46.216327, 14.793456 46.216541, 14.793242 46.21667, 14.793026 46.216842, 14.792511 46.217636, 14.791354 46.218475, 14.791182 46.218688, 14.790967 46.218902, 14.790645 46.219181, 14.789958 46.219482, 14.789637 46.219589, 14.78925 46.21976, 14.788907 46.21989, 14.788199 46.220169, 14.787447 46.220428, 14.785731 46.22092, 14.785152 46.221199, 14.784701 46.221519, 14.784316 46.221755, 14.783671 46.222099, 14.783371 46.222206, 14.782663 46.222358, 14.782234 46.2224, 14.78129 46.2224, 14.78056 46.222377, 14.78011 46.222335, 14.779894 46.22227, 14.779745 46.222251, 14.77908 46.221862, 14.778479 46.221565, 14.778286 46.221435, 14.777599 46.221069, 14.776762 46.220813, 14.775002 46.220428, 14.774294 46.220298, 14.773565 46.220127, 14.77335 46.220104, 14.772728 46.219974, 14.771784 46.219783, 14.77026 46.219631, 14.769874 46.219589, 14.7676 46.219223, 14.765904 46.219055, 14.765068 46.218944, 14.764575 46.218925, 14.763866 46.218944, 14.763287 46.218925, 14.762235 46.218772, 14.761806 46.218772, 14.760668 46.218902, 14.760347 46.218879, 14.759789 46.218624, 14.759167 46.218368, 14.757643 46.218067, 14.755798 46.217807, 14.755261 46.217788, 14.75479 46.217849, 14.753309 46.218238, 14.752965 46.218303, 14.752129 46.218345, 14.751764 46.218322, 14.749424 46.218238, 14.747 46.218368, 14.74685 46.218387, 14.745991 46.218429, 14.744726 46.218536, 14.744211 46.218559, 14.743481 46.218559, 14.742859 46.218536, 14.742386 46.218494, 14.741057 46.21828, 14.739061 46.217872, 14.738975 46.217872, 14.73831 46.217681, 14.737216 46.21725, 14.736464 46.216735, 14.736292 46.216541, 14.735177 46.215232, 14.734919 46.214954, 14.734555 46.214633, 14.73419 46.214462, 14.733675 46.214355, 14.733139 46.214355, 14.732474 46.214397, 14.731765 46.214439, 14.731293 46.214504, 14.73095 46.214591, 14.730306 46.214847, 14.72962 46.215084, 14.728976 46.215278, 14.727688 46.215599, 14.726552 46.215835, 14.725113 46.216072, 14.724749 46.216072, 14.723719 46.216114, 14.723182 46.216197, 14.723053 46.21622, 14.722432 46.216197, 14.722023 46.216243, 14.721723 46.216262, 14.720757 46.216072, 14.72035 46.216049, 14.720028 46.216091, 14.719256 46.216369, 14.718848 46.216541, 14.718462 46.216629, 14.717753 46.21667, 14.717238 46.216777, 14.716767 46.216842, 14.716465 46.216842, 14.71535 46.216651, 14.714663 46.216564, 14.713827 46.216693, 14.713461 46.216777, 14.712968 46.21693, 14.712689 46.217037, 14.712453 46.217208, 14.712002 46.2177, 14.71181 46.217872, 14.711573 46.217979, 14.71093 46.218151, 14.709836 46.218475, 14.709492 46.218517, 14.70872 46.218429, 14.708591 46.21841, 14.707797 46.218303, 14.705781 46.218086, 14.705329 46.218086, 14.704878 46.218044, 14.703742 46.21783, 14.703527 46.217357, 14.703291 46.217144, 14.703398 46.216972, 14.703398 46.216777, 14.703355 46.216629, 14.703227 46.216564, 14.702755 46.216392, 14.702647 46.216327, 14.702347 46.215984, 14.702176 46.216007, 14.70196 46.216007, 14.701661 46.216114, 14.701081 46.216327, 14.700888 46.216285, 14.700758 46.216178, 14.701146 46.215362, 14.701445 46.215042, 14.701617 46.214805, 14.701725 46.214462, 14.701725 46.214183, 14.701617 46.213108, 14.701531 46.212699, 14.701424 46.212486, 14.701167 46.212165, 14.700866 46.211994, 14.700415 46.211822, 14.699343 46.211479, 14.698291 46.210769, 14.69739 46.210189, 14.697154 46.209999, 14.696703 46.209377, 14.696339 46.208755, 14.696082 46.208152, 14.695845 46.207725, 14.695523 46.207359, 14.694815 46.206478, 14.694579 46.206092, 14.694558 46.205898, 14.694579 46.205726, 14.694644 46.205642, 14.69503 46.205383, 14.695244 46.205276, 14.695459 46.205104, 14.695609 46.20502, 14.695652 46.204891, 14.695523 46.204784, 14.695201 46.204719, 14.694729 46.204505, 14.694558 46.204525, 14.6943 46.204742, 14.694043 46.204826, 14.69372 46.204742, 14.693313 46.204399, 14.692391 46.203948, 14.692197 46.203666, 14.692026 46.203517, 14.691768 46.203388, 14.691511 46.203067, 14.691854 46.202873, 14.692197 46.202743, 14.692412 46.202529, 14.692477 46.202293, 14.69269 46.202056, 14.693549 46.201351, 14.693828 46.201221, 14.694043 46.201091, 14.694214 46.200794, 14.694644 46.199871, 14.694858 46.199569, 14.694901 46.19942, 14.695179 46.199203, 14.695481 46.199012, 14.695716 46.198883, 14.696467 46.198581, 14.696639 46.19841, 14.696618 46.198261, 14.696553 46.198112, 14.696597 46.197895, 14.696597 46.197723, 14.69636 46.197059, 14.696038 46.196971, 14.694836 46.196823, 14.694644 46.196758, 14.694343 46.196693, 14.694193 46.196628, 14.69385 46.196414, 14.693635 46.196178, 14.693549 46.195964, 14.693571 46.195793, 14.69372 46.195514, 14.693871 46.195278, 14.694086 46.195064, 14.694235 46.194934, 14.694815 46.194656, 14.695158 46.194568, 14.695373 46.194442, 14.695459 46.194248, 14.695481 46.194053, 14.695395 46.193862, 14.694966 46.193603, 14.694429 46.193241, 14.694171 46.192787, 14.694022 46.192188, 14.693764 46.192146, 14.693442 46.191802, 14.693184 46.191631, 14.69297 46.191394, 14.692669 46.191288, 14.692412 46.191246, 14.692004 46.191223, 14.691682 46.191181, 14.691082 46.190856, 14.691039 46.19075, 14.690888 46.190559, 14.69063 46.190063, 14.690438 46.189849, 14.690094 46.189529, 14.6896 46.189163, 14.688786 46.188667, 14.687928 46.188217, 14.687562 46.187873, 14.687348 46.187595, 14.687262 46.187423, 14.687133 46.186439, 14.686962 46.186244, 14.686747 46.186096, 14.686469 46.185966, 14.686211 46.185813, 14.685932 46.185577, 14.685566 46.185321, 14.685309 46.185192, 14.684924 46.185066, 14.684473 46.184955, 14.683421 46.184783, 14.682906 46.184741, 14.68252 46.184635, 14.682284 46.184528, 14.681769 46.184333, 14.68134 46.18412, 14.680502 46.183734, 14.679902 46.183368, 14.679623 46.183284, 14.679387 46.183303, 14.679065 46.18341, 14.678636 46.183734, 14.678401 46.18386, 14.678185 46.184013, 14.677907 46.18412, 14.676834 46.184333, 14.676319 46.184463, 14.67589 46.184547, 14.675761 46.18457, 14.674796 46.184806, 14.674494 46.184848, 14.674259 46.184848, 14.674044 46.184806, 14.67383 46.184783, 14.673658 46.184741, 14.673615 46.184741, 14.673572 46.184722, 14.672671 46.184505, 14.672606 46.184268, 14.67252 46.18399, 14.672349 46.183734, 14.672221 46.183456, 14.672478 46.182769, 14.67252 46.182533, 14.672456 46.182296, 14.671919 46.181674, 14.671662 46.181415, 14.670975 46.180366, 14.670632 46.179935, 14.670482 46.179656, 14.670353 46.179336, 14.670311 46.179164, 14.67046 46.178497, 14.670611 46.178005, 14.670676 46.177703, 14.670611 46.177402, 14.67059 46.177253, 14.670418 46.176868, 14.670225 46.176544, 14.670075 46.176139, 14.669989 46.175815, 14.669796 46.175342, 14.669646 46.175064, 14.66956 46.174957, 14.669388 46.174591, 14.66943 46.174591, 14.669732 46.174591, 14.669967 46.174613, 14.670161 46.174678, 14.670825 46.17485, 14.671169 46.174957, 14.67207 46.17517, 14.672843 46.175277, 14.673293 46.1753, 14.673486 46.175258, 14.673658 46.17517, 14.67383 46.175022, 14.673915 46.174915, 14.673958 46.17472, 14.674065 46.174312, 14.674302 46.174057, 14.674559 46.173969, 14.674881 46.17395, 14.675482 46.17382, 14.675696 46.173713, 14.676125 46.173393, 14.676276 46.17324, 14.676684 46.17279, 14.676791 46.172595, 14.676941 46.172382, 14.677134 46.172275, 14.677714 46.171997, 14.678271 46.17176, 14.678442 46.171653, 14.678464 46.171394, 14.678336 46.171245, 14.677928 46.171051, 14.677584 46.170795, 14.677371 46.170581, 14.677285 46.170429, 14.677263 46.17028, 14.677091 46.169357, 14.677091 46.169078, 14.676919 46.16867, 14.676748 46.168476, 14.676468 46.168243, 14.676276 46.167984, 14.676147 46.167747, 14.676125 46.167167, 14.676169 46.16687, 14.676255 46.166526, 14.67619 46.166244, 14.676169 46.165901, 14.675696 46.165409, 14.675503 46.165195, 14.674967 46.16468, 14.674624 46.164249, 14.67458 46.164077, 14.674624 46.163822, 14.674817 46.163093, 14.67486 46.162811, 14.673293 46.162574, 14.672843 46.16249, 14.672049 46.162384, 14.67046 46.162231, 14.669495 46.16217, 14.669323 46.162189, 14.667178 46.162124, 14.665825 46.162017, 14.663873 46.161804, 14.663552 46.161762, 14.663358 46.161739, 14.662478 46.161655, 14.66029 46.16159, 14.658466 46.161609, 14.657757 46.161632, 14.656212 46.16159, 14.655633 46.161567, 14.653208 46.161437, 14.6522 46.161312, 14.650913 46.161075, 14.650827 46.161033, 14.650398 46.160945, 14.649476 46.16069, 14.647136 46.159915, 14.646513 46.15966, 14.644625 46.158798, 14.644304 46.158672, 14.643509 46.158435, 14.642888 46.158264, 14.642544 46.158199, 14.641858 46.158027, 14.639326 46.157382, 14.638617 46.157169, 14.638038 46.15702, 14.637415 46.156826, 14.637051 46.156738, 14.636751 46.156631, 14.63542 46.156227, 14.634562 46.15599, 14.633254 46.155582, 14.629305 46.154399, 14.628597 46.154167, 14.62806 46.153991, 14.627974 46.153972, 14.627138 46.153694, 14.626279 46.153434, 14.626151 46.153392, 14.625335 46.153137, 14.625056 46.153072, 14.624091 46.152748, 14.622954 46.152385, 14.622353 46.152214, 14.62098 46.151763, 14.616452 46.150344, 14.614821 46.149829, 14.61304 46.149272, 14.61289 46.14923, 14.612353 46.149101, 14.611731 46.148906, 14.610938 46.148609, 14.609908 46.148307, 14.608534 46.147941, 14.607397 46.147644, 14.606045 46.147319, 14.605036 46.147106, 14.604844 46.147083, 14.604092 46.146934, 14.601303 46.146503, 14.600487 46.146354, 14.598877 46.146095, 14.59772 46.145862, 14.596776 46.145584, 14.596474 46.145496, 14.595488 46.14513, 14.595273 46.145088, 14.595059 46.145088, 14.594908 46.145111, 14.59463 46.14524, 14.5945 46.145324, 14.594115 46.145366, 14.594093 46.145366, 14.594029 46.145347, 14.593899 46.145259, 14.593771 46.14513, 14.593707 46.144981, 14.593899 46.144489, 14.593985 46.144271, 14.594157 46.143608, 14.594307 46.142837, 14.594414 46.141933, 14.594372 46.141761, 14.594265 46.140861, 14.594115 46.139896, 14.594115 46.139488, 14.594136 46.138736, 14.593921 46.137855, 14.593964 46.137298, 14.594029 46.13702, 14.594157 46.136096, 14.594372 46.135025, 14.59463 46.133911, 14.594672 46.133567, 14.594822 46.133029, 14.595037 46.132148, 14.595101 46.131805, 14.595123 46.13084, 14.595015 46.129554, 14.595015 46.129402, 14.594973 46.127494, 14.594908 46.127063, 14.594887 46.126701, 14.594716 46.125949, 14.5945 46.125305, 14.594286 46.124595, 14.594136 46.123996, 14.593964 46.123458, 14.593835 46.123203, 14.593535 46.122558, 14.59332 46.122257, 14.593063 46.121871, 14.592955 46.121593, 14.592548 46.120262, 14.592548 46.120197, 14.592354 46.119167, 14.59229 46.118522, 14.592141 46.11771, 14.592097 46.117172, 14.592141 46.116313, 14.59229 46.114791, 14.592505 46.11301, 14.592591 46.112514, 14.592698 46.112472, 14.592913 46.11185, 14.592999 46.111484, 14.593171 46.110969, 14.593277 46.111057, 14.593428 46.111377, 14.5936 46.111442, 14.593985 46.111484, 14.594328 46.111595, 14.594779 46.111701, 14.595059 46.111827, 14.595358 46.111938)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testMiddleLong1H10() throws Exception {
    Route route=test(38574, 17774, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(1.0));

    logger.info("Found route="+route);

    String result="LINESTRING (15.102147 46.369964, 15.102576 46.369728, 15.102919 46.369491, 15.103606 46.36885, 15.103777 46.368721, 15.103863 46.368614, 15.104763 46.367649, 15.105193 46.367153, 15.105558 46.36679, 15.105666 46.366638, 15.107017 46.365093, 15.107275 46.364921, 15.106802 46.364536, 15.106245 46.364105, 15.106159 46.363807, 15.106181 46.363548, 15.10648 46.362991, 15.106566 46.362819, 15.10661 46.362625, 15.106545 46.362327, 15.106502 46.362155, 15.106545 46.361938, 15.106781 46.361724, 15.107125 46.361511, 15.107296 46.361316, 15.10736 46.361125, 15.107424 46.360801, 15.107575 46.360179, 15.107747 46.359622, 15.107897 46.359214, 15.108047 46.358699, 15.108155 46.358333, 15.108176 46.358184, 15.108756 46.3581, 15.109034 46.358036, 15.10867 46.357391, 15.108454 46.357154, 15.108176 46.357006, 15.107939 46.356918, 15.107747 46.356746, 15.107403 46.356296, 15.107275 46.356018, 15.107211 46.355758, 15.107211 46.355461, 15.107189 46.355117, 15.107232 46.355052, 15.107232 46.354793, 15.107382 46.354579, 15.107446 46.354366, 15.10736 46.354171, 15.107167 46.353958, 15.107081 46.353805, 15.107081 46.353637, 15.107146 46.353508, 15.107275 46.353336, 15.107618 46.353118, 15.107682 46.352951, 15.10764 46.35284, 15.107554 46.352691, 15.107403 46.352542, 15.107232 46.352413, 15.107103 46.352371, 15.106995 46.352306, 15.106652 46.352153, 15.106416 46.351898, 15.10633 46.351619, 15.106352 46.351467, 15.106416 46.351295, 15.106631 46.350997, 15.106974 46.350654, 15.107253 46.350482, 15.10751 46.350395, 15.108219 46.350265, 15.108476 46.350181, 15.10854 46.350009, 15.108562 46.349838, 15.108476 46.349536, 15.108519 46.3493, 15.108648 46.349063, 15.108626 46.348892, 15.108648 46.348785, 15.108691 46.348613, 15.108454 46.348487, 15.108262 46.348293, 15.10809 46.348056, 15.107897 46.347862, 15.107682 46.34769, 15.107403 46.347606, 15.106717 46.347564, 15.106137 46.347347, 15.106073 46.34692, 15.106116 46.346576, 15.106073 46.346233, 15.106095 46.345954, 15.105751 46.345848, 15.105215 46.34576, 15.104871 46.345741, 15.104614 46.345783, 15.104334 46.345695, 15.104185 46.345676, 15.103755 46.345676, 15.103541 46.345611, 15.103434 46.345523, 15.103369 46.345397, 15.103155 46.345161, 15.102875 46.344837, 15.102704 46.344535, 15.102296 46.34428, 15.102168 46.34415, 15.102125 46.344001, 15.101824 46.343723, 15.101567 46.343616, 15.101352 46.343444, 15.10133 46.343315, 15.101244 46.343162, 15.101288 46.343055, 15.10146 46.342971, 15.101738 46.342929, 15.102662 46.342929, 15.103476 46.342758, 15.103755 46.34267, 15.103819 46.342586, 15.103841 46.342414, 15.103648 46.341426, 15.103692 46.340847, 15.103262 46.340805, 15.102789 46.340805, 15.101632 46.34074, 15.101052 46.34074, 15.100151 46.340675, 15.099378 46.340591, 15.098799 46.340503, 15.098198 46.340374, 15.097983 46.34029, 15.097447 46.339988, 15.097254 46.339775, 15.097039 46.339603, 15.096825 46.339279, 15.096695 46.339218, 15.096589 46.339, 15.096417 46.338764, 15.09646 46.338573, 15.096568 46.338401, 15.09676 46.338165, 15.096911 46.337844, 15.096997 46.337562, 15.097104 46.337112, 15.097275 46.336448, 15.097576 46.335395, 15.097769 46.334987, 15.09852 46.334194, 15.099013 46.333831, 15.099699 46.333251, 15.100258 46.332843, 15.100644 46.3325, 15.100666 46.332412, 15.100644 46.332286, 15.100472 46.332157, 15.099593 46.33179, 15.098799 46.331489, 15.097361 46.330997, 15.096889 46.33076, 15.096695 46.330612, 15.096653 46.330417, 15.096717 46.330245, 15.096825 46.330009, 15.096803 46.329795, 15.096546 46.329322, 15.096503 46.329132, 15.096524 46.328979, 15.09676 46.328422, 15.096739 46.32825, 15.096589 46.327842, 15.096674 46.327369, 15.096159 46.326641, 15.095837 46.326339, 15.095387 46.326084, 15.095194 46.325847, 15.095087 46.325634, 15.095129 46.325332, 15.095366 46.325141, 15.095816 46.324966, 15.095967 46.324752, 15.095881 46.32439, 15.095902 46.324001, 15.095837 46.32381, 15.095473 46.323486, 15.09543 46.323314, 15.095473 46.323081, 15.095665 46.322544, 15.09573 46.322177, 15.095665 46.321922, 15.095602 46.321792, 15.095494 46.321662, 15.095023 46.321254, 15.094807 46.321041, 15.094743 46.320911, 15.094614 46.320739, 15.094357 46.319927, 15.094314 46.319881, 15.0944 46.318962, 15.094336 46.318744, 15.093971 46.318252, 15.093928 46.318058, 15.094014 46.317821, 15.094057 46.317607, 15.093971 46.317371, 15.093713 46.317008, 15.093585 46.316772, 15.093413 46.316577, 15.093134 46.316406, 15.092426 46.316085, 15.092232 46.315979, 15.092168 46.315849, 15.092146 46.315696, 15.092276 46.315399, 15.092232 46.315162, 15.092125 46.314949, 15.091954 46.314712, 15.091675 46.314498, 15.091482 46.314369, 15.091224 46.314239, 15.091074 46.31409, 15.090817 46.313961, 15.090559 46.313854, 15.090237 46.313766, 15.090044 46.313747, 15.089851 46.313659, 15.089615 46.313404, 15.0894 46.313144, 15.089207 46.31306, 15.088478 46.312889, 15.088263 46.312801, 15.088091 46.312671, 15.088006 46.3125, 15.088156 46.312072, 15.087963 46.311599, 15.087877 46.310955, 15.087769 46.310398, 15.087683 46.310119, 15.087683 46.309906, 15.087813 46.309711, 15.088112 46.309368, 15.088306 46.309005, 15.088414 46.308662, 15.088414 46.308597, 15.088606 46.308082, 15.088843 46.307693, 15.089743 46.306556, 15.089894 46.306278, 15.090001 46.306041, 15.090172 46.304283, 15.090086 46.303939, 15.08998 46.303638, 15.089787 46.303253, 15.089722 46.302993, 15.089743 46.302761, 15.089808 46.302524, 15.089808 46.302265, 15.089636 46.30162, 15.089657 46.301406, 15.089873 46.301086, 15.090065 46.300678, 15.090194 46.300483, 15.090216 46.300376, 15.090108 46.300098, 15.089915 46.299778, 15.089679 46.299453, 15.089015 46.298961, 15.088606 46.298316, 15.08837 46.297801, 15.088156 46.297565, 15.087941 46.297374, 15.08747 46.297031, 15.087254 46.296813, 15.086546 46.295848, 15.086461 46.295291, 15.086396 46.295097, 15.085903 46.294754, 15.085495 46.29441, 15.085259 46.294155, 15.08513 46.293899, 15.085108 46.293556, 15.085173 46.293319, 15.085259 46.293125, 15.085989 46.292331, 15.086546 46.291839, 15.086869 46.291496, 15.08704 46.291236, 15.08734 46.290443, 15.087405 46.290077, 15.087212 46.28984, 15.085645 46.28862, 15.084873 46.28804, 15.084229 46.287525, 15.0841 46.28746, 15.083027 46.286643, 15.082727 46.286388, 15.08232 46.286109, 15.081805 46.285678, 15.081225 46.28527, 15.080302 46.284713, 15.07895 46.283962, 15.077469 46.283317, 15.076741 46.282932, 15.076247 46.282783, 15.075882 46.282695, 15.075474 46.282524, 15.075217 46.282394, 15.075066 46.282226, 15.074767 46.281967, 15.074509 46.281795, 15.074101 46.28244, 15.073822 46.28276, 15.073565 46.282955, 15.073222 46.283145, 15.07292 46.283317, 15.072299 46.283706, 15.07189 46.28405, 15.071633 46.284221, 15.071397 46.284263, 15.070904 46.284286, 15.070561 46.284263, 15.070324 46.284286, 15.069617 46.284519, 15.068844 46.284778, 15.068007 46.285121, 15.067234 46.285335, 15.066956 46.285141, 15.066247 46.284542, 15.065904 46.28424, 15.06511 46.283641, 15.064788 46.28347, 15.064553 46.283382, 15.064059 46.283145, 15.062792 46.282611, 15.06187 46.282203, 15.060926 46.281753, 15.061355 46.280487, 15.061463 46.28012, 15.061635 46.279647, 15.061698 46.279479, 15.062042 46.278556, 15.062042 46.278469, 15.06202 46.278297, 15.06187 46.278018, 15.06172 46.27787, 15.061269 46.27761, 15.060797 46.277397, 15.060347 46.277267, 15.059746 46.27703, 15.05951 46.276901, 15.058157 46.27626, 15.056656 46.275402, 15.056162 46.275142, 15.055862 46.274929, 15.05569 46.274757, 15.055325 46.274543, 15.055111 46.274372, 15.054896 46.274242, 15.054489 46.273811, 15.05391 46.273277, 15.05363 46.272975, 15.053201 46.272697, 15.052966 46.272483, 15.052729 46.272331, 15.052493 46.272052, 15.052343 46.271797, 15.052343 46.271583, 15.052257 46.271324, 15.052063 46.271152, 15.051291 46.270851, 15.051141 46.270721, 15.050992 46.270423, 15.05097 46.269565, 15.051399 46.268257, 15.051463 46.267932, 15.051485 46.267417, 15.051421 46.267269, 15.051485 46.267055, 15.051828 46.266323, 15.052472 46.265487, 15.0526 46.265209, 15.052922 46.264694, 15.05303 46.264434, 15.053115 46.263984, 15.053179 46.263835, 15.053309 46.263622, 15.053502 46.263362, 15.053652 46.263168, 15.053437 46.262546, 15.0526 46.262439, 15.052343 46.262439, 15.052 46.262332, 15.051678 46.262096, 15.051527 46.261905, 15.051442 46.261711, 15.051421 46.261627, 15.051442 46.261451, 15.051506 46.261283, 15.051527 46.261108, 15.051678 46.260787, 15.051699 46.260551, 15.051678 46.260444, 15.051506 46.260231, 15.051163 46.260124, 15.050841 46.260078, 15.050755 46.260017, 15.050197 46.259887, 15.050004 46.259887, 15.04979 46.259822, 15.049447 46.259628, 15.049317 46.259544, 15.048201 46.259544, 15.047858 46.259521, 15.047514 46.259456, 15.047193 46.259437, 15.046592 46.259284, 15.045991 46.259071, 15.045756 46.259029, 15.044876 46.258941, 15.044339 46.258769, 15.044167 46.258472, 15.044146 46.258106, 15.043931 46.257633, 15.043782 46.257484, 15.043738 46.256366, 15.043717 46.256347, 15.043631 46.255252, 15.043545 46.254287, 15.043502 46.253856, 15.043438 46.253471, 15.043202 46.252826, 15.042429 46.252998, 15.042043 46.253063, 15.041249 46.253276, 15.040692 46.253406, 15.040305 46.253536, 15.040005 46.253601, 15.039618 46.25362, 15.039039 46.253578, 15.037688 46.253406, 15.037558 46.25317, 15.037215 46.252784, 15.037087 46.252525, 15.037087 46.252399, 15.037173 46.252162, 15.037215 46.251777, 15.037215 46.251476, 15.037259 46.251239, 15.037194 46.250808, 15.037022 46.250358, 15.0367 46.249996, 15.036636 46.249866, 15.0364 46.249717, 15.036335 46.249629, 15.036185 46.249481, 15.036013 46.249374, 15.035498 46.249092, 15.035199 46.249008, 15.034919 46.248836, 15.034533 46.248558, 15.034404 46.248321, 15.03434 46.248043, 15.034276 46.24744, 15.033246 46.247486, 15.031744 46.247486, 15.030478 46.247505, 15.030091 46.24755, 15.028889 46.247741, 15.027773 46.247894, 15.026336 46.248062, 15.022109 46.248363, 15.021765 46.248363, 15.021272 46.248386, 15.020693 46.248363, 15.019985 46.248279, 15.019104 46.248126, 15.018504 46.248001, 15.018397 46.247978, 15.017861 46.247894, 15.014706 46.247741, 15.013741 46.247592, 15.012667 46.247505, 15.012282 46.24744, 15.011981 46.247398, 15.01123 46.247356, 15.010758 46.247356, 15.009985 46.24744, 15.009084 46.247592, 15.00829 46.247634, 15.007946 46.247612, 15.007218 46.247676, 15.004492 46.247955, 15.00387 46.24802, 15.003827 46.248043, 15.002497 46.248126, 15.002003 46.248172, 15.001681 46.248172, 15.000759 46.248191, 14.999986 46.248191, 14.997497 46.248321, 14.994708 46.248516, 14.992325 46.248641, 14.991231 46.248794, 14.990866 46.248836, 14.990416 46.248878, 14.989686 46.248901, 14.988914 46.248878, 14.988678 46.248878, 14.98812 46.248859, 14.987197 46.248901, 14.98576 46.248901, 14.984686 46.248859, 14.984429 46.248859, 14.983678 46.248813, 14.98312 46.248836, 14.98267 46.248878, 14.980975 46.249137, 14.979816 46.249328, 14.978915 46.249588, 14.977584 46.250061, 14.976233 46.250511, 14.975739 46.250617, 14.97473 46.250808, 14.974173 46.250896, 14.97325 46.250984, 14.972714 46.250984, 14.972199 46.250961, 14.971619 46.250873, 14.971211 46.250789, 14.970524 46.250595, 14.969988 46.250381, 14.969688 46.250232, 14.96958 46.250144, 14.968937 46.249759, 14.968358 46.249458, 14.967307 46.24905, 14.965848 46.248558, 14.965203 46.248321, 14.964409 46.247955, 14.9634 46.247398, 14.962414 46.246864, 14.961728 46.246521, 14.961105 46.246261, 14.960698 46.246067, 14.960225 46.245834, 14.959689 46.245597, 14.957779 46.244716, 14.956942 46.244331, 14.956341 46.24403, 14.954839 46.243343, 14.953294 46.242614, 14.950913 46.24139, 14.949904 46.240684, 14.949604 46.240425, 14.949238 46.240039, 14.948745 46.239437, 14.948424 46.238967, 14.947865 46.237743, 14.947651 46.237075, 14.94735 46.236434, 14.9472 46.236045, 14.946879 46.235359, 14.946213 46.23418, 14.945698 46.233257, 14.94542 46.232593, 14.945226 46.23225, 14.944861 46.231239, 14.944025 46.229053, 14.94351 46.227851, 14.943274 46.227893, 14.942931 46.227893, 14.942544 46.227787, 14.942136 46.227527, 14.941879 46.227401, 14.941535 46.227314, 14.940634 46.227249, 14.939862 46.227272, 14.938853 46.227272, 14.938509 46.227249, 14.93808 46.227184, 14.937523 46.227058, 14.937072 46.226905, 14.936106 46.226627, 14.93542 46.226478, 14.932523 46.22594, 14.931815 46.225791, 14.930506 46.225425, 14.929519 46.225105, 14.928275 46.224716, 14.927395 46.224502, 14.926836 46.224437, 14.925549 46.224395, 14.924927 46.224266, 14.923683 46.223987, 14.923125 46.223922, 14.922287 46.22388, 14.920872 46.223922, 14.918833 46.224029, 14.918297 46.224029, 14.917481 46.22388, 14.916902 46.223686, 14.916559 46.223495, 14.91613 46.223194, 14.915529 46.222679, 14.915271 46.222507, 14.914692 46.222229, 14.91407 46.221969, 14.913083 46.221519, 14.91231 46.221134, 14.912138 46.220985, 14.911924 46.220771, 14.911581 46.220169, 14.911173 46.219654, 14.910508 46.219097, 14.909778 46.218559, 14.90922 46.218109, 14.908898 46.217849, 14.908513 46.217617, 14.908083 46.217399, 14.90761 46.21725, 14.907032 46.217144, 14.904091 46.216693, 14.903534 46.216587, 14.901753 46.216178, 14.901345 46.216114, 14.900057 46.216072, 14.899329 46.215877, 14.899071 46.215877, 14.898792 46.215965, 14.898277 46.216156, 14.898127 46.216178, 14.897097 46.216178, 14.895874 46.216285, 14.895466 46.21635, 14.895037 46.216369, 14.892291 46.216369, 14.891303 46.216415, 14.890895 46.216457, 14.890358 46.216541, 14.889586 46.216606, 14.888621 46.216629, 14.887742 46.21667, 14.887033 46.216758, 14.886904 46.216758, 14.886839 46.216777, 14.88551 46.21693, 14.884909 46.21693, 14.883921 46.216819, 14.883063 46.216606, 14.88184 46.216178, 14.881218 46.216007, 14.880402 46.2159, 14.877442 46.215835, 14.876282 46.215835, 14.875767 46.215877, 14.875403 46.215942, 14.874845 46.216026, 14.874545 46.216049, 14.874201 46.216007, 14.873707 46.2159, 14.873621 46.215854, 14.873408 46.215747, 14.872592 46.215255, 14.871863 46.215103, 14.871498 46.215084, 14.870811 46.214977, 14.868472 46.21429, 14.868001 46.214183, 14.867206 46.21416, 14.865511 46.214031, 14.86476 46.214031, 14.863881 46.214096, 14.863022 46.214118, 14.862034 46.214073, 14.86094 46.214138, 14.860597 46.214054, 14.860082 46.213882, 14.859717 46.213729, 14.859417 46.213668, 14.859074 46.213668, 14.858065 46.213775, 14.856242 46.213752, 14.855662 46.213794, 14.854975 46.213924, 14.854697 46.214054, 14.854374 46.214183, 14.854138 46.214309, 14.853645 46.214504, 14.853216 46.214591, 14.851134 46.214698, 14.849911 46.214698, 14.848881 46.214611, 14.848044 46.214504, 14.847529 46.214504, 14.847144 46.214611, 14.8468 46.21474, 14.846457 46.214954, 14.845877 46.215362, 14.845641 46.215492, 14.844332 46.216156, 14.84386 46.216308, 14.843646 46.21635, 14.843237 46.216392, 14.842916 46.216369, 14.842616 46.216327, 14.842144 46.216156, 14.841435 46.21577, 14.841199 46.215663, 14.840856 46.215576, 14.840491 46.215557, 14.840106 46.215599, 14.839676 46.215705, 14.839203 46.215789, 14.838646 46.215854, 14.837573 46.216007, 14.8368 46.216072, 14.836415 46.216049, 14.835277 46.215877, 14.834719 46.215812, 14.834355 46.215705, 14.833882 46.215557, 14.833475 46.215362, 14.833045 46.21519, 14.832638 46.215019, 14.832209 46.214889, 14.83193 46.214847, 14.831629 46.214847, 14.831265 46.21487, 14.829978 46.215042, 14.82914 46.215126, 14.828668 46.215103, 14.827896 46.214912, 14.827595 46.214805, 14.826844 46.214611, 14.826329 46.214546, 14.826115 46.214527, 14.825728 46.214546, 14.825299 46.214591, 14.824784 46.214675, 14.823926 46.214912, 14.82296 46.215148, 14.822552 46.215232, 14.822188 46.215255, 14.821522 46.215168, 14.8209 46.215148, 14.819977 46.215168, 14.819656 46.215126, 14.818841 46.214977, 14.818154 46.214912, 14.816931 46.21474, 14.816823 46.21474, 14.816094 46.214611, 14.815171 46.214416, 14.814913 46.214374, 14.813605 46.21429, 14.813262 46.214225, 14.811888 46.214031, 14.811459 46.213989, 14.810515 46.214012, 14.809292 46.214073, 14.808905 46.214012, 14.80852 46.213859, 14.808283 46.213729, 14.807875 46.213687, 14.807511 46.213752, 14.807274 46.213882, 14.807189 46.214054, 14.807339 46.214504, 14.807232 46.214675, 14.806931 46.214824, 14.805602 46.215255, 14.805386 46.215297, 14.805001 46.21532, 14.804678 46.215297, 14.802897 46.215255, 14.80221 46.215148, 14.80176 46.215103, 14.801374 46.215126, 14.800967 46.215213, 14.800538 46.215339, 14.80015 46.215492, 14.7997 46.215621, 14.79925 46.215705, 14.798176 46.215812, 14.797984 46.215812, 14.797575 46.215854, 14.797146 46.215877, 14.796868 46.215877, 14.79601 46.215942, 14.795366 46.216007, 14.79498 46.216072, 14.79395 46.216327, 14.793456 46.216541, 14.793242 46.21667, 14.793026 46.216842, 14.792511 46.217636, 14.791354 46.218475, 14.791182 46.218688, 14.790967 46.218902, 14.790645 46.219181, 14.789958 46.219482, 14.789637 46.219589, 14.78925 46.21976, 14.788907 46.21989, 14.788199 46.220169, 14.787447 46.220428, 14.785731 46.22092, 14.785152 46.221199, 14.784701 46.221519, 14.784316 46.221755, 14.783671 46.222099, 14.783371 46.222206, 14.782663 46.222358, 14.782234 46.2224, 14.78129 46.2224, 14.78056 46.222377, 14.78011 46.222335, 14.779894 46.22227, 14.779745 46.222251, 14.77908 46.221862, 14.778479 46.221565, 14.778286 46.221435, 14.777599 46.221069, 14.776762 46.220813, 14.775002 46.220428, 14.774294 46.220298, 14.773565 46.220127, 14.77335 46.220104, 14.772728 46.219974, 14.771784 46.219783, 14.77026 46.219631, 14.769874 46.219589, 14.7676 46.219223, 14.765904 46.219055, 14.765068 46.218944, 14.764575 46.218925, 14.763866 46.218944, 14.763287 46.218925, 14.762235 46.218772, 14.761806 46.218772, 14.760668 46.218902, 14.760347 46.218879, 14.759789 46.218624, 14.759167 46.218368, 14.757643 46.218067, 14.755798 46.217807, 14.755261 46.217788, 14.75479 46.217849, 14.753309 46.218238, 14.752965 46.218303, 14.752129 46.218345, 14.751764 46.218322, 14.749424 46.218238, 14.747 46.218368, 14.74685 46.218387, 14.745991 46.218429, 14.744726 46.218536, 14.744211 46.218559, 14.743481 46.218559, 14.742859 46.218536, 14.742386 46.218494, 14.741057 46.21828, 14.739061 46.217872, 14.738975 46.217872, 14.73831 46.217681, 14.737216 46.21725, 14.736228 46.21693, 14.73565 46.216865, 14.735091 46.216758, 14.73462 46.216629, 14.734318 46.216587, 14.733997 46.216434, 14.733632 46.216178, 14.733396 46.2159, 14.733224 46.215641, 14.733031 46.215492, 14.732731 46.21532, 14.732172 46.215061, 14.731851 46.214889, 14.731336 46.214698, 14.73095 46.214591, 14.730306 46.214847, 14.72962 46.215084, 14.728976 46.215278, 14.727688 46.215599, 14.726552 46.215835, 14.725113 46.216072, 14.724749 46.216072, 14.723719 46.216114, 14.723182 46.216197, 14.723053 46.21622, 14.722432 46.216197, 14.722023 46.216243, 14.721723 46.216262, 14.720757 46.216072, 14.72035 46.216049, 14.720028 46.216091, 14.719256 46.216369, 14.718848 46.216541, 14.718462 46.216629, 14.717753 46.21667, 14.717238 46.216777, 14.716767 46.216842, 14.716465 46.216842, 14.71535 46.216651, 14.714663 46.216564, 14.713827 46.216693, 14.713461 46.216777, 14.712968 46.21693, 14.712689 46.217037, 14.712453 46.217208, 14.712002 46.2177, 14.71181 46.217872, 14.711573 46.217979, 14.71093 46.218151, 14.709836 46.218475, 14.709492 46.218517, 14.70872 46.218429, 14.708591 46.21841, 14.707797 46.218303, 14.705781 46.218086, 14.705329 46.218086, 14.704878 46.218044, 14.703742 46.21783, 14.703527 46.217357, 14.703291 46.217144, 14.703398 46.216972, 14.703398 46.216777, 14.703355 46.216629, 14.703227 46.216564, 14.702755 46.216392, 14.702647 46.216327, 14.702347 46.215984, 14.702176 46.216007, 14.70196 46.216007, 14.701661 46.216114, 14.701081 46.216327, 14.700888 46.216285, 14.700758 46.216178, 14.701146 46.215362, 14.701445 46.215042, 14.701617 46.214805, 14.701725 46.214462, 14.701725 46.214183, 14.701617 46.213108, 14.701531 46.212699, 14.701424 46.212486, 14.701167 46.212165, 14.700866 46.211994, 14.700415 46.211822, 14.699343 46.211479, 14.698291 46.210769, 14.69739 46.210189, 14.697154 46.209999, 14.696703 46.209377, 14.696339 46.208755, 14.696082 46.208152, 14.695845 46.207725, 14.695523 46.207359, 14.694815 46.206478, 14.694579 46.206092, 14.694558 46.205898, 14.694579 46.205726, 14.694644 46.205642, 14.69503 46.205383, 14.695244 46.205276, 14.695459 46.205104, 14.695609 46.20502, 14.695652 46.204891, 14.695523 46.204784, 14.695201 46.204719, 14.694729 46.204505, 14.694558 46.204525, 14.6943 46.204742, 14.694043 46.204826, 14.69372 46.204742, 14.693313 46.204399, 14.692391 46.203948, 14.692197 46.203666, 14.692026 46.203517, 14.691768 46.203388, 14.691511 46.203067, 14.691854 46.202873, 14.692197 46.202743, 14.692412 46.202529, 14.692477 46.202293, 14.69269 46.202056, 14.693549 46.201351, 14.693828 46.201221, 14.694043 46.201091, 14.694214 46.200794, 14.694644 46.199871, 14.694858 46.199569, 14.694901 46.19942, 14.695179 46.199203, 14.695481 46.199012, 14.695716 46.198883, 14.696467 46.198581, 14.696639 46.19841, 14.696618 46.198261, 14.696553 46.198112, 14.696597 46.197895, 14.696597 46.197723, 14.69636 46.197059, 14.696038 46.196971, 14.694836 46.196823, 14.694644 46.196758, 14.694343 46.196693, 14.694193 46.196628, 14.69385 46.196414, 14.693635 46.196178, 14.693549 46.195964, 14.693571 46.195793, 14.69372 46.195514, 14.693871 46.195278, 14.694086 46.195064, 14.694235 46.194934, 14.694815 46.194656, 14.695158 46.194568, 14.695373 46.194442, 14.695459 46.194248, 14.695481 46.194053, 14.695395 46.193862, 14.694966 46.193603, 14.694429 46.193241, 14.694171 46.192787, 14.694022 46.192188, 14.693764 46.192146, 14.693442 46.191802, 14.693184 46.191631, 14.69297 46.191394, 14.692669 46.191288, 14.692412 46.191246, 14.692004 46.191223, 14.691682 46.191181, 14.691082 46.190856, 14.691039 46.19075, 14.690888 46.190559, 14.69063 46.190063, 14.690438 46.189849, 14.690094 46.189529, 14.6896 46.189163, 14.688786 46.188667, 14.687928 46.188217, 14.687562 46.187873, 14.687348 46.187595, 14.687262 46.187423, 14.687133 46.186439, 14.686962 46.186244, 14.686747 46.186096, 14.686469 46.185966, 14.686211 46.185813, 14.685932 46.185577, 14.685566 46.185321, 14.685309 46.185192, 14.684924 46.185066, 14.684473 46.184955, 14.683421 46.184783, 14.682906 46.184741, 14.68252 46.184635, 14.682284 46.184528, 14.681769 46.184333, 14.68134 46.18412, 14.680502 46.183734, 14.679902 46.183368, 14.679623 46.183284, 14.679387 46.183303, 14.679065 46.18341, 14.678636 46.183734, 14.678401 46.18386, 14.678185 46.184013, 14.677907 46.18412, 14.676834 46.184333, 14.676319 46.184463, 14.67589 46.184547, 14.675761 46.18457, 14.674796 46.184806, 14.674494 46.184848, 14.674259 46.184848, 14.674044 46.184806, 14.67383 46.184783, 14.673658 46.184741, 14.673615 46.184741, 14.673572 46.184722, 14.672671 46.184505, 14.672606 46.184268, 14.67252 46.18399, 14.672349 46.183734, 14.672221 46.183456, 14.672478 46.182769, 14.67252 46.182533, 14.672456 46.182296, 14.671919 46.181674, 14.671662 46.181415, 14.670975 46.180366, 14.670632 46.179935, 14.670482 46.179656, 14.670353 46.179336, 14.670311 46.179164, 14.67046 46.178497, 14.670611 46.178005, 14.670676 46.177703, 14.670611 46.177402, 14.67059 46.177253, 14.670418 46.176868, 14.670225 46.176544, 14.670075 46.176139, 14.669989 46.175815, 14.669796 46.175342, 14.669646 46.175064, 14.66956 46.174957, 14.669388 46.174591, 14.66943 46.174591, 14.669732 46.174591, 14.669967 46.174613, 14.670161 46.174678, 14.670825 46.17485, 14.671169 46.174957, 14.67207 46.17517, 14.672843 46.175277, 14.673293 46.1753, 14.673486 46.175258, 14.673658 46.17517, 14.67383 46.175022, 14.673915 46.174915, 14.673958 46.17472, 14.674065 46.174312, 14.674302 46.174057, 14.674559 46.173969, 14.674881 46.17395, 14.675482 46.17382, 14.675696 46.173713, 14.676125 46.173393, 14.676276 46.17324, 14.676684 46.17279, 14.676791 46.172595, 14.676941 46.172382, 14.677134 46.172275, 14.677714 46.171997, 14.678271 46.17176, 14.678442 46.171653, 14.678464 46.171394, 14.678336 46.171245, 14.677928 46.171051, 14.677584 46.170795, 14.677371 46.170581, 14.677285 46.170429, 14.677263 46.17028, 14.677091 46.169357, 14.677091 46.169078, 14.676919 46.16867, 14.676748 46.168476, 14.676468 46.168243, 14.676276 46.167984, 14.676147 46.167747, 14.676125 46.167167, 14.676169 46.16687, 14.676255 46.166526, 14.67619 46.166244, 14.676169 46.165901, 14.675696 46.165409, 14.675503 46.165195, 14.674967 46.16468, 14.674624 46.164249, 14.67458 46.164077, 14.674624 46.163822, 14.674817 46.163093, 14.67486 46.162811, 14.673293 46.162574, 14.672843 46.16249, 14.672049 46.162384, 14.67046 46.162231, 14.669495 46.16217, 14.669323 46.162189, 14.667178 46.162124, 14.665825 46.162017, 14.663873 46.161804, 14.663552 46.161762, 14.663358 46.161739, 14.662478 46.161655, 14.66029 46.16159, 14.658466 46.161609, 14.657757 46.161632, 14.656212 46.16159, 14.655633 46.161567, 14.653208 46.161437, 14.6522 46.161312, 14.650913 46.161075, 14.650827 46.161033, 14.650398 46.160945, 14.649476 46.16069, 14.647136 46.159915, 14.646513 46.15966, 14.644625 46.158798, 14.644304 46.158672, 14.643509 46.158435, 14.642888 46.158264, 14.642544 46.158199, 14.641858 46.158027, 14.639326 46.157382, 14.638617 46.157169, 14.638038 46.15702, 14.637415 46.156826, 14.637051 46.156738, 14.636751 46.156631, 14.63542 46.156227, 14.634562 46.15599, 14.633254 46.155582, 14.629305 46.154399, 14.628597 46.154167, 14.62806 46.153991, 14.627974 46.153972, 14.627138 46.153694, 14.626279 46.153434, 14.626151 46.153392, 14.625335 46.153137, 14.625056 46.153072, 14.624091 46.152748, 14.622954 46.152385, 14.622353 46.152214, 14.62098 46.151763, 14.616452 46.150344, 14.614821 46.149829, 14.61304 46.149272, 14.61289 46.14923, 14.612353 46.149101, 14.611731 46.148906, 14.610938 46.148609, 14.609908 46.148307, 14.608534 46.147941, 14.607397 46.147644, 14.606045 46.147319, 14.605036 46.147106, 14.604844 46.147083, 14.604092 46.146934, 14.601303 46.146503, 14.601538 46.146034, 14.600745 46.14524, 14.600273 46.144721, 14.599758 46.144229, 14.599114 46.143886, 14.598664 46.14363, 14.59847 46.143543, 14.598105 46.143306, 14.597312 46.142837, 14.596303 46.142429, 14.595767 46.142234, 14.595316 46.142127, 14.594887 46.142021, 14.594414 46.141933, 14.594372 46.141761, 14.594265 46.140861, 14.594115 46.139896, 14.594115 46.139488, 14.594136 46.138736, 14.593921 46.137855, 14.593964 46.137298, 14.594029 46.13702, 14.594157 46.136096, 14.594372 46.135025, 14.59463 46.133911, 14.594672 46.133567, 14.594822 46.133029, 14.595037 46.132148, 14.595101 46.131805, 14.595123 46.13084, 14.595015 46.129554, 14.595015 46.129402, 14.594973 46.127494, 14.594908 46.127063, 14.594887 46.126701, 14.594716 46.125949, 14.5945 46.125305, 14.594286 46.124595, 14.594136 46.123996, 14.593964 46.123458, 14.593835 46.123203, 14.593535 46.122558, 14.59332 46.122257, 14.593063 46.121871, 14.592955 46.121593, 14.592548 46.120262, 14.592548 46.120197, 14.592354 46.119167, 14.59229 46.118522, 14.592141 46.11771, 14.592097 46.117172, 14.592141 46.116313, 14.59229 46.114791, 14.592505 46.11301, 14.592591 46.112514, 14.592698 46.112472, 14.592913 46.11185, 14.592999 46.111484, 14.593171 46.110969, 14.593277 46.111057, 14.593428 46.111377, 14.5936 46.111442, 14.593985 46.111484, 14.594328 46.111595, 14.594779 46.111701, 14.595059 46.111827, 14.595358 46.111938)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testMiddleLong1H12() throws Exception {
    Route route=test(38574, 17774, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(1.2));

    logger.info("Found route="+route);

    String result="LINESTRING (15.102147 46.369964, 15.102576 46.369728, 15.102919 46.369491, 15.103606 46.36885, 15.103283 46.368141, 15.103219 46.367904, 15.102747 46.367324, 15.102725 46.367027, 15.102768 46.366683, 15.102811 46.366081, 15.102983 46.365524, 15.104614 46.364856, 15.104979 46.364643, 15.1053 46.364494, 15.105879 46.36428, 15.106245 46.364105, 15.106159 46.363807, 15.106181 46.363548, 15.10648 46.362991, 15.106566 46.362819, 15.10661 46.362625, 15.106545 46.362327, 15.106502 46.362155, 15.106545 46.361938, 15.106781 46.361724, 15.107125 46.361511, 15.107296 46.361316, 15.10736 46.361125, 15.107424 46.360801, 15.107575 46.360179, 15.107747 46.359622, 15.107897 46.359214, 15.108047 46.358699, 15.108155 46.358333, 15.108176 46.358184, 15.108756 46.3581, 15.109034 46.358036, 15.10867 46.357391, 15.108454 46.357154, 15.108176 46.357006, 15.107939 46.356918, 15.107747 46.356746, 15.107403 46.356296, 15.107275 46.356018, 15.107211 46.355758, 15.107211 46.355461, 15.107189 46.355117, 15.107232 46.355052, 15.107232 46.354793, 15.107382 46.354579, 15.107446 46.354366, 15.10736 46.354171, 15.107167 46.353958, 15.107081 46.353805, 15.107081 46.353637, 15.107146 46.353508, 15.107275 46.353336, 15.107618 46.353118, 15.107682 46.352951, 15.10764 46.35284, 15.107554 46.352691, 15.107403 46.352542, 15.107232 46.352413, 15.107103 46.352371, 15.106995 46.352306, 15.106652 46.352153, 15.106416 46.351898, 15.10633 46.351619, 15.106352 46.351467, 15.106416 46.351295, 15.106631 46.350997, 15.106974 46.350654, 15.107253 46.350482, 15.10751 46.350395, 15.108219 46.350265, 15.108476 46.350181, 15.10854 46.350009, 15.108562 46.349838, 15.108476 46.349536, 15.108519 46.3493, 15.108648 46.349063, 15.108626 46.348892, 15.108648 46.348785, 15.108691 46.348613, 15.108454 46.348487, 15.108262 46.348293, 15.10809 46.348056, 15.107897 46.347862, 15.107682 46.34769, 15.107403 46.347606, 15.106717 46.347564, 15.106137 46.347347, 15.106073 46.34692, 15.106116 46.346576, 15.106073 46.346233, 15.106095 46.345954, 15.105751 46.345848, 15.105215 46.34576, 15.104871 46.345741, 15.104614 46.345783, 15.104334 46.345695, 15.104185 46.345676, 15.103755 46.345676, 15.103541 46.345611, 15.103434 46.345523, 15.103369 46.345397, 15.103155 46.345161, 15.102875 46.344837, 15.102704 46.344535, 15.102296 46.34428, 15.102168 46.34415, 15.102125 46.344001, 15.101824 46.343723, 15.101567 46.343616, 15.101352 46.343444, 15.10133 46.343315, 15.101244 46.343162, 15.101288 46.343055, 15.10146 46.342971, 15.101738 46.342929, 15.102662 46.342929, 15.103476 46.342758, 15.103755 46.34267, 15.103819 46.342586, 15.103841 46.342414, 15.103648 46.341426, 15.103692 46.340847, 15.103262 46.340805, 15.102789 46.340805, 15.101632 46.34074, 15.101052 46.34074, 15.100151 46.340675, 15.099378 46.340591, 15.098799 46.340503, 15.098198 46.340374, 15.097983 46.34029, 15.097447 46.339988, 15.097254 46.339775, 15.097039 46.339603, 15.096825 46.339279, 15.096695 46.339218, 15.096589 46.339, 15.096417 46.338764, 15.09646 46.338573, 15.096568 46.338401, 15.09676 46.338165, 15.096911 46.337844, 15.096997 46.337562, 15.097104 46.337112, 15.097275 46.336448, 15.097576 46.335395, 15.097769 46.334987, 15.09852 46.334194, 15.099013 46.333831, 15.099699 46.333251, 15.100258 46.332843, 15.100644 46.3325, 15.100666 46.332412, 15.100644 46.332286, 15.100472 46.332157, 15.099593 46.33179, 15.098799 46.331489, 15.097361 46.330997, 15.096889 46.33076, 15.096695 46.330612, 15.096653 46.330417, 15.096717 46.330245, 15.096825 46.330009, 15.096803 46.329795, 15.096546 46.329322, 15.096503 46.329132, 15.096524 46.328979, 15.09676 46.328422, 15.096739 46.32825, 15.096589 46.327842, 15.096674 46.327369, 15.096159 46.326641, 15.095837 46.326339, 15.095387 46.326084, 15.095194 46.325847, 15.095087 46.325634, 15.095129 46.325332, 15.095366 46.325141, 15.095816 46.324966, 15.095967 46.324752, 15.095881 46.32439, 15.095902 46.324001, 15.095837 46.32381, 15.095473 46.323486, 15.09543 46.323314, 15.095473 46.323081, 15.095665 46.322544, 15.09573 46.322177, 15.095665 46.321922, 15.095602 46.321792, 15.095494 46.321662, 15.095023 46.321254, 15.094807 46.321041, 15.094743 46.320911, 15.094614 46.320739, 15.094357 46.319927, 15.094314 46.319881, 15.0944 46.318962, 15.094336 46.318744, 15.093971 46.318252, 15.093928 46.318058, 15.094014 46.317821, 15.094057 46.317607, 15.093971 46.317371, 15.093713 46.317008, 15.093585 46.316772, 15.093413 46.316577, 15.093134 46.316406, 15.092426 46.316085, 15.092232 46.315979, 15.092168 46.315849, 15.092146 46.315696, 15.092276 46.315399, 15.092232 46.315162, 15.092125 46.314949, 15.091954 46.314712, 15.091675 46.314498, 15.091482 46.314369, 15.091224 46.314239, 15.091074 46.31409, 15.090817 46.313961, 15.090559 46.313854, 15.090237 46.313766, 15.090044 46.313747, 15.089851 46.313659, 15.089615 46.313404, 15.0894 46.313144, 15.089207 46.31306, 15.088478 46.312889, 15.088263 46.312801, 15.088091 46.312671, 15.088006 46.3125, 15.088156 46.312072, 15.087963 46.311599, 15.087877 46.310955, 15.087769 46.310398, 15.087683 46.310119, 15.087683 46.309906, 15.087813 46.309711, 15.088112 46.309368, 15.088306 46.309005, 15.088414 46.308662, 15.088414 46.308597, 15.088606 46.308082, 15.088843 46.307693, 15.089743 46.306556, 15.089894 46.306278, 15.090001 46.306041, 15.090172 46.304283, 15.090086 46.303939, 15.08998 46.303638, 15.089787 46.303253, 15.089722 46.302993, 15.089743 46.302761, 15.089808 46.302524, 15.089808 46.302265, 15.089636 46.30162, 15.089657 46.301406, 15.089873 46.301086, 15.090065 46.300678, 15.090194 46.300483, 15.090216 46.300376, 15.090108 46.300098, 15.089915 46.299778, 15.089679 46.299453, 15.089015 46.298961, 15.088606 46.298316, 15.08837 46.297801, 15.088156 46.297565, 15.087941 46.297374, 15.08747 46.297031, 15.087254 46.296813, 15.086546 46.295848, 15.086461 46.295291, 15.086396 46.295097, 15.085903 46.294754, 15.085495 46.29441, 15.085259 46.294155, 15.08513 46.293899, 15.085108 46.293556, 15.085173 46.293319, 15.085259 46.293125, 15.085989 46.292331, 15.086546 46.291839, 15.086869 46.291496, 15.08704 46.291236, 15.08734 46.290443, 15.087405 46.290077, 15.087212 46.28984, 15.085645 46.28862, 15.084873 46.28804, 15.084229 46.287525, 15.0841 46.28746, 15.083027 46.286643, 15.082727 46.286388, 15.08232 46.286109, 15.081805 46.285678, 15.081225 46.28527, 15.080302 46.284713, 15.07895 46.283962, 15.077469 46.283317, 15.076741 46.282932, 15.076247 46.282783, 15.075882 46.282695, 15.075474 46.282524, 15.075217 46.282394, 15.075066 46.282226, 15.074767 46.281967, 15.074509 46.281795, 15.074101 46.28244, 15.073822 46.28276, 15.073565 46.282955, 15.073222 46.283145, 15.07292 46.283317, 15.072299 46.283706, 15.07189 46.28405, 15.071633 46.284221, 15.071397 46.284263, 15.070904 46.284286, 15.070561 46.284263, 15.070324 46.284286, 15.069617 46.284519, 15.068844 46.284778, 15.068007 46.285121, 15.067234 46.285335, 15.066956 46.285141, 15.066247 46.284542, 15.065904 46.28424, 15.06511 46.283641, 15.064788 46.28347, 15.064553 46.283382, 15.064059 46.283145, 15.062792 46.282611, 15.06187 46.282203, 15.060926 46.281753, 15.061355 46.280487, 15.061463 46.28012, 15.061635 46.279647, 15.061698 46.279479, 15.062042 46.278556, 15.062042 46.278469, 15.06202 46.278297, 15.06187 46.278018, 15.06172 46.27787, 15.061269 46.27761, 15.060797 46.277397, 15.060347 46.277267, 15.059746 46.27703, 15.05951 46.276901, 15.058157 46.27626, 15.056656 46.275402, 15.056162 46.275142, 15.055862 46.274929, 15.05569 46.274757, 15.055325 46.274543, 15.055111 46.274372, 15.054896 46.274242, 15.054489 46.273811, 15.05391 46.273277, 15.05363 46.272975, 15.053201 46.272697, 15.052966 46.272483, 15.052729 46.272331, 15.052493 46.272052, 15.052343 46.271797, 15.052343 46.271583, 15.052257 46.271324, 15.052063 46.271152, 15.051291 46.270851, 15.051141 46.270721, 15.050992 46.270423, 15.05097 46.269565, 15.051399 46.268257, 15.051463 46.267932, 15.051485 46.267417, 15.051421 46.267269, 15.051485 46.267055, 15.051828 46.266323, 15.052472 46.265487, 15.0526 46.265209, 15.052922 46.264694, 15.05303 46.264434, 15.053115 46.263984, 15.053179 46.263835, 15.053309 46.263622, 15.053502 46.263362, 15.053652 46.263168, 15.053437 46.262546, 15.0526 46.262439, 15.052343 46.262439, 15.052 46.262332, 15.051678 46.262096, 15.051527 46.261905, 15.051442 46.261711, 15.051421 46.261627, 15.051442 46.261451, 15.051506 46.261283, 15.051527 46.261108, 15.051678 46.260787, 15.051699 46.260551, 15.051678 46.260444, 15.051506 46.260231, 15.051163 46.260124, 15.050841 46.260078, 15.050755 46.260017, 15.050197 46.259887, 15.050004 46.259887, 15.04979 46.259822, 15.049447 46.259628, 15.049317 46.259544, 15.048201 46.259544, 15.047858 46.259521, 15.047514 46.259456, 15.047193 46.259437, 15.046592 46.259284, 15.045991 46.259071, 15.045756 46.259029, 15.044876 46.258941, 15.044339 46.258769, 15.044167 46.258472, 15.044146 46.258106, 15.043931 46.257633, 15.043782 46.257484, 15.043738 46.256366, 15.043717 46.256347, 15.043631 46.255252, 15.043545 46.254287, 15.043502 46.253856, 15.043438 46.253471, 15.043202 46.252826, 15.042429 46.252998, 15.042043 46.253063, 15.041249 46.253276, 15.040692 46.253406, 15.040305 46.253536, 15.040005 46.253601, 15.039618 46.25362, 15.039039 46.253578, 15.037688 46.253406, 15.037558 46.25317, 15.037215 46.252784, 15.037087 46.252525, 15.037087 46.252399, 15.037173 46.252162, 15.037215 46.251777, 15.037215 46.251476, 15.037259 46.251239, 15.037194 46.250808, 15.037022 46.250358, 15.0367 46.249996, 15.036636 46.249866, 15.0364 46.249717, 15.036335 46.249629, 15.036185 46.249481, 15.036013 46.249374, 15.035498 46.249092, 15.035199 46.249008, 15.034919 46.248836, 15.034533 46.248558, 15.034404 46.248321, 15.03434 46.248043, 15.034276 46.24744, 15.033246 46.247486, 15.031744 46.247486, 15.030478 46.247505, 15.030091 46.24755, 15.028889 46.247741, 15.027773 46.247894, 15.026336 46.248062, 15.022109 46.248363, 15.021765 46.248363, 15.021272 46.248386, 15.020693 46.248363, 15.019985 46.248279, 15.019104 46.248126, 15.018504 46.248001, 15.018397 46.247978, 15.017861 46.247894, 15.014706 46.247741, 15.013741 46.247592, 15.012667 46.247505, 15.012282 46.24744, 15.011981 46.247398, 15.01123 46.247356, 15.010758 46.247356, 15.009985 46.24744, 15.009084 46.247592, 15.00829 46.247634, 15.007946 46.247612, 15.007218 46.247676, 15.004492 46.247955, 15.00387 46.24802, 15.003827 46.248043, 15.002497 46.248126, 15.002003 46.248172, 15.001681 46.248172, 15.000759 46.248191, 14.999986 46.248191, 14.997497 46.248321, 14.994708 46.248516, 14.992325 46.248641, 14.991231 46.248794, 14.990866 46.248836, 14.990416 46.248878, 14.989686 46.248901, 14.988914 46.248878, 14.988678 46.248878, 14.98812 46.248859, 14.987197 46.248901, 14.98576 46.248901, 14.984686 46.248859, 14.984429 46.248859, 14.983678 46.248813, 14.98312 46.248836, 14.98267 46.248878, 14.980975 46.249137, 14.979816 46.249328, 14.978915 46.249588, 14.977584 46.250061, 14.976233 46.250511, 14.975739 46.250617, 14.97473 46.250808, 14.974173 46.250896, 14.97325 46.250984, 14.972714 46.250984, 14.972199 46.250961, 14.971619 46.250873, 14.971211 46.250789, 14.970524 46.250595, 14.969988 46.250381, 14.969688 46.250232, 14.96958 46.250144, 14.968937 46.249759, 14.968358 46.249458, 14.967307 46.24905, 14.965848 46.248558, 14.965203 46.248321, 14.964409 46.247955, 14.9634 46.247398, 14.962414 46.246864, 14.961728 46.246521, 14.961105 46.246261, 14.960698 46.246067, 14.960225 46.245834, 14.959689 46.245597, 14.957779 46.244716, 14.956942 46.244331, 14.956341 46.24403, 14.954839 46.243343, 14.953294 46.242614, 14.950913 46.24139, 14.949904 46.240684, 14.949604 46.240425, 14.949238 46.240039, 14.948745 46.239437, 14.948424 46.238967, 14.947865 46.237743, 14.947651 46.237075, 14.94735 46.236434, 14.9472 46.236045, 14.946879 46.235359, 14.946213 46.23418, 14.945698 46.233257, 14.94542 46.232593, 14.945226 46.23225, 14.944861 46.231239, 14.944025 46.229053, 14.94351 46.227851, 14.943274 46.227893, 14.942931 46.227893, 14.942544 46.227787, 14.942136 46.227527, 14.941879 46.227401, 14.941535 46.227314, 14.940634 46.227249, 14.939862 46.227272, 14.938853 46.227272, 14.938509 46.227249, 14.93808 46.227184, 14.937523 46.227058, 14.937072 46.226905, 14.936106 46.226627, 14.93542 46.226478, 14.932523 46.22594, 14.931815 46.225791, 14.930506 46.225425, 14.929519 46.225105, 14.928275 46.224716, 14.927395 46.224502, 14.926836 46.224437, 14.925549 46.224395, 14.924927 46.224266, 14.923683 46.223987, 14.923125 46.223922, 14.922287 46.22388, 14.920872 46.223922, 14.918833 46.224029, 14.918297 46.224029, 14.917481 46.22388, 14.916902 46.223686, 14.916559 46.223495, 14.91613 46.223194, 14.915529 46.222679, 14.915271 46.222507, 14.914692 46.222229, 14.91407 46.221969, 14.913083 46.221519, 14.91231 46.221134, 14.912138 46.220985, 14.911924 46.220771, 14.911581 46.220169, 14.911173 46.219654, 14.910508 46.219097, 14.909778 46.218559, 14.90922 46.218109, 14.908898 46.217849, 14.908513 46.217617, 14.908083 46.217399, 14.90761 46.21725, 14.907032 46.217144, 14.904091 46.216693, 14.903534 46.216587, 14.901753 46.216178, 14.901345 46.216114, 14.900057 46.216072, 14.899329 46.215877, 14.899071 46.215877, 14.898792 46.215965, 14.898277 46.216156, 14.898127 46.216178, 14.897097 46.216178, 14.895874 46.216285, 14.895466 46.21635, 14.895037 46.216369, 14.892291 46.216369, 14.891303 46.216415, 14.890895 46.216457, 14.890358 46.216541, 14.889586 46.216606, 14.888621 46.216629, 14.887742 46.21667, 14.887033 46.216758, 14.886904 46.216758, 14.886839 46.216777, 14.88551 46.21693, 14.884909 46.21693, 14.883921 46.216819, 14.883063 46.216606, 14.88184 46.216178, 14.881218 46.216007, 14.880402 46.2159, 14.877442 46.215835, 14.876282 46.215835, 14.875767 46.215877, 14.875403 46.215942, 14.874845 46.216026, 14.874545 46.216049, 14.874201 46.216007, 14.873707 46.2159, 14.873621 46.215854, 14.873408 46.215747, 14.872592 46.215255, 14.871863 46.215103, 14.871498 46.215084, 14.870811 46.214977, 14.868472 46.21429, 14.868001 46.214183, 14.867206 46.21416, 14.865511 46.214031, 14.86476 46.214031, 14.863881 46.214096, 14.863022 46.214118, 14.862034 46.214073, 14.86094 46.214138, 14.860597 46.214054, 14.860082 46.213882, 14.859717 46.213729, 14.859417 46.213668, 14.859074 46.213668, 14.858065 46.213775, 14.856242 46.213752, 14.855662 46.213794, 14.854975 46.213924, 14.854697 46.214054, 14.854374 46.214183, 14.854138 46.214309, 14.853645 46.214504, 14.853216 46.214591, 14.851134 46.214698, 14.849911 46.214698, 14.848881 46.214611, 14.848044 46.214504, 14.847529 46.214504, 14.847144 46.214611, 14.8468 46.21474, 14.846457 46.214954, 14.845877 46.215362, 14.845641 46.215492, 14.844332 46.216156, 14.84386 46.216308, 14.843646 46.21635, 14.843237 46.216392, 14.842916 46.216369, 14.842616 46.216327, 14.842144 46.216156, 14.841435 46.21577, 14.841199 46.215663, 14.840856 46.215576, 14.840491 46.215557, 14.840106 46.215599, 14.839676 46.215705, 14.839203 46.215789, 14.838646 46.215854, 14.837573 46.216007, 14.8368 46.216072, 14.836415 46.216049, 14.835277 46.215877, 14.834719 46.215812, 14.834355 46.215705, 14.833882 46.215557, 14.833475 46.215362, 14.833045 46.21519, 14.832638 46.215019, 14.832209 46.214889, 14.83193 46.214847, 14.831629 46.214847, 14.831265 46.21487, 14.829978 46.215042, 14.82914 46.215126, 14.828668 46.215103, 14.827896 46.214912, 14.827595 46.214805, 14.826844 46.214611, 14.826329 46.214546, 14.826115 46.214527, 14.825728 46.214546, 14.825299 46.214591, 14.824784 46.214675, 14.823926 46.214912, 14.82296 46.215148, 14.822552 46.215232, 14.822188 46.215255, 14.821522 46.215168, 14.8209 46.215148, 14.819977 46.215168, 14.819656 46.215126, 14.818841 46.214977, 14.818154 46.214912, 14.816931 46.21474, 14.816823 46.21474, 14.816094 46.214611, 14.815171 46.214416, 14.814913 46.214374, 14.813605 46.21429, 14.813262 46.214225, 14.811888 46.214031, 14.811459 46.213989, 14.810515 46.214012, 14.809292 46.214073, 14.808905 46.214012, 14.80852 46.213859, 14.808283 46.213729, 14.807875 46.213687, 14.807511 46.213752, 14.807274 46.213882, 14.807189 46.214054, 14.807339 46.214504, 14.807232 46.214675, 14.806931 46.214824, 14.805602 46.215255, 14.805386 46.215297, 14.805001 46.21532, 14.804678 46.215297, 14.802897 46.215255, 14.80221 46.215148, 14.80176 46.215103, 14.801374 46.215126, 14.800967 46.215213, 14.800538 46.215339, 14.80015 46.215492, 14.7997 46.215621, 14.79925 46.215705, 14.798176 46.215812, 14.797984 46.215812, 14.797575 46.215854, 14.797146 46.215877, 14.796868 46.215877, 14.79601 46.215942, 14.795366 46.216007, 14.79498 46.216072, 14.79395 46.216327, 14.793456 46.216541, 14.793242 46.21667, 14.793026 46.216842, 14.792511 46.217636, 14.791354 46.218475, 14.791182 46.218688, 14.790967 46.218902, 14.790645 46.219181, 14.789958 46.219482, 14.789637 46.219589, 14.78925 46.21976, 14.788907 46.21989, 14.788199 46.220169, 14.787447 46.220428, 14.785731 46.22092, 14.785152 46.221199, 14.784701 46.221519, 14.784316 46.221755, 14.783671 46.222099, 14.783371 46.222206, 14.782663 46.222358, 14.782234 46.2224, 14.78129 46.2224, 14.78056 46.222377, 14.78011 46.222335, 14.779894 46.22227, 14.779745 46.222251, 14.77908 46.221862, 14.778479 46.221565, 14.778286 46.221435, 14.777599 46.221069, 14.776762 46.220813, 14.775002 46.220428, 14.774294 46.220298, 14.773565 46.220127, 14.77335 46.220104, 14.772728 46.219974, 14.771784 46.219783, 14.77026 46.219631, 14.769874 46.219589, 14.7676 46.219223, 14.765904 46.219055, 14.765068 46.218944, 14.764575 46.218925, 14.763866 46.218944, 14.763287 46.218925, 14.762235 46.218772, 14.761806 46.218772, 14.760668 46.218902, 14.760347 46.218879, 14.759789 46.218624, 14.759167 46.218368, 14.757643 46.218067, 14.755798 46.217807, 14.755261 46.217788, 14.75479 46.217849, 14.753309 46.218238, 14.752965 46.218303, 14.752129 46.218345, 14.751764 46.218322, 14.749424 46.218238, 14.747 46.218368, 14.74685 46.218387, 14.745991 46.218429, 14.744726 46.218536, 14.744211 46.218559, 14.743481 46.218559, 14.742859 46.218536, 14.742386 46.218494, 14.741057 46.21828, 14.739061 46.217872, 14.738975 46.217872, 14.73831 46.217681, 14.737216 46.21725, 14.736228 46.21693, 14.73565 46.216865, 14.735091 46.216758, 14.73462 46.216629, 14.734318 46.216587, 14.733997 46.216434, 14.733632 46.216178, 14.733396 46.2159, 14.733224 46.215641, 14.733031 46.215492, 14.732731 46.21532, 14.732172 46.215061, 14.731851 46.214889, 14.731336 46.214698, 14.73095 46.214591, 14.730306 46.214847, 14.72962 46.215084, 14.728976 46.215278, 14.727688 46.215599, 14.726552 46.215835, 14.725113 46.216072, 14.724749 46.216072, 14.723719 46.216114, 14.723182 46.216197, 14.723053 46.21622, 14.722432 46.216197, 14.722023 46.216243, 14.721723 46.216262, 14.720757 46.216072, 14.72035 46.216049, 14.720028 46.216091, 14.719256 46.216369, 14.718848 46.216541, 14.718462 46.216629, 14.717753 46.21667, 14.717238 46.216777, 14.716767 46.216842, 14.716465 46.216842, 14.71535 46.216651, 14.714663 46.216564, 14.713827 46.216693, 14.713461 46.216777, 14.712968 46.21693, 14.712689 46.217037, 14.712453 46.217208, 14.712002 46.2177, 14.71181 46.217872, 14.711573 46.217979, 14.71093 46.218151, 14.709836 46.218475, 14.709492 46.218517, 14.70872 46.218429, 14.708591 46.21841, 14.707797 46.218303, 14.705781 46.218086, 14.705329 46.218086, 14.704878 46.218044, 14.703742 46.21783, 14.703527 46.217357, 14.703291 46.217144, 14.703398 46.216972, 14.703398 46.216777, 14.703355 46.216629, 14.703227 46.216564, 14.702755 46.216392, 14.702647 46.216327, 14.702347 46.215984, 14.702176 46.216007, 14.70196 46.216007, 14.701661 46.216114, 14.701081 46.216327, 14.700888 46.216285, 14.700758 46.216178, 14.701146 46.215362, 14.701445 46.215042, 14.701617 46.214805, 14.701725 46.214462, 14.701725 46.214183, 14.701617 46.213108, 14.701531 46.212699, 14.701424 46.212486, 14.701167 46.212165, 14.700866 46.211994, 14.700415 46.211822, 14.699343 46.211479, 14.698291 46.210769, 14.69739 46.210189, 14.697154 46.209999, 14.696703 46.209377, 14.696339 46.208755, 14.696082 46.208152, 14.695845 46.207725, 14.695523 46.207359, 14.694815 46.206478, 14.694579 46.206092, 14.694558 46.205898, 14.694579 46.205726, 14.694644 46.205642, 14.69503 46.205383, 14.695244 46.205276, 14.695459 46.205104, 14.695609 46.20502, 14.695652 46.204891, 14.695523 46.204784, 14.695201 46.204719, 14.694729 46.204505, 14.694558 46.204525, 14.6943 46.204742, 14.694043 46.204826, 14.69372 46.204742, 14.693313 46.204399, 14.692391 46.203948, 14.692197 46.203666, 14.692026 46.203517, 14.691768 46.203388, 14.691511 46.203067, 14.691854 46.202873, 14.692197 46.202743, 14.692412 46.202529, 14.692477 46.202293, 14.69269 46.202056, 14.693549 46.201351, 14.693828 46.201221, 14.694043 46.201091, 14.694214 46.200794, 14.694644 46.199871, 14.694858 46.199569, 14.694901 46.19942, 14.695179 46.199203, 14.695481 46.199012, 14.695716 46.198883, 14.696467 46.198581, 14.696639 46.19841, 14.696618 46.198261, 14.696553 46.198112, 14.696597 46.197895, 14.696597 46.197723, 14.69636 46.197059, 14.696038 46.196971, 14.694836 46.196823, 14.694644 46.196758, 14.694343 46.196693, 14.694193 46.196628, 14.69385 46.196414, 14.693635 46.196178, 14.693549 46.195964, 14.693571 46.195793, 14.69372 46.195514, 14.693871 46.195278, 14.694086 46.195064, 14.694235 46.194934, 14.694815 46.194656, 14.695158 46.194568, 14.695373 46.194442, 14.695459 46.194248, 14.695481 46.194053, 14.695395 46.193862, 14.694966 46.193603, 14.694429 46.193241, 14.694171 46.192787, 14.694022 46.192188, 14.693764 46.192146, 14.693442 46.191802, 14.693184 46.191631, 14.69297 46.191394, 14.692669 46.191288, 14.692412 46.191246, 14.692004 46.191223, 14.691682 46.191181, 14.691082 46.190856, 14.691039 46.19075, 14.690888 46.190559, 14.69063 46.190063, 14.690438 46.189849, 14.690094 46.189529, 14.6896 46.189163, 14.688786 46.188667, 14.687928 46.188217, 14.687562 46.187873, 14.687348 46.187595, 14.687262 46.187423, 14.687133 46.186439, 14.686962 46.186244, 14.686747 46.186096, 14.686469 46.185966, 14.686211 46.185813, 14.685932 46.185577, 14.685566 46.185321, 14.685309 46.185192, 14.684924 46.185066, 14.684473 46.184955, 14.683421 46.184783, 14.682906 46.184741, 14.68252 46.184635, 14.682284 46.184528, 14.681769 46.184333, 14.68134 46.18412, 14.680502 46.183734, 14.679902 46.183368, 14.679623 46.183284, 14.679387 46.183303, 14.679065 46.18341, 14.678636 46.183734, 14.678401 46.18386, 14.678185 46.184013, 14.677907 46.18412, 14.676834 46.184333, 14.676319 46.184463, 14.67589 46.184547, 14.675761 46.18457, 14.674796 46.184806, 14.674494 46.184848, 14.674259 46.184848, 14.674044 46.184806, 14.67383 46.184783, 14.673658 46.184741, 14.673615 46.184741, 14.673572 46.184722, 14.672671 46.184505, 14.672606 46.184268, 14.67252 46.18399, 14.672349 46.183734, 14.672221 46.183456, 14.672478 46.182769, 14.67252 46.182533, 14.672456 46.182296, 14.671919 46.181674, 14.671662 46.181415, 14.670975 46.180366, 14.670632 46.179935, 14.670482 46.179656, 14.670353 46.179336, 14.670311 46.179164, 14.67046 46.178497, 14.670611 46.178005, 14.670676 46.177703, 14.670611 46.177402, 14.67059 46.177253, 14.670418 46.176868, 14.670225 46.176544, 14.670075 46.176139, 14.669989 46.175815, 14.669796 46.175342, 14.669646 46.175064, 14.66956 46.174957, 14.669388 46.174591, 14.66943 46.174591, 14.669732 46.174591, 14.669967 46.174613, 14.670161 46.174678, 14.670825 46.17485, 14.671169 46.174957, 14.67207 46.17517, 14.672843 46.175277, 14.673293 46.1753, 14.673486 46.175258, 14.673658 46.17517, 14.67383 46.175022, 14.673915 46.174915, 14.673958 46.17472, 14.674065 46.174312, 14.674302 46.174057, 14.674559 46.173969, 14.674881 46.17395, 14.675482 46.17382, 14.675696 46.173713, 14.676125 46.173393, 14.676276 46.17324, 14.676684 46.17279, 14.676791 46.172595, 14.676941 46.172382, 14.677134 46.172275, 14.677714 46.171997, 14.678271 46.17176, 14.678442 46.171653, 14.678464 46.171394, 14.678336 46.171245, 14.677928 46.171051, 14.677584 46.170795, 14.677371 46.170581, 14.677285 46.170429, 14.677263 46.17028, 14.677091 46.169357, 14.677091 46.169078, 14.676919 46.16867, 14.676748 46.168476, 14.676468 46.168243, 14.676276 46.167984, 14.676147 46.167747, 14.676125 46.167167, 14.676169 46.16687, 14.676255 46.166526, 14.67619 46.166244, 14.676169 46.165901, 14.675696 46.165409, 14.675503 46.165195, 14.674967 46.16468, 14.674624 46.164249, 14.67458 46.164077, 14.674624 46.163822, 14.674817 46.163093, 14.67486 46.162811, 14.673293 46.162574, 14.672843 46.16249, 14.672049 46.162384, 14.67046 46.162231, 14.669495 46.16217, 14.669323 46.162189, 14.667178 46.162124, 14.665825 46.162017, 14.663873 46.161804, 14.663552 46.161762, 14.663358 46.161739, 14.662478 46.161655, 14.66029 46.16159, 14.658466 46.161609, 14.657757 46.161632, 14.656212 46.16159, 14.655633 46.161567, 14.653208 46.161437, 14.6522 46.161312, 14.650913 46.161075, 14.650827 46.161033, 14.650398 46.160945, 14.649476 46.16069, 14.647136 46.159915, 14.646513 46.15966, 14.644625 46.158798, 14.644304 46.158672, 14.643509 46.158435, 14.642888 46.158264, 14.642544 46.158199, 14.641858 46.158027, 14.639326 46.157382, 14.638617 46.157169, 14.638038 46.15702, 14.637415 46.156826, 14.637051 46.156738, 14.636751 46.156631, 14.63542 46.156227, 14.634562 46.15599, 14.633254 46.155582, 14.629305 46.154399, 14.628597 46.154167, 14.62806 46.153991, 14.627974 46.153972, 14.627138 46.153694, 14.626279 46.153434, 14.626151 46.153392, 14.625335 46.153137, 14.625056 46.153072, 14.624091 46.152748, 14.622954 46.152385, 14.622353 46.152214, 14.62098 46.151763, 14.616452 46.150344, 14.614821 46.149829, 14.61304 46.149272, 14.61289 46.14923, 14.612353 46.149101, 14.611731 46.148906, 14.610938 46.148609, 14.609908 46.148307, 14.608534 46.147941, 14.607397 46.147644, 14.606045 46.147319, 14.605036 46.147106, 14.604844 46.147083, 14.604092 46.146934, 14.601303 46.146503, 14.601538 46.146034, 14.600745 46.14524, 14.600273 46.144721, 14.599758 46.144229, 14.599114 46.143886, 14.598664 46.14363, 14.59847 46.143543, 14.598105 46.143306, 14.597312 46.142837, 14.596303 46.142429, 14.595767 46.142234, 14.595316 46.142127, 14.594887 46.142021, 14.594414 46.141933, 14.594372 46.141761, 14.594265 46.140861, 14.594115 46.139896, 14.594115 46.139488, 14.594136 46.138736, 14.593921 46.137855, 14.593964 46.137298, 14.594029 46.13702, 14.594157 46.136096, 14.594372 46.135025, 14.59463 46.133911, 14.594672 46.133567, 14.594822 46.133029, 14.595037 46.132148, 14.595101 46.131805, 14.595123 46.13084, 14.595015 46.129554, 14.595015 46.129402, 14.594973 46.127494, 14.594908 46.127063, 14.594887 46.126701, 14.594716 46.125949, 14.5945 46.125305, 14.594286 46.124595, 14.594136 46.123996, 14.593964 46.123458, 14.593835 46.123203, 14.593535 46.122558, 14.59332 46.122257, 14.593063 46.121871, 14.592955 46.121593, 14.592548 46.120262, 14.592548 46.120197, 14.592354 46.119167, 14.59229 46.118522, 14.592141 46.11771, 14.592097 46.117172, 14.592141 46.116313, 14.59229 46.114791, 14.592505 46.11301, 14.592591 46.112514, 14.592698 46.112472, 14.592913 46.11185, 14.592999 46.111484, 14.593171 46.110969, 14.593277 46.111057, 14.593428 46.111377, 14.5936 46.111442, 14.593985 46.111484, 14.594328 46.111595, 14.594779 46.111701, 14.595059 46.111827, 14.595358 46.111938)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testMiddleLong1H20() throws Exception {
    Route route=test(38574, 17774, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(2.0));

    logger.info("Found route="+route);

    String result="LINESTRING (15.102147 46.369964, 15.102576 46.369728, 15.102919 46.369491, 15.103606 46.36885, 15.103283 46.368141, 15.103219 46.367904, 15.102747 46.367324, 15.102725 46.367027, 15.102768 46.366683, 15.102811 46.366081, 15.102983 46.365524, 15.104614 46.364856, 15.104979 46.364643, 15.1053 46.364494, 15.105879 46.36428, 15.106245 46.364105, 15.106159 46.363807, 15.106181 46.363548, 15.10648 46.362991, 15.106566 46.362819, 15.10661 46.362625, 15.106545 46.362327, 15.105257 46.362754, 15.103692 46.363357, 15.103112 46.363636, 15.102961 46.363677, 15.102725 46.363613, 15.10249 46.363594, 15.102125 46.363594, 15.10161 46.363742, 15.100923 46.362884, 15.100516 46.362369, 15.099442 46.362731, 15.095387 46.364192, 15.094014 46.362411, 15.093691 46.362262, 15.093262 46.361938, 15.09307 46.362003, 15.092726 46.36222, 15.092168 46.362606, 15.091997 46.362754, 15.091825 46.362861, 15.091653 46.363033, 15.091224 46.363334, 15.090645 46.363636, 15.090323 46.363742, 15.089808 46.363979, 15.0894 46.364234, 15.089015 46.364559, 15.088821 46.364749, 15.087212 46.363979, 15.087018 46.363891, 15.086847 46.363784, 15.086718 46.363441, 15.086567 46.363163, 15.086461 46.363014, 15.086246 46.362842, 15.085688 46.362541, 15.085559 46.362453, 15.085474 46.362369, 15.085431 46.362091, 15.085345 46.361984, 15.085238 46.361896, 15.085022 46.361831, 15.084486 46.361812, 15.084294 46.361576, 15.083992 46.361423, 15.0838 46.361251, 15.083585 46.361061, 15.08322 46.36061, 15.082877 46.360309, 15.082727 46.360328, 15.082555 46.360309, 15.082341 46.360328, 15.081761 46.359966, 15.081525 46.359771, 15.081376 46.359474, 15.081376 46.359149, 15.081439 46.358722, 15.081311 46.357883, 15.081311 46.357627, 15.081311 46.357349, 15.081354 46.357112, 15.081439 46.356918, 15.081461 46.356918, 15.081633 46.356597, 15.081783 46.356166, 15.081805 46.355976, 15.081761 46.355823, 15.081761 46.355716, 15.081783 46.355567, 15.081891 46.355415, 15.081933 46.355243, 15.081933 46.355117, 15.081869 46.355011, 15.08174 46.3549, 15.081118 46.354686, 15.08071 46.354621, 15.080195 46.354515, 15.079831 46.354492, 15.079529 46.354278, 15.079316 46.354087, 15.07923 46.353958, 15.079272 46.353336, 15.079529 46.352928, 15.079701 46.352821, 15.079787 46.352668, 15.079787 46.352561, 15.079637 46.35239, 15.079422 46.352046, 15.079337 46.351921, 15.079316 46.351726, 15.080109 46.351104, 15.080302 46.351017, 15.080538 46.351081, 15.080775 46.351169, 15.080988 46.351276, 15.081719 46.35136, 15.081976 46.351234, 15.082104 46.351081, 15.082255 46.350997, 15.082856 46.350524, 15.083371 46.350074, 15.083264 46.349945, 15.082791 46.349838, 15.082684 46.349624, 15.082705 46.349517, 15.08277 46.349407, 15.082921 46.349258, 15.083092 46.349128, 15.083671 46.348892, 15.083779 46.348873, 15.084057 46.348678, 15.0841 46.348529, 15.084078 46.3484, 15.084208 46.347885, 15.084208 46.347713, 15.084014 46.347263, 15.083951 46.347026, 15.083907 46.346576, 15.083929 46.346233, 15.083886 46.345867, 15.083779 46.345611, 15.083628 46.345352, 15.083328 46.345031, 15.08307 46.344665, 15.083006 46.344387, 15.082984 46.344215, 15.083027 46.344043, 15.083156 46.34383, 15.083328 46.343185, 15.083264 46.342735, 15.083285 46.342475, 15.083199 46.342308, 15.083134 46.342025, 15.083156 46.341747, 15.083285 46.341598, 15.083242 46.341384, 15.083264 46.341018, 15.083392 46.340805, 15.083628 46.340545, 15.083714 46.340461, 15.083693 46.340072, 15.083714 46.339881, 15.083649 46.339561, 15.083477 46.339302, 15.08322 46.339, 15.083092 46.338893, 15.082899 46.338615, 15.082877 46.338443, 15.082921 46.338314, 15.08307 46.338123, 15.083092 46.337886, 15.083242 46.337413, 15.08335 46.337158, 15.083477 46.336963, 15.083563 46.336769, 15.083649 46.336471, 15.083735 46.336254, 15.083886 46.33604, 15.083971 46.335868, 15.084078 46.33559, 15.084078 46.334968, 15.0841 46.334472, 15.084229 46.334323, 15.084315 46.334152, 15.084658 46.334022, 15.085066 46.333702, 15.085152 46.333507, 15.085324 46.333251, 15.085516 46.332992, 15.085667 46.332862, 15.08586 46.332778, 15.086203 46.332565, 15.086396 46.332477, 15.086589 46.332115, 15.086611 46.331878, 15.086567 46.3316, 15.086503 46.331298, 15.086525 46.330375, 15.086546 46.329837, 15.086653 46.32954, 15.086697 46.32928, 15.086847 46.328979, 15.087212 46.328464, 15.087426 46.328231, 15.088134 46.327072, 15.088541 46.326683, 15.088778 46.326469, 15.088864 46.326339, 15.088971 46.326042, 15.089207 46.325763, 15.089358 46.325527, 15.089358 46.325397, 15.089293 46.325248, 15.089056 46.325031, 15.088864 46.324882, 15.088521 46.324752, 15.088284 46.324581, 15.088091 46.324432, 15.087705 46.324047, 15.087405 46.32336, 15.087298 46.323188, 15.08719 46.322887, 15.087126 46.322628, 15.087104 46.322307, 15.087061 46.322135, 15.087147 46.321899, 15.087405 46.321426, 15.087641 46.32117, 15.087899 46.320957, 15.088134 46.320846, 15.08837 46.32059, 15.088392 46.320354, 15.08807 46.32059, 15.087941 46.320632, 15.087641 46.320678, 15.087491 46.32072, 15.086739 46.320785, 15.086482 46.320892, 15.08631 46.320934, 15.086052 46.321235, 15.085946 46.321491, 15.085731 46.321834, 15.085559 46.3222, 15.085388 46.322479, 15.084551 46.32278, 15.084315 46.322845, 15.084036 46.322845, 15.083456 46.32278, 15.083306 46.322738, 15.082899 46.322673, 15.082705 46.322586, 15.082577 46.322502, 15.082469 46.322219, 15.082298 46.322135, 15.081805 46.3222, 15.081482 46.3222, 15.081096 46.322242, 15.080517 46.322242, 15.080302 46.322219, 15.07998 46.322093, 15.079831 46.322006, 15.079573 46.321922, 15.079251 46.321857, 15.078972 46.321922, 15.078693 46.321941, 15.078221 46.322135, 15.077555 46.32233, 15.077277 46.322372, 15.077062 46.322349, 15.076397 46.322029, 15.076096 46.321857, 15.075646 46.321491, 15.075066 46.321105, 15.074595 46.320869, 15.074122 46.320697, 15.073693 46.320503, 15.073286 46.320442, 15.072685 46.320377, 15.071955 46.320377, 15.071719 46.320335, 15.071591 46.32027, 15.071591 46.320159, 15.071547 46.319992, 15.071419 46.319881, 15.07026 46.319305, 15.069938 46.319194, 15.069703 46.319217, 15.069273 46.319324, 15.068608 46.31969, 15.068371 46.319904, 15.068093 46.320117, 15.067878 46.32027, 15.067427 46.320526, 15.067277 46.320526, 15.067149 46.320484, 15.06702 46.320247, 15.066805 46.319992, 15.066613 46.319946, 15.066699 46.320312, 15.066699 46.320461, 15.066634 46.320526, 15.066376 46.320442, 15.066397 46.320335, 15.066311 46.320098, 15.06614 46.319862, 15.065947 46.319648, 15.065796 46.319412, 15.065561 46.31911, 15.064917 46.318164, 15.064745 46.318038, 15.064337 46.317478, 15.06408 46.317199, 15.063844 46.316577, 15.063651 46.315399, 15.063544 46.314991, 15.063501 46.314689, 15.063523 46.314453, 15.063565 46.314174, 15.063523 46.313919, 15.063565 46.313682, 15.063651 46.313404, 15.063801 46.313102, 15.063887 46.312843, 15.063844 46.312629, 15.063694 46.312438, 15.063393 46.312221, 15.063072 46.312072, 15.062579 46.311878, 15.062428 46.311729, 15.062428 46.311599, 15.062557 46.311515, 15.0626 46.31145, 15.062814 46.311386, 15.062943 46.311298, 15.063029 46.311149, 15.062943 46.310806, 15.062857 46.310657, 15.062493 46.310421, 15.062235 46.310314, 15.061676 46.310184, 15.060904 46.309906, 15.060411 46.309692, 15.060175 46.309497, 15.059918 46.309219, 15.05966 46.308662, 15.059467 46.308444, 15.05878 46.307865, 15.05863 46.307674, 15.058179 46.307201, 15.058115 46.306987, 15.058115 46.306793, 15.057965 46.306686, 15.057793 46.306621, 15.057621 46.306621, 15.057515 46.306644, 15.056935 46.306816, 15.056784 46.306816, 15.056591 46.30677, 15.056506 46.306686, 15.056313 46.306602, 15.056141 46.306491, 15.05597 46.306343, 15.055884 46.306236, 15.055819 46.306083, 15.055819 46.305892, 15.055884 46.305572, 15.055776 46.305377, 15.055712 46.305141, 15.055776 46.304885, 15.055776 46.30471, 15.055819 46.304477, 15.055712 46.304325, 15.055239 46.30381, 15.055111 46.303573, 15.055047 46.303039, 15.054896 46.302246, 15.054918 46.301902, 15.055283 46.301429, 15.05539 46.301235, 15.055776 46.300891, 15.055905 46.30072, 15.055991 46.300571, 15.056141 46.300399, 15.056227 46.300163, 15.056162 46.300033, 15.056141 46.299861, 15.056205 46.299755, 15.056313 46.299476, 15.056463 46.29924, 15.056549 46.299049, 15.056698 46.298339, 15.05672 46.29808, 15.056698 46.297908, 15.056784 46.297676, 15.056742 46.297073, 15.056806 46.296665, 15.057042 46.296173, 15.057515 46.295528, 15.057664 46.295421, 15.057793 46.295291, 15.058051 46.294818, 15.058394 46.294326, 15.05863 46.293941, 15.058974 46.29351, 15.059638 46.292758, 15.061076 46.291496, 15.061247 46.291172, 15.061419 46.290977, 15.062213 46.290378, 15.062685 46.289863, 15.062943 46.289348, 15.062922 46.289199, 15.062943 46.289112, 15.062922 46.288661, 15.063008 46.28836, 15.063094 46.288169, 15.063437 46.287761, 15.06363 46.287525, 15.063952 46.287311, 15.064359 46.287158, 15.064766 46.287052, 15.065303 46.286987, 15.06584 46.28688, 15.066376 46.28688, 15.066699 46.286838, 15.066956 46.286708, 15.067106 46.286537, 15.06717 46.286323, 15.067234 46.285892, 15.067255 46.285572, 15.067234 46.285423, 15.067234 46.285335, 15.066956 46.285141, 15.066247 46.284542, 15.065904 46.28424, 15.06511 46.283641, 15.064788 46.28347, 15.064553 46.283382, 15.064059 46.283145, 15.062792 46.282611, 15.06187 46.282203, 15.060926 46.281753, 15.061355 46.280487, 15.061463 46.28012, 15.061635 46.279647, 15.061698 46.279479, 15.062042 46.278556, 15.062042 46.278469, 15.06202 46.278297, 15.06187 46.278018, 15.06172 46.27787, 15.061269 46.27761, 15.060797 46.277397, 15.060347 46.277267, 15.059746 46.27703, 15.061141 46.27581, 15.061441 46.275615, 15.062042 46.275272, 15.062535 46.274951, 15.062922 46.274456, 15.063115 46.274112, 15.063286 46.273876, 15.063694 46.273082, 15.064617 46.272052, 15.064852 46.27169, 15.065003 46.27098, 15.065003 46.270767, 15.065068 46.270378, 15.065175 46.270099, 15.065281 46.269844, 15.065475 46.269565, 15.065625 46.269393, 15.06629 46.268836, 15.066333 46.268791, 15.067471 46.267784, 15.067599 46.267631, 15.067729 46.267482, 15.068158 46.266838, 15.068371 46.266368, 15.068651 46.265617, 15.069401 46.263298, 15.069745 46.262355, 15.070024 46.261753, 15.070345 46.261238, 15.070561 46.260894, 15.070647 46.260704, 15.071183 46.25978, 15.07114 46.259758, 15.069445 46.259479, 15.068393 46.25933, 15.067835 46.259201, 15.066699 46.259159, 15.065432 46.259136, 15.061827 46.259006, 15.059918 46.258964, 15.058243 46.258899, 15.05775 46.258899, 15.055604 46.258792, 15.054961 46.25875, 15.053566 46.258621, 15.052944 46.258579, 15.051656 46.258449, 15.050776 46.258342, 15.049059 46.258106, 15.046614 46.25772, 15.045133 46.257507, 15.044146 46.257377, 15.043995 46.257377, 15.042515 46.257205, 15.040584 46.257053, 15.037087 46.256881, 15.036765 46.256881, 15.034147 46.256881, 15.032881 46.256927, 15.029919 46.257076, 15.027838 46.25727, 15.023633 46.257675, 15.021208 46.257934, 15.019556 46.258083, 15.016959 46.258277, 15.013934 46.258384, 15.01241 46.258384, 15.010607 46.258342, 15.009062 46.258277, 15.008698 46.258277, 15.006188 46.258083, 15.004321 46.257892, 15.001595 46.257568, 14.999986 46.257396, 14.998227 46.257141, 14.995866 46.256732, 14.993721 46.256412, 14.991425 46.256134, 14.989022 46.255916, 14.984257 46.255661, 14.983721 46.255638, 14.98312 46.255554, 14.982648 46.255508, 14.981189 46.255271, 14.979601 46.254886, 14.978099 46.254436, 14.976941 46.254009, 14.975461 46.253322, 14.974924 46.253063, 14.974022 46.252525, 14.973572 46.252227, 14.972821 46.251861, 14.972477 46.251754, 14.972199 46.251731, 14.971812 46.251754, 14.971362 46.251754, 14.971147 46.251712, 14.971018 46.251647, 14.970912 46.251625, 14.970761 46.25156, 14.970524 46.251369, 14.970438 46.251262, 14.970397 46.251091, 14.970397 46.250961, 14.970438 46.250747, 14.970524 46.250595, 14.969988 46.250381, 14.969688 46.250232, 14.96958 46.250144, 14.968937 46.249759, 14.968358 46.249458, 14.967307 46.24905, 14.966919 46.249073, 14.966083 46.249286, 14.965568 46.249267, 14.965096 46.249179, 14.964796 46.249073, 14.964087 46.248664, 14.963723 46.248493, 14.963551 46.248386, 14.963422 46.248237, 14.963294 46.247978, 14.963165 46.247783, 14.963015 46.247634, 14.962779 46.247486, 14.962607 46.24744, 14.962349 46.247463, 14.962049 46.247463, 14.961834 46.247375, 14.961491 46.247226, 14.960976 46.246925, 14.960547 46.246711, 14.960332 46.246669, 14.959925 46.24654, 14.959388 46.246303, 14.959239 46.246219, 14.958937 46.246112, 14.958422 46.246047, 14.957478 46.245834, 14.955483 46.245552, 14.954324 46.245597, 14.953959 46.245681, 14.953358 46.245918, 14.953101 46.245853, 14.952479 46.245468, 14.952328 46.245273, 14.951642 46.244804, 14.951191 46.244461, 14.950033 46.243942, 14.94941 46.243556, 14.948895 46.242935, 14.948723 46.242744, 14.948081 46.242225, 14.947995 46.242034, 14.948166 46.241821, 14.948359 46.241649, 14.948466 46.241455, 14.948488 46.241241, 14.948316 46.240768, 14.948252 46.240509, 14.948208 46.240016, 14.948102 46.239608, 14.94748 46.239223, 14.947243 46.239051, 14.946965 46.238731, 14.946685 46.238105, 14.946578 46.237701, 14.946514 46.237075, 14.946406 46.236671, 14.946278 46.236347, 14.946536 46.236156, 14.947114 46.236045, 14.9472 46.236045, 14.946879 46.235359, 14.946213 46.23418, 14.945698 46.233257, 14.94542 46.232593, 14.945226 46.23225, 14.944861 46.231239, 14.944025 46.229053, 14.94351 46.227851, 14.943274 46.227893, 14.942931 46.227893, 14.942544 46.227787, 14.942136 46.227527, 14.941879 46.227401, 14.941535 46.227314, 14.940634 46.227249, 14.939862 46.227272, 14.938853 46.227272, 14.938509 46.227249, 14.93808 46.227184, 14.937523 46.227058, 14.937072 46.226905, 14.936106 46.226627, 14.93542 46.226478, 14.932523 46.22594, 14.931815 46.225791, 14.930506 46.225425, 14.929519 46.225105, 14.928275 46.224716, 14.927395 46.224502, 14.926836 46.224437, 14.925549 46.224395, 14.924927 46.224266, 14.923683 46.223987, 14.923125 46.223922, 14.922287 46.22388, 14.920872 46.223922, 14.918833 46.224029, 14.918297 46.224029, 14.917481 46.22388, 14.916902 46.223686, 14.916559 46.223495, 14.91613 46.223194, 14.915529 46.222679, 14.915271 46.222507, 14.914692 46.222229, 14.91407 46.221969, 14.913083 46.221519, 14.91231 46.221134, 14.912138 46.220985, 14.911924 46.220771, 14.911581 46.220169, 14.911173 46.219654, 14.910508 46.219097, 14.909778 46.218559, 14.90922 46.218109, 14.908898 46.217849, 14.908513 46.217617, 14.908083 46.217399, 14.90761 46.21725, 14.907032 46.217144, 14.904091 46.216693, 14.903534 46.216587, 14.901753 46.216178, 14.901345 46.216114, 14.900057 46.216072, 14.899329 46.215877, 14.899071 46.215877, 14.898792 46.215965, 14.898277 46.216156, 14.898127 46.216178, 14.897097 46.216178, 14.895874 46.216285, 14.895466 46.21635, 14.895037 46.216369, 14.892291 46.216369, 14.891303 46.216415, 14.890895 46.216457, 14.890358 46.216541, 14.889586 46.216606, 14.888621 46.216629, 14.887742 46.21667, 14.887033 46.216758, 14.886904 46.216758, 14.886839 46.216777, 14.88551 46.21693, 14.884909 46.21693, 14.883921 46.216819, 14.883063 46.216606, 14.88184 46.216178, 14.881218 46.216007, 14.880402 46.2159, 14.877442 46.215835, 14.876282 46.215835, 14.875767 46.215877, 14.875403 46.215942, 14.874845 46.216026, 14.874545 46.216049, 14.874201 46.216007, 14.873707 46.2159, 14.873621 46.215854, 14.873408 46.215747, 14.872592 46.215255, 14.871863 46.215103, 14.871498 46.215084, 14.870811 46.214977, 14.868472 46.21429, 14.868001 46.214183, 14.867206 46.21416, 14.865511 46.214031, 14.86476 46.214031, 14.863881 46.214096, 14.863022 46.214118, 14.862034 46.214073, 14.86094 46.214138, 14.860597 46.214054, 14.860082 46.213882, 14.859717 46.213729, 14.859417 46.213668, 14.859074 46.213668, 14.858065 46.213775, 14.856242 46.213752, 14.855662 46.213794, 14.854975 46.213924, 14.854697 46.214054, 14.854374 46.214183, 14.854138 46.214309, 14.853645 46.214504, 14.853216 46.214591, 14.851134 46.214698, 14.849911 46.214698, 14.848881 46.214611, 14.848044 46.214504, 14.847529 46.214504, 14.847144 46.214611, 14.8468 46.21474, 14.846457 46.214954, 14.845877 46.215362, 14.845641 46.215492, 14.844332 46.216156, 14.84386 46.216308, 14.843646 46.21635, 14.843237 46.216392, 14.842916 46.216369, 14.842616 46.216327, 14.842144 46.216156, 14.841435 46.21577, 14.841199 46.215663, 14.840856 46.215576, 14.840491 46.215557, 14.840106 46.215599, 14.839676 46.215705, 14.839203 46.215789, 14.838646 46.215854, 14.837573 46.216007, 14.8368 46.216072, 14.836415 46.216049, 14.835277 46.215877, 14.834719 46.215812, 14.834355 46.215705, 14.833882 46.215557, 14.833475 46.215362, 14.833045 46.21519, 14.832638 46.215019, 14.832209 46.214889, 14.83193 46.214847, 14.831629 46.214847, 14.831265 46.21487, 14.829978 46.215042, 14.82914 46.215126, 14.828668 46.215103, 14.827896 46.214912, 14.827595 46.214805, 14.826844 46.214611, 14.826329 46.214546, 14.826115 46.214527, 14.825728 46.214546, 14.825299 46.214591, 14.824784 46.214675, 14.823926 46.214912, 14.82296 46.215148, 14.822552 46.215232, 14.822188 46.215255, 14.821522 46.215168, 14.8209 46.215148, 14.819977 46.215168, 14.819656 46.215126, 14.818841 46.214977, 14.818154 46.214912, 14.816931 46.21474, 14.816823 46.21474, 14.816094 46.214611, 14.815171 46.214416, 14.814913 46.214374, 14.813605 46.21429, 14.813262 46.214225, 14.811888 46.214031, 14.811459 46.213989, 14.810515 46.214012, 14.809292 46.214073, 14.808905 46.214012, 14.80852 46.213859, 14.808283 46.213729, 14.807875 46.213687, 14.807511 46.213752, 14.807274 46.213882, 14.807189 46.214054, 14.807339 46.214504, 14.807232 46.214675, 14.806931 46.214824, 14.805602 46.215255, 14.805386 46.215297, 14.805001 46.21532, 14.804678 46.215297, 14.802897 46.215255, 14.80221 46.215148, 14.80176 46.215103, 14.801374 46.215126, 14.800967 46.215213, 14.800538 46.215339, 14.80015 46.215492, 14.7997 46.215621, 14.79925 46.215705, 14.798176 46.215812, 14.797984 46.215812, 14.797575 46.215854, 14.797146 46.215877, 14.796868 46.215877, 14.79601 46.215942, 14.795366 46.216007, 14.79498 46.216072, 14.79395 46.216327, 14.793456 46.216541, 14.793242 46.21667, 14.793026 46.216842, 14.792511 46.217636, 14.791354 46.218475, 14.791182 46.218688, 14.790967 46.218902, 14.790645 46.219181, 14.789958 46.219482, 14.789637 46.219589, 14.78925 46.21976, 14.788907 46.21989, 14.788199 46.220169, 14.787447 46.220428, 14.785731 46.22092, 14.785152 46.221199, 14.784701 46.221519, 14.784316 46.221755, 14.783671 46.222099, 14.783371 46.222206, 14.782663 46.222358, 14.782234 46.2224, 14.78129 46.2224, 14.78056 46.222377, 14.78011 46.222335, 14.779894 46.22227, 14.779745 46.222251, 14.77908 46.221862, 14.778479 46.221565, 14.778157 46.221778, 14.777599 46.221755, 14.776934 46.221927, 14.776655 46.222057, 14.776118 46.222377, 14.775882 46.222614, 14.775647 46.222999, 14.775582 46.22311, 14.775496 46.223365, 14.775367 46.223537, 14.775153 46.223709, 14.774916 46.223796, 14.774659 46.223945, 14.773758 46.224353, 14.773093 46.224845, 14.772427 46.225318, 14.771977 46.225597, 14.771397 46.225921, 14.770689 46.2262, 14.770281 46.226371, 14.770154 46.226478, 14.77011 46.226608, 14.769617 46.226715, 14.769273 46.226757, 14.769102 46.226497, 14.768801 46.2262, 14.768609 46.226135, 14.768265 46.22607, 14.768179 46.22607, 14.767857 46.226047, 14.76775 46.225811, 14.766913 46.224525, 14.766698 46.224246, 14.766291 46.22388, 14.765347 46.223365, 14.76496 46.223217, 14.764681 46.223045, 14.764338 46.222892, 14.76406 46.222702, 14.763201 46.221862, 14.76303 46.221778, 14.762836 46.221649, 14.762407 46.221458, 14.761656 46.22105, 14.761269 46.220771, 14.761076 46.220554, 14.760905 46.220298, 14.760476 46.219547, 14.760325 46.219009, 14.760347 46.218879, 14.759789 46.218624, 14.759167 46.218368, 14.757643 46.218067, 14.755798 46.217807, 14.755261 46.217788, 14.75479 46.217849, 14.753309 46.218238, 14.752965 46.218303, 14.752129 46.218345, 14.751764 46.218322, 14.749424 46.218238, 14.747 46.218368, 14.74685 46.218387, 14.745991 46.218429, 14.744726 46.218536, 14.744211 46.218559, 14.743481 46.218559, 14.742859 46.218536, 14.742386 46.218494, 14.741057 46.21828, 14.739061 46.217872, 14.738975 46.217872, 14.73831 46.217681, 14.737216 46.21725, 14.736464 46.216735, 14.736292 46.216541, 14.735177 46.215232, 14.734919 46.214954, 14.734555 46.214633, 14.73419 46.214462, 14.733675 46.214355, 14.733139 46.214355, 14.732474 46.214397, 14.731765 46.214439, 14.731293 46.214504, 14.73095 46.214591, 14.730843 46.214481, 14.730542 46.214267, 14.73037 46.214054, 14.730349 46.213966, 14.730349 46.213729, 14.730414 46.213474, 14.730521 46.213195, 14.730435 46.212959, 14.730242 46.212638, 14.729769 46.212314, 14.729556 46.21223, 14.728568 46.211971, 14.72816 46.211822, 14.727731 46.211608, 14.72713 46.211265, 14.726723 46.210792, 14.726637 46.210556, 14.726637 46.210296, 14.726787 46.210063, 14.727259 46.209785, 14.72756 46.209655, 14.728096 46.209484, 14.72859 46.209289, 14.729019 46.209098, 14.729384 46.209011, 14.730177 46.208774, 14.730692 46.208538, 14.73125 46.208194, 14.731465 46.207958, 14.731679 46.207786, 14.731873 46.207595, 14.732045 46.207489, 14.732131 46.207466, 14.73228 46.207382, 14.732516 46.207359, 14.73301 46.207164, 14.733332 46.207099, 14.733525 46.206993, 14.733825 46.206928, 14.734254 46.206779, 14.735671 46.206542, 14.735928 46.206459, 14.736186 46.206306, 14.736572 46.205898, 14.736829 46.20549, 14.737023 46.204956, 14.737387 46.204399, 14.737452 46.204116, 14.737322 46.203666, 14.737408 46.203453, 14.737559 46.203369, 14.737859 46.203262, 14.73831 46.203002, 14.73846 46.202919, 14.737967 46.202575, 14.737624 46.202465, 14.737044 46.202186, 14.736636 46.202079, 14.736378 46.202037, 14.736057 46.201931, 14.735799 46.201801, 14.735756 46.201671, 14.735348 46.201221, 14.735156 46.201049, 14.734747 46.200813, 14.734232 46.200576, 14.734105 46.20045, 14.734061 46.20034, 14.733761 46.199871, 14.73359 46.199718, 14.733418 46.199611, 14.733181 46.199546, 14.732646 46.199569, 14.732216 46.199504, 14.732001 46.19942, 14.731915 46.199268, 14.731959 46.199161, 14.732045 46.199119, 14.732474 46.199119, 14.732667 46.199096, 14.732795 46.199012, 14.732838 46.198905, 14.732646 46.198753, 14.731829 46.198345, 14.731444 46.198173, 14.731186 46.198001, 14.730821 46.197875, 14.730649 46.197769, 14.729963 46.197551, 14.729512 46.197551, 14.728632 46.197551, 14.727925 46.197467, 14.727108 46.197273, 14.726765 46.197143, 14.725865 46.196674, 14.725522 46.196437, 14.725264 46.196178, 14.725092 46.196094, 14.724899 46.195899, 14.724384 46.195579, 14.724041 46.195384, 14.723805 46.195278, 14.722624 46.194442, 14.722044 46.193946, 14.721551 46.193584, 14.721358 46.193412, 14.720994 46.193241, 14.720585 46.193111, 14.720006 46.193046, 14.719685 46.192962, 14.719126 46.192897, 14.717818 46.192832, 14.716681 46.192852, 14.716509 46.192787, 14.716101 46.192573, 14.71595 46.192466, 14.715607 46.192443, 14.715394 46.192401, 14.715006 46.192295, 14.714599 46.192211, 14.71417 46.192039, 14.714062 46.191951, 14.71387 46.191589, 14.713569 46.191158, 14.713312 46.191051, 14.712946 46.190921, 14.712689 46.190773, 14.71241 46.190643, 14.712218 46.190494, 14.71196 46.190429, 14.711638 46.190383, 14.711188 46.190448, 14.710866 46.190578, 14.710028 46.190856, 14.709707 46.191028, 14.709406 46.191158, 14.708956 46.191265, 14.708205 46.191352, 14.707947 46.191436, 14.707196 46.19178, 14.706274 46.191932, 14.705952 46.191951, 14.705565 46.191909, 14.704943 46.191909, 14.704686 46.191867, 14.704128 46.191909, 14.703312 46.192146, 14.702969 46.192188, 14.702197 46.192253, 14.701939 46.192317, 14.701767 46.192401, 14.701381 46.192508, 14.701016 46.192619, 14.700137 46.192962, 14.6999 46.193069, 14.699687 46.193195, 14.699214 46.193325, 14.698784 46.193325, 14.698571 46.193305, 14.698098 46.193023, 14.697862 46.192962, 14.697026 46.192916, 14.696724 46.192852, 14.696789 46.193004, 14.696724 46.193111, 14.696446 46.193241, 14.695888 46.193195, 14.695609 46.193111, 14.695223 46.192916, 14.695116 46.192768, 14.695008 46.192443, 14.694901 46.192253, 14.694794 46.192165, 14.694579 46.1921, 14.694022 46.192188, 14.693764 46.192146, 14.693442 46.191802, 14.693184 46.191631, 14.69297 46.191394, 14.692669 46.191288, 14.692412 46.191246, 14.692004 46.191223, 14.691682 46.191181, 14.691082 46.190856, 14.691039 46.19075, 14.690888 46.190559, 14.69063 46.190063, 14.690438 46.189849, 14.690094 46.189529, 14.6896 46.189163, 14.688786 46.188667, 14.687928 46.188217, 14.687562 46.187873, 14.687348 46.187595, 14.687262 46.187423, 14.687133 46.186439, 14.686962 46.186244, 14.686747 46.186096, 14.686469 46.185966, 14.686211 46.185813, 14.685932 46.185577, 14.685566 46.185321, 14.685309 46.185192, 14.684924 46.185066, 14.684473 46.184955, 14.683421 46.184783, 14.682906 46.184741, 14.68252 46.184635, 14.682284 46.184528, 14.681769 46.184333, 14.68134 46.18412, 14.680502 46.183734, 14.679902 46.183368, 14.679623 46.183284, 14.679387 46.183303, 14.679065 46.18341, 14.678636 46.183734, 14.678401 46.18386, 14.678185 46.184013, 14.677907 46.18412, 14.676834 46.184333, 14.676319 46.184463, 14.67589 46.184547, 14.675761 46.18457, 14.674796 46.184806, 14.674494 46.184848, 14.674259 46.184848, 14.674044 46.184806, 14.67383 46.184783, 14.673658 46.184741, 14.673615 46.184741, 14.673572 46.184722, 14.672671 46.184505, 14.672606 46.184268, 14.67252 46.18399, 14.672349 46.183734, 14.672221 46.183456, 14.672478 46.182769, 14.67252 46.182533, 14.672456 46.182296, 14.671919 46.181674, 14.671662 46.181415, 14.670975 46.180366, 14.670632 46.179935, 14.670482 46.179656, 14.670353 46.179336, 14.670311 46.179164, 14.67046 46.178497, 14.670611 46.178005, 14.670676 46.177703, 14.670611 46.177402, 14.67059 46.177253, 14.670418 46.176868, 14.670225 46.176544, 14.670075 46.176139, 14.669989 46.175815, 14.669796 46.175342, 14.669646 46.175064, 14.66956 46.174957, 14.669388 46.174591, 14.66943 46.174591, 14.669732 46.174591, 14.669967 46.174613, 14.670161 46.174678, 14.670825 46.17485, 14.671169 46.174957, 14.67207 46.17517, 14.672843 46.175277, 14.673293 46.1753, 14.673486 46.175258, 14.673658 46.17517, 14.67383 46.175022, 14.673915 46.174915, 14.673958 46.17472, 14.674065 46.174312, 14.674302 46.174057, 14.674559 46.173969, 14.674881 46.17395, 14.675482 46.17382, 14.675696 46.173713, 14.676125 46.173393, 14.676276 46.17324, 14.676684 46.17279, 14.676791 46.172595, 14.676941 46.172382, 14.677134 46.172275, 14.677714 46.171997, 14.678271 46.17176, 14.678442 46.171653, 14.678464 46.171394, 14.678336 46.171245, 14.677928 46.171051, 14.677584 46.170795, 14.677371 46.170581, 14.677285 46.170429, 14.677263 46.17028, 14.677091 46.169357, 14.677091 46.169078, 14.676919 46.16867, 14.676748 46.168476, 14.676468 46.168243, 14.676276 46.167984, 14.676147 46.167747, 14.676125 46.167167, 14.676169 46.16687, 14.676255 46.166526, 14.67619 46.166244, 14.676169 46.165901, 14.675696 46.165409, 14.675503 46.165195, 14.674967 46.16468, 14.674624 46.164249, 14.67458 46.164077, 14.674624 46.163822, 14.674817 46.163093, 14.67486 46.162811, 14.673293 46.162574, 14.672843 46.16249, 14.672049 46.162384, 14.67046 46.162231, 14.669495 46.16217, 14.669323 46.162189, 14.667178 46.162124, 14.665825 46.162017, 14.663873 46.161804, 14.663552 46.161762, 14.663358 46.161739, 14.662478 46.161655, 14.66029 46.16159, 14.658466 46.161609, 14.657757 46.161632, 14.656212 46.16159, 14.655633 46.161567, 14.653208 46.161437, 14.6522 46.161312, 14.650913 46.161075, 14.650827 46.161033, 14.650398 46.160945, 14.649476 46.16069, 14.647136 46.159915, 14.646513 46.15966, 14.644625 46.158798, 14.644304 46.158672, 14.643509 46.158435, 14.642888 46.158264, 14.642544 46.158199, 14.641858 46.158027, 14.639326 46.157382, 14.638617 46.157169, 14.638038 46.15702, 14.637415 46.156826, 14.637051 46.156738, 14.636751 46.156631, 14.63542 46.156227, 14.634562 46.15599, 14.633254 46.155582, 14.633254 46.155323, 14.632952 46.154293, 14.632888 46.154079, 14.632781 46.153823, 14.632695 46.153457, 14.632567 46.153091, 14.632373 46.152942, 14.631944 46.152793, 14.631665 46.152771, 14.631108 46.152683, 14.630721 46.152576, 14.629369 46.152168, 14.628983 46.152019, 14.628403 46.151718, 14.62806 46.151504, 14.626988 46.151031, 14.626151 46.150558, 14.626022 46.150432, 14.625828 46.150108, 14.625615 46.150024, 14.625313 46.149982, 14.624005 46.149658, 14.623898 46.149593, 14.623104 46.14923, 14.622717 46.149078, 14.622481 46.148952, 14.62201 46.148799, 14.621495 46.148609, 14.620615 46.14822, 14.618662 46.147319, 14.617503 46.146656, 14.616838 46.146377, 14.616087 46.146011, 14.616108 46.145988, 14.616215 46.145709, 14.61628 46.145366, 14.616345 46.145175, 14.616366 46.144981, 14.616237 46.144531, 14.616044 46.14421, 14.616023 46.144035, 14.616194 46.143821, 14.61628 46.143672, 14.616151 46.143585, 14.61598 46.143566, 14.61583 46.143524, 14.615529 46.143222, 14.615185 46.142944, 14.614521 46.142536, 14.613942 46.142276, 14.613576 46.142086, 14.613405 46.142021, 14.61304 46.141784, 14.61274 46.141506, 14.61261 46.141353, 14.612418 46.141033, 14.612289 46.140777, 14.612267 46.140304, 14.612267 46.140281, 14.612311 46.14011, 14.612246 46.139724, 14.612246 46.139553, 14.612203 46.139251, 14.612053 46.138481, 14.612181 46.137535, 14.612181 46.137256, 14.61201 46.136806, 14.611774 46.136398, 14.611453 46.136077, 14.610787 46.135604, 14.610251 46.13541, 14.609371 46.13541, 14.606667 46.135345, 14.605937 46.135345, 14.605101 46.135517, 14.604865 46.135581, 14.604435 46.135646, 14.603878 46.135688, 14.603105 46.135776, 14.602612 46.135795, 14.601668 46.13586, 14.601303 46.135925, 14.601067 46.13599, 14.60053 46.136096, 14.600444 46.13586, 14.600466 46.135581, 14.600573 46.135303, 14.600209 46.134208, 14.599972 46.133823, 14.599951 46.133674, 14.599801 46.133651, 14.599179 46.133674, 14.598685 46.133693, 14.599093 46.133159, 14.598985 46.132965, 14.598899 46.132705, 14.598942 46.132514, 14.599221 46.132106, 14.599329 46.131893, 14.599522 46.131206, 14.599522 46.131034, 14.599586 46.130756, 14.59965 46.130538, 14.599715 46.130432, 14.599822 46.130153, 14.599907 46.129833, 14.599907 46.129615, 14.599972 46.129447, 14.600037 46.129253, 14.60023 46.128974, 14.600937 46.128566, 14.601045 46.128479, 14.601045 46.128135, 14.600831 46.127899, 14.600638 46.12775, 14.600487 46.127536, 14.600616 46.12717, 14.600638 46.127021, 14.600552 46.126441, 14.600573 46.12614, 14.600616 46.125949, 14.600573 46.125732, 14.600487 46.125606, 14.600401 46.125495, 14.600273 46.125347, 14.599951 46.125198, 14.599715 46.125152, 14.5992 46.125175, 14.599265 46.125003, 14.599157 46.124511, 14.599286 46.124317, 14.599093 46.124404, 14.598835 46.124382, 14.598599 46.124252, 14.598362 46.12408, 14.598105 46.123973, 14.597933 46.123954, 14.598105 46.123718, 14.59832 46.123458, 14.598556 46.123245, 14.598792 46.122879, 14.598578 46.122772, 14.59817 46.122665, 14.597933 46.122623, 14.597569 46.1226, 14.596668 46.12247, 14.595552 46.12228, 14.594994 46.122238, 14.594801 46.122257, 14.59332 46.122257, 14.593063 46.121871, 14.592955 46.121593, 14.592548 46.120262, 14.592548 46.120197, 14.592354 46.119167, 14.59229 46.118522, 14.592141 46.11771, 14.592097 46.117172, 14.592141 46.116313, 14.59229 46.114791, 14.592505 46.11301, 14.592591 46.112514, 14.592698 46.112472, 14.592913 46.11185, 14.592999 46.111484, 14.593171 46.110969, 14.593277 46.111057, 14.593428 46.111377, 14.5936 46.111442, 14.593985 46.111484, 14.594328 46.111595, 14.594779 46.111701, 14.595059 46.111827, 14.595358 46.111938)";

    assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testLong1H10() throws Exception {
    Route route=test(33828, 32811, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(1.0));

    logger.info("Found route="+route);

    // String
    // result="LINESTRING (16.244529 46.707901, 16.244573 46.707622, 16.244594 46.70713, 16.244529 46.707, 16.244274 46.706722, 16.244079 46.70655, 16.243801 46.706337, 16.24335 46.706054, 16.243135 46.705947, 16.242792 46.705799, 16.242214 46.705692, 16.241912 46.705669, 16.241247 46.705711, 16.240924 46.705711, 16.240604 46.705757, 16.240346 46.705692, 16.240066 46.70565, 16.239337 46.705348, 16.239101 46.705154, 16.238651 46.704898, 16.238286 46.70462, 16.237857 46.704425, 16.237319 46.704231, 16.237085 46.704124, 16.236097 46.703804, 16.235519 46.703567, 16.235004 46.703289, 16.234273 46.702388, 16.234037 46.701892, 16.233909 46.701465, 16.233737 46.701034, 16.233587 46.700477, 16.23335 46.70005, 16.233158 46.699813, 16.232557 46.698974, 16.231204 46.697601, 16.231056 46.697429, 16.230583 46.697109, 16.230133 46.696743, 16.229961 46.696487, 16.229831 46.696144, 16.22966 46.69522, 16.229532 46.694942, 16.229381 46.694683, 16.229103 46.694404, 16.228652 46.694084, 16.228309 46.693759, 16.228029 46.693332, 16.227579 46.692817, 16.227064 46.69226, 16.22672 46.692001, 16.226312 46.691787, 16.225475 46.691421, 16.224317 46.691036, 16.223737 46.690864, 16.223352 46.690715, 16.223009 46.690521, 16.222965 46.689857, 16.222965 46.689277, 16.222988 46.688846, 16.223051 46.688461, 16.223222 46.687774, 16.223287 46.687515, 16.223287 46.68711, 16.223115 46.686336, 16.222988 46.6861, 16.222858 46.685672, 16.22245 46.685672, 16.221786 46.685691, 16.221464 46.685672, 16.220647 46.685565, 16.220197 46.685455, 16.21936 46.685134, 16.218545 46.685028, 16.217258 46.684944, 16.216184 46.684791, 16.215971 46.684791, 16.215047 46.684791, 16.214317 46.684749, 16.213567 46.684661, 16.211593 46.684536, 16.210197 46.684536, 16.209083 46.6846, 16.208095 46.684707, 16.206787 46.684898, 16.20539 46.685112, 16.203975 46.685348, 16.203331 46.685436, 16.202924 46.68552, 16.202152 46.685756, 16.201637 46.6861, 16.200563 46.686893, 16.199619 46.687408, 16.199361 46.687515, 16.199018 46.687625, 16.198633 46.68769, 16.197837 46.687667, 16.197431 46.687603, 16.197067 46.687515, 16.19668 46.687366, 16.196357 46.687194, 16.195972 46.687023, 16.195564 46.686874, 16.194555 46.686573, 16.194126 46.686401, 16.193889 46.686271, 16.193181 46.685798, 16.192947 46.685672, 16.192131 46.685371, 16.191444 46.685157, 16.191186 46.685157, 16.190113 46.685478, 16.189641 46.685565, 16.189083 46.68563, 16.188762 46.685649, 16.187709 46.685798, 16.187067 46.685974, 16.186552 46.686141, 16.18608 46.686229, 16.185672 46.686271, 16.18535 46.686271, 16.18505 46.686252, 16.184856 46.686206, 16.184835 46.686187, 16.184556 46.6861, 16.184341 46.685974, 16.184104 46.685798, 16.183849 46.68552, 16.183612 46.685306, 16.183355 46.685112, 16.182195 46.684536, 16.181573 46.684146, 16.181037 46.683719, 16.180736 46.683441, 16.180458 46.683162, 16.180156 46.682731, 16.180049 46.682216, 16.179985 46.68153, 16.180072 46.680866, 16.180286 46.679729, 16.180244 46.679405, 16.180028 46.678699, 16.179985 46.678398, 16.179922 46.677215, 16.179836 46.676704, 16.179792 46.676403, 16.179386 46.675178, 16.179126 46.674579, 16.178848 46.674125, 16.178548 46.673675, 16.178483 46.673439, 16.178504 46.673206, 16.17859 46.672988, 16.178827 46.672691, 16.179813 46.671852, 16.179985 46.671615, 16.180114 46.671253, 16.180114 46.670928, 16.179943 46.670524, 16.179792 46.670242, 16.179664 46.670051, 16.179149 46.668956, 16.178913 46.668525, 16.178848 46.668441, 16.178504 46.668121, 16.178226 46.667945, 16.177433 46.66756, 16.176853 46.667198, 16.176723 46.66711, 16.175973 46.666404, 16.175373 46.665737, 16.175266 46.665588, 16.17505 46.665435, 16.174556 46.665245, 16.173313 46.665008, 16.172626 46.66492, 16.172197 46.664855, 16.171274 46.664707, 16.169986 46.664535, 16.169643 46.66447, 16.169021 46.664344, 16.16784 46.664108, 16.166103 46.663784, 16.165351 46.663635, 16.164857 46.663463, 16.163764 46.663505, 16.163656 46.662261, 16.163528 46.661209, 16.163484 46.660503, 16.163312 46.659172, 16.16327 46.65887, 16.162454 46.658615, 16.16222 46.658592, 16.161832 46.658527, 16.161382 46.658397, 16.161018 46.658119, 16.160631 46.658142, 16.160287 46.658119, 16.159536 46.657989, 16.158935 46.657733, 16.158615 46.657562, 16.157991 46.657218, 16.15578 46.656059, 16.155502 46.655952, 16.155202 46.655845, 16.153892 46.655544, 16.151769 46.654686, 16.149517 46.653743, 16.144302 46.651596, 16.1434 46.651252, 16.142414 46.650825, 16.140396 46.649986, 16.139881 46.649795, 16.139581 46.649665, 16.139001 46.649517, 16.138036 46.649322, 16.136104 46.648895, 16.135417 46.648784, 16.134817 46.648742, 16.133443 46.6487, 16.132949 46.648658, 16.132499 46.648593, 16.13117 46.648292, 16.129989 46.64812, 16.128574 46.647884, 16.12808 46.647777, 16.128015 46.647777, 16.126426 46.647392, 16.125656 46.647197, 16.125097 46.647026, 16.124839 46.646896, 16.123723 46.646019, 16.123209 46.645568, 16.122673 46.645137, 16.122543 46.645008, 16.120805 46.643699, 16.119991 46.643119, 16.119604 46.642948, 16.119325 46.642883, 16.118982 46.642841, 16.118553 46.642883, 16.118209 46.642971, 16.117523 46.643249, 16.114025 46.644836, 16.111085 46.646255, 16.109647 46.646919, 16.109239 46.647132, 16.106557 46.648399, 16.106235 46.648571, 16.105077 46.649108, 16.104261 46.649517, 16.103918 46.649688, 16.10321 46.650093, 16.102331 46.65063, 16.10205 46.65076, 16.101837 46.65089, 16.101535 46.651016, 16.100786 46.651233, 16.100334 46.651405, 16.09967 46.651767, 16.099025 46.652111, 16.098445 46.652412, 16.097352 46.652797, 16.095829 46.653419, 16.09482 46.653808, 16.094305 46.654045, 16.094005 46.654151, 16.093639 46.654212, 16.093339 46.654212, 16.092781 46.654087, 16.092546 46.654064, 16.091516 46.654151, 16.090679 46.654129, 16.089885 46.654064, 16.088897 46.654064, 16.088275 46.654151, 16.087867 46.6543, 16.087245 46.654495, 16.087032 46.654537, 16.08686 46.654537, 16.086666 46.654472, 16.086517 46.654365, 16.086408 46.654258, 16.08613 46.653892, 16.085914 46.653678, 16.085378 46.653228, 16.085014 46.653015, 16.084735 46.65282, 16.084457 46.652477, 16.083984 46.651767, 16.08362 46.651298, 16.083448 46.651123, 16.083168 46.650997, 16.082975 46.650974, 16.082761 46.650974, 16.08259 46.650997, 16.081539 46.651145, 16.080787 46.65121, 16.080186 46.65121, 16.079671 46.651187, 16.078556 46.650955, 16.078041 46.650779, 16.077503 46.650566, 16.077159 46.650459, 16.07656 46.650394, 16.075679 46.650394, 16.075315 46.650375, 16.074628 46.650287, 16.074048 46.650115, 16.073577 46.649944, 16.073169 46.649772, 16.071903 46.649429, 16.071645 46.649299, 16.071239 46.649021, 16.070959 46.648895, 16.07068 46.648807, 16.070293 46.648765, 16.069866 46.648765, 16.068233 46.649086, 16.067806 46.64915, 16.067247 46.64928, 16.066045 46.649879, 16.06553 46.65018, 16.063491 46.651275, 16.06302 46.651512, 16.060144 46.653057, 16.059865 46.653228, 16.059543 46.653377, 16.059114 46.653636, 16.058321 46.654045, 16.057956 46.654258, 16.05772 46.654277, 16.05759 46.654193, 16.057483 46.654045, 16.057163 46.653419, 16.057075 46.653312, 16.056926 46.653183, 16.056669 46.653057, 16.054759 46.65237, 16.054372 46.652263, 16.054115 46.652217, 16.05347 46.652069, 16.053171 46.651939, 16.052398 46.651531, 16.05199 46.651275, 16.051389 46.650909, 16.050983 46.650547, 16.050724 46.650157, 16.050552 46.649902, 16.049781 46.648784, 16.049545 46.648464, 16.049243 46.648185, 16.048879 46.647926, 16.048492 46.647689, 16.047399 46.64722, 16.046775 46.646984, 16.046669 46.646919, 16.046562 46.646854, 16.045724 46.646492, 16.044523 46.645889, 16.04388 46.645504, 16.043493 46.645286, 16.042871 46.644924, 16.042228 46.644516, 16.04182 46.644321, 16.041477 46.644195, 16.040468 46.643978, 16.040252 46.643913, 16.039373 46.644001, 16.039115 46.644043, 16.038988 46.644043, 16.037849 46.644302, 16.037014 46.644451, 16.036735 46.644451, 16.035768 46.644432, 16.035146 46.644409, 16.035081 46.644172, 16.034868 46.643894, 16.034738 46.644023, 16.034481 46.644256, 16.034374 46.644386, 16.03416 46.644557, 16.033773 46.644924, 16.033365 46.645332, 16.032915 46.645694, 16.032035 46.646511, 16.031455 46.647003, 16.030662 46.647563, 16.030275 46.6478, 16.029653 46.648227, 16.029502 46.648292, 16.028644 46.64883, 16.027721 46.649536, 16.027593 46.649601, 16.027185 46.649837, 16.0268 46.649967, 16.026327 46.650093, 16.025684 46.650245, 16.025297 46.650375, 16.025083 46.650459, 16.024782 46.650653, 16.024353 46.651039, 16.024139 46.651298, 16.023967 46.651683, 16.023689 46.65166, 16.023538 46.651554, 16.023174 46.651466, 16.02268 46.651317, 16.022443 46.651252, 16.021863 46.650997, 16.021457 46.650779, 16.020942 46.650459, 16.020511 46.650093, 16.020126 46.649642, 16.019868 46.649406, 16.019762 46.64928, 16.019569 46.64915, 16.019397 46.649002, 16.019161 46.648872, 16.01843 46.648765, 16.017744 46.648571, 16.017122 46.648185, 16.01695 46.648014, 16.0165 46.647647, 16.016349 46.647563, 16.01622 46.647392, 16.015985 46.647239, 16.015705 46.647003, 16.015254 46.646747, 16.014612 46.646533, 16.014375 46.646492, 16.013517 46.646533, 16.012786 46.646427, 16.011608 46.646148, 16.011178 46.646083, 16.010877 46.646083, 16.010383 46.64619, 16.009696 46.646404, 16.008817 46.646404, 16.008432 46.646362, 16.007616 46.646339, 16.005384 46.646038, 16.004913 46.645912, 16.004291 46.645694, 16.003646 46.645504, 16.002595 46.645072, 16.002231 46.644943, 16.001737 46.644882, 16.001844 46.644474, 16.001823 46.644279, 16.001865 46.644107, 16.001844 46.643764, 16.001758 46.643486, 16.001501 46.643165, 16.001113 46.642799, 16.000492 46.642284, 16.00032 46.642196, 16 46.641918, 16 46.641899, 15.999356 46.641426, 15.999141 46.641231, 15.998776 46.640998, 15.998477 46.640911, 15.998089 46.640846, 15.997361 46.64059, 15.997145 46.640419, 15.997059 46.640247, 15.997081 46.640052, 15.99721 46.639816, 15.997746 46.638916, 15.998025 46.638378, 15.998197 46.638141, 15.998391 46.637908, 15.99854 46.637779, 15.99869 46.637542, 15.998776 46.637264, 15.99882 46.637134, 15.99882 46.636898, 15.998884 46.63636, 15.998862 46.636146, 15.998948 46.635955, 15.99897 46.635784, 15.998884 46.635566, 15.998776 46.635162, 15.998862 46.634731, 15.998884 46.634536, 15.997768 46.633506, 15.997639 46.633335, 15.997425 46.633144, 15.997189 46.63282, 15.997081 46.632629, 15.996995 46.632392, 15.996974 46.631877, 15.997253 46.630954, 15.997339 46.630416, 15.997317 46.630245, 15.997145 46.629837, 15.996824 46.628917, 15.996588 46.628356, 15.996588 46.62825, 15.996115 46.627029, 15.996008 46.626663, 15.996008 46.626579, 15.995858 46.626213, 15.99573 46.625675, 15.995708 46.625354, 15.995665 46.625183, 15.995665 46.624668, 15.995965 46.623573, 15.996115 46.623207, 15.996309 46.622608, 15.996373 46.622306, 15.996417 46.621555, 15.99648 46.621147, 15.996395 46.620182, 15.996309 46.619518, 15.99618 46.618873, 15.99618 46.618228, 15.996073 46.617351, 15.996094 46.617008, 15.996159 46.616836, 15.99618 46.616558, 15.996115 46.616233, 15.995772 46.615417, 15.995644 46.614688, 15.995686 46.614044, 15.99573 46.613895, 15.996245 46.61267, 15.996266 46.61235, 15.996223 46.612113, 15.996051 46.611835, 15.995579 46.611362, 15.995322 46.611084, 15.995064 46.61061, 15.995064 46.610504, 15.99485 46.609348, 15.994528 46.60849, 15.994185 46.60791, 15.993948 46.607585, 15.993734 46.607372, 15.99354 46.607223, 15.992898 46.606643, 15.991181 46.605205, 15.989571 46.603809, 15.989314 46.60353, 15.988327 46.60263, 15.98764 46.602008, 15.986867 46.601364, 15.98558 46.600185, 15.984442 46.599067, 15.983456 46.598018, 15.982684 46.597267, 15.982361 46.59703, 15.981352 46.596172, 15.981225 46.594905, 15.98116 46.59349, 15.981053 46.592994, 15.980923 46.592609, 15.980731 46.592159, 15.980452 46.591728, 15.980109 46.59143, 15.9794 46.590934, 15.978842 46.59053, 15.978542 46.590293, 15.978284 46.590141, 15.977984 46.589885, 15.977362 46.589454, 15.977105 46.589263, 15.976889 46.589069, 15.976482 46.588855, 15.976053 46.588684, 15.974508 46.588233, 15.974186 46.588104, 15.974015 46.587951, 15.973971 46.587696, 15.974015 46.587051, 15.973971 46.586814, 15.973843 46.586708, 15.973607 46.586624, 15.973113 46.586517, 15.97247 46.586322, 15.972254 46.586235, 15.971955 46.586067, 15.971675 46.585872, 15.970881 46.585441, 15.970109 46.585079, 15.969573 46.5848, 15.968886 46.584392, 15.967276 46.58332, 15.966955 46.583061, 15.966246 46.582611, 15.965925 46.582309, 15.96571 46.582115, 15.965539 46.581836, 15.965346 46.581665, 15.964787 46.581321, 15.963908 46.581043, 15.96335 46.580894, 15.962342 46.580551, 15.962212 46.580398, 15.961032 46.580616, 15.960454 46.580699, 15.960002 46.580806, 15.959316 46.581001, 15.958887 46.581131, 15.957749 46.581451, 15.957062 46.581645, 15.956248 46.581752, 15.955647 46.581729, 15.953565 46.581729, 15.952449 46.581836, 15.951634 46.581947, 15.951334 46.582073, 15.950755 46.582458, 15.950539 46.582569, 15.950261 46.582675, 15.950068 46.582801, 15.94966 46.582977, 15.949296 46.583168, 15.948781 46.583297, 15.948158 46.583511, 15.947621 46.583641, 15.947192 46.583663, 15.946248 46.583789, 15.945862 46.583877, 15.945368 46.584049, 15.945004 46.584198, 15.944575 46.584327, 15.944124 46.584434, 15.943415 46.584518, 15.942601 46.584754, 15.942042 46.584842, 15.941678 46.584842, 15.941377 46.5848, 15.941227 46.584671, 15.940948 46.58422, 15.940819 46.583896, 15.940605 46.583511, 15.940433 46.583297, 15.940304 46.583038, 15.940154 46.582912, 15.939982 46.582824, 15.939918 46.582824, 15.939403 46.582782, 15.938866 46.582801, 15.937729 46.582675, 15.937214 46.582569, 15.936635 46.582569, 15.936099 46.582588, 15.935669 46.582569, 15.934962 46.582634, 15.934082 46.582675, 15.932279 46.582977, 15.9317 46.583126, 15.930626 46.583274, 15.930133 46.583297, 15.929575 46.583297, 15.929167 46.583255, 15.928588 46.583145, 15.928116 46.583103, 15.927536 46.583084, 15.927086 46.583061, 15.926507 46.583103, 15.925435 46.58332, 15.924297 46.583576, 15.923632 46.583725, 15.923245 46.583747, 15.922795 46.583705, 15.921894 46.583641, 15.921507 46.583705, 15.921207 46.583725, 15.920756 46.583725, 15.919211 46.583618, 15.91874 46.583534, 15.918438 46.583427, 15.91816 46.583297, 15.917881 46.583126, 15.917666 46.582954, 15.917065 46.582374, 15.916937 46.582202, 15.916852 46.581966, 15.91683 46.581687, 15.916744 46.581474, 15.916594 46.581195, 15.916464 46.581024, 15.916315 46.580917, 15.914919 46.580574, 15.914534 46.580356, 15.913783 46.580013, 15.913418 46.579906, 15.912989 46.579887, 15.912238 46.579929, 15.911294 46.580078, 15.910564 46.580249, 15.910071 46.580337, 15.909448 46.580509, 15.908762 46.580635, 15.908161 46.580787, 15.907753 46.580917, 15.90668 46.581131, 15.906015 46.581237, 15.905564 46.58126, 15.904942 46.58126, 15.904234 46.581195, 15.903697 46.581131, 15.90314 46.581024, 15.901059 46.580463, 15.900586 46.580398, 15.900157 46.580356, 15.899727 46.580356, 15.898333 46.580463, 15.898055 46.580143, 15.895995 46.578254, 15.895844 46.578125, 15.895264 46.577632, 15.894814 46.577289, 15.893956 46.576774, 15.89329 46.576408, 15.892905 46.576278, 15.892603 46.576259, 15.892368 46.576301, 15.891875 46.576515, 15.891467 46.576732, 15.89123 46.576904, 15.890866 46.57703, 15.89048 46.577117, 15.889407 46.577289, 15.888892 46.577438, 15.887948 46.577762, 15.887433 46.577911, 15.886746 46.578083, 15.886145 46.578189, 15.885609 46.578254, 15.883978 46.578338, 15.883635 46.578403, 15.883227 46.578514, 15.88224 46.578964, 15.881939 46.579071, 15.881682 46.579135, 15.88106 46.5792, 15.88093 46.5792, 15.880373 46.579177, 15.879965 46.57909, 15.878914 46.578983, 15.878335 46.578964, 15.877112 46.578941, 15.87679 46.578918, 15.876661 46.579113, 15.876146 46.579605, 15.875481 46.580013, 15.874858 46.580337, 15.8743 46.580787, 15.874043 46.580936, 15.873721 46.580978, 15.873399 46.580978, 15.872347 46.581024, 15.871661 46.580806, 15.871575 46.580764, 15.871468 46.580741, 15.870867 46.580528, 15.870438 46.580398, 15.869279 46.580249, 15.868721 46.58023, 15.867884 46.580272, 15.867413 46.580337, 15.866876 46.580486, 15.865395 46.580806, 15.864237 46.580871, 15.863572 46.580764, 15.8631 46.580616, 15.862648 46.580463, 15.862155 46.580356, 15.861598 46.580337, 15.86134 46.580356, 15.86104 46.580421, 15.860696 46.580486, 15.860074 46.580635, 15.85986 46.580657, 15.859559 46.580741, 15.85928 46.580722, 15.858894 46.580593, 15.858636 46.580528, 15.857842 46.58012, 15.857457 46.579971, 15.856726 46.579822, 15.855976 46.579757, 15.854838 46.579757, 15.854323 46.579776, 15.853852 46.579887, 15.853035 46.580143, 15.85265 46.58023, 15.852371 46.580291, 15.851964 46.580291, 15.851792 46.580272, 15.851255 46.580207, 15.850783 46.580078, 15.850676 46.580013, 15.850419 46.579669, 15.850182 46.579605, 15.849925 46.579586, 15.849646 46.579544, 15.84868 46.579479, 15.847736 46.579498, 15.84735 46.579544, 15.847135 46.579586, 15.846856 46.57965, 15.84647 46.579692, 15.84559 46.579757, 15.844967 46.579822, 15.844496 46.579948, 15.843552 46.580078, 15.843144 46.580078, 15.842929 46.580036, 15.842844 46.580036, 15.842307 46.580207, 15.842049 46.580444, 15.841706 46.580657, 15.84147 46.580764, 15.840891 46.58126, 15.840376 46.581474, 15.839861 46.581581, 15.839668 46.581604, 15.838852 46.581665, 15.838358 46.581665, 15.838037 46.582309, 15.836664 46.580936, 15.836298 46.580593, 15.835677 46.579948, 15.835205 46.579479, 15.834346 46.578598, 15.834175 46.578449, 15.83351 46.577762, 15.833231 46.577484, 15.832844 46.577159, 15.832372 46.576946, 15.831728 46.576709, 15.831536 46.576622, 15.831234 46.576538, 15.830956 46.576431, 15.830312 46.576129, 15.829926 46.575916, 15.829626 46.575744, 15.829282 46.575508, 15.828789 46.575271, 15.828145 46.575099, 15.827437 46.574905, 15.826622 46.574626, 15.82617 46.574501, 15.825741 46.574436, 15.825291 46.574413, 15.824991 46.574436, 15.824326 46.57452, 15.823897 46.574626, 15.822566 46.574886, 15.822438 46.574928, 15.821923 46.575035, 15.821558 46.57508, 15.82115 46.575099, 15.820721 46.575099, 15.820378 46.57508, 15.819476 46.57497, 15.818661 46.574905, 15.817889 46.574821, 15.816945 46.574737, 15.816794 46.574737, 15.81585 46.574607, 15.812867 46.574306, 15.81203 46.574199, 15.810915 46.574028, 15.810056 46.573963, 15.809155 46.574028, 15.808855 46.574092, 15.807246 46.57452, 15.806666 46.574649, 15.806365 46.574737, 15.805701 46.574844, 15.804949 46.574886, 15.804348 46.574886, 15.803576 46.574779, 15.802761 46.574607, 15.802138 46.574455, 15.801667 46.574371, 15.795357 46.572933, 15.794821 46.572784, 15.792118 46.571926, 15.790293 46.571323, 15.789778 46.571174, 15.788964 46.570896, 15.786453 46.570098, 15.785187 46.569671, 15.784886 46.569587, 15.78465 46.569541, 15.784972 46.569156, 15.785208 46.569049, 15.785359 46.568943, 15.785895 46.568321, 15.786561 46.56731, 15.78686 46.566841, 15.786968 46.566558, 15.786925 46.566368, 15.786882 46.566196, 15.786688 46.566001, 15.786603 46.56583, 15.786173 46.565273, 15.785744 46.564907, 15.785187 46.564586, 15.784865 46.564456, 15.784607 46.564308, 15.784221 46.564155, 15.783814 46.563941, 15.783728 46.56377, 15.783277 46.563468, 15.781882 46.56274, 15.781668 46.56261, 15.781538 46.562461, 15.781389 46.562183, 15.781411 46.561904, 15.781239 46.560829, 15.781239 46.560596, 15.781088 46.559845, 15.781045 46.559455, 15.780745 46.558235, 15.780745 46.557956, 15.78068 46.557678, 15.780702 46.557033, 15.780852 46.556648, 15.781088 46.556198, 15.781045 46.555961, 15.780917 46.555809, 15.780509 46.555511, 15.780144 46.555145, 15.779973 46.554931, 15.779586 46.554565, 15.779415 46.554435, 15.779243 46.554245, 15.779135 46.55418, 15.778922 46.554115, 15.77862 46.554073, 15.778449 46.55405, 15.778105 46.55405, 15.777827 46.553962, 15.777441 46.553684, 15.777247 46.553428, 15.777054 46.553257, 15.776883 46.552932, 15.776862 46.552783, 15.776968 46.552589, 15.777119 46.55244, 15.777504 46.552162, 15.77759 46.551967, 15.777526 46.551605, 15.777526 46.55141, 15.777655 46.551155, 15.77787 46.55096, 15.778041 46.550811, 15.778149 46.550594, 15.778191 46.550403, 15.778599 46.549156, 15.778857 46.548877, 15.7789 46.548858, 15.779135 46.548492, 15.779564 46.547958, 15.779908 46.547676, 15.779973 46.547485, 15.779973 46.547378, 15.7795 46.546669, 15.779351 46.546176, 15.779329 46.545852, 15.779071 46.54536, 15.778922 46.545188, 15.77862 46.544929, 15.778363 46.544803, 15.777762 46.544609, 15.777011 46.544479, 15.776411 46.544437, 15.775896 46.544307, 15.77566 46.544136, 15.775403 46.54388, 15.775295 46.543663, 15.775209 46.543407, 15.774973 46.542957, 15.774823 46.542743, 15.774608 46.542354, 15.774436 46.542163, 15.774329 46.541904, 15.774201 46.541755, 15.774093 46.541561, 15.773686 46.541069, 15.773235 46.540809, 15.772977 46.540618, 15.772548 46.540016, 15.772591 46.539844, 15.772548 46.539672, 15.77257 46.539543, 15.772634 46.539417, 15.773213 46.539032, 15.773964 46.538623, 15.774222 46.538406, 15.774373 46.53828, 15.774522 46.538085, 15.774586 46.537872, 15.774586 46.537719, 15.774544 46.537529, 15.774415 46.537292, 15.774243 46.536861, 15.774093 46.536197, 15.774007 46.535961, 15.773879 46.535789, 15.773642 46.535598, 15.773385 46.535488, 15.77272 46.535511, 15.772011 46.535404, 15.771347 46.535232, 15.770188 46.534458, 15.769587 46.534267, 15.768193 46.534008, 15.767742 46.533966, 15.767334 46.533966, 15.766712 46.533924, 15.766325 46.533943, 15.765961 46.53405, 15.765189 46.534523, 15.764952 46.53461, 15.764566 46.534694, 15.764073 46.534736, 15.763815 46.534736, 15.76315 46.534824, 15.762249 46.535018, 15.761734 46.535018, 15.761111 46.534889, 15.760918 46.534782, 15.760575 46.534503, 15.760468 46.534351, 15.760211 46.53416, 15.76006 46.533988, 15.759416 46.533794, 15.759202 46.533752, 15.758923 46.533729, 15.758322 46.533771, 15.757185 46.533924, 15.756777 46.533924, 15.756498 46.533943, 15.756304 46.534008, 15.756047 46.53416, 15.755811 46.534416, 15.755682 46.53461, 15.755618 46.534801, 15.755576 46.535038, 15.755425 46.535167, 15.755232 46.535297, 15.754374 46.534996, 15.754095 46.534847, 15.753945 46.534736, 15.75358 46.534568, 15.753344 46.534481, 15.753001 46.534393, 15.75255 46.534374, 15.752142 46.534351, 15.751799 46.534309, 15.751048 46.534072, 15.750811 46.533943, 15.750554 46.533836, 15.749889 46.533428, 15.749202 46.533084, 15.748215 46.532657, 15.747228 46.532119, 15.746821 46.531711, 15.74667 46.53152, 15.746499 46.531391, 15.746198 46.531219, 15.745791 46.531089, 15.745362 46.530792, 15.744717 46.530555, 15.744052 46.530166, 15.74358 46.529823, 15.743216 46.52946, 15.74255 46.528923, 15.742207 46.528816, 15.741778 46.528709, 15.741541 46.528621, 15.741349 46.52843, 15.741177 46.528194, 15.741005 46.527999, 15.740813 46.527721, 15.740555 46.527465, 15.739975 46.526992, 15.739418 46.526607, 15.738903 46.526371, 15.738388 46.526176, 15.738001 46.525985, 15.737787 46.52581, 15.737315 46.525318, 15.736628 46.524761, 15.736199 46.524459, 15.735835 46.524246, 15.735447 46.524093, 15.735104 46.523986, 15.734783 46.523944, 15.733731 46.523925, 15.732573 46.523925, 15.732014 46.523902, 15.731543 46.523967, 15.731392 46.523967, 15.731006 46.524051, 15.730212 46.524288, 15.729611 46.523323, 15.729525 46.523063, 15.729439 46.52272, 15.729418 46.521842, 15.729332 46.521499, 15.729204 46.521263, 15.728753 46.520854, 15.728603 46.52066, 15.728323 46.520019, 15.728217 46.519847, 15.72811 46.519611, 15.728002 46.51931, 15.727959 46.518989, 15.727808 46.518196, 15.727637 46.517292, 15.727595 46.517101, 15.727423 46.516883, 15.727187 46.516799, 15.726843 46.516757, 15.726414 46.516757, 15.725878 46.516716, 15.725319 46.516651, 15.724912 46.516563, 15.724547 46.516456, 15.724183 46.516178, 15.723968 46.515834, 15.723796 46.515705, 15.723603 46.515598, 15.723453 46.515384, 15.723002 46.514568, 15.722874 46.514266, 15.722766 46.514118, 15.722552 46.513904, 15.722294 46.513774, 15.721865 46.513668, 15.721307 46.51358, 15.72105 46.51358, 15.720042 46.513732, 15.719462 46.513774, 15.718861 46.513904, 15.718389 46.514095, 15.718281 46.514461, 15.718089 46.514698, 15.717831 46.514911, 15.717531 46.515083, 15.71723 46.51519, 15.716973 46.515232, 15.716736 46.515254, 15.716393 46.515254, 15.713711 46.514781, 15.713131 46.514762, 15.712767 46.514781, 15.712424 46.514869, 15.712403 46.514888, 15.71208 46.515041, 15.711694 46.515254, 15.709033 46.517143, 15.708991 46.517185, 15.706824 46.518665, 15.704892 46.520019, 15.704484 46.520381, 15.704249 46.520641, 15.704033 46.520942, 15.703884 46.521221, 15.70369 46.52182, 15.703626 46.522293, 15.70369 46.522701, 15.703798 46.523128, 15.703906 46.523494, 15.704119 46.524051, 15.704313 46.524589, 15.704377 46.524868, 15.704335 46.525104, 15.704227 46.525318, 15.704012 46.525512, 15.703583 46.525726, 15.703133 46.52581, 15.702081 46.525939, 15.701458 46.526046, 15.70163 46.525531, 15.701695 46.525081, 15.701352 46.523925, 15.701201 46.523239, 15.700816 46.522033, 15.700644 46.521327, 15.700343 46.520404, 15.699721 46.518001, 15.699656 46.517551, 15.699635 46.51712, 15.699592 46.516929, 15.699313 46.516735, 15.699034 46.516628, 15.698927 46.516605, 15.698584 46.516479, 15.697961 46.516071, 15.697833 46.515941, 15.696888 46.515403, 15.695773 46.514823, 15.695279 46.514698, 15.6947 46.514568, 15.69352 46.514225, 15.693369 46.514183, 15.693133 46.514118, 15.692017 46.513904, 15.691867 46.513858, 15.691782 46.513858, 15.690987 46.513538, 15.690516 46.513301, 15.689765 46.512916, 15.68955 46.512657, 15.689292 46.51242, 15.688756 46.511909, 15.687941 46.511158, 15.687919 46.511005, 15.687983 46.510898, 15.688005 46.510768, 15.687983 46.51062, 15.687576 46.509696, 15.687512 46.508987, 15.687275 46.508731, 15.687233 46.508625, 15.687233 46.50856, 15.687296 46.508346, 15.687554 46.507873, 15.68764 46.507659, 15.687684 46.507446, 15.687726 46.50727, 15.687747 46.506908, 15.687662 46.50637, 15.687447 46.505729, 15.687382 46.505577, 15.687275 46.505428, 15.687211 46.505363, 15.686889 46.505168, 15.685452 46.504524, 15.684721 46.504268, 15.684529 46.504161, 15.684057 46.503948, 15.683713 46.503837, 15.68307 46.503559, 15.682383 46.503345, 15.682168 46.503303, 15.681761 46.503173, 15.680409 46.502681, 15.679314 46.502273, 15.677062 46.501415, 15.676011 46.501007, 15.675152 46.500728, 15.674895 46.50064, 15.674358 46.50045, 15.673972 46.500469, 15.672963 46.500579, 15.672169 46.50064, 15.671805 46.500598, 15.67116 46.500534, 15.670796 46.50045, 15.670474 46.500404, 15.670903 46.497444, 15.670882 46.496715, 15.670731 46.495662, 15.67071 46.495365, 15.67013 46.495449, 15.669981 46.495449, 15.669938 46.495319, 15.669745 46.494762, 15.669402 46.494075, 15.668693 46.493026, 15.668028 46.492275, 15.667555 46.491802, 15.666697 46.491027, 15.665989 46.490512, 15.663394 46.488605, 15.661226 46.486824, 15.660668 46.486328, 15.659401 46.484848, 15.658865 46.484054, 15.658587 46.483581, 15.658114 46.482662, 15.657921 46.482208, 15.657535 46.480964, 15.657256 46.479312, 15.657235 46.478691, 15.657256 46.478305, 15.657235 46.477939, 15.657341 46.477016, 15.657599 46.475322, 15.657643 46.474807, 15.657793 46.473754, 15.6579 46.473197, 15.658007 46.472126, 15.65805 46.471889, 15.658093 46.470581, 15.658072 46.469272, 15.658072 46.468906, 15.6579 46.466632, 15.657449 46.464443, 15.657128 46.463218, 15.656848 46.462295, 15.656763 46.462081, 15.65672 46.461868, 15.656312 46.460815, 15.65524 46.458328, 15.654059 46.456226, 15.653201 46.454853, 15.651613 46.452384, 15.651549 46.452255, 15.650454 46.450622, 15.650368 46.450515, 15.650111 46.450065, 15.649102 46.448562, 15.648631 46.447875, 15.647836 46.446739, 15.647278 46.446052, 15.646463 46.444786, 15.645754 46.443779, 15.644597 46.442062, 15.643738 46.440902, 15.641634 46.437984, 15.641571 46.437942, 15.641119 46.437297, 15.639124 46.434661, 15.637902 46.433052, 15.635798 46.430347, 15.635712 46.430259, 15.635648 46.430198, 15.634682 46.429016, 15.633566 46.4276, 15.632387 46.42612, 15.631528 46.425109, 15.631228 46.424705, 15.630627 46.424018, 15.629812 46.423007, 15.629017 46.422107, 15.628932 46.421958, 15.627945 46.420864, 15.627774 46.420627, 15.626744 46.419425, 15.625155 46.417621, 15.62419 46.416591, 15.623246 46.415519, 15.620971 46.413158, 15.619598 46.411766, 15.618504 46.410713, 15.615736 46.408203, 15.611959 46.405006, 15.610522 46.403888, 15.607625 46.401702, 15.605436 46.400135, 15.603934 46.399169, 15.602947 46.398483, 15.600693 46.397045, 15.5999 46.39653, 15.597861 46.395286, 15.596209 46.394275, 15.593312 46.392601, 15.59299 46.392387, 15.591145 46.391357, 15.585888 46.388656, 15.581982 46.386764, 15.580502 46.3861, 15.57782 46.384964, 15.575395 46.383998, 15.573529 46.383312, 15.572992 46.383098, 15.571597 46.382583, 15.566683 46.380886, 15.565375 46.380416, 15.565181 46.380371, 15.563099 46.379642, 15.562606 46.379447, 15.557135 46.377559, 15.556448 46.377368, 15.551856 46.375953, 15.549689 46.375328, 15.546664 46.374534, 15.545977 46.374385, 15.545462 46.374237, 15.544324 46.373954, 15.541492 46.373291, 15.53823 46.372497, 15.536021 46.371917, 15.531879 46.370758, 15.530291 46.370307, 15.524949 46.368591, 15.524326 46.368377, 15.523146 46.367969, 15.521107 46.367218, 15.516816 46.365501, 15.513812 46.364192, 15.511923 46.363311, 15.511645 46.363185, 15.509713 46.362262, 15.507567 46.36119, 15.505786 46.360244, 15.503577 46.358936, 15.502868 46.358486, 15.501366 46.357604, 15.499972 46.356746, 15.496603 46.354621, 15.495315 46.353786, 15.493513 46.352542, 15.493105 46.352283, 15.491066 46.350891, 15.489221 46.349559, 15.485466 46.346771, 15.483277 46.345054, 15.482934 46.344772, 15.482827 46.344711, 15.480702 46.342948, 15.478858 46.341362, 15.476712 46.339496, 15.475466 46.338272, 15.475402 46.338188, 15.475359 46.338165, 15.474394 46.337219, 15.472827 46.335609, 15.472827 46.33559, 15.470231 46.332801, 15.468708 46.331039, 15.467528 46.329624, 15.46536 46.326835, 15.464159 46.32516, 15.463558 46.324279, 15.461733 46.321491, 15.460725 46.319797, 15.459845 46.318229, 15.459438 46.317478, 15.459288 46.317134, 15.458687 46.315547, 15.45815 46.314044, 15.457957 46.313529, 15.457206 46.312393, 15.45579 46.310398, 15.454717 46.309646, 15.450125 46.307331, 15.448838 46.306602, 15.448022 46.306083, 15.447464 46.305656, 15.446541 46.304798, 15.445984 46.304134, 15.445769 46.303833, 15.445061 46.302696, 15.444825 46.302158, 15.44446 46.301429, 15.443473 46.299819, 15.442421 46.298488, 15.442208 46.298252, 15.441585 46.297031, 15.441113 46.29615, 15.44077 46.29557, 15.440169 46.294433, 15.439589 46.293254, 15.43886 46.29216, 15.438345 46.291645, 15.437937 46.291278, 15.437014 46.290615, 15.434461 46.289093, 15.434353 46.289005, 15.433388 46.288425, 15.431435 46.287139, 15.430577 46.28656, 15.429633 46.285873, 15.427702 46.284542, 15.427144 46.284091, 15.424912 46.282161, 15.422295 46.279972, 15.420428 46.278385, 15.418282 46.276622, 15.417338 46.275894, 15.417016 46.275615, 15.415213 46.274307, 15.414463 46.273792, 15.411652 46.271968, 15.409163 46.270553, 15.406502 46.269157, 15.403734 46.267868, 15.40339 46.267696, 15.401867 46.267032, 15.39867 46.265747, 15.396245 46.264865, 15.393305 46.263877, 15.393113 46.263813, 15.392962 46.263793, 15.392361 46.263576, 15.391781 46.263404, 15.389829 46.262805, 15.387555 46.262161, 15.384293 46.261325, 15.383799 46.261219, 15.381911 46.260746, 15.379744 46.260253, 15.376482 46.259586, 15.374186 46.259159, 15.372857 46.258941, 15.370217 46.258472, 15.369401 46.258342, 15.365775 46.257785, 15.364981 46.257633, 15.364595 46.257591, 15.364123 46.257526, 15.361205 46.257034, 15.359982 46.256862, 15.356549 46.256301, 15.351677 46.255554, 15.351485 46.255508, 15.348008 46.255039, 15.343932 46.254631, 15.342193 46.254501, 15.337279 46.254264, 15.335155 46.254241, 15.332108 46.254264, 15.329876 46.254306, 15.32434 46.254459, 15.320821 46.254524, 15.319233 46.254524, 15.316873 46.254501, 15.316616 46.254478, 15.314663 46.254459, 15.311059 46.254352, 15.304986 46.254222, 15.304492 46.254241, 15.304148 46.254241, 15.303935 46.254222, 15.297497 46.254241, 15.296746 46.254287, 15.295544 46.254306, 15.294042 46.254329, 15.292777 46.254394, 15.287669 46.254566, 15.286532 46.254585, 15.286017 46.254566, 15.285459 46.254585, 15.285309 46.254585, 15.285202 46.254608, 15.28415 46.254608, 15.282498 46.254608, 15.281403 46.254585, 15.275546 46.254371, 15.272305 46.254158, 15.271984 46.254158, 15.269237 46.253944, 15.267327 46.253814, 15.264967 46.253749, 15.262221 46.25373, 15.260525 46.253772, 15.260268 46.253791, 15.259818 46.253791, 15.2578 46.253921, 15.256405 46.254028, 15.254603 46.254222, 15.250933 46.254737, 15.247372 46.255447, 15.245204 46.255939, 15.240162 46.257118, 15.236685 46.257911, 15.235291 46.258235, 15.23514 46.258255, 15.235033 46.2583, 15.234519 46.258407, 15.228682 46.259822, 15.228339 46.259929, 15.227952 46.260017, 15.22542 46.260765, 15.223597 46.261344, 15.218661 46.263061, 15.2151 46.264221, 15.212417 46.264995, 15.211365 46.265274, 15.210743 46.265445, 15.209435 46.265766, 15.208189 46.266044, 15.204778 46.266731, 15.203211 46.26699, 15.201731 46.267204, 15.19877 46.267589, 15.197632 46.267719, 15.194006 46.268039, 15.191067 46.268257, 15.187462 46.268554, 15.182141 46.268878, 15.179694 46.269069, 15.176218 46.269306, 15.175917 46.269348, 15.174351 46.269458, 15.170875 46.269649, 15.168729 46.269737, 15.166176 46.269802, 15.164352 46.269821, 15.162056 46.269821, 15.161627 46.269802, 15.15873 46.269802, 15.155855 46.269844, 15.15225 46.269992, 15.150683 46.27008, 15.146735 46.270401, 15.145018 46.270572, 15.14298 46.270809, 15.13959 46.271259, 15.138581 46.271408, 15.138259 46.27143, 15.137595 46.271518, 15.137444 46.271518, 15.137423 46.271537, 15.136543 46.271625, 15.134655 46.271751, 15.132466 46.271816, 15.129376 46.271774, 15.127209 46.271602, 15.126608 46.271537, 15.125321 46.271366, 15.12399 46.271152, 15.123003 46.270938, 15.121651 46.270614, 15.120257 46.27021, 15.118862 46.269756, 15.117596 46.269264, 15.116566 46.268814, 15.115343 46.268234, 15.113562 46.267227, 15.112768 46.266689, 15.111695 46.265914, 15.110708 46.265167, 15.108862 46.263729, 15.10809 46.263191, 15.106287 46.262096, 15.105815 46.261859, 15.104678 46.261344, 15.103927 46.261047, 15.102576 46.260551, 15.102425 46.260509, 15.10236 46.260509, 15.101803 46.260337, 15.101502 46.260253, 15.100108 46.259929, 15.099528 46.259822, 15.099314 46.259799, 15.098949 46.259716, 15.097468 46.259521, 15.096374 46.259437, 15.094657 46.259372, 15.093564 46.259391, 15.091095 46.259521, 15.088627 46.259693, 15.087298 46.259716, 15.085537 46.259716, 15.083178 46.259651, 15.082899 46.259628, 15.081633 46.259609, 15.078693 46.259502, 15.077019 46.259456, 15.075453 46.259414, 15.074509 46.259372, 15.071354 46.259307, 15.071118 46.259284, 15.067835 46.259201, 15.066699 46.259159, 15.065432 46.259136, 15.061827 46.259006, 15.059918 46.258964, 15.058243 46.258899, 15.05775 46.258899, 15.055604 46.258792, 15.054961 46.25875, 15.053566 46.258621, 15.052944 46.258579, 15.051656 46.258449, 15.050776 46.258342, 15.049059 46.258106, 15.046614 46.25772, 15.045133 46.257507, 15.044146 46.257377, 15.043995 46.257377, 15.042515 46.257205, 15.040584 46.257053, 15.037087 46.256881, 15.036765 46.256881, 15.034147 46.256881, 15.032881 46.256927, 15.029919 46.257076, 15.027838 46.25727, 15.023633 46.257675, 15.021208 46.257934, 15.019556 46.258083, 15.016959 46.258277, 15.013934 46.258384, 15.01241 46.258384, 15.010607 46.258342, 15.009062 46.258277, 15.008698 46.258277, 15.006188 46.258083, 15.004321 46.257892, 15.001595 46.257568, 14.999986 46.257396, 14.998227 46.257141, 14.995866 46.256732, 14.993721 46.256412, 14.991425 46.256134, 14.989022 46.255916, 14.984257 46.255661, 14.983721 46.255638, 14.98312 46.255554, 14.982648 46.255508, 14.981189 46.255271, 14.979601 46.254886, 14.978099 46.254436, 14.976941 46.254009, 14.975461 46.253322, 14.974924 46.253063, 14.974022 46.252525, 14.973572 46.252227, 14.972821 46.251861, 14.972477 46.251754, 14.972199 46.251731, 14.971812 46.251754, 14.971362 46.251754, 14.971147 46.251712, 14.971018 46.251647, 14.970912 46.251625, 14.970761 46.25156, 14.970524 46.251369, 14.970438 46.251262, 14.970397 46.251091, 14.970397 46.250961, 14.970438 46.250747, 14.970524 46.250595, 14.969988 46.250381, 14.969688 46.250232, 14.96958 46.250144, 14.968937 46.249759, 14.968358 46.249458, 14.967307 46.24905, 14.965848 46.248558, 14.965203 46.248321, 14.964409 46.247955, 14.9634 46.247398, 14.962414 46.246864, 14.961728 46.246521, 14.961105 46.246261, 14.960698 46.246067, 14.960225 46.245834, 14.959689 46.245597, 14.957779 46.244716, 14.956942 46.244331, 14.956341 46.24403, 14.954839 46.243343, 14.953294 46.242614, 14.950913 46.24139, 14.949904 46.240684, 14.949604 46.240425, 14.949238 46.240039, 14.948745 46.239437, 14.948424 46.238967, 14.947865 46.237743, 14.947651 46.237075, 14.94735 46.236434, 14.9472 46.236045, 14.946879 46.235359, 14.946213 46.23418, 14.945698 46.233257, 14.94542 46.232593, 14.945226 46.23225, 14.944861 46.231239, 14.944025 46.229053, 14.94351 46.227851, 14.943274 46.227893, 14.942931 46.227893, 14.942544 46.227787, 14.942136 46.227527, 14.941879 46.227401, 14.941535 46.227314, 14.940634 46.227249, 14.939862 46.227272, 14.938853 46.227272, 14.938509 46.227249, 14.93808 46.227184, 14.937523 46.227058, 14.937072 46.226905, 14.936106 46.226627, 14.93542 46.226478, 14.932523 46.22594, 14.931815 46.225791, 14.930506 46.225425, 14.929519 46.225105, 14.928275 46.224716, 14.927395 46.224502, 14.926836 46.224437, 14.925549 46.224395, 14.924927 46.224266, 14.923683 46.223987, 14.923125 46.223922, 14.922287 46.22388, 14.920872 46.223922, 14.918833 46.224029, 14.918297 46.224029, 14.917481 46.22388, 14.916902 46.223686, 14.916559 46.223495, 14.91613 46.223194, 14.915529 46.222679, 14.915271 46.222507, 14.914692 46.222229, 14.91407 46.221969, 14.913083 46.221519, 14.91231 46.221134, 14.912138 46.220985, 14.911924 46.220771, 14.911581 46.220169, 14.911173 46.219654, 14.910508 46.219097, 14.909778 46.218559, 14.90922 46.218109, 14.908898 46.217849, 14.908513 46.217617, 14.908083 46.217399, 14.90761 46.21725, 14.907032 46.217144, 14.904091 46.216693, 14.903534 46.216587, 14.901753 46.216178, 14.901345 46.216114, 14.900057 46.216072, 14.899329 46.215877, 14.899071 46.215877, 14.898792 46.215965, 14.898277 46.216156, 14.898127 46.216178, 14.897097 46.216178, 14.895874 46.216285, 14.895466 46.21635, 14.895037 46.216369, 14.892291 46.216369, 14.891303 46.216415, 14.890895 46.216457, 14.890358 46.216541, 14.889586 46.216606, 14.888621 46.216629, 14.887742 46.21667, 14.887033 46.216758, 14.886904 46.216758, 14.886839 46.216777, 14.88551 46.21693, 14.884909 46.21693, 14.883921 46.216819, 14.883063 46.216606, 14.88184 46.216178, 14.881218 46.216007, 14.880402 46.2159, 14.877442 46.215835, 14.876282 46.215835, 14.875767 46.215877, 14.875403 46.215942, 14.874845 46.216026, 14.874545 46.216049, 14.874201 46.216007, 14.873707 46.2159, 14.873621 46.215854, 14.873408 46.215747, 14.872592 46.215255, 14.871863 46.215103, 14.871498 46.215084, 14.870811 46.214977, 14.868472 46.21429, 14.868001 46.214183, 14.867206 46.21416, 14.865511 46.214031, 14.86476 46.214031, 14.863881 46.214096, 14.863022 46.214118, 14.862034 46.214073, 14.86094 46.214138, 14.860597 46.214054, 14.860082 46.213882, 14.859717 46.213729, 14.859417 46.213668, 14.859074 46.213668, 14.858065 46.213775, 14.856242 46.213752, 14.855662 46.213794, 14.854975 46.213924, 14.854697 46.214054, 14.854374 46.214183, 14.854138 46.214309, 14.853645 46.214504, 14.853216 46.214591, 14.851134 46.214698, 14.849911 46.214698, 14.848881 46.214611, 14.848044 46.214504, 14.847529 46.214504, 14.847144 46.214611, 14.8468 46.21474, 14.846457 46.214954, 14.845877 46.215362, 14.845641 46.215492, 14.844332 46.216156, 14.84386 46.216308, 14.843646 46.21635, 14.843237 46.216392, 14.842916 46.216369, 14.842616 46.216327, 14.842144 46.216156, 14.841435 46.21577, 14.841199 46.215663, 14.840856 46.215576, 14.840491 46.215557, 14.840106 46.215599, 14.839676 46.215705, 14.839203 46.215789, 14.838646 46.215854, 14.837573 46.216007, 14.8368 46.216072, 14.836415 46.216049, 14.835277 46.215877, 14.834719 46.215812, 14.834355 46.215705, 14.833882 46.215557, 14.833475 46.215362, 14.833045 46.21519, 14.832638 46.215019, 14.832209 46.214889, 14.83193 46.214847, 14.831629 46.214847, 14.831265 46.21487, 14.829978 46.215042, 14.82914 46.215126, 14.828668 46.215103, 14.827896 46.214912, 14.827595 46.214805, 14.826844 46.214611, 14.826329 46.214546, 14.826115 46.214527, 14.825728 46.214546, 14.825299 46.214591, 14.824784 46.214675, 14.823926 46.214912, 14.82296 46.215148, 14.822552 46.215232, 14.822188 46.215255, 14.821522 46.215168, 14.8209 46.215148, 14.819977 46.215168, 14.819656 46.215126, 14.818841 46.214977, 14.818154 46.214912, 14.816931 46.21474, 14.816823 46.21474, 14.816094 46.214611, 14.815171 46.214416, 14.814913 46.214374, 14.813605 46.21429, 14.813262 46.214225, 14.811888 46.214031, 14.811459 46.213989, 14.810515 46.214012, 14.809292 46.214073, 14.808905 46.214012, 14.80852 46.213859, 14.808283 46.213729, 14.807875 46.213687, 14.807511 46.213752, 14.807274 46.213882, 14.807189 46.214054, 14.807339 46.214504, 14.807232 46.214675, 14.806931 46.214824, 14.805602 46.215255, 14.805386 46.215297, 14.805001 46.21532, 14.804678 46.215297, 14.802897 46.215255, 14.80221 46.215148, 14.80176 46.215103, 14.801374 46.215126, 14.800967 46.215213, 14.800538 46.215339, 14.80015 46.215492, 14.7997 46.215621, 14.79925 46.215705, 14.798176 46.215812, 14.797984 46.215812, 14.797575 46.215854, 14.797146 46.215877, 14.796868 46.215877, 14.79601 46.215942, 14.795366 46.216007, 14.79498 46.216072, 14.79395 46.216327, 14.793456 46.216541, 14.793242 46.21667, 14.793026 46.216842, 14.792511 46.217636, 14.791354 46.218475, 14.791182 46.218688, 14.790967 46.218902, 14.790645 46.219181, 14.789958 46.219482, 14.789637 46.219589, 14.78925 46.21976, 14.788907 46.21989, 14.788199 46.220169, 14.787447 46.220428, 14.785731 46.22092, 14.785152 46.221199, 14.784701 46.221519, 14.784316 46.221755, 14.783671 46.222099, 14.783371 46.222206, 14.782663 46.222358, 14.782234 46.2224, 14.78129 46.2224, 14.78056 46.222377, 14.78011 46.222335, 14.779894 46.22227, 14.779745 46.222251, 14.77908 46.221862, 14.778479 46.221565, 14.778286 46.221435, 14.777599 46.221069, 14.776762 46.220813, 14.775002 46.220428, 14.774294 46.220298, 14.773565 46.220127, 14.77335 46.220104, 14.772728 46.219974, 14.771784 46.219783, 14.77026 46.219631, 14.769874 46.219589, 14.7676 46.219223, 14.765904 46.219055, 14.765068 46.218944, 14.764575 46.218925, 14.763866 46.218944, 14.763287 46.218925, 14.762235 46.218772, 14.761806 46.218772, 14.760668 46.218902, 14.760347 46.218879, 14.759789 46.218624, 14.759167 46.218368, 14.757643 46.218067, 14.755798 46.217807, 14.755261 46.217788, 14.75479 46.217849, 14.753309 46.218238, 14.752965 46.218303, 14.752129 46.218345, 14.751764 46.218322, 14.749424 46.218238, 14.747 46.218368, 14.74685 46.218387, 14.745991 46.218429, 14.744726 46.218536, 14.744211 46.218559, 14.743481 46.218559, 14.742859 46.218536, 14.742386 46.218494, 14.741057 46.21828, 14.739061 46.217872, 14.738975 46.217872, 14.73831 46.217681, 14.737216 46.21725, 14.736228 46.21693, 14.73565 46.216865, 14.735091 46.216758, 14.73462 46.216629, 14.734318 46.216587, 14.733997 46.216434, 14.733632 46.216178, 14.733396 46.2159, 14.733224 46.215641, 14.733031 46.215492, 14.732731 46.21532, 14.732172 46.215061, 14.731851 46.214889, 14.731336 46.214698, 14.73095 46.214591, 14.730306 46.214847, 14.72962 46.215084, 14.728976 46.215278, 14.727688 46.215599, 14.726552 46.215835, 14.725113 46.216072, 14.724749 46.216072, 14.723719 46.216114, 14.723182 46.216197, 14.723053 46.21622, 14.722432 46.216197, 14.722023 46.216243, 14.721723 46.216262, 14.720757 46.216072, 14.72035 46.216049, 14.720028 46.216091, 14.719256 46.216369, 14.718848 46.216541, 14.718462 46.216629, 14.717753 46.21667, 14.717238 46.216777, 14.716767 46.216842, 14.716465 46.216842, 14.71535 46.216651, 14.714663 46.216564, 14.713827 46.216693, 14.713461 46.216777, 14.712968 46.21693, 14.712689 46.217037, 14.712453 46.217208, 14.712002 46.2177, 14.71181 46.217872, 14.711573 46.217979, 14.71093 46.218151, 14.709836 46.218475, 14.709492 46.218517, 14.70872 46.218429, 14.708591 46.21841, 14.707797 46.218303, 14.705781 46.218086, 14.705329 46.218086, 14.704878 46.218044, 14.703742 46.21783, 14.703527 46.217357, 14.703291 46.217144, 14.703398 46.216972, 14.703398 46.216777, 14.703355 46.216629, 14.703227 46.216564, 14.702755 46.216392, 14.702647 46.216327, 14.702347 46.215984, 14.702176 46.216007, 14.70196 46.216007, 14.701661 46.216114, 14.701081 46.216327, 14.700888 46.216285, 14.700758 46.216178, 14.701146 46.215362, 14.701445 46.215042, 14.701617 46.214805, 14.701725 46.214462, 14.701725 46.214183, 14.701617 46.213108, 14.701531 46.212699, 14.701424 46.212486, 14.701167 46.212165, 14.700866 46.211994, 14.700415 46.211822, 14.699343 46.211479, 14.698291 46.210769, 14.69739 46.210189, 14.697154 46.209999, 14.696703 46.209377, 14.696339 46.208755, 14.696082 46.208152, 14.695845 46.207725, 14.695523 46.207359, 14.694815 46.206478, 14.694579 46.206092, 14.694558 46.205898, 14.694579 46.205726, 14.694644 46.205642, 14.69503 46.205383, 14.695244 46.205276, 14.695459 46.205104, 14.695609 46.20502, 14.695652 46.204891, 14.695523 46.204784, 14.695201 46.204719, 14.694729 46.204505, 14.694558 46.204525, 14.6943 46.204742, 14.694043 46.204826, 14.69372 46.204742, 14.693313 46.204399, 14.692391 46.203948, 14.692197 46.203666, 14.692026 46.203517, 14.691768 46.203388, 14.691511 46.203067, 14.691854 46.202873, 14.692197 46.202743, 14.692412 46.202529, 14.692477 46.202293, 14.69269 46.202056, 14.693549 46.201351, 14.693828 46.201221, 14.694043 46.201091, 14.694214 46.200794, 14.694644 46.199871, 14.694858 46.199569, 14.694901 46.19942, 14.695179 46.199203, 14.695481 46.199012, 14.695716 46.198883, 14.696467 46.198581, 14.696639 46.19841, 14.696618 46.198261, 14.696553 46.198112, 14.696597 46.197895, 14.696597 46.197723, 14.69636 46.197059, 14.696038 46.196971, 14.694836 46.196823, 14.694644 46.196758, 14.694343 46.196693, 14.694193 46.196628, 14.69385 46.196414, 14.693635 46.196178, 14.693549 46.195964, 14.693571 46.195793, 14.69372 46.195514, 14.693871 46.195278, 14.694086 46.195064, 14.694235 46.194934, 14.694815 46.194656, 14.695158 46.194568, 14.695373 46.194442, 14.695459 46.194248, 14.695481 46.194053, 14.695395 46.193862, 14.694966 46.193603, 14.694429 46.193241, 14.694171 46.192787, 14.694022 46.192188, 14.693764 46.192146, 14.693442 46.191802, 14.693184 46.191631, 14.69297 46.191394, 14.692669 46.191288, 14.692412 46.191246, 14.692004 46.191223, 14.691682 46.191181, 14.691082 46.190856, 14.691039 46.19075, 14.690888 46.190559, 14.69063 46.190063, 14.690438 46.189849, 14.690094 46.189529, 14.6896 46.189163, 14.688786 46.188667, 14.687928 46.188217, 14.687562 46.187873, 14.687348 46.187595, 14.687262 46.187423, 14.687133 46.186439, 14.686962 46.186244, 14.686747 46.186096, 14.686469 46.185966, 14.686211 46.185813, 14.685932 46.185577, 14.685566 46.185321, 14.685309 46.185192, 14.684924 46.185066, 14.684473 46.184955, 14.683421 46.184783, 14.682906 46.184741, 14.68252 46.184635, 14.682284 46.184528, 14.681769 46.184333, 14.68134 46.18412, 14.680502 46.183734, 14.679902 46.183368, 14.679623 46.183284, 14.679387 46.183303, 14.679065 46.18341, 14.678636 46.183734, 14.678401 46.18386, 14.678185 46.184013, 14.677907 46.18412, 14.676834 46.184333, 14.676319 46.184463, 14.67589 46.184547, 14.675761 46.18457, 14.674796 46.184806, 14.674494 46.184848, 14.674259 46.184848, 14.674044 46.184806, 14.67383 46.184783, 14.673658 46.184741, 14.673615 46.184741, 14.673572 46.184722, 14.672671 46.184505, 14.672606 46.184268, 14.67252 46.18399, 14.672349 46.183734, 14.672221 46.183456, 14.672478 46.182769, 14.67252 46.182533, 14.672456 46.182296, 14.671919 46.181674, 14.671662 46.181415, 14.670975 46.180366, 14.670632 46.179935, 14.670482 46.179656, 14.670353 46.179336, 14.670311 46.179164, 14.67046 46.178497, 14.670611 46.178005, 14.670676 46.177703, 14.670611 46.177402, 14.67059 46.177253, 14.670418 46.176868, 14.670225 46.176544, 14.670075 46.176139, 14.669989 46.175815, 14.669796 46.175342, 14.669646 46.175064, 14.66956 46.174957, 14.669388 46.174591, 14.66943 46.174591, 14.669732 46.174591, 14.669967 46.174613, 14.670161 46.174678, 14.670825 46.17485, 14.671169 46.174957, 14.67207 46.17517, 14.672843 46.175277, 14.673293 46.1753, 14.673486 46.175258, 14.673658 46.17517, 14.67383 46.175022, 14.673915 46.174915, 14.673958 46.17472, 14.674065 46.174312, 14.674302 46.174057, 14.674559 46.173969, 14.674881 46.17395, 14.675482 46.17382, 14.675696 46.173713, 14.676125 46.173393, 14.676276 46.17324, 14.676684 46.17279, 14.676791 46.172595, 14.676941 46.172382, 14.677134 46.172275, 14.677714 46.171997, 14.678271 46.17176, 14.678442 46.171653, 14.678464 46.171394, 14.678336 46.171245, 14.677928 46.171051, 14.677584 46.170795, 14.677371 46.170581, 14.677285 46.170429, 14.677263 46.17028, 14.677091 46.169357, 14.677091 46.169078, 14.676919 46.16867, 14.676748 46.168476, 14.676468 46.168243, 14.676276 46.167984, 14.676147 46.167747, 14.676125 46.167167, 14.676169 46.16687, 14.676255 46.166526, 14.67619 46.166244, 14.676169 46.165901, 14.675696 46.165409, 14.675503 46.165195, 14.674967 46.16468, 14.674624 46.164249, 14.67458 46.164077, 14.674624 46.163822, 14.674817 46.163093, 14.67486 46.162811, 14.673293 46.162574, 14.672843 46.16249, 14.672049 46.162384, 14.67046 46.162231, 14.669495 46.16217, 14.669323 46.162189, 14.667178 46.162124, 14.665825 46.162017, 14.663873 46.161804, 14.663552 46.161762, 14.663358 46.161739, 14.662478 46.161655, 14.66029 46.16159, 14.658466 46.161609, 14.657757 46.161632, 14.656212 46.16159, 14.655633 46.161567, 14.653208 46.161437, 14.6522 46.161312, 14.650913 46.161075, 14.650827 46.161033, 14.650398 46.160945, 14.649476 46.16069, 14.647136 46.159915, 14.646513 46.15966, 14.644625 46.158798, 14.644304 46.158672, 14.643509 46.158435, 14.642888 46.158264, 14.642544 46.158199, 14.641858 46.158027, 14.639326 46.157382, 14.638617 46.157169, 14.638038 46.15702, 14.637415 46.156826, 14.637051 46.156738, 14.636751 46.156631, 14.63542 46.156227, 14.634562 46.15599, 14.633254 46.155582, 14.629305 46.154399, 14.628597 46.154167, 14.62806 46.153991, 14.627974 46.153972, 14.627138 46.153694, 14.626279 46.153434, 14.626151 46.153392, 14.625335 46.153137, 14.625056 46.153072, 14.624091 46.152748, 14.622954 46.152385, 14.622353 46.152214, 14.62098 46.151763, 14.616452 46.150344, 14.614821 46.149829, 14.61304 46.149272, 14.61289 46.14923, 14.612353 46.149101, 14.611731 46.148906, 14.610938 46.148609, 14.609908 46.148307, 14.608534 46.147941, 14.607397 46.147644, 14.606045 46.147319, 14.605036 46.147106, 14.604844 46.147083, 14.604092 46.146934, 14.601303 46.146503, 14.600487 46.146354, 14.598877 46.146095, 14.59772 46.145862, 14.596776 46.145584, 14.596474 46.145496, 14.595488 46.14513, 14.594887 46.144874, 14.594608 46.144721, 14.594286 46.144573, 14.593771 46.144294, 14.592913 46.143756, 14.59257 46.143566, 14.589779 46.141807, 14.588536 46.140945, 14.587634 46.140369, 14.586476 46.139617, 14.585939 46.139251, 14.585532 46.138996, 14.584952 46.138587, 14.583514 46.137664, 14.5833 46.137512, 14.582849 46.137214, 14.581991 46.136718, 14.581454 46.136482, 14.580897 46.136268, 14.579888 46.13599, 14.579695 46.135948, 14.578472 46.135734, 14.578386 46.135734, 14.573236 46.135002, 14.572335 46.134853, 14.571562 46.134746, 14.569352 46.134315, 14.569008 46.134208, 14.567743 46.133716, 14.566735 46.133224, 14.565876 46.132705, 14.565168 46.132301, 14.563302 46.131118, 14.561778 46.130195, 14.560555 46.129402, 14.559889 46.128929, 14.55916 46.128372, 14.558495 46.127857, 14.557379 46.126827, 14.556799 46.126228, 14.554654 46.123909, 14.554204 46.123394, 14.553366 46.122364, 14.553066 46.121955, 14.551241 46.119533, 14.551028 46.119445, 14.549504 46.117385, 14.548646 46.116271, 14.547787 46.11507, 14.547293 46.114318, 14.547101 46.11404, 14.546778 46.113437, 14.54665 46.113201, 14.545985 46.111743, 14.545748 46.11095, 14.545684 46.110626, 14.54562 46.110069, 14.545599 46.107818, 14.545427 46.107021, 14.545169 46.106445, 14.544848 46.105949, 14.544419 46.105457, 14.543968 46.105026, 14.543431 46.104682, 14.542916 46.104362, 14.541457 46.103569, 14.540856 46.10316, 14.540513 46.102882, 14.540105 46.102474, 14.539934 46.10226, 14.539719 46.102001, 14.539441 46.101573, 14.53929 46.101356, 14.538538 46.099899, 14.537594 46.097988, 14.534269 46.090995, 14.533646 46.089729, 14.533217 46.088912, 14.532187 46.087219, 14.531737 46.086551, 14.529462 46.083354, 14.528904 46.082603, 14.528733 46.082412, 14.528475 46.082069, 14.528368 46.081958, 14.528197 46.081874, 14.527768 46.08181, 14.527231 46.081832, 14.526887 46.08181, 14.526587 46.081851, 14.524656 46.082412, 14.524249 46.082496, 14.523325 46.082756, 14.52221 46.083034, 14.521845 46.083076, 14.52073 46.08329, 14.520043 46.083396, 14.520021 46.082691, 14.519978 46.08213, 14.519957 46.081851, 14.519849 46.080093, 14.519785 46.07904, 14.519742 46.078742, 14.5197 46.078182, 14.51957 46.076725, 14.519506 46.075778, 14.519398 46.074428, 14.519334 46.073333, 14.519206 46.072132, 14.51912 46.071533, 14.518797 46.070758, 14.518454 46.070415, 14.518069 46.069965, 14.517682 46.069515, 14.517575 46.069408, 14.517252 46.069023, 14.517017 46.068763, 14.51545 46.067005, 14.514464 46.065952, 14.513819 46.065223, 14.51324 46.064559, 14.513091 46.064384, 14.512983 46.064258, 14.512897 46.063915, 14.511158 46.063549, 14.510236 46.063442, 14.509657 46.063442, 14.508906 46.063549, 14.508542 46.063655, 14.508477 46.063507, 14.506953 46.05973, 14.506653 46.058956, 14.506073 46.057693, 14.505923 46.05767, 14.505194 46.0578, 14.504464 46.057861, 14.503821 46.057819, 14.503541 46.057777, 14.503306 46.057712, 14.502898 46.057582, 14.502597 46.057456, 14.502018 46.057132, 14.501418 46.056663, 14.501288 46.056575, 14.500709 46.055995, 14.499571 46.054771, 14.499529 46.054729, 14.498477 46.05355, 14.498349 46.053421, 14.497555 46.05252, 14.497104 46.052047, 14.496675 46.05162, 14.496182 46.051212, 14.495302 46.050651, 14.495087 46.050525, 14.49483 46.050373, 14.494421 46.05001, 14.494164 46.049453, 14.494122 46.049171, 14.494122 46.048915, 14.494272 46.048465, 14.495001 46.047435, 14.49395 46.047069, 14.493628 46.046962, 14.493199 46.046791, 14.49189 46.046363, 14.490302 46.045803, 14.488757 46.045288, 14.487749 46.045032, 14.487513 46.044967, 14.486933 46.044815, 14.485689 46.044429, 14.48498 46.044258, 14.484293 46.044044, 14.482813 46.043659, 14.48189 46.04338, 14.480882 46.043121, 14.478693 46.042476, 14.477213 46.041984, 14.475625 46.041297, 14.475132 46.041038, 14.474059 46.040527, 14.473844 46.040416, 14.471398 46.039196, 14.470261 46.038658, 14.46893 46.038059, 14.468351 46.037841, 14.466033 46.037029, 14.465905 46.036964, 14.465819 46.036899, 14.464832 46.036533, 14.463952 46.03619, 14.462492 46.035762, 14.461956 46.035591, 14.461634 46.035526, 14.460712 46.035419, 14.460111 46.035377, 14.459553 46.035377, 14.458609 46.035331, 14.45788 46.035266, 14.456957 46.035053, 14.456271 46.034839, 14.455155 46.034496, 14.454854 46.034431, 14.454489 46.034301, 14.453803 46.034088, 14.453352 46.033958, 14.452666 46.033744, 14.45258 46.033721, 14.451893 46.033508, 14.451185 46.033271, 14.450819 46.033164, 14.450648 46.033123, 14.449769 46.032821, 14.449339 46.032691, 14.448374 46.032394, 14.446957 46.031944, 14.445799 46.031555, 14.445456 46.031471, 14.444447 46.031105, 14.443975 46.030933, 14.443739 46.030891, 14.443438 46.030784, 14.442945 46.03059, 14.442751 46.030525, 14.442408 46.030395, 14.441829 46.030204, 14.4411 46.029945, 14.440691 46.029819, 14.439877 46.029541, 14.438053 46.028896, 14.437109 46.028594, 14.43625 46.028293, 14.434899 46.027801, 14.433804 46.027435, 14.433933 46.026771, 14.434019 46.026107, 14.434019 46.025848, 14.433933 46.025505, 14.433718 46.025077, 14.433461 46.024795, 14.43316 46.024559, 14.432581 46.024303, 14.432366 46.024238, 14.43213 46.024173, 14.431422 46.024154, 14.43022 46.024345, 14.429856 46.024391, 14.428719 46.024368, 14.426615 46.02396, 14.422088 46.023166, 14.417174 46.022331, 14.414557 46.021858, 14.409106 46.020935, 14.405995 46.020378, 14.400223 46.01939, 14.398335 46.019065, 14.391962 46.017929, 14.389987 46.017478, 14.388506 46.017093, 14.387284 46.016708, 14.38694 46.016578, 14.385718 46.016128, 14.384774 46.01572, 14.383786 46.01527, 14.382692 46.014713, 14.382477 46.014583, 14.380524 46.013446, 14.378787 46.012393, 14.37649 46.010955, 14.374173 46.00954, 14.37265 46.008659, 14.371041 46.007888, 14.369646 46.007308, 14.368787 46.007007, 14.367414 46.00658, 14.366212 46.006237, 14.364518 46.005893, 14.364367 46.005851, 14.3622 46.005508, 14.360869 46.005378, 14.359131 46.0051, 14.358166 46.00497, 14.356943 46.004776, 14.354475 46.004261, 14.353552 46.004024, 14.35336 46.003959, 14.352952 46.003875, 14.35117 46.003337, 14.350483 46.003124, 14.349905 46.002887, 14.349003 46.002567, 14.347308 46.001899, 14.346085 46.001388, 14.343531 46.000205, 14.343102 45.999992, 14.34042 45.998703, 14.338103 45.997524, 14.335828 45.996299, 14.33306 45.994777, 14.332503 45.994476, 14.328704 45.992244, 14.326409 45.990806, 14.325207 45.990036, 14.321881 45.987762, 14.321023 45.987117, 14.320229 45.98656, 14.317353 45.984348, 14.315057 45.98246, 14.314285 45.981754, 14.312997 45.980636, 14.311474 45.97924, 14.30995 45.977699, 14.309264 45.976902, 14.308448 45.975894, 14.30774 45.974929, 14.307161 45.97409, 14.306538 45.973041, 14.306045 45.972076, 14.30568 45.97126, 14.30538 45.970508, 14.305209 45.969951, 14.304907 45.968597, 14.304843 45.968277, 14.30478 45.967826, 14.304715 45.966819, 14.304715 45.965808, 14.304843 45.964523, 14.304951 45.963901, 14.305573 45.961368, 14.305788 45.960208, 14.305851 45.959587, 14.305895 45.95845, 14.305809 45.957569, 14.305638 45.956691, 14.305209 45.95536, 14.304629 45.954135, 14.304371 45.953666, 14.303921 45.953041, 14.30347 45.952419, 14.303063 45.951927, 14.302526 45.951347, 14.302011 45.950832, 14.301474 45.950359, 14.300272 45.949417, 14.298642 45.948387, 14.298514 45.948322, 14.297869 45.947979, 14.297097 45.947612, 14.295917 45.947097, 14.295102 45.9468, 14.293728 45.946369, 14.292484 45.946048, 14.290681 45.945705, 14.288621 45.945446, 14.287162 45.945297, 14.28536 45.945018, 14.284587 45.944847, 14.283944 45.944694, 14.282313 45.944202, 14.281497 45.943901, 14.280703 45.94358, 14.280167 45.943321, 14.279072 45.942657, 14.278643 45.942314, 14.278042 45.941776, 14.277657 45.941368, 14.27727 45.940811, 14.277098 45.940555, 14.276627 45.939716, 14.276004 45.938774, 14.275146 45.93785, 14.273966 45.936779, 14.273386 45.936218, 14.272829 45.935619, 14.272336 45.935016, 14.271948 45.934459, 14.271541 45.933708, 14.271477 45.933536, 14.271176 45.93283, 14.270918 45.932144, 14.270726 45.931457, 14.270554 45.93077, 14.270082 45.928131, 14.269888 45.927291, 14.26931 45.925449, 14.268944 45.924484, 14.268 45.922573, 14.267657 45.921928, 14.267272 45.921306, 14.266391 45.920021, 14.265705 45.919097, 14.265597 45.918991, 14.263688 45.916694, 14.263237 45.916198, 14.261628 45.914184, 14.260426 45.912487, 14.259331 45.910793, 14.258902 45.910064, 14.258795 45.909828, 14.258366 45.909076, 14.257851 45.908088, 14.257679 45.907722, 14.257293 45.906887, 14.25695 45.905944, 14.256649 45.905063, 14.256499 45.904548, 14.256177 45.90324, 14.255942 45.901992, 14.255812 45.900684, 14.255812 45.900169, 14.25577 45.899547, 14.255791 45.898345, 14.25592 45.897018, 14.25607 45.896072, 14.256199 45.89532, 14.256285 45.894977, 14.2568 45.893196, 14.257271 45.89191, 14.257551 45.891288, 14.258044 45.890258, 14.258345 45.889678, 14.258774 45.888927, 14.260641 45.886138, 14.260855 45.885837, 14.261435 45.884891, 14.261842 45.884098, 14.262057 45.883628, 14.262486 45.882423, 14.262679 45.881568, 14.262808 45.880622, 14.262915 45.878154, 14.263001 45.877449, 14.263087 45.87704, 14.263301 45.876354, 14.263581 45.875686, 14.264074 45.874828, 14.264567 45.874164, 14.265104 45.873542, 14.265576 45.873092, 14.266928 45.87191, 14.267679 45.871223, 14.268429 45.870471, 14.269438 45.869163, 14.269824 45.868629, 14.27006 45.86824, 14.270661 45.866996, 14.271433 45.865215, 14.272142 45.863777, 14.272271 45.863563, 14.272336 45.863498, 14.272528 45.863155, 14.272978 45.862556, 14.273429 45.861976, 14.27418 45.861118, 14.276154 45.859077, 14.277314 45.857921, 14.27991 45.855258, 14.282034 45.85305, 14.282442 45.852619, 14.283278 45.851654, 14.285038 45.84951, 14.286025 45.848266, 14.286819 45.847385, 14.288342 45.845924, 14.288879 45.845474, 14.289244 45.84513, 14.290961 45.843479, 14.291583 45.84275, 14.292269 45.841869, 14.293042 45.840713, 14.293385 45.840133, 14.293814 45.839275, 14.293857 45.839229, 14.294458 45.837707, 14.294758 45.836677, 14.294887 45.836055, 14.295081 45.834831, 14.295123 45.833953, 14.294995 45.832859, 14.294865 45.832344, 14.294672 45.831806, 14.294544 45.831527, 14.294458 45.831314, 14.294308 45.831035, 14.293965 45.830497, 14.2936 45.830005, 14.293128 45.829467, 14.29257 45.828952, 14.291647 45.828178, 14.289694 45.826721, 14.289458 45.826507, 14.289179 45.826313, 14.288428 45.825649, 14.287871 45.825069, 14.287055 45.824058, 14.28654 45.823246, 14.286068 45.82228, 14.285832 45.821655, 14.285617 45.820949, 14.285489 45.820262, 14.285403 45.819168, 14.285424 45.818717, 14.285553 45.8176, 14.28566 45.817237, 14.285746 45.816806, 14.28611 45.81554, 14.28624 45.814941, 14.286347 45.814231, 14.286347 45.813739, 14.286261 45.813072, 14.286175 45.812622, 14.285961 45.812023, 14.285789 45.811679, 14.285489 45.811187, 14.28521 45.810798, 14.285038 45.810585, 14.284544 45.81007, 14.284029 45.809619, 14.283579 45.809276, 14.28302 45.808891, 14.28227 45.808418, 14.282162 45.808376, 14.282034 45.808288, 14.280725 45.807579, 14.280124 45.807281, 14.279566 45.807022, 14.27888 45.806743, 14.278214 45.806442, 14.277335 45.806121, 14.276691 45.805908, 14.275038 45.805393, 14.272721 45.804706, 14.272164 45.804576, 14.271927 45.804489, 14.270812 45.804191, 14.27019 45.803997, 14.269503 45.80376, 14.269181 45.803676, 14.26813 45.80331, 14.267657 45.803115, 14.267056 45.802902, 14.264117 45.801635, 14.262615 45.800949, 14.26195 45.80067, 14.260705 45.800178, 14.258881 45.799533, 14.257079 45.799041, 14.256499 45.798912, 14.255211 45.798675, 14.254569 45.798591, 14.253666 45.798439, 14.251908 45.798202, 14.245363 45.797302, 14.243002 45.796958, 14.242209 45.796829, 14.240942 45.796592, 14.240191 45.796424, 14.239526 45.796295, 14.238282 45.79597, 14.236973 45.795608, 14.235556 45.795112, 14.235235 45.794986, 14.234934 45.794876, 14.233668 45.794342, 14.232552 45.793785, 14.232081 45.793567, 14.230686 45.792774, 14.230021 45.792346, 14.229613 45.792087, 14.229227 45.791786, 14.228454 45.791252, 14.228068 45.79095, 14.227252 45.790245, 14.226674 45.789707, 14.226394 45.789382, 14.225815 45.788784, 14.225021 45.787841, 14.22412 45.786575, 14.223282 45.785095, 14.222983 45.78445, 14.22264 45.783592, 14.222382 45.782775, 14.222167 45.781852, 14.222081 45.781467, 14.222081 45.781318, 14.221953 45.780502, 14.221888 45.779426, 14.221931 45.778205, 14.221995 45.777668, 14.222231 45.77621, 14.222424 45.77518, 14.222575 45.774108, 14.222618 45.773033, 14.222532 45.772026, 14.222274 45.771038, 14.222081 45.770523, 14.221716 45.769859, 14.221073 45.768871, 14.220644 45.768379, 14.220408 45.768142, 14.219979 45.767692, 14.218841 45.766811, 14.218218 45.76638, 14.217146 45.765758, 14.216546 45.765438, 14.215923 45.765136, 14.2153 45.764858, 14.214721 45.764644, 14.213605 45.764194, 14.210795 45.763271, 14.209614 45.762905, 14.209013 45.762756, 14.208842 45.762691, 14.20764 45.762348, 14.206803 45.762134, 14.205237 45.761768, 14.204765 45.761638, 14.203606 45.761383, 14.202576 45.761188, 14.20101 45.760868, 14.19998 45.760696, 14.198134 45.760395, 14.196932 45.760246, 14.195924 45.760093, 14.194851 45.759967, 14.193114 45.759796, 14.189616 45.759536, 14.186354 45.759407, 14.185367 45.759407, 14.184445 45.759407, 14.183779 45.759388, 14.181139 45.759407, 14.178114 45.759471, 14.176741 45.759536, 14.175818 45.759559, 14.175604 45.759578, 14.174724 45.759601, 14.172085 45.759773, 14.17084 45.759838, 14.167664 45.760093, 14.165798 45.760288, 14.165626 45.760288, 14.163652 45.760501, 14.159575 45.760997, 14.159446 45.760997, 14.159275 45.761039, 14.158974 45.761062, 14.157879 45.761211, 14.156571 45.761425, 14.155798 45.761554, 14.153845 45.761856, 14.151464 45.76226, 14.150477 45.762413, 14.149747 45.762542, 14.148911 45.762714, 14.146743 45.763057, 14.14316 45.763656, 14.14095 45.763977, 14.138246 45.764343, 14.138117 45.764343, 14.13625 45.764556, 14.133805 45.764774, 14.133482 45.764793, 14.131423 45.764923, 14.130393 45.764946, 14.129663 45.764987, 14.129105 45.764987, 14.128848 45.765007, 14.124707 45.765007, 14.122904 45.764946, 14.120844 45.764793, 14.120651 45.764793, 14.1202 45.764751, 14.118076 45.764579, 14.116059 45.764366, 14.112025 45.763786, 14.111724 45.763721, 14.110694 45.763549, 14.108592 45.763122, 14.106123 45.762584, 14.104922 45.762306, 14.102154 45.761619, 14.101983 45.761596, 14.100159 45.761146, 14.098528 45.76078, 14.09548 45.760158, 14.094666 45.760009, 14.092648 45.759689, 14.091061 45.759494, 14.090653 45.759429, 14.088442 45.759216, 14.087434 45.759128, 14.086039 45.759044, 14.083336 45.758956, 14.081318 45.758956, 14.079538 45.759002, 14.077027 45.759151, 14.075246 45.759281, 14.07383 45.759407, 14.073508 45.759452, 14.070418 45.759666, 14.067693 45.75975, 14.064775 45.759708, 14.062973 45.759624, 14.061985 45.759559, 14.061213 45.759471, 14.060398 45.759388, 14.059668 45.759281, 14.058402 45.759044, 14.057801 45.758892, 14.057136 45.758701, 14.056235 45.758377, 14.055871 45.758228, 14.05514 45.757885, 14.054518 45.757564, 14.053359 45.756832, 14.052094 45.755954, 14.0513 45.755394, 14.050527 45.754943, 14.049818 45.754581, 14.048317 45.753913, 14.048081 45.753788, 14.047437 45.753486, 14.046321 45.752864, 14.045721 45.752456, 14.045098 45.75196, 14.044541 45.751468, 14.044047 45.750953, 14.043661 45.750526, 14.042974 45.749496, 14.041944 45.747177, 14.041687 45.746643, 14.041365 45.746124, 14.041086 45.745719, 14.040399 45.74488, 14.039649 45.744174, 14.039326 45.743915, 14.038253 45.743145, 14.037589 45.742736, 14.036343 45.742027, 14.035014 45.741317, 14.033469 45.740589, 14.02628 45.737457, 14.02495 45.73677, 14.024241 45.736362, 14.022718 45.735313, 14.021967 45.73471, 14.021345 45.734153, 14.020658 45.733444, 14.020207 45.73291, 14.019692 45.732242, 14.019263 45.731597, 14.017997 45.729454, 14.017525 45.728702, 14.01686 45.72776, 14.01671 45.727607, 14.016624 45.727478, 14.015723 45.726448, 14.014392 45.725139, 14.013834 45.72467, 14.012375 45.723575, 14.011517 45.723014, 14.01083 45.722587, 14.010787 45.722545, 14.010724 45.722522, 14.009608 45.721859, 14.009157 45.72158, 14.006903 45.720207, 14.006238 45.719776, 14.005552 45.719284, 14.005251 45.719024, 14.004436 45.718254, 14.004115 45.71791, 14.003921 45.717716, 14.00332 45.716899, 14.002999 45.716365, 14.002697 45.715744, 14.002247 45.714561, 14.002055 45.713489, 14.002011 45.713123, 14.002011 45.712501, 14.002076 45.711151, 14.002076 45.709991, 14.002033 45.709541, 14.001904 45.708854, 14.001753 45.708274, 14.001475 45.707546, 14.001282 45.707138, 14.000809 45.706321, 14.000531 45.705913, 14.000016 45.705268, 13.999394 45.704605, 13.999007 45.704238, 13.998342 45.703704, 13.997999 45.703468, 13.997118 45.702888, 13.996518 45.702522, 13.996047 45.702289, 13.994952 45.701816, 13.994286 45.701557, 13.993664 45.701366, 13.992871 45.701149, 13.991175 45.700805, 13.988643 45.700355, 13.987763 45.700164, 13.986927 45.699951, 13.986262 45.699714, 13.985725 45.699497, 13.985102 45.699218, 13.984374 45.698833, 13.983557 45.698295, 13.983172 45.698017, 13.981648 45.69675, 13.980403 45.6954, 13.980038 45.694927, 13.97976 45.694477, 13.979395 45.693897, 13.978966 45.693084, 13.978795 45.692695, 13.978601 45.692161, 13.978322 45.691387, 13.978043 45.689971, 13.977871 45.688278, 13.977807 45.687011, 13.977593 45.685249, 13.977485 45.68478, 13.977034 45.683277, 13.976756 45.68259, 13.976176 45.681346, 13.97491 45.679115, 13.974696 45.678772, 13.973343 45.676368, 13.973044 45.675895, 13.968279 45.667568, 13.968152 45.667377, 13.968087 45.667224, 13.967958 45.667034, 13.966778 45.664909, 13.966113 45.663749, 13.964396 45.660488, 13.963237 45.658042, 13.962336 45.656047, 13.961735 45.654544, 13.961521 45.654071, 13.96092 45.652336, 13.960748 45.651821, 13.960383 45.650466, 13.960211 45.649482, 13.960083 45.648151, 13.960083 45.647506, 13.960125 45.64669, 13.960211 45.645896, 13.960534 45.644287, 13.961521 45.640575, 13.961692 45.639587, 13.961714 45.638946, 13.961778 45.638301, 13.961692 45.636779, 13.961543 45.635662, 13.961241 45.634288, 13.960619 45.632358, 13.959976 45.630748, 13.959847 45.630489, 13.959439 45.629505, 13.958087 45.626823, 13.957315 45.625469, 13.956736 45.624546, 13.956414 45.624076, 13.955748 45.623172, 13.955598 45.622959, 13.954611 45.621799, 13.953753 45.620899, 13.952938 45.620082, 13.951071 45.618537, 13.949139 45.617187, 13.947788 45.616371, 13.947402 45.616157, 13.945534 45.615257, 13.944912 45.614997, 13.944011 45.614677, 13.942873 45.614269, 13.940127 45.613559, 13.939033 45.613304, 13.936544 45.612766, 13.935643 45.612552, 13.935428 45.612487, 13.934827 45.612274, 13.933969 45.61193, 13.932424 45.611114, 13.931094 45.610214, 13.929741 45.60897, 13.929656 45.608882, 13.928883 45.607681, 13.928304 45.606502, 13.928154 45.605899, 13.927939 45.604396, 13.927939 45.603862, 13.928133 45.602916, 13.928218 45.60268, 13.928498 45.601779, 13.929312 45.599956, 13.929463 45.599571, 13.929677 45.598884, 13.929849 45.597938, 13.929892 45.597446, 13.929871 45.596458, 13.92972 45.59457, 13.929741 45.593627, 13.929827 45.592681, 13.930107 45.590965, 13.930107 45.58955, 13.929999 45.588775, 13.929785 45.58794, 13.929635 45.587574, 13.929077 45.586502, 13.928326 45.585449, 13.92824 45.585342, 13.927489 45.584613, 13.926953 45.58414, 13.925686 45.583282, 13.924356 45.582511, 13.923647 45.582191, 13.923262 45.582038, 13.922747 45.581848, 13.92236 45.581737, 13.921587 45.581504, 13.919936 45.581096, 13.918498 45.580795, 13.917382 45.580642, 13.915945 45.580516, 13.913992 45.580451, 13.912791 45.580516, 13.911932 45.5806, 13.911009 45.58073, 13.909829 45.581031, 13.908455 45.581459, 13.906246 45.582275, 13.903242 45.583328, 13.901503 45.58382, 13.900088 45.584186, 13.897341 45.58485, 13.894938 45.585407, 13.894187 45.585514, 13.893565 45.585514, 13.89305 45.585536, 13.892062 45.585472, 13.89071 45.585323, 13.88865 45.584999, 13.886891 45.584655, 13.885131 45.58427, 13.884445 45.584075, 13.883049 45.583713, 13.881355 45.583198, 13.880217 45.582748, 13.879317 45.582317, 13.878586 45.581867, 13.87805 45.581504, 13.877471 45.58105, 13.876656 45.580387, 13.87584 45.579593, 13.87436 45.57796, 13.873673 45.577148, 13.872321 45.575473, 13.87185 45.57468, 13.870798 45.573005, 13.870261 45.571933, 13.86979 45.570579, 13.869424 45.569423, 13.869317 45.568862, 13.86921 45.568069, 13.869231 45.566761, 13.869553 45.565753, 13.870089 45.564552, 13.871249 45.562213, 13.871742 45.561031, 13.871892 45.560432, 13.871956 45.559959, 13.871892 45.559379, 13.871592 45.5582, 13.871163 45.557277, 13.870583 45.556396, 13.870025 45.555709, 13.869467 45.555194, 13.868652 45.554595, 13.867493 45.553974, 13.866506 45.553501, 13.865498 45.553176, 13.86464 45.552963, 13.863845 45.552791, 13.861935 45.552578, 13.859875 45.552642, 13.858996 45.552749, 13.858288 45.552879, 13.857687 45.553028, 13.857451 45.55307, 13.857172 45.553028, 13.856957 45.552963, 13.856764 45.552791, 13.856658 45.552619, 13.856593 45.552448, 13.856593 45.552299, 13.856636 45.552104, 13.856764 45.551933, 13.857001 45.551742, 13.857322 45.55157, 13.858073 45.551311, 13.858889 45.551181, 13.859296 45.551162, 13.858953 45.551116, 13.85861 45.550991, 13.858416 45.550861, 13.858266 45.550773, 13.858138 45.550582, 13.857794 45.550281, 13.857193 45.550025, 13.856829 45.549896, 13.85655 45.549766, 13.855949 45.549575, 13.855477 45.549465, 13.854704 45.549015, 13.853674 45.54837, 13.853352 45.548221, 13.852538 45.548072, 13.852387 45.548027, 13.852473 45.547901, 13.852644 45.547813, 13.853053 45.547748, 13.853224 45.547706, 13.853417 45.547622, 13.853524 45.547515, 13.853524 45.547363, 13.853438 45.547233, 13.853266 45.546871, 13.853417 45.546527, 13.853438 45.546421, 13.85331 45.546226, 13.853266 45.546142, 13.853117 45.546054, 13.852923 45.54586, 13.852559 45.545623, 13.852387 45.54541, 13.852194 45.545089, 13.852108 45.544876, 13.851979 45.544681, 13.851872 45.544574, 13.851529 45.544338, 13.851422 45.544208, 13.851314 45.544036, 13.851164 45.543628, 13.850198 45.543437, 13.849812 45.543396, 13.849404 45.543457, 13.84846 45.543781, 13.847989 45.543888, 13.847516 45.543952, 13.847388 45.544082, 13.847172 45.544208, 13.846959 45.544296, 13.846657 45.54438, 13.8464 45.544403, 13.846358 45.544425, 13.845027 45.544425, 13.844791 45.544403, 13.844254 45.54425, 13.842774 45.543907, 13.842087 45.543651, 13.841809 45.543502, 13.841143 45.543201, 13.84035 45.543006, 13.839856 45.542942, 13.838761 45.5429, 13.838503 45.542877, 13.838074 45.5429, 13.837624 45.542964, 13.836615 45.543113, 13.835629 45.543308, 13.835135 45.543373, 13.834942 45.543415, 13.834555 45.543437, 13.834448 45.543457, 13.832839 45.543609, 13.832367 45.543674, 13.83198 45.543781, 13.830543 45.544315, 13.830092 45.544509, 13.829256 45.544746, 13.828248 45.544853, 13.827946 45.544895, 13.827389 45.545024, 13.826531 45.545173, 13.825865 45.545173, 13.825479 45.545131, 13.824856 45.545024, 13.82432 45.54483, 13.823699 45.544551, 13.823162 45.54425, 13.82269 45.544017, 13.822003 45.543758, 13.821681 45.543674, 13.821038 45.543563, 13.820694 45.543563, 13.820372 45.543586, 13.818848 45.543823, 13.817626 45.543865, 13.816939 45.543952, 13.815179 45.544124, 13.814278 45.54425, 13.813914 45.544315, 13.813571 45.544403, 13.813056 45.544574, 13.812798 45.544658, 13.812455 45.544811, 13.811381 45.545326, 13.811209 45.545433, 13.810652 45.545669, 13.810093 45.545883, 13.809107 45.546142, 13.807798 45.546421, 13.807518 45.546505, 13.806704 45.546699, 13.806017 45.546913, 13.805137 45.54734, 13.804794 45.547576, 13.804215 45.548072, 13.803828 45.548564, 13.803528 45.549144, 13.803185 45.549938, 13.802991 45.550304, 13.802454 45.551139, 13.801898 45.551742, 13.80164 45.551979, 13.801125 45.552383, 13.800631 45.552707, 13.800267 45.552944, 13.800073 45.553028, 13.799773 45.553199, 13.799129 45.553501, 13.798657 45.553672, 13.797884 45.553909, 13.795588 45.554573, 13.794365 45.554893, 13.793335 45.555194, 13.793185 45.555236, 13.791855 45.555603, 13.790932 45.555904, 13.790139 45.556182, 13.78943 45.556461, 13.788701 45.556785, 13.788594 45.556846, 13.788165 45.557041, 13.787863 45.557212, 13.787671 45.557296, 13.787327 45.557407, 13.786898 45.557514, 13.78662 45.557556, 13.785075 45.557533, 13.784795 45.557514, 13.784495 45.557449, 13.784152 45.557342, 13.782799 45.556591, 13.781985 45.556205, 13.781105 45.555797, 13.779603 45.555152, 13.779109 45.554893, 13.778401 45.554489, 13.777886 45.554122, 13.7775 45.553844, 13.777049 45.553413, 13.775504 45.551742, 13.775139 45.551399, 13.774496 45.550903, 13.774024 45.550582, 13.7731 45.550087, 13.772693 45.549896, 13.771406 45.549423, 13.768036 45.548328, 13.76411 45.547019, 13.763767 45.54689, 13.763531 45.546829, 13.76087 45.545906, 13.76072 45.54586, 13.758832 45.545131, 13.758102 45.544811, 13.757029 45.54438, 13.755934 45.543952, 13.752823 45.542598, 13.746537 45.539917, 13.743167 45.538501, 13.741966 45.538051, 13.740914 45.537727, 13.740141 45.537559, 13.739391 45.537429, 13.73894 45.537364, 13.738253 45.5373, 13.737696 45.537128, 13.737566 45.536979, 13.737417 45.536678, 13.737245 45.536441, 13.73688 45.53614, 13.736601 45.535842, 13.735829 45.53498, 13.735636 45.53479, 13.735465 45.53466, 13.735185 45.53466, 13.735185 45.53421, 13.735163 45.533481, 13.735163 45.533008, 13.735142 45.5326, 13.735185 45.531871, 13.735185 45.53086, 13.735207 45.530475, 13.735228 45.529811, 13.735228 45.529319, 13.735185 45.528633, 13.735099 45.528244, 13.734584 45.527084, 13.734456 45.526828, 13.734327 45.526634, 13.734219 45.526443, 13.733898 45.525947, 13.733532 45.525497, 13.733168 45.525112, 13.732502 45.52449, 13.731409 45.523738, 13.730679 45.52333, 13.729563 45.522644, 13.728684 45.522129, 13.72746 45.521358, 13.727181 45.521141, 13.726666 45.52069, 13.726087 45.520133, 13.725915 45.519874, 13.72568 45.519596, 13.72555 45.519405, 13.724928 45.518352, 13.723792 45.516315, 13.723512 45.5158, 13.723126 45.515155, 13.722203 45.513782, 13.721881 45.513267, 13.721667 45.512966, 13.721452 45.51258, 13.721045 45.511291, 13.720766 45.510971, 13.720465 45.510734, 13.720251 45.510604, 13.719328 45.510135, 13.719157 45.510005, 13.719006 45.509876, 13.718706 45.509426, 13.718512 45.509212, 13.718298 45.50904, 13.718041 45.508888, 13.717375 45.508674, 13.716989 45.508483, 13.71656 45.508266, 13.716366 45.508224, 13.716131 45.508201, 13.715937 45.508247, 13.715594 45.508419, 13.715466 45.508544, 13.715401 45.50872, 13.715401 45.508888, 13.715487 45.50904, 13.715594 45.50917, 13.715765 45.509277, 13.716474 45.509513, 13.716795 45.509685, 13.716946 45.509834, 13.717053 45.510005, 13.717118 45.510219, 13.717118 45.510414, 13.717097 45.510757, 13.717161 45.511058, 13.717225 45.511207, 13.717375 45.511444, 13.717676 45.51181, 13.717825 45.512043, 13.71789 45.512432, 13.717847 45.51271, 13.717762 45.512901, 13.717568 45.513351, 13.717568 45.513526, 13.71759 45.513717, 13.717654 45.51387, 13.717804 45.514102, 13.71789 45.514381, 13.71789 45.514575, 13.717804 45.5149, 13.717804 45.515113, 13.717933 45.515522, 13.717933 45.515735, 13.717847 45.515865, 13.71759 45.516056, 13.717418 45.51612, 13.716989 45.51612, 13.716795 45.516056, 13.716582 45.515907, 13.716496 45.515735, 13.716452 45.515586, 13.716474 45.515285, 13.716452 45.515048, 13.71641 45.514877, 13.71628 45.514682, 13.716088 45.514492, 13.715765 45.514297, 13.715487 45.51419, 13.714757 45.513996, 13.714028 45.51374, 13.713492 45.513439, 13.712698 45.512924, 13.712183 45.51271, 13.710873 45.512386, 13.710316 45.51213, 13.709951 45.511936, 13.709757 45.51181, 13.709221 45.511528, 13.70817 45.511207, 13.707741 45.511013, 13.707462 45.510887, 13.707247 45.510757, 13.706454 45.510349, 13.705574 45.509899, 13.70493 45.509532, 13.704801 45.509407, 13.704672 45.509212, 13.704672 45.50917, 13.704651 45.508911, 13.704737 45.508609, 13.704844 45.508331, 13.704973 45.508094, 13.705059 45.507797, 13.705059 45.50756, 13.705016 45.507453, 13.704995 45.507324, 13.704844 45.507087, 13.704629 45.506893, 13.704436 45.506767, 13.704136 45.506637, 13.703793 45.50653, 13.703106 45.506423, 13.701196 45.50608, 13.70081 45.506057, 13.700123 45.506057, 13.699523 45.505992, 13.698987 45.505844, 13.698729 45.505756, 13.698342 45.505584, 13.697957 45.505393, 13.697527 45.505134, 13.697248 45.505004, 13.696927 45.50492, 13.695961 45.5046, 13.695446 45.504447, 13.694952 45.504257, 13.694266 45.504039, 13.693793 45.503932, 13.693386 45.503891, 13.692313 45.503803, 13.69199 45.503761, 13.690445 45.503311, 13.689952 45.503269, 13.689202 45.503269, 13.6883 45.503227, 13.686926 45.503009, 13.684781 45.502819, 13.684138 45.502773, 13.68373 45.502708, 13.683258 45.502601, 13.682871 45.502475, 13.682571 45.502304, 13.682356 45.502109, 13.68225 45.501979, 13.682164 45.501831, 13.681862 45.501121, 13.681691 45.500801, 13.681433 45.500564, 13.680983 45.500007, 13.680683 45.499362, 13.680447 45.498931, 13.679996 45.498012, 13.679631 45.497344, 13.679417 45.497131, 13.679052 45.496871, 13.67858 45.496639, 13.678301 45.496444, 13.67813 45.49625, 13.677764 45.495285, 13.677421 45.494705, 13.677292 45.494533, 13.676563 45.493762, 13.676455 45.493614, 13.676283 45.493289, 13.676219 45.492969, 13.676219 45.492496, 13.676348 45.491809, 13.675619 45.491767, 13.675469 45.491722, 13.675168 45.491554, 13.67504 45.491378, 13.675018 45.49123, 13.674932 45.491081, 13.674738 45.490928, 13.674589 45.490821, 13.673559 45.490543, 13.673409 45.490459, 13.673344 45.490371, 13.673365 45.49007, 13.673451 45.48962, 13.673409 45.489513, 13.673237 45.489494, 13.673044 45.48962, 13.6727 45.490028, 13.6724 45.490436, 13.672207 45.490459, 13.672163 45.490306, 13.672185 45.490051, 13.672163 45.489921, 13.672057 45.489749, 13.671885 45.489662, 13.670726 45.489536, 13.668644 45.489364, 13.668151 45.489276, 13.667529 45.489128, 13.667185 45.489063, 13.666756 45.488914, 13.666564 45.488956, 13.666756 45.489063, 13.6677 45.489685, 13.668774 45.490135, 13.668774 45.490287, 13.668409 45.4902, 13.667916 45.490116, 13.667121 45.490005, 13.666564 45.489879, 13.665791 45.489536, 13.665083 45.489257, 13.664332 45.488826, 13.663666 45.488353, 13.663367 45.488056, 13.663065 45.487796, 13.662744 45.48756, 13.66268 45.487583, 13.66283 45.487777, 13.663044 45.488311, 13.663259 45.488613, 13.663409 45.488891, 13.663688 45.489318, 13.663731 45.489578, 13.663838 45.489727, 13.663774 45.489749, 13.663538 45.489662, 13.663345 45.489749, 13.663237 45.489707, 13.662315 45.48904, 13.662165 45.488891, 13.661606 45.488376, 13.661091 45.488204, 13.660813 45.488334, 13.660491 45.488311, 13.659546 45.488182, 13.658732 45.488121, 13.658001 45.488163, 13.657637 45.488246, 13.65693 45.488483, 13.6565 45.488742, 13.656693 45.489212, 13.656736 45.489471, 13.656822 45.489749, 13.656822 45.490028, 13.65665 45.49018, 13.656371 45.490348, 13.656027 45.490306, 13.655684 45.490306, 13.654805 45.490135, 13.654482 45.490116, 13.654204 45.490135, 13.653882 45.490222, 13.653582 45.490413, 13.653582 45.490585, 13.653624 45.490844, 13.65371 45.491123, 13.653818 45.491359, 13.653967 45.491508, 13.654139 45.491767, 13.654269 45.492065, 13.654333 45.492408, 13.653818 45.492691, 13.653367 45.492797, 13.653174 45.492797, 13.652874 45.492733, 13.652337 45.492431, 13.652015 45.492366, 13.651629 45.492366, 13.651286 45.492301, 13.651071 45.492218, 13.65062 45.492176, 13.649762 45.492046, 13.648903 45.491939, 13.648067 45.492065, 13.647616 45.492176, 13.646951 45.492301, 13.646716 45.492389, 13.646544 45.492519, 13.646136 45.493248, 13.646007 45.493503, 13.645707 45.493762, 13.645149 45.494171, 13.644934 45.494255, 13.644526 45.494468, 13.644484 45.494491, 13.644097 45.494644, 13.643454 45.495029, 13.643132 45.495265, 13.642531 45.495929, 13.642402 45.496185, 13.642338 45.49636, 13.642273 45.496765, 13.642273 45.496894, 13.642188 45.497753, 13.642059 45.498119, 13.641909 45.49887, 13.641909 45.499214, 13.64193 45.49945, 13.641522 45.499469, 13.640965 45.499362, 13.64045 45.49932, 13.63987 45.499275, 13.63929 45.499191, 13.637982 45.499042, 13.637316 45.498912, 13.636437 45.498699, 13.636029 45.498569, 13.635901 45.498741, 13.635815 45.498996, 13.635879 45.499256, 13.635965 45.499469, 13.636073 45.499599, 13.636222 45.499794, 13.63648 45.500026, 13.637188 45.500759, 13.637639 45.501079, 13.637982 45.501293, 13.638196 45.501487, 13.638411 45.501617, 13.638905 45.502021, 13.639012 45.502281, 13.638905 45.502601, 13.638926 45.502925, 13.638905 45.503074, 13.63884 45.503246, 13.638626 45.503459, 13.638454 45.503719, 13.638068 45.503696, 13.63766 45.503631, 13.637424 45.503826, 13.636823 45.503932, 13.636351 45.504104, 13.635901 45.504276, 13.635536 45.504341, 13.635235 45.504341, 13.634849 45.504299, 13.634442 45.504318, 13.634162 45.504299, 13.633626 45.504169, 13.633218 45.504039, 13.632789 45.503955, 13.632338 45.503803, 13.631651 45.503631, 13.63118 45.503547, 13.629893 45.50299, 13.629098 45.502666, 13.628047 45.502323, 13.626867 45.502021, 13.626223 45.501853, 13.624743 45.501552, 13.624292 45.50151, 13.623348 45.50138, 13.622811 45.501251, 13.622168 45.501274, 13.621116 45.501487, 13.620816 45.501529, 13.620086 45.501678, 13.619292 45.501937, 13.61897 45.502021, 13.618241 45.502304, 13.617254 45.502773, 13.616932 45.502883, 13.61616 45.503376, 13.615859 45.503589, 13.615473 45.503482, 13.614507 45.503505, 13.614207 45.503677, 13.613842 45.503761, 13.613649 45.503761, 13.613541 45.503677, 13.613649 45.503311, 13.61352 45.503227, 13.613328 45.503395, 13.612876 45.503738, 13.612683 45.503913, 13.612555 45.504062, 13.612254 45.504234, 13.611611 45.504577, 13.611203 45.504833, 13.610902 45.504833, 13.610667 45.504898, 13.61043 45.505092, 13.609894 45.505477, 13.609787 45.505607, 13.609744 45.505714, 13.609443 45.505886, 13.609229 45.505973, 13.607577 45.506294, 13.606911 45.506359, 13.606633 45.506359, 13.606396 45.506378, 13.60571 45.506378, 13.605109 45.506378, 13.604551 45.506485, 13.604271 45.506572, 13.60395 45.506744, 13.603671 45.50698, 13.603306 45.507366, 13.60277 45.507923, 13.602383 45.507709, 13.602513 45.507431, 13.602469 45.507347, 13.602255 45.507217, 13.601933 45.507236, 13.601804 45.507324, 13.601633 45.507514, 13.60114 45.507923, 13.60071 45.508331, 13.600388 45.508697, 13.600281 45.508869, 13.600195 45.509124, 13.600152 45.509407, 13.600216 45.510349, 13.600195 45.510669, 13.600131 45.510929, 13.59983 45.511337, 13.599595 45.511573, 13.599122 45.511936, 13.598994 45.512023, 13.598263 45.512645, 13.598114 45.512859, 13.598006 45.513118, 13.597985 45.51329, 13.598006 45.513526, 13.598114 45.513759, 13.598263 45.513996, 13.598692 45.514339, 13.60071 45.515819, 13.601933 45.516742, 13.602319 45.517021, 13.602534 45.51715, 13.604014 45.518287, 13.60425 45.518547, 13.604508 45.518909, 13.604851 45.51947, 13.605902 45.521247, 13.605902 45.5214, 13.605881 45.521507, 13.605731 45.52172, 13.605581 45.521785, 13.605387 45.52185, 13.604765 45.521892, 13.604594 45.521957, 13.604465 45.52211, 13.604465 45.522277, 13.604594 45.522388, 13.60483 45.522472, 13.605774 45.522731, 13.606182 45.522857, 13.606547 45.523075, 13.606825 45.523353, 13.607148 45.523738, 13.607555 45.524169, 13.60779 45.524337, 13.608305 45.524597, 13.608735 45.524726, 13.609122 45.524791, 13.609936 45.524982, 13.610774 45.525177, 13.611697 45.525367, 13.613348 45.525756, 13.614679 45.526012, 13.617918 45.526741, 13.618413 45.52687, 13.619614 45.527107, 13.620387 45.527278, 13.622296 45.527687, 13.62279 45.527858, 13.623154 45.52803, 13.623434 45.528224, 13.623691 45.528415, 13.62412 45.5288, 13.624464 45.52906, 13.624936 45.529403, 13.625322 45.529575, 13.625579 45.529663, 13.626009 45.529724, 13.62749 45.529853, 13.628669 45.529983, 13.62957 45.530048, 13.630772 45.530155, 13.631459 45.530197, 13.63221 45.530284, 13.633733 45.53041, 13.634077 45.53041, 13.634827 45.530326, 13.635085 45.530239, 13.636094 45.529811, 13.636759 45.529533, 13.637017 45.529426, 13.637553 45.529254, 13.638476 45.528995, 13.638861 45.528823, 13.639205 45.528633, 13.639613 45.528373, 13.640879 45.527343, 13.641844 45.526485, 13.642488 45.525947, 13.642917 45.525711, 13.643218 45.525627, 13.64354 45.525604, 13.644097 45.525669, 13.64444 45.525775, 13.644955 45.525886, 13.646136 45.526035, 13.647145 45.526012, 13.647852 45.525947, 13.648732 45.525798, 13.64944 45.525585, 13.650105 45.525348, 13.651351 45.524745, 13.652123 45.52436, 13.653239 45.523887, 13.654183 45.523609, 13.655684 45.523307, 13.656886 45.523181, 13.658131 45.523159, 13.658453 45.523181, 13.659161 45.523223, 13.659804 45.523288, 13.661457 45.523651, 13.662121 45.523845, 13.662787 45.524082, 13.663367 45.524318, 13.664053 45.524639, 13.665791 45.52552, 13.666928 45.526142, 13.668517 45.527107, 13.669203 45.527603, 13.669846 45.528114, 13.670319 45.528568, 13.670662 45.52893, 13.671048 45.529426, 13.671263 45.529811, 13.671456 45.530239, 13.67167 45.530841, 13.672057 45.532344, 13.672293 45.532943, 13.672465 45.533287, 13.672615 45.5335, 13.672958 45.533889, 13.67313 45.53408, 13.67343 45.534359, 13.674996 45.535476, 13.675275 45.535732, 13.675662 45.536163, 13.675854 45.536399, 13.676048 45.536762, 13.676305 45.537364, 13.676348 45.537708, 13.676369 45.53833, 13.676348 45.538589, 13.676219 45.5391, 13.676005 45.539638, 13.675962 45.539703, 13.675683 45.54013, 13.67549 45.540325, 13.67519 45.540473, 13.674589 45.540538, 13.673945 45.540538, 13.673473 45.540496, 13.672936 45.540348, 13.67285 45.540283, 13.672443 45.540023, 13.67195 45.539661, 13.671241 45.539188, 13.67107 45.5391, 13.670233 45.538589, 13.669353 45.538093, 13.668946 45.538352, 13.668644 45.538589, 13.668473 45.538696, 13.668151 45.538845, 13.667722 45.538993, 13.667551 45.539016, 13.666649 45.539253, 13.666049 45.539382, 13.665233 45.539596, 13.66474 45.53968, 13.664375 45.539768, 13.66386 45.539875, 13.663666 45.539939, 13.663559 45.539596, 13.662658 45.539768, 13.662594 45.539554, 13.661843 45.539703, 13.660491 45.540004)";
    //
    // assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testLong1H20() throws Exception {
    Route route=test(33828, 32811, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction(2.0));

    logger.info("Found route="+route);

    // String
    // result="LINESTRING (13.660491 45.540004, 13.661843 45.539703, 13.662594 45.539554, 13.662658 45.539768, 13.663559 45.539596, 13.663666 45.539939, 13.66386 45.539875, 13.664375 45.539768, 13.66474 45.53968, 13.665233 45.539596, 13.666049 45.539382, 13.666649 45.539253, 13.667551 45.539016, 13.667722 45.538993, 13.668151 45.538845, 13.668473 45.538696, 13.668644 45.538589, 13.668946 45.538352, 13.669353 45.538093, 13.670233 45.538589, 13.67107 45.5391, 13.671241 45.539188, 13.67195 45.539661, 13.672443 45.540023, 13.67285 45.540283, 13.672936 45.540348, 13.673473 45.540496, 13.673945 45.540538, 13.674589 45.540538, 13.67519 45.540473, 13.67549 45.540325, 13.675683 45.54013, 13.675962 45.539703, 13.676005 45.539638, 13.676219 45.5391, 13.676348 45.538589, 13.676369 45.53833, 13.676348 45.537708, 13.676305 45.537364, 13.676048 45.536762, 13.675854 45.536399, 13.675662 45.536163, 13.675275 45.535732, 13.674996 45.535476, 13.67343 45.534359, 13.67313 45.53408, 13.672958 45.533889, 13.672615 45.5335, 13.672465 45.533287, 13.672293 45.532943, 13.672057 45.532344, 13.67167 45.530841, 13.671456 45.530239, 13.671263 45.529811, 13.671048 45.529426, 13.670662 45.52893, 13.670319 45.528568, 13.669846 45.528114, 13.669203 45.527603, 13.668517 45.527107, 13.666928 45.526142, 13.665791 45.52552, 13.664053 45.524639, 13.663367 45.524318, 13.662787 45.524082, 13.662121 45.523845, 13.661457 45.523651, 13.659804 45.523288, 13.659161 45.523223, 13.658453 45.523181, 13.658131 45.523159, 13.656886 45.523181, 13.655684 45.523307, 13.654183 45.523609, 13.653239 45.523887, 13.652123 45.52436, 13.651351 45.524745, 13.650105 45.525348, 13.64944 45.525585, 13.648732 45.525798, 13.647852 45.525947, 13.647145 45.526012, 13.646136 45.526035, 13.644955 45.525886, 13.64444 45.525775, 13.644097 45.525669, 13.64354 45.525604, 13.643218 45.525627, 13.642917 45.525711, 13.642488 45.525947, 13.641844 45.526485, 13.640879 45.527343, 13.639613 45.528373, 13.639205 45.528633, 13.638861 45.528823, 13.638476 45.528995, 13.637553 45.529254, 13.637017 45.529426, 13.636759 45.529533, 13.636094 45.529811, 13.635085 45.530239, 13.634827 45.530326, 13.634077 45.53041, 13.633733 45.53041, 13.63221 45.530284, 13.631459 45.530197, 13.630772 45.530155, 13.62957 45.530048, 13.628669 45.529983, 13.62749 45.529853, 13.626009 45.529724, 13.625579 45.529663, 13.625322 45.529575, 13.624936 45.529403, 13.624464 45.52906, 13.62412 45.5288, 13.623691 45.528415, 13.623434 45.528224, 13.623154 45.52803, 13.62279 45.527858, 13.622296 45.527687, 13.620387 45.527278, 13.619614 45.527107, 13.618413 45.52687, 13.617918 45.526741, 13.614679 45.526012, 13.613348 45.525756, 13.611697 45.525367, 13.610774 45.525177, 13.609936 45.524982, 13.609122 45.524791, 13.608735 45.524726, 13.608305 45.524597, 13.60779 45.524337, 13.607555 45.524169, 13.607148 45.523738, 13.606825 45.523353, 13.606547 45.523075, 13.606182 45.522857, 13.605774 45.522731, 13.60483 45.522472, 13.604594 45.522388, 13.604465 45.522277, 13.604465 45.52211, 13.604594 45.521957, 13.604765 45.521892, 13.605387 45.52185, 13.605581 45.521785, 13.605731 45.52172, 13.605881 45.521507, 13.605902 45.5214, 13.605902 45.521247, 13.604851 45.51947, 13.604508 45.518909, 13.60425 45.518547, 13.604014 45.518287, 13.602534 45.51715, 13.602319 45.517021, 13.601933 45.516742, 13.60071 45.515819, 13.598692 45.514339, 13.598263 45.513996, 13.598114 45.513759, 13.598006 45.513526, 13.597985 45.51329, 13.598006 45.513118, 13.598114 45.512859, 13.598263 45.512645, 13.598994 45.512023, 13.599122 45.511936, 13.599595 45.511573, 13.59983 45.511337, 13.600131 45.510929, 13.600195 45.510669, 13.600216 45.510349, 13.600152 45.509407, 13.600195 45.509124, 13.600281 45.508869, 13.600388 45.508697, 13.60071 45.508331, 13.60114 45.507923, 13.601633 45.507514, 13.601804 45.507324, 13.601933 45.507236, 13.602255 45.507217, 13.602469 45.507347, 13.602513 45.507431, 13.602383 45.507709, 13.60277 45.507923, 13.603306 45.507366, 13.603671 45.50698, 13.60395 45.506744, 13.604271 45.506572, 13.604551 45.506485, 13.605109 45.506378, 13.60571 45.506378, 13.606396 45.506378, 13.606633 45.506359, 13.606911 45.506359, 13.607577 45.506294, 13.609229 45.505973, 13.609443 45.505886, 13.609744 45.505714, 13.609787 45.505607, 13.609894 45.505477, 13.61043 45.505092, 13.610667 45.504898, 13.610902 45.504833, 13.611203 45.504833, 13.611611 45.504577, 13.612254 45.504234, 13.612555 45.504062, 13.612683 45.503913, 13.612876 45.503738, 13.613328 45.503395, 13.61352 45.503227, 13.613649 45.503311, 13.613541 45.503677, 13.613649 45.503761, 13.613842 45.503761, 13.614207 45.503677, 13.614507 45.503505, 13.615473 45.503482, 13.615859 45.503589, 13.61616 45.503376, 13.616932 45.502883, 13.617254 45.502773, 13.618241 45.502304, 13.61897 45.502021, 13.619292 45.501937, 13.620086 45.501678, 13.620816 45.501529, 13.621116 45.501487, 13.622168 45.501274, 13.622811 45.501251, 13.623348 45.50138, 13.624292 45.50151, 13.624743 45.501552, 13.626223 45.501853, 13.626867 45.502021, 13.628047 45.502323, 13.629098 45.502666, 13.629893 45.50299, 13.63118 45.503547, 13.631651 45.503631, 13.632338 45.503803, 13.632789 45.503955, 13.633218 45.504039, 13.633626 45.504169, 13.634162 45.504299, 13.634442 45.504318, 13.634849 45.504299, 13.635235 45.504341, 13.635536 45.504341, 13.635901 45.504276, 13.636351 45.504104, 13.636823 45.503932, 13.637424 45.503826, 13.63766 45.503631, 13.638068 45.503696, 13.638454 45.503719, 13.638626 45.503459, 13.63884 45.503246, 13.638905 45.503074, 13.638926 45.502925, 13.638905 45.502601, 13.639012 45.502281, 13.638905 45.502021, 13.638411 45.501617, 13.638196 45.501487, 13.637982 45.501293, 13.637639 45.501079, 13.637188 45.500759, 13.63648 45.500026, 13.636222 45.499794, 13.636073 45.499599, 13.635965 45.499469, 13.635879 45.499256, 13.635815 45.498996, 13.635901 45.498741, 13.636029 45.498569, 13.636437 45.498699, 13.637316 45.498912, 13.637982 45.499042, 13.63929 45.499191, 13.63987 45.499275, 13.64045 45.49932, 13.640965 45.499362, 13.641522 45.499469, 13.64193 45.49945, 13.641909 45.499214, 13.641909 45.49887, 13.642059 45.498119, 13.642188 45.497753, 13.642273 45.496894, 13.642273 45.496765, 13.642338 45.49636, 13.642402 45.496185, 13.642531 45.495929, 13.643132 45.495265, 13.643454 45.495029, 13.644097 45.494644, 13.644484 45.494491, 13.644526 45.494468, 13.644934 45.494255, 13.645149 45.494171, 13.645707 45.493762, 13.646007 45.493503, 13.646136 45.493248, 13.646544 45.492519, 13.646716 45.492389, 13.646951 45.492301, 13.647616 45.492176, 13.648067 45.492065, 13.648903 45.491939, 13.649762 45.492046, 13.65062 45.492176, 13.651071 45.492218, 13.651286 45.492301, 13.651629 45.492366, 13.652015 45.492366, 13.652337 45.492431, 13.652874 45.492733, 13.653174 45.492797, 13.653367 45.492797, 13.653818 45.492691, 13.654333 45.492408, 13.654269 45.492065, 13.654139 45.491767, 13.653967 45.491508, 13.653818 45.491359, 13.65371 45.491123, 13.653624 45.490844, 13.653582 45.490585, 13.653582 45.490413, 13.653882 45.490222, 13.654204 45.490135, 13.654482 45.490116, 13.654805 45.490135, 13.655684 45.490306, 13.656027 45.490306, 13.656371 45.490348, 13.65665 45.49018, 13.656822 45.490028, 13.656822 45.489749, 13.656736 45.489471, 13.656693 45.489212, 13.6565 45.488742, 13.65693 45.488483, 13.657637 45.488246, 13.658001 45.488163, 13.658732 45.488121, 13.659546 45.488182, 13.660491 45.488311, 13.660813 45.488334, 13.661091 45.488204, 13.661606 45.488376, 13.662165 45.488891, 13.662315 45.48904, 13.663237 45.489707, 13.663345 45.489749, 13.663538 45.489662, 13.663774 45.489749, 13.663838 45.489727, 13.663731 45.489578, 13.663688 45.489318, 13.663409 45.488891, 13.663259 45.488613, 13.663044 45.488311, 13.66283 45.487777, 13.66268 45.487583, 13.662744 45.48756, 13.663065 45.487796, 13.663367 45.488056, 13.663666 45.488353, 13.664332 45.488826, 13.665083 45.489257, 13.665791 45.489536, 13.666564 45.489879, 13.667121 45.490005, 13.667916 45.490116, 13.668409 45.4902, 13.668774 45.490287, 13.668774 45.490135, 13.6677 45.489685, 13.666756 45.489063, 13.666564 45.488956, 13.666756 45.488914, 13.667185 45.489063, 13.667529 45.489128, 13.668151 45.489276, 13.668644 45.489364, 13.670726 45.489536, 13.671885 45.489662, 13.672057 45.489749, 13.672163 45.489921, 13.672185 45.490051, 13.672163 45.490306, 13.672207 45.490459, 13.6724 45.490436, 13.6727 45.490028, 13.673044 45.48962, 13.673237 45.489494, 13.673409 45.489513, 13.673451 45.48962, 13.673365 45.49007, 13.673344 45.490371, 13.673409 45.490459, 13.673559 45.490543, 13.674589 45.490821, 13.674738 45.490928, 13.674932 45.491081, 13.675018 45.49123, 13.67504 45.491378, 13.675168 45.491554, 13.675469 45.491722, 13.675619 45.491767, 13.676348 45.491809, 13.676219 45.492496, 13.676219 45.492969, 13.676283 45.493289, 13.676455 45.493614, 13.676563 45.493762, 13.677292 45.494533, 13.677421 45.494705, 13.677764 45.495285, 13.67813 45.49625, 13.678301 45.496444, 13.67858 45.496639, 13.679052 45.496871, 13.679417 45.497131, 13.679631 45.497344, 13.679996 45.498012, 13.680447 45.498931, 13.680683 45.499362, 13.680983 45.500007, 13.681433 45.500564, 13.681691 45.500801, 13.681862 45.501121, 13.682164 45.501831, 13.68225 45.501979, 13.682356 45.502109, 13.682571 45.502304, 13.682871 45.502475, 13.683258 45.502601, 13.68373 45.502708, 13.684138 45.502773, 13.684781 45.502819, 13.686926 45.503009, 13.6883 45.503227, 13.689202 45.503269, 13.689952 45.503269, 13.690445 45.503311, 13.69199 45.503761, 13.692313 45.503803, 13.693386 45.503891, 13.693793 45.503932, 13.694266 45.504039, 13.694952 45.504257, 13.695446 45.504447, 13.695961 45.5046, 13.696927 45.50492, 13.697248 45.505004, 13.697527 45.505134, 13.697957 45.505393, 13.698342 45.505584, 13.698729 45.505756, 13.698987 45.505844, 13.699523 45.505992, 13.700123 45.506057, 13.70081 45.506057, 13.701196 45.50608, 13.703106 45.506423, 13.703793 45.50653, 13.704136 45.506637, 13.704436 45.506767, 13.704629 45.506893, 13.704844 45.507087, 13.704995 45.507324, 13.705016 45.507453, 13.705059 45.50756, 13.705059 45.507797, 13.704973 45.508094, 13.704844 45.508331, 13.704737 45.508609, 13.704651 45.508911, 13.704672 45.50917, 13.704672 45.509212, 13.704801 45.509407, 13.70493 45.509532, 13.705574 45.509899, 13.706454 45.510349, 13.707247 45.510757, 13.707462 45.510887, 13.707741 45.511013, 13.70817 45.511207, 13.709221 45.511528, 13.709757 45.51181, 13.709951 45.511936, 13.710316 45.51213, 13.710873 45.512386, 13.712183 45.51271, 13.712698 45.512924, 13.713492 45.513439, 13.714028 45.51374, 13.714757 45.513996, 13.715487 45.51419, 13.715765 45.514297, 13.716088 45.514492, 13.71628 45.514682, 13.71641 45.514877, 13.716452 45.515048, 13.716474 45.515285, 13.716452 45.515586, 13.716496 45.515735, 13.716582 45.515907, 13.716795 45.516056, 13.716989 45.51612, 13.717418 45.51612, 13.71759 45.516056, 13.717847 45.515865, 13.717933 45.515735, 13.717933 45.515522, 13.717804 45.515113, 13.717804 45.5149, 13.71789 45.514575, 13.71789 45.514381, 13.717804 45.514102, 13.717654 45.51387, 13.71759 45.513717, 13.717568 45.513526, 13.717568 45.513351, 13.717762 45.512901, 13.717847 45.51271, 13.71789 45.512432, 13.717825 45.512043, 13.717676 45.51181, 13.717375 45.511444, 13.717225 45.511207, 13.717161 45.511058, 13.717097 45.510757, 13.717118 45.510414, 13.717118 45.510219, 13.717053 45.510005, 13.716946 45.509834, 13.716795 45.509685, 13.716474 45.509513, 13.715765 45.509277, 13.715594 45.50917, 13.715487 45.50904, 13.715401 45.508888, 13.715401 45.50872, 13.715466 45.508544, 13.715594 45.508419, 13.715937 45.508247, 13.716131 45.508201, 13.716366 45.508224, 13.71656 45.508266, 13.716989 45.508483, 13.717375 45.508674, 13.718041 45.508888, 13.718298 45.50904, 13.718512 45.509212, 13.718706 45.509426, 13.719006 45.509876, 13.719157 45.510005, 13.719328 45.510135, 13.720251 45.510604, 13.720465 45.510734, 13.720766 45.510971, 13.721045 45.511291, 13.721452 45.51258, 13.721667 45.512966, 13.721881 45.513267, 13.722203 45.513782, 13.723126 45.515155, 13.723512 45.5158, 13.723792 45.516315, 13.724928 45.518352, 13.72555 45.519405, 13.72568 45.519596, 13.725915 45.519874, 13.726087 45.520133, 13.726666 45.52069, 13.727181 45.521141, 13.72746 45.521358, 13.728684 45.522129, 13.729563 45.522644, 13.730679 45.52333, 13.731409 45.523738, 13.732502 45.52449, 13.733168 45.525112, 13.733532 45.525497, 13.733898 45.525947, 13.734219 45.526443, 13.734327 45.526634, 13.734456 45.526828, 13.734584 45.527084, 13.735099 45.528244, 13.735185 45.528633, 13.735228 45.529319, 13.735228 45.529811, 13.735207 45.530475, 13.735185 45.53086, 13.735185 45.531871, 13.735142 45.5326, 13.735163 45.533008, 13.735163 45.533481, 13.735185 45.53421, 13.735185 45.53466, 13.735465 45.53466, 13.735636 45.53479, 13.735829 45.53498, 13.736601 45.535842, 13.73688 45.53614, 13.737245 45.536441, 13.737417 45.536678, 13.737674 45.536697, 13.738339 45.536808, 13.738854 45.536933, 13.739584 45.537128, 13.740271 45.537364, 13.741086 45.537601, 13.742567 45.538051, 13.74306 45.538223, 13.743575 45.538414, 13.74409 45.53865, 13.745099 45.539058, 13.746858 45.539897, 13.747609 45.54026, 13.747931 45.540367, 13.748317 45.540538, 13.749261 45.54084, 13.749991 45.541053, 13.750399 45.541141, 13.751 45.541313, 13.751278 45.541397, 13.751729 45.541484, 13.751965 45.541568, 13.752415 45.541633, 13.753531 45.541957, 13.754519 45.542278, 13.755699 45.542579, 13.75615 45.542663, 13.757222 45.542793, 13.758381 45.54322, 13.758445 45.54322, 13.758467 45.543243, 13.759046 45.543693, 13.759432 45.544101, 13.759604 45.54425, 13.760226 45.545002, 13.760634 45.545391, 13.760849 45.545516, 13.761192 45.545455, 13.761599 45.545303, 13.76205 45.545173, 13.762694 45.54496, 13.763274 45.544811, 13.763617 45.544937, 13.764668 45.54541, 13.764346 45.545799, 13.764218 45.545967, 13.764174 45.546119, 13.764925 45.546463, 13.765398 45.546741, 13.765805 45.547172, 13.766041 45.54747, 13.766256 45.547599, 13.766857 45.547748, 13.767801 45.547966, 13.768809 45.548221, 13.769002 45.54837, 13.769753 45.548736, 13.770096 45.548824, 13.771277 45.549186, 13.772092 45.549423, 13.773015 45.549724, 13.773659 45.550025, 13.774474 45.55056, 13.775139 45.551055, 13.77574 45.551548, 13.775891 45.551742, 13.776041 45.551891, 13.77632 45.552146, 13.776492 45.552406, 13.777092 45.553199, 13.77735 45.553436, 13.777543 45.553649, 13.777886 45.553863, 13.77838 45.554206, 13.779023 45.554531, 13.780869 45.555088, 13.781426 45.555603, 13.781512 45.555797, 13.781985 45.556205, 13.782799 45.556591, 13.784152 45.557342, 13.784495 45.557449, 13.784795 45.557514, 13.785075 45.557533, 13.78662 45.557556, 13.786898 45.557514, 13.787327 45.557407, 13.787671 45.557296, 13.787863 45.557212, 13.788165 45.557041, 13.788594 45.556846, 13.788701 45.556785, 13.78943 45.556461, 13.790139 45.556182, 13.790932 45.555904, 13.791855 45.555603, 13.793185 45.555236, 13.793335 45.555194, 13.794365 45.554893, 13.795588 45.554573, 13.797884 45.553909, 13.798657 45.553672, 13.799129 45.553501, 13.799773 45.553199, 13.800073 45.553028, 13.800267 45.552944, 13.800631 45.552707, 13.801125 45.552383, 13.80164 45.551979, 13.801898 45.551742, 13.802454 45.551139, 13.802991 45.550304, 13.803185 45.549938, 13.803528 45.549144, 13.803828 45.548564, 13.804215 45.548072, 13.804794 45.547576, 13.805137 45.54734, 13.806017 45.546913, 13.806704 45.546699, 13.807518 45.546505, 13.807798 45.546421, 13.809107 45.546142, 13.810093 45.545883, 13.810652 45.545669, 13.811209 45.545433, 13.811381 45.545326, 13.812455 45.544811, 13.812798 45.544658, 13.813056 45.544574, 13.813571 45.544403, 13.813914 45.544315, 13.814278 45.54425, 13.815179 45.544124, 13.816939 45.543952, 13.817626 45.543865, 13.818848 45.543823, 13.820372 45.543586, 13.820694 45.543563, 13.821038 45.543563, 13.821681 45.543674, 13.822003 45.543758, 13.82269 45.544017, 13.823162 45.54425, 13.823699 45.544551, 13.82432 45.54483, 13.824856 45.545024, 13.825479 45.545131, 13.825865 45.545173, 13.826531 45.545173, 13.827389 45.545024, 13.827946 45.544895, 13.828248 45.544853, 13.829256 45.544746, 13.830092 45.544509, 13.830543 45.544315, 13.83198 45.543781, 13.832367 45.543674, 13.832839 45.543609, 13.834448 45.543457, 13.834555 45.543437, 13.834942 45.543415, 13.835135 45.543373, 13.835629 45.543308, 13.836615 45.543113, 13.837624 45.542964, 13.838074 45.5429, 13.838503 45.542877, 13.838761 45.5429, 13.839856 45.542942, 13.84035 45.543006, 13.841143 45.543201, 13.841809 45.543502, 13.842087 45.543651, 13.842774 45.543907, 13.844254 45.54425, 13.844791 45.544403, 13.845027 45.544425, 13.846358 45.544425, 13.8464 45.544403, 13.846657 45.54438, 13.846959 45.544296, 13.847172 45.544208, 13.847388 45.544082, 13.847516 45.543952, 13.847989 45.543888, 13.84846 45.543781, 13.849404 45.543457, 13.849812 45.543396, 13.850198 45.543437, 13.851164 45.543628, 13.851314 45.544036, 13.851422 45.544208, 13.851529 45.544338, 13.851872 45.544574, 13.851979 45.544681, 13.852108 45.544876, 13.852194 45.545089, 13.852387 45.54541, 13.852559 45.545623, 13.852923 45.54586, 13.853117 45.546054, 13.853266 45.546142, 13.85331 45.546226, 13.853438 45.546421, 13.853417 45.546527, 13.853266 45.546871, 13.853438 45.547233, 13.853524 45.547363, 13.853524 45.547515, 13.853417 45.547622, 13.853224 45.547706, 13.853053 45.547748, 13.852644 45.547813, 13.852473 45.547901, 13.852387 45.548027, 13.852538 45.548072, 13.853352 45.548221, 13.853674 45.54837, 13.854704 45.549015, 13.855477 45.549465, 13.855949 45.549575, 13.85655 45.549766, 13.856829 45.549896, 13.857193 45.550025, 13.857794 45.550281, 13.858138 45.550582, 13.858266 45.550773, 13.858416 45.550861, 13.85861 45.550991, 13.858953 45.551116, 13.859296 45.551162, 13.858889 45.551181, 13.858073 45.551311, 13.857322 45.55157, 13.857001 45.551742, 13.856764 45.551933, 13.856636 45.552104, 13.856593 45.552299, 13.856593 45.552448, 13.856658 45.552619, 13.856764 45.552791, 13.856957 45.552963, 13.857172 45.553028, 13.857451 45.55307, 13.857687 45.553028, 13.858288 45.552879, 13.858996 45.552749, 13.859875 45.552642, 13.861935 45.552578, 13.863845 45.552791, 13.86464 45.552963, 13.865498 45.553176, 13.866506 45.553501, 13.867493 45.553974, 13.868652 45.554595, 13.869467 45.555194, 13.870025 45.555709, 13.870583 45.556396, 13.871163 45.557277, 13.871592 45.5582, 13.871892 45.559379, 13.871956 45.559959, 13.871892 45.560432, 13.871742 45.561031, 13.871249 45.562213, 13.870089 45.564552, 13.869553 45.565753, 13.869231 45.566761, 13.86921 45.568069, 13.869317 45.568862, 13.869424 45.569423, 13.86979 45.570579, 13.870261 45.571933, 13.870798 45.573005, 13.87185 45.57468, 13.872321 45.575473, 13.873673 45.577148, 13.87436 45.57796, 13.87584 45.579593, 13.876656 45.580387, 13.877471 45.58105, 13.87805 45.581504, 13.878586 45.581867, 13.879317 45.582317, 13.880217 45.582748, 13.881355 45.583198, 13.883049 45.583713, 13.884445 45.584075, 13.885131 45.58427, 13.886891 45.584655, 13.88865 45.584999, 13.89071 45.585323, 13.892062 45.585472, 13.89305 45.585536, 13.893565 45.585514, 13.894187 45.585514, 13.894938 45.585407, 13.897341 45.58485, 13.900088 45.584186, 13.901503 45.58382, 13.903242 45.583328, 13.906246 45.582275, 13.908455 45.581459, 13.909829 45.581031, 13.911009 45.58073, 13.911932 45.5806, 13.912791 45.580516, 13.913992 45.580451, 13.915945 45.580516, 13.917382 45.580642, 13.918498 45.580795, 13.919936 45.581096, 13.921587 45.581504, 13.92236 45.581737, 13.922747 45.581848, 13.923262 45.582038, 13.923647 45.582191, 13.924356 45.582511, 13.925686 45.583282, 13.926953 45.58414, 13.927489 45.584613, 13.92824 45.585342, 13.928326 45.585449, 13.929077 45.586502, 13.929635 45.587574, 13.929785 45.58794, 13.929999 45.588775, 13.930107 45.58955, 13.930107 45.590965, 13.929827 45.592681, 13.929741 45.593627, 13.92972 45.59457, 13.929871 45.596458, 13.929892 45.597446, 13.929849 45.597938, 13.929677 45.598884, 13.929463 45.599571, 13.929312 45.599956, 13.928498 45.601779, 13.928218 45.60268, 13.928133 45.602916, 13.927939 45.603862, 13.927939 45.604396, 13.928154 45.605899, 13.928304 45.606502, 13.928883 45.607681, 13.929656 45.608882, 13.929741 45.60897, 13.931094 45.610214, 13.932424 45.611114, 13.933969 45.61193, 13.934827 45.612274, 13.935428 45.612487, 13.935643 45.612552, 13.936544 45.612766, 13.939033 45.613304, 13.940127 45.613559, 13.942873 45.614269, 13.944011 45.614677, 13.944912 45.614997, 13.945534 45.615257, 13.947402 45.616157, 13.947788 45.616371, 13.949139 45.617187, 13.951071 45.618537, 13.952938 45.620082, 13.953753 45.620899, 13.954611 45.621799, 13.955598 45.622959, 13.955748 45.623172, 13.956414 45.624076, 13.956736 45.624546, 13.957315 45.625469, 13.958087 45.626823, 13.959439 45.629505, 13.959847 45.630489, 13.959976 45.630748, 13.960619 45.632358, 13.961241 45.634288, 13.961543 45.635662, 13.961692 45.636779, 13.961778 45.638301, 13.961714 45.638946, 13.961692 45.639587, 13.961521 45.640575, 13.960534 45.644287, 13.960211 45.645896, 13.960125 45.64669, 13.960083 45.647506, 13.960083 45.648151, 13.960211 45.649482, 13.960383 45.650466, 13.960748 45.651821, 13.96092 45.652336, 13.961521 45.654071, 13.961735 45.654544, 13.962336 45.656047, 13.963237 45.658042, 13.964396 45.660488, 13.966113 45.663749, 13.966778 45.664909, 13.967958 45.667034, 13.968087 45.667224, 13.968152 45.667377, 13.968279 45.667568, 13.973044 45.675895, 13.973343 45.676368, 13.974696 45.678772, 13.97491 45.679115, 13.976176 45.681346, 13.976756 45.68259, 13.977034 45.683277, 13.977485 45.68478, 13.977593 45.685249, 13.977807 45.687011, 13.977871 45.688278, 13.978043 45.689971, 13.978322 45.691387, 13.978601 45.692161, 13.978795 45.692695, 13.978966 45.693084, 13.979395 45.693897, 13.97976 45.694477, 13.980038 45.694927, 13.980403 45.6954, 13.981648 45.69675, 13.983172 45.698017, 13.983557 45.698295, 13.984374 45.698833, 13.985102 45.699218, 13.985725 45.699497, 13.986262 45.699714, 13.986927 45.699951, 13.987763 45.700164, 13.988643 45.700355, 13.991175 45.700805, 13.992871 45.701149, 13.993664 45.701366, 13.994286 45.701557, 13.994952 45.701816, 13.996047 45.702289, 13.996518 45.702522, 13.997118 45.702888, 13.997999 45.703468, 13.998342 45.703704, 13.999007 45.704238, 13.999394 45.704605, 14.000016 45.705268, 14.000531 45.705913, 14.000809 45.706321, 14.001282 45.707138, 14.001475 45.707546, 14.001753 45.708274, 14.001904 45.708854, 14.002033 45.709541, 14.002076 45.709991, 14.002076 45.711151, 14.002011 45.712501, 14.002011 45.713123, 14.002055 45.713489, 14.002247 45.714561, 14.002697 45.715744, 14.002999 45.716365, 14.00332 45.716899, 14.003921 45.717716, 14.004115 45.71791, 14.004436 45.718254, 14.005251 45.719024, 14.005552 45.719284, 14.006238 45.719776, 14.006903 45.720207, 14.009157 45.72158, 14.009608 45.721859, 14.010724 45.722522, 14.010787 45.722545, 14.01083 45.722587, 14.011517 45.723014, 14.012375 45.723575, 14.013834 45.72467, 14.014392 45.725139, 14.015723 45.726448, 14.016624 45.727478, 14.01671 45.727607, 14.01686 45.72776, 14.017525 45.728702, 14.017997 45.729454, 14.019263 45.731597, 14.019692 45.732242, 14.020207 45.73291, 14.020658 45.733444, 14.021345 45.734153, 14.021967 45.73471, 14.022718 45.735313, 14.024241 45.736362, 14.02495 45.73677, 14.02628 45.737457, 14.033469 45.740589, 14.035014 45.741317, 14.036343 45.742027, 14.037589 45.742736, 14.038253 45.743145, 14.039326 45.743915, 14.039649 45.744174, 14.040399 45.74488, 14.041086 45.745719, 14.041365 45.746124, 14.041687 45.746643, 14.041944 45.747177, 14.042974 45.749496, 14.043661 45.750526, 14.044047 45.750953, 14.044541 45.751468, 14.045098 45.75196, 14.045721 45.752456, 14.046321 45.752864, 14.047437 45.753486, 14.048081 45.753788, 14.048317 45.753913, 14.049818 45.754581, 14.050527 45.754943, 14.0513 45.755394, 14.052094 45.755954, 14.053359 45.756832, 14.054518 45.757564, 14.05514 45.757885, 14.055871 45.758228, 14.056235 45.758377, 14.057136 45.758701, 14.057801 45.758892, 14.058402 45.759044, 14.059668 45.759281, 14.060398 45.759388, 14.061213 45.759471, 14.061985 45.759559, 14.062973 45.759624, 14.064775 45.759708, 14.067693 45.75975, 14.070418 45.759666, 14.073508 45.759452, 14.07383 45.759407, 14.075246 45.759281, 14.077027 45.759151, 14.079538 45.759002, 14.081318 45.758956, 14.083336 45.758956, 14.086039 45.759044, 14.087434 45.759128, 14.088442 45.759216, 14.090653 45.759429, 14.091061 45.759494, 14.092648 45.759689, 14.094666 45.760009, 14.09548 45.760158, 14.098528 45.76078, 14.100159 45.761146, 14.101983 45.761596, 14.102154 45.761619, 14.104922 45.762306, 14.106123 45.762584, 14.108592 45.763122, 14.110694 45.763549, 14.111724 45.763721, 14.112025 45.763786, 14.116059 45.764366, 14.118076 45.764579, 14.1202 45.764751, 14.120651 45.764793, 14.120844 45.764793, 14.122904 45.764946, 14.124707 45.765007, 14.128848 45.765007, 14.129105 45.764987, 14.129663 45.764987, 14.130393 45.764946, 14.131423 45.764923, 14.133482 45.764793, 14.133805 45.764774, 14.13625 45.764556, 14.138117 45.764343, 14.138246 45.764343, 14.14095 45.763977, 14.14316 45.763656, 14.146743 45.763057, 14.148911 45.762714, 14.149747 45.762542, 14.150477 45.762413, 14.151464 45.76226, 14.153845 45.761856, 14.155798 45.761554, 14.156571 45.761425, 14.157879 45.761211, 14.158974 45.761062, 14.159275 45.761039, 14.159446 45.760997, 14.159575 45.760997, 14.163652 45.760501, 14.165626 45.760288, 14.165798 45.760288, 14.167664 45.760093, 14.17084 45.759838, 14.172085 45.759773, 14.174724 45.759601, 14.175604 45.759578, 14.175818 45.759559, 14.176741 45.759536, 14.178114 45.759471, 14.181139 45.759407, 14.183779 45.759388, 14.184445 45.759407, 14.185367 45.759407, 14.186354 45.759407, 14.189616 45.759536, 14.193114 45.759796, 14.194851 45.759967, 14.195924 45.760093, 14.196932 45.760246, 14.198134 45.760395, 14.19998 45.760696, 14.20101 45.760868, 14.202576 45.761188, 14.203606 45.761383, 14.204765 45.761638, 14.205237 45.761768, 14.206803 45.762134, 14.20764 45.762348, 14.208842 45.762691, 14.209013 45.762756, 14.209614 45.762905, 14.210795 45.763271, 14.213605 45.764194, 14.214721 45.764644, 14.2153 45.764858, 14.215923 45.765136, 14.216546 45.765438, 14.217146 45.765758, 14.218218 45.76638, 14.218841 45.766811, 14.219979 45.767692, 14.220408 45.768142, 14.220644 45.768379, 14.221073 45.768871, 14.221716 45.769859, 14.222081 45.770523, 14.222274 45.771038, 14.222532 45.772026, 14.222618 45.773033, 14.222575 45.774108, 14.222424 45.77518, 14.222231 45.77621, 14.221995 45.777668, 14.221931 45.778205, 14.221888 45.779426, 14.221953 45.780502, 14.222081 45.781318, 14.222081 45.781467, 14.222167 45.781852, 14.222382 45.782775, 14.22264 45.783592, 14.222983 45.78445, 14.223282 45.785095, 14.22412 45.786575, 14.225021 45.787841, 14.225815 45.788784, 14.226394 45.789382, 14.226674 45.789707, 14.227252 45.790245, 14.228068 45.79095, 14.228454 45.791252, 14.229227 45.791786, 14.229613 45.792087, 14.230021 45.792346, 14.230686 45.792774, 14.232081 45.793567, 14.232552 45.793785, 14.233668 45.794342, 14.234934 45.794876, 14.235235 45.794986, 14.235556 45.795112, 14.236973 45.795608, 14.238282 45.79597, 14.239526 45.796295, 14.240191 45.796424, 14.240942 45.796592, 14.242209 45.796829, 14.243002 45.796958, 14.245363 45.797302, 14.251908 45.798202, 14.253666 45.798439, 14.254569 45.798591, 14.255211 45.798675, 14.256499 45.798912, 14.257079 45.799041, 14.258881 45.799533, 14.260705 45.800178, 14.26195 45.80067, 14.262615 45.800949, 14.264117 45.801635, 14.267056 45.802902, 14.267657 45.803115, 14.26813 45.80331, 14.269181 45.803676, 14.269503 45.80376, 14.27019 45.803997, 14.270812 45.804191, 14.271927 45.804489, 14.272164 45.804576, 14.272721 45.804706, 14.275038 45.805393, 14.276691 45.805908, 14.277335 45.806121, 14.278214 45.806442, 14.27888 45.806743, 14.279566 45.807022, 14.280124 45.807281, 14.280725 45.807579, 14.282034 45.808288, 14.282162 45.808376, 14.28227 45.808418, 14.28302 45.808891, 14.283579 45.809276, 14.284029 45.809619, 14.284544 45.81007, 14.285038 45.810585, 14.28521 45.810798, 14.285489 45.811187, 14.285789 45.811679, 14.285961 45.812023, 14.286175 45.812622, 14.286261 45.813072, 14.286347 45.813739, 14.286347 45.814231, 14.28624 45.814941, 14.28611 45.81554, 14.285746 45.816806, 14.28566 45.817237, 14.285553 45.8176, 14.285424 45.818717, 14.285403 45.819168, 14.285489 45.820262, 14.285617 45.820949, 14.285832 45.821655, 14.286068 45.82228, 14.28654 45.823246, 14.287055 45.824058, 14.287398 45.824317, 14.287549 45.824401, 14.28772 45.824447, 14.288128 45.824424, 14.288493 45.824234, 14.288558 45.824058, 14.288558 45.823932, 14.288514 45.82378, 14.287978 45.823329, 14.287184 45.822795, 14.287591 45.822429, 14.289008 45.821033, 14.289566 45.820583, 14.290144 45.820156, 14.290424 45.819919, 14.291089 45.819274, 14.291325 45.819084, 14.292805 45.818309, 14.293063 45.818222, 14.293364 45.818161, 14.294672 45.817943, 14.295874 45.817688, 14.297247 45.817344, 14.297762 45.817192, 14.298556 45.816978, 14.298921 45.816848, 14.300638 45.816337, 14.302033 45.815906, 14.302762 45.815628, 14.303664 45.815326, 14.305123 45.814918, 14.306839 45.814384, 14.307654 45.814189, 14.308491 45.814125, 14.308877 45.814147, 14.309607 45.814231, 14.309865 45.814212, 14.310294 45.814102, 14.310723 45.813953, 14.311324 45.813804, 14.31156 45.813758, 14.312247 45.813591, 14.312676 45.813354, 14.313341 45.813201, 14.313834 45.813159, 14.314199 45.813095, 14.315014 45.813072, 14.315293 45.813137, 14.315958 45.813182, 14.317117 45.813159, 14.317203 45.813159, 14.317976 45.813137, 14.318984 45.813011, 14.319285 45.812965, 14.320099 45.812751, 14.32055 45.812622, 14.32098 45.812515, 14.321644 45.812278, 14.322696 45.811698, 14.323061 45.811466, 14.323469 45.811141, 14.323576 45.811012, 14.323683 45.811058, 14.324133 45.811313, 14.324413 45.811058, 14.325357 45.810779, 14.325786 45.810714, 14.326107 45.810627, 14.326794 45.810562, 14.327137 45.810501, 14.327674 45.81052, 14.328576 45.810733, 14.329327 45.810886, 14.329756 45.810905, 14.3301 45.810886, 14.330764 45.810756, 14.330958 45.810691, 14.331108 45.810668, 14.331537 45.810478, 14.33188 45.810264, 14.332138 45.810134, 14.332287 45.810112, 14.332524 45.809898, 14.332631 45.809768, 14.332738 45.809577, 14.332974 45.809318, 14.333317 45.809169, 14.33379 45.809062, 14.334326 45.808891, 14.334755 45.808826, 14.335185 45.808761, 14.335764 45.808719, 14.336022 45.808673, 14.336321 45.808609, 14.336772 45.808483, 14.337202 45.808265, 14.337545 45.80801, 14.337802 45.80788, 14.338017 45.807731, 14.338295 45.807601, 14.338553 45.807453, 14.339691 45.807045, 14.339926 45.806938, 14.340313 45.806831, 14.3413 45.806358, 14.342137 45.805908, 14.343038 45.80537, 14.344262 45.804771, 14.34527 45.804382, 14.345591 45.804298, 14.346278 45.804039, 14.3466 45.80389, 14.346986 45.803741, 14.347201 45.803611, 14.347781 45.803333, 14.349089 45.802989, 14.349282 45.80286, 14.349539 45.802753, 14.349819 45.802581, 14.350033 45.80241, 14.35027 45.80228, 14.35072 45.801872, 14.350956 45.8017, 14.351149 45.801528, 14.351643 45.801292, 14.351986 45.801166, 14.352543 45.800907, 14.35278 45.800735, 14.352909 45.800605, 14.353188 45.800201, 14.353295 45.800006, 14.353638 45.799728, 14.354432 45.79932, 14.354861 45.79906, 14.35587 45.798954, 14.356342 45.79887, 14.356749 45.798828, 14.357501 45.798805, 14.358144 45.798889, 14.358702 45.798996, 14.359196 45.798996, 14.359646 45.798934, 14.359883 45.79887, 14.360011 45.798763, 14.360398 45.798419, 14.360633 45.798267, 14.361234 45.798053, 14.361663 45.798053, 14.362736 45.797946, 14.362779 45.798183, 14.362822 45.79829, 14.363122 45.798828, 14.363337 45.799083, 14.363637 45.799362, 14.363852 45.799514, 14.363873 45.799556, 14.365483 45.800521, 14.365933 45.800823, 14.366234 45.800972, 14.36662 45.801208, 14.367178 45.801464, 14.368294 45.80183, 14.370933 45.802925, 14.371169 45.803115, 14.371233 45.803226, 14.371191 45.803783, 14.371127 45.804126, 14.371127 45.804534, 14.371212 45.804878, 14.371384 45.805221, 14.371663 45.8055, 14.372284 45.80595, 14.372607 45.806442, 14.37265 45.807128, 14.37265 45.807754, 14.372521 45.80801, 14.372199 45.808353, 14.371941 45.808589, 14.37162 45.809127, 14.371598 45.809471, 14.371684 45.809875, 14.371813 45.810134, 14.372157 45.810627, 14.3725 45.810821, 14.372971 45.811035, 14.373873 45.811485, 14.374131 45.811679, 14.37443 45.812152, 14.374646 45.81245, 14.375075 45.812858, 14.375247 45.812988, 14.375418 45.813159, 14.375868 45.813888, 14.376061 45.814083, 14.376276 45.814212, 14.376941 45.814491, 14.37765 45.815025, 14.3781 45.815197, 14.378422 45.815261, 14.379001 45.815261, 14.37958 45.815391, 14.380267 45.815628, 14.381018 45.816101, 14.382649 45.817024, 14.382756 45.817558, 14.382756 45.818031, 14.382735 45.81874, 14.382756 45.819126, 14.382649 45.820411, 14.382971 45.820583, 14.383143 45.820583, 14.3837 45.820518, 14.384966 45.820156, 14.386125 45.819683, 14.386704 45.819469, 14.386961 45.819488, 14.387434 45.819553, 14.387842 45.819488, 14.389301 45.819191, 14.389752 45.819061, 14.390159 45.818908, 14.390267 45.818847, 14.390588 45.818565, 14.390868 45.818458, 14.391253 45.818416, 14.392047 45.818653, 14.392283 45.818782, 14.39297 45.819297, 14.393227 45.819427, 14.393528 45.819446, 14.393764 45.819362, 14.393893 45.819274, 14.394085 45.818866, 14.394257 45.818759, 14.394515 45.81874, 14.394708 45.818824, 14.394772 45.819274, 14.39488 45.819553, 14.395159 45.819747, 14.395566 45.819896, 14.395888 45.819938, 14.396253 45.819854, 14.396447 45.819877, 14.396639 45.820003, 14.397048 45.820564, 14.397219 45.820842, 14.397326 45.821121, 14.397906 45.821571, 14.398507 45.822086, 14.39872 45.822322, 14.39872 45.822452, 14.398613 45.822624, 14.398571 45.82275, 14.398656 45.822879, 14.398828 45.822902, 14.398914 45.822921, 14.398978 45.822902, 14.399107 45.822814, 14.400223 45.821807, 14.400502 45.821594, 14.400652 45.821548, 14.400952 45.821506, 14.401274 45.821872, 14.401532 45.822044, 14.402112 45.822128, 14.402476 45.822258, 14.403399 45.822773, 14.404192 45.823287, 14.404213 45.823436, 14.403828 45.823909, 14.403828 45.824039, 14.403935 45.824104, 14.404107 45.824146, 14.404557 45.824123, 14.404665 45.824211, 14.404601 45.824382, 14.404407 45.824619, 14.404192 45.82481, 14.403849 45.82492, 14.403656 45.825027, 14.403656 45.825153, 14.403828 45.825305, 14.403935 45.825454, 14.403914 45.825756, 14.403784 45.826183, 14.403806 45.826248, 14.403956 45.8264, 14.404364 45.826507, 14.404515 45.826614, 14.404707 45.827087, 14.404858 45.827556, 14.404793 45.827793, 14.404622 45.827922, 14.404579 45.828029, 14.404622 45.82846, 14.404471 45.828609, 14.404235 45.828929, 14.404086 45.829444, 14.404128 45.829574, 14.404192 45.829875, 14.403806 45.830326, 14.40372 45.830474, 14.403763 45.830646, 14.403849 45.830757, 14.404042 45.830776, 14.404343 45.830669, 14.404557 45.830669, 14.4049 45.830734, 14.405501 45.830925, 14.405866 45.831012, 14.406273 45.831035, 14.406788 45.831035, 14.407068 45.831077, 14.407046 45.831207, 14.406874 45.831314, 14.406639 45.831378, 14.406531 45.831504, 14.406489 45.831611, 14.406381 45.831764, 14.406102 45.831913, 14.405802 45.832, 14.405415 45.832019, 14.405265 45.832084, 14.405265 45.832214, 14.405566 45.832408, 14.405545 45.832515, 14.405222 45.832859, 14.404536 45.833286, 14.404385 45.83348, 14.404128 45.833736, 14.404042 45.833908, 14.404 45.833972, 14.404064 45.83406, 14.404235 45.834102, 14.404364 45.834102, 14.404515 45.834037, 14.404707 45.833889, 14.404707 45.833866, 14.404944 45.833652, 14.405631 45.833438, 14.405823 45.833351, 14.406124 45.83316, 14.406317 45.833179, 14.406424 45.833328, 14.406317 45.833503, 14.40606 45.833694, 14.406016 45.833866, 14.406146 45.833953, 14.406273 45.833972, 14.406724 45.833931, 14.406982 45.833972, 14.407303 45.834232, 14.40754 45.834297, 14.407862 45.834232, 14.408247 45.83419, 14.408484 45.834274, 14.408549 45.834423, 14.408484 45.83451, 14.408034 45.834789, 14.407904 45.835002, 14.407818 45.83522, 14.407818 45.835433, 14.407926 45.835582, 14.408226 45.83554, 14.408505 45.835563, 14.408699 45.83567, 14.40902 45.835796, 14.409128 45.835968, 14.409106 45.836162, 14.408827 45.836418, 14.408269 45.836761, 14.407948 45.83702, 14.407948 45.837127, 14.408076 45.837257, 14.408333 45.837406, 14.408527 45.837558, 14.40857 45.838714, 14.408635 45.838951, 14.408827 45.839122, 14.409128 45.839164, 14.409363 45.839229, 14.409385 45.839424, 14.409449 45.839618, 14.409665 45.839683, 14.409836 45.839637, 14.410008 45.839553, 14.410201 45.839424, 14.410479 45.839382, 14.41063 45.839466, 14.410866 45.839809, 14.41121 45.839916, 14.411896 45.839637, 14.41224 45.83953, 14.412583 45.839466, 14.413054 45.839424, 14.413934 45.83953, 14.41417 45.839466, 14.414492 45.839447, 14.414835 45.839572, 14.415329 45.839962, 14.415908 45.840477, 14.416102 45.840732, 14.41623 45.840946, 14.416401 45.84114, 14.416789 45.841419, 14.417218 45.841613, 14.417453 45.841743, 14.417539 45.841869, 14.417775 45.842857, 14.417946 45.843242, 14.418032 45.843521, 14.418011 45.843845, 14.418097 45.844337, 14.418076 45.844596, 14.41799 45.844875, 14.417775 45.84513, 14.417453 45.845455, 14.417304 45.845645, 14.41726 45.845817, 14.417218 45.846054, 14.417389 45.846504, 14.417453 45.846782, 14.417475 45.847148, 14.417431 45.847427, 14.418076 45.847747, 14.418097 45.847835, 14.417819 45.848327, 14.417668 45.848545, 14.417561 45.848758, 14.417539 45.848995, 14.417603 45.849296, 14.417775 45.849487, 14.417797 45.849594, 14.417754 45.849872, 14.417797 45.84996, 14.417968 45.850044, 14.418076 45.850067, 14.418526 45.850002, 14.419041 45.84983, 14.419342 45.849872, 14.419556 45.850345, 14.419793 45.85099, 14.419771 45.851139, 14.419707 45.851333, 14.419535 45.851524, 14.418698 45.852233, 14.418226 45.852493, 14.418076 45.852619, 14.41741 45.853714, 14.41726 45.853908, 14.417067 45.85408, 14.41681 45.854209, 14.416252 45.8544, 14.41608 45.854507, 14.416016 45.85453, 14.415865 45.854724, 14.41578 45.854957, 14.415759 45.855602, 14.415844 45.855773, 14.415844 45.856567, 14.415715 45.856784, 14.415565 45.856891, 14.415329 45.857017, 14.415179 45.857212, 14.415072 45.857597, 14.414942 45.857769, 14.41432 45.858371, 14.414214 45.858627, 14.414278 45.858757, 14.414385 45.858821, 14.415028 45.858844, 14.415543 45.859058, 14.415694 45.859252, 14.415651 45.859466, 14.415543 45.85968, 14.415543 45.859874, 14.415501 45.860107, 14.415308 45.860282, 14.415028 45.86045, 14.414771 45.860645, 14.414535 45.860858, 14.414256 45.861053, 14.414084 45.861095, 14.413891 45.861076, 14.41372 45.86116, 14.413677 45.861248, 14.413741 45.861312, 14.413891 45.861354, 14.414214 45.861354, 14.414471 45.861373, 14.414664 45.861461, 14.414942 45.86148, 14.415457 45.861312, 14.415801 45.86129, 14.416058 45.861331, 14.416166 45.861419, 14.416209 45.861545, 14.416252 45.861804, 14.416252 45.862213, 14.416102 45.862621, 14.416037 45.863136, 14.416102 45.863349, 14.416552 45.863716, 14.416617 45.863822, 14.416487 45.864078, 14.416487 45.864208, 14.416573 45.864421, 14.416595 45.864593, 14.416487 45.864746, 14.41623 45.864978, 14.415822 45.865474, 14.415844 45.865646, 14.415908 45.86586, 14.415844 45.866031, 14.415865 45.866245, 14.416144 45.866588, 14.416401 45.867126, 14.416401 45.86734, 14.416209 45.867553, 14.416209 45.867683, 14.416595 45.867919, 14.416638 45.868305, 14.416703 45.868392, 14.417024 45.868629, 14.417088 45.868865, 14.41726 45.869098, 14.417475 45.869293, 14.417603 45.869487, 14.417754 45.869766, 14.417882 45.870109, 14.417925 45.870407, 14.418076 45.870624, 14.418312 45.870815, 14.418419 45.870945, 14.418461 45.871093, 14.418375 45.871246, 14.418334 45.871524, 14.418397 45.871654, 14.418526 45.871891, 14.418569 45.872146, 14.418461 45.872234, 14.418183 45.872383, 14.418054 45.872554, 14.418097 45.872749, 14.418204 45.872833, 14.418375 45.872856, 14.418526 45.872833, 14.418763 45.872749, 14.419062 45.872703, 14.419278 45.872703, 14.419599 45.872962, 14.419857 45.873218, 14.419899 45.873264, 14.419857 45.873436, 14.419599 45.873821, 14.419491 45.874057, 14.419513 45.874336, 14.419148 45.874656, 14.419213 45.874744, 14.419342 45.874763, 14.419663 45.874527, 14.41992 45.874294, 14.420178 45.874229, 14.420521 45.874294, 14.420715 45.874507, 14.420994 45.874721, 14.421338 45.874916, 14.421595 45.875171, 14.421895 45.875324, 14.422088 45.875556, 14.422131 45.875774, 14.421853 45.876075, 14.421874 45.876289, 14.42198 45.876419, 14.422152 45.876544, 14.422389 45.876632, 14.422603 45.876804, 14.422732 45.876995, 14.422861 45.877273, 14.422925 45.87751, 14.423054 45.877574, 14.423697 45.877639, 14.424706 45.877704, 14.424984 45.877853, 14.42507 45.877983, 14.425092 45.878135, 14.42507 45.878349, 14.425114 45.878456, 14.42535 45.878562, 14.425585 45.878604, 14.425757 45.878585, 14.425993 45.878479, 14.426337 45.878242, 14.426594 45.878196, 14.426917 45.878196, 14.427238 45.878284, 14.427861 45.878349, 14.428332 45.878498, 14.42874 45.878692, 14.428912 45.878841, 14.429104 45.879119, 14.429126 45.879333, 14.428933 45.879852, 14.428891 45.880043, 14.428955 45.880237, 14.429427 45.880687, 14.429384 45.880901, 14.42947 45.881137, 14.429598 45.881351, 14.429641 45.88161, 14.429212 45.882102, 14.429104 45.882255, 14.429104 45.882339, 14.429427 45.882534, 14.429856 45.88264, 14.430113 45.882896, 14.430306 45.883197, 14.430563 45.883327, 14.430563 45.883392, 14.430521 45.883518, 14.430306 45.883647, 14.429985 45.883754, 14.429899 45.883842, 14.429856 45.883926, 14.430156 45.884315, 14.430092 45.884677, 14.429963 45.884765, 14.429813 45.884807, 14.429255 45.884826, 14.429104 45.884891, 14.42904 45.885002, 14.429062 45.885108, 14.429555 45.885429, 14.429985 45.885814, 14.430907 45.886543, 14.431487 45.887039, 14.432173 45.887809, 14.432302 45.888069, 14.432345 45.888282, 14.432302 45.888584, 14.432044 45.889335, 14.432194 45.889526, 14.432753 45.889892, 14.433053 45.890041, 14.433332 45.890193, 14.433632 45.8903, 14.434126 45.890342, 14.435328 45.890277, 14.43625 45.890258, 14.436937 45.890277, 14.437538 45.890319, 14.437817 45.890365, 14.43816 45.890384, 14.438525 45.890449, 14.439404 45.89075, 14.439641 45.89088, 14.439534 45.891159, 14.438954 45.89178, 14.438504 45.892189, 14.43831 45.892318, 14.437709 45.892662, 14.436722 45.893585, 14.436422 45.893905, 14.436228 45.894248, 14.435993 45.894977, 14.435799 45.895214, 14.435542 45.895469, 14.43537 45.895729, 14.435349 45.895858, 14.435349 45.896221, 14.435435 45.896587, 14.435542 45.897232, 14.435478 45.897422, 14.435027 45.898239, 14.434556 45.898902, 14.434168 45.899528, 14.433997 45.899955, 14.433761 45.900428, 14.433246 45.901157, 14.433182 45.901565, 14.433053 45.901973, 14.432881 45.902317, 14.432452 45.902854, 14.431744 45.903625, 14.431208 45.904289, 14.431036 45.904525, 14.4308 45.905147, 14.430779 45.9053, 14.4308 45.905471, 14.430951 45.906436, 14.430993 45.906566, 14.430821 45.907188, 14.4308 45.907508, 14.430886 45.907917, 14.4311 45.908283, 14.43138 45.908454, 14.431723 45.90858, 14.432173 45.908691, 14.432731 45.908798, 14.433761 45.908882, 14.434298 45.908969, 14.434727 45.909118, 14.435821 45.909332, 14.436293 45.909378, 14.436551 45.909439, 14.436765 45.909568, 14.43756 45.909397, 14.439576 45.909484, 14.440091 45.909568, 14.440434 45.909656, 14.440949 45.909935, 14.441206 45.910125, 14.441314 45.91032, 14.441357 45.910491, 14.441422 45.911048, 14.441679 45.911563, 14.441765 45.911972, 14.441786 45.912403, 14.441893 45.912681, 14.442065 45.912872, 14.442236 45.913154, 14.442344 45.913433, 14.442494 45.913665, 14.442816 45.913967, 14.443074 45.914268, 14.443117 45.914417, 14.443074 45.914634, 14.442923 45.914718, 14.442837 45.914913, 14.442881 45.915103, 14.442859 45.91534, 14.442645 45.915664, 14.442516 45.916069, 14.442516 45.916244, 14.442602 45.916458, 14.442773 45.916671, 14.442967 45.916992, 14.443224 45.917316, 14.443482 45.91753, 14.443717 45.917678, 14.444018 45.917743, 14.444598 45.917743, 14.444855 45.917808, 14.44552 45.918128, 14.446249 45.918365, 14.446679 45.91843, 14.447065 45.91843, 14.44773 45.918472, 14.448117 45.918838, 14.449704 45.919117, 14.450047 45.919158, 14.450241 45.919246, 14.45039 45.919738, 14.450498 45.919975, 14.450756 45.920318, 14.451142 45.921241, 14.451356 45.921672, 14.451528 45.921863, 14.451743 45.921993, 14.451828 45.922206, 14.4517 45.922679, 14.4517 45.923, 14.451914 45.923622, 14.452129 45.923881, 14.452301 45.92403, 14.452752 45.924224, 14.453931 45.924591, 14.454532 45.924739, 14.454918 45.92517, 14.455004 45.925361, 14.455069 45.926025, 14.454961 45.926479, 14.45464 45.92712, 14.453974 45.928173, 14.453824 45.928344, 14.45348 45.928604, 14.452987 45.928901, 14.452837 45.929138, 14.452816 45.929397, 14.452837 45.929588, 14.452923 45.929889, 14.453116 45.930103, 14.453288 45.930339, 14.453288 45.930534, 14.453202 45.930789, 14.453137 45.931285, 14.453309 45.932079, 14.453502 45.932594, 14.454039 45.933578, 14.454318 45.934265, 14.454489 45.934825, 14.454424 45.935295, 14.454596 45.935684, 14.454682 45.935962, 14.454747 45.936584, 14.454918 45.936927, 14.455111 45.937271, 14.455369 45.937549, 14.455584 45.937744, 14.456978 45.93856, 14.457386 45.938774, 14.457665 45.93901, 14.457943 45.939373, 14.458373 45.939868, 14.458716 45.940082, 14.458952 45.940212, 14.459231 45.940444, 14.459339 45.940746, 14.459618 45.941154, 14.459961 45.941455, 14.460111 45.941669, 14.460197 45.941841, 14.460326 45.941993, 14.460798 45.942249, 14.461077 45.942462, 14.461163 45.942699, 14.461077 45.943214, 14.460884 45.943515, 14.460798 45.943836, 14.460841 45.944072, 14.460991 45.944351, 14.461313 45.94448, 14.461484 45.944629, 14.461656 45.944931, 14.461999 45.945316, 14.462193 45.945488, 14.46348 45.946067, 14.464339 45.946689, 14.464574 45.946842, 14.464918 45.947013, 14.465754 45.947204, 14.466033 45.947399, 14.466205 45.947719, 14.466313 45.948085, 14.466355 45.94873, 14.466399 45.948967, 14.466506 45.94905, 14.466698 45.949115, 14.467171 45.949092, 14.467728 45.949203, 14.468115 45.949245, 14.468672 45.949436, 14.469059 45.949607, 14.469273 45.949802, 14.469832 45.949996, 14.470175 45.950016, 14.470411 45.94989, 14.470625 45.949737, 14.47084 45.949501, 14.471161 45.948986, 14.471183 45.948814, 14.471355 45.9486, 14.471441 45.948516, 14.47172 45.948471, 14.471913 45.948535, 14.472836 45.949028, 14.473243 45.949157, 14.473286 45.94918, 14.47365 45.94931, 14.474251 45.949436, 14.474745 45.949459, 14.475153 45.949607, 14.475346 45.949779, 14.475281 45.950058, 14.475195 45.950294, 14.475132 45.950531, 14.475068 45.951068, 14.475046 45.951217, 14.474724 45.952182, 14.474553 45.952526, 14.474402 45.952655, 14.473822 45.953323, 14.473694 45.953601, 14.473501 45.953857, 14.473286 45.954135, 14.473157 45.954395, 14.473135 45.954631, 14.473221 45.954822, 14.473286 45.955059, 14.473221 45.955188, 14.473072 45.955318, 14.472771 45.955444, 14.47187 45.95549, 14.471269 45.955551, 14.470175 45.955894, 14.469832 45.955917, 14.469489 45.955768, 14.469166 45.955596, 14.468888 45.955272, 14.468716 45.955123, 14.468437 45.954975, 14.468179 45.95491, 14.467965 45.95491, 14.467793 45.954994, 14.467686 45.955165, 14.466978 45.955852, 14.466956 45.956024, 14.467041 45.956131, 14.467235 45.956283, 14.467343 45.956474, 14.467299 45.956645, 14.466892 45.956817, 14.466485 45.956882, 14.465884 45.956924, 14.465411 45.957035, 14.465218 45.957183, 14.465067 45.957355, 14.464896 45.957462, 14.464724 45.957611, 14.464531 45.957721, 14.464209 45.957847, 14.464037 45.957977, 14.463845 45.958385, 14.463824 45.958641, 14.463738 45.959007, 14.463608 45.959308, 14.46348 45.959522, 14.462944 45.959972, 14.462664 45.960144, 14.461936 45.960422, 14.461399 45.96083, 14.461119 45.960918, 14.460841 45.961067, 14.460669 45.961261, 14.46054 45.961517, 14.460454 45.961841, 14.460604 45.962184, 14.460604 45.96257, 14.460626 45.962718, 14.461055 45.963577, 14.461163 45.964115, 14.461506 45.964435, 14.461634 45.96463, 14.461634 45.964759, 14.461527 45.964908, 14.461291 45.965122, 14.460712 45.965744, 14.460583 45.96585, 14.459639 45.96643, 14.45921 45.966648, 14.458888 45.966819, 14.458588 45.96701, 14.458266 45.96746, 14.458094 45.967933, 14.457837 45.968791, 14.457708 45.969051, 14.457622 45.969223, 14.457536 45.969501, 14.457514 45.969779, 14.4576 45.970035, 14.457751 45.970336, 14.457644 45.970615, 14.457407 45.970851, 14.456978 45.971218, 14.456828 45.971454, 14.456742 45.971668, 14.456678 45.971881, 14.456614 45.972675, 14.4567 45.972892, 14.456913 45.97306, 14.457193 45.973278, 14.457579 45.973468, 14.457965 45.973533, 14.458245 45.973514, 14.45876 45.973426, 14.459639 45.973236, 14.46024 45.97319, 14.460305 45.973171, 14.462686 45.973018, 14.46333 45.973018, 14.46363 45.972976, 14.463824 45.972934, 14.464209 45.972782, 14.464789 45.972461, 14.465153 45.972267, 14.466141 45.971645, 14.466441 45.971408, 14.466698 45.971237, 14.467385 45.970703, 14.467728 45.970359, 14.468866 45.969051, 14.470282 45.967632, 14.470862 45.967205, 14.47157 45.966476, 14.47172 45.966495, 14.471848 45.966453, 14.472149 45.966518, 14.472814 45.966838, 14.473114 45.966861, 14.473307 45.966838, 14.473501 45.966838, 14.473908 45.966945, 14.47481 45.967269, 14.475668 45.967697, 14.476247 45.967868, 14.476934 45.96804, 14.477471 45.968147, 14.478221 45.968212, 14.478994 45.968341, 14.479423 45.968406, 14.480774 45.96849, 14.481547 45.968578, 14.482642 45.968833, 14.483006 45.968879, 14.483435 45.968856, 14.484487 45.968727, 14.485946 45.96862, 14.486418 45.96849, 14.487062 45.968193, 14.487341 45.968147, 14.487771 45.96817, 14.488478 45.968254, 14.489165 45.968383, 14.489316 45.968429, 14.48998 45.968277, 14.490431 45.968147, 14.491311 45.967762, 14.491697 45.967548, 14.492362 45.967224, 14.492555 45.96701, 14.49277 45.966903, 14.49307 45.966796, 14.493477 45.966774, 14.4941 45.966861, 14.494421 45.966945, 14.494936 45.967033, 14.495194 45.966991, 14.496224 45.966537, 14.496503 45.966369, 14.498048 45.966476, 14.499228 45.966537, 14.499293 45.966518, 14.499979 45.966304, 14.500388 45.966087, 14.500645 45.965915, 14.501095 45.965766, 14.501524 45.965683, 14.501675 45.96566, 14.502575 45.965274, 14.506695 45.96392, 14.507983 45.9636, 14.50824 45.963493, 14.508498 45.963405, 14.508906 45.963214, 14.510258 45.962741, 14.51043 45.962699, 14.511395 45.962291, 14.512103 45.96212, 14.512833 45.96199, 14.513369 45.961925, 14.514056 45.961753, 14.515364 45.961368, 14.515944 45.961174, 14.517274 45.960575, 14.517596 45.960445, 14.51764 45.960468, 14.518197 45.96051, 14.518841 45.96096, 14.519185 45.961109, 14.519614 45.961174, 14.521073 45.96109, 14.521566 45.961044, 14.52251 45.960895, 14.522768 45.960876, 14.523368 45.960746, 14.524055 45.96051, 14.52457 45.96038, 14.524935 45.96025, 14.525193 45.960102, 14.52545 45.959781, 14.525686 45.95961, 14.525965 45.959457, 14.527209 45.958751, 14.527274 45.958473, 14.527445 45.957935, 14.527682 45.957378, 14.52781 45.957248, 14.528132 45.957248, 14.528325 45.957267, 14.528475 45.957225, 14.528582 45.957141, 14.528798 45.957035, 14.529183 45.956947, 14.529419 45.956924, 14.529634 45.956863, 14.530321 45.956581, 14.531179 45.956005, 14.531458 45.955875, 14.531973 45.95568, 14.533625 45.95504, 14.534054 45.954757, 14.534398 45.954586, 14.534548 45.954544, 14.53517 45.954372, 14.5359 45.954223, 14.536286 45.954158, 14.5365 45.954093, 14.537144 45.953964, 14.538002 45.95375, 14.538303 45.95375, 14.53856 45.953987, 14.538689 45.954288, 14.53884 45.95491, 14.538968 45.955253, 14.539053 45.955532, 14.539183 45.955787, 14.539441 45.956131, 14.53987 45.956668, 14.540449 45.957332, 14.541543 45.95845, 14.542766 45.959907, 14.543259 45.960639, 14.544032 45.961689, 14.545126 45.963214, 14.546328 45.965103, 14.547036 45.966133, 14.547207 45.966346, 14.547744 45.96669, 14.547959 45.966796, 14.550812 45.968513, 14.551864 45.969051, 14.553216 45.969757, 14.557593 45.971969, 14.56019 45.973255, 14.562142 45.974266, 14.564675 45.975528, 14.56667 45.976493, 14.568387 45.977375, 14.571434 45.978897, 14.57315 45.979736, 14.573729 45.979885, 14.574953 45.979972, 14.575296 45.980037, 14.575725 45.980808, 14.57594 45.9813, 14.575961 45.981796, 14.575897 45.982162, 14.575553 45.982868, 14.574931 45.983791, 14.574845 45.983898, 14.574889 45.984005, 14.574996 45.984069, 14.575146 45.984092, 14.575854 45.983879, 14.576669 45.983726, 14.57712 45.983661, 14.578 45.983619, 14.5786 45.983619, 14.579781 45.983661, 14.580789 45.983749, 14.581927 45.983814, 14.582613 45.983879, 14.583386 45.983898, 14.584673 45.983833, 14.585789 45.983703, 14.586861 45.983535, 14.58757 45.98336, 14.588793 45.982997, 14.589673 45.982631, 14.590123 45.982418, 14.590939 45.982009, 14.591861 45.981494, 14.592955 45.980915, 14.593149 45.980808, 14.593771 45.980529, 14.594951 45.980056, 14.596067 45.979759, 14.597054 45.979583, 14.597912 45.979476, 14.598706 45.979434, 14.600981 45.979434, 14.601195 45.979415, 14.601689 45.979415, 14.603191 45.979305, 14.603985 45.979198, 14.604221 45.979133, 14.604714 45.978984, 14.605057 45.978855, 14.605444 45.978664, 14.605959 45.97834, 14.606474 45.977912, 14.606839 45.977653, 14.607161 45.977481, 14.60744 45.977397, 14.607848 45.977355, 14.608061 45.977355, 14.60892 45.977375, 14.609499 45.977417, 14.610272 45.977417, 14.610636 45.977397, 14.611109 45.97731, 14.611495 45.977291, 14.612031 45.977161, 14.612483 45.977073, 14.614735 45.976539, 14.615614 45.976303, 14.619263 45.975166, 14.619864 45.975013, 14.620293 45.974887, 14.620786 45.974777, 14.621108 45.974693, 14.624262 45.974006, 14.625164 45.973747, 14.625979 45.973468, 14.62658 45.973236, 14.62716 45.972999, 14.627867 45.972656, 14.630571 45.971195, 14.631493 45.970787, 14.63173 45.970703, 14.632138 45.97055, 14.633339 45.970188, 14.633747 45.970081, 14.63482 45.969863, 14.635334 45.969779, 14.636385 45.96965, 14.637008 45.969608, 14.638081 45.969585, 14.640956 45.969627, 14.642244 45.969608, 14.642888 45.969566, 14.64366 45.969478, 14.646235 45.969116, 14.646815 45.969051, 14.647437 45.968986, 14.648381 45.96894, 14.652243 45.968986, 14.654454 45.968921, 14.655676 45.968879, 14.656234 45.968833, 14.657929 45.968833, 14.65956 45.96894, 14.660204 45.969005, 14.660483 45.969028, 14.660569 45.969051, 14.660891 45.969116, 14.661406 45.969177, 14.661877 45.969284, 14.6628 45.969459, 14.664131 45.969821, 14.664881 45.970058, 14.665869 45.970401, 14.668723 45.971626, 14.669688 45.971988, 14.669989 45.972076, 14.670525 45.972206, 14.671233 45.972312, 14.671855 45.972354, 14.672863 45.972396, 14.673722 45.972332, 14.674001 45.97229, 14.674902 45.972076, 14.675482 45.971881, 14.677027 45.971302, 14.677821 45.971065, 14.678401 45.970939, 14.679945 45.97068, 14.680331 45.970638, 14.681319 45.970443, 14.681597 45.970359, 14.682177 45.970165, 14.682606 45.96997, 14.682971 45.969779, 14.684021 45.969051, 14.68458 45.968685, 14.685288 45.968318, 14.685696 45.96817, 14.686554 45.96791, 14.687499 45.96772, 14.688185 45.967525, 14.688614 45.967376, 14.689193 45.96714, 14.690545 45.96656, 14.691296 45.966346, 14.691918 45.966239, 14.692519 45.966194, 14.69327 45.966194, 14.693764 45.966239, 14.694171 45.966304, 14.69533 45.96656, 14.696446 45.966774, 14.697347 45.966861, 14.698012 45.966838, 14.699086 45.966774, 14.700223 45.966732, 14.702497 45.966945, 14.703227 45.966991, 14.704257 45.966903, 14.70518 45.96669, 14.705544 45.96656, 14.706059 45.966346, 14.706767 45.965938, 14.707175 45.965618, 14.708076 45.96463, 14.708312 45.964328, 14.708677 45.96392, 14.708977 45.963642, 14.709707 45.963127, 14.710587 45.962677, 14.711058 45.962505, 14.711724 45.96231, 14.712904 45.962162, 14.713461 45.962139, 14.715844 45.962204, 14.716252 45.962184, 14.716939 45.962097, 14.719298 45.961689, 14.719835 45.961624, 14.721852 45.961563, 14.72271 45.961475, 14.723719 45.961303, 14.724384 45.961154, 14.724921 45.960983, 14.725436 45.960746, 14.726508 45.960166, 14.726959 45.959972, 14.727002 45.95993, 14.727152 45.959888, 14.727538 45.959716, 14.727925 45.959587, 14.728354 45.95948, 14.728847 45.959392, 14.729877 45.959308, 14.730392 45.959327, 14.731358 45.959457, 14.731787 45.959545, 14.733181 45.959671, 14.733975 45.959693, 14.735306 45.95961, 14.735799 45.959522, 14.736443 45.959522, 14.736701 45.959564, 14.737301 45.959735, 14.73758 45.959758, 14.738245 45.959651, 14.738846 45.95948, 14.739426 45.95935, 14.741807 45.958641, 14.742365 45.958515, 14.743224 45.958278, 14.744275 45.958042, 14.744683 45.957935, 14.745219 45.957828, 14.746185 45.957592, 14.748309 45.956924, 14.74951 45.956474, 14.751077 45.955787, 14.75157 45.955551, 14.752751 45.954952, 14.753566 45.954502, 14.754768 45.953792, 14.756442 45.95272, 14.756634 45.952571, 14.756764 45.952507, 14.757193 45.952182, 14.758351 45.951305, 14.758587 45.951087, 14.759231 45.950618, 14.75981 45.95021, 14.760067 45.950058, 14.76039 45.949844, 14.761248 45.949371, 14.762343 45.948879, 14.763308 45.948535, 14.764553 45.948173, 14.765433 45.947979, 14.766183 45.947849, 14.76805 45.947654, 14.768651 45.947635, 14.769745 45.947547, 14.771548 45.947334, 14.772341 45.947185, 14.773651 45.946884, 14.774852 45.946456, 14.77511 45.946346, 14.775431 45.94622, 14.776655 45.945594, 14.777363 45.945167, 14.777771 45.944908, 14.778864 45.944007, 14.779809 45.942977, 14.780174 45.942462, 14.780667 45.941585, 14.781418 45.939888, 14.781741 45.939224, 14.782191 45.938537, 14.782599 45.938064, 14.782984 45.937679, 14.783307 45.937377, 14.783757 45.937034, 14.784766 45.936389, 14.785882 45.935874, 14.786525 45.935638, 14.787104 45.93547, 14.788156 45.935188, 14.790388 45.934696, 14.792576 45.934352, 14.794873 45.934158, 14.795666 45.934116, 14.797082 45.934116, 14.798456 45.934204, 14.799464 45.93431, 14.79955 45.93433, 14.800837 45.934459, 14.802404 45.934566, 14.803691 45.934631, 14.804442 45.934654, 14.806824 45.934719, 14.80691 45.934738, 14.811051 45.934738, 14.812424 45.934719, 14.81251 45.934696, 14.814141 45.934608, 14.815407 45.934482, 14.816695 45.934288, 14.818047 45.934051, 14.818219 45.934009, 14.819806 45.933731, 14.820943 45.933559, 14.82178 45.933452, 14.822896 45.933364, 14.824227 45.933364, 14.824892 45.933387, 14.826157 45.933494, 14.826608 45.933578, 14.827231 45.933666, 14.828175 45.93386, 14.82869 45.933986, 14.831265 45.934524, 14.832402 45.934673, 14.833045 45.934738, 14.834097 45.934803, 14.835299 45.934803, 14.837165 45.934631, 14.839289 45.934181, 14.840427 45.933815, 14.840964 45.933624, 14.841414 45.933429, 14.842423 45.932937, 14.843646 45.932205, 14.845427 45.931026, 14.846543 45.930381, 14.847422 45.929931, 14.848002 45.929695, 14.848816 45.929416, 14.848902 45.929374, 14.850319 45.928966, 14.851649 45.928665, 14.854246 45.928173, 14.857765 45.927551, 14.859739 45.92712, 14.862743 45.926368, 14.864052 45.92609, 14.864266 45.926071, 14.865426 45.925876, 14.867142 45.925704, 14.868729 45.925598, 14.869717 45.925556, 14.871712 45.925449, 14.873086 45.925338, 14.874524 45.92517, 14.876069 45.924911, 14.877957 45.924419, 14.881154 45.923751, 14.883514 45.923515, 14.885231 45.923454, 14.887097 45.923538, 14.888449 45.92358, 14.889136 45.92358, 14.890724 45.923561, 14.892054 45.923515, 14.894007 45.923366, 14.895724 45.923278, 14.897032 45.923278, 14.898083 45.923408, 14.898814 45.92358, 14.901087 45.924308, 14.901904 45.924526, 14.902912 45.924716, 14.905207 45.924953, 14.905787 45.925041, 14.906302 45.925148, 14.907847 45.92564, 14.908705 45.926025, 14.910079 45.926692, 14.911795 45.927764, 14.912611 45.928302, 14.913619 45.928836, 14.914734 45.929267, 14.915271 45.929504, 14.916516 45.929931, 14.917868 45.930255, 14.91894 45.930469, 14.920056 45.930553, 14.920936 45.930599, 14.921601 45.930599, 14.923619 45.930511, 14.924455 45.930362, 14.925399 45.930084, 14.926644 45.929611, 14.928296 45.928646, 14.928875 45.928173, 14.92939 45.9277, 14.929884 45.927101, 14.930785 45.926326, 14.931708 45.925746, 14.931965 45.925682, 14.932695 45.925254, 14.932995 45.925125, 14.933318 45.924953, 14.933725 45.924804, 14.934841 45.924438, 14.936128 45.924118, 14.936686 45.924011, 14.937158 45.923881, 14.938102 45.923515, 14.938382 45.923389, 14.938853 45.92313, 14.940119 45.922424, 14.941386 45.921798, 14.942458 45.921371, 14.942952 45.921199, 14.943424 45.921051, 14.944561 45.920749, 14.945312 45.920597, 14.946921 45.920425, 14.949303 45.920425, 14.949561 45.920425, 14.950312 45.920383, 14.950612 45.920341, 14.951212 45.920234, 14.951727 45.920105, 14.952372 45.91991, 14.95308 45.919612, 14.953981 45.919139, 14.954281 45.918968, 14.955075 45.918388, 14.955676 45.917785, 14.955955 45.9174, 14.956041 45.917209, 14.956213 45.916713, 14.95632 45.916263, 14.956427 45.91534, 14.956534 45.914806, 14.956706 45.914375, 14.956835 45.914138, 14.957093 45.913795, 14.957135 45.913711, 14.957478 45.913345, 14.95765 45.913196, 14.958187 45.912811, 14.958981 45.912357, 14.959624 45.912059, 14.960332 45.911758, 14.961426 45.91135, 14.962414 45.911029, 14.962993 45.910877, 14.964259 45.910621, 14.964881 45.910556, 14.965568 45.910469, 14.966426 45.910408, 14.966727 45.910408, 14.966856 45.910385, 14.967843 45.910385, 14.969559 45.910469, 14.970847 45.91064, 14.971125 45.910705, 14.971276 45.910728, 14.97222 45.910964, 14.973013 45.911201, 14.97428 45.911521, 14.975181 45.911693, 14.975932 45.911735, 14.976426 45.911735, 14.97737 45.911628, 14.978142 45.911479, 14.978872 45.911243, 14.979838 45.910793, 14.980095 45.91064, 14.980567 45.91032, 14.981018 45.909976, 14.982219 45.909118, 14.983271 45.908454, 14.984129 45.908004, 14.98458 45.907787, 14.985116 45.907573, 14.985716 45.907318, 14.98606 45.907207, 14.986897 45.906974, 14.987541 45.906822, 14.988614 45.906673, 14.988892 45.90665, 14.988936 45.906631, 14.989623 45.906585, 14.990395 45.906566, 14.991597 45.906566, 14.992411 45.906543, 14.992905 45.90652, 14.994042 45.906414, 14.994858 45.906288, 14.995265 45.9062, 14.998463 45.905406, 14.999321 45.905235, 14.999986 45.905128, 15.001338 45.904998, 15.00166 45.904975, 15.002218 45.904975, 15.00327 45.904975, 15.00402 45.905021, 15.004128 45.905021, 15.004492 45.90504, 15.005479 45.905105, 15.006037 45.905105, 15.006445 45.905147, 15.006681 45.905212, 15.00681 45.905319, 15.006831 45.905426, 15.006789 45.90549, 15.006703 45.905555, 15.006445 45.905601, 15.006123 45.905685, 15.005994 45.906394, 15.005845 45.907295, 15.005801 45.907638, 15.005823 45.90768, 15.00623 45.90768, 15.006617 45.907661, 15.007496 45.907703, 15.008398 45.90781, 15.008955 45.907959, 15.009299 45.908046, 15.009793 45.908111, 15.010543 45.908111, 15.011359 45.908153, 15.011681 45.908195, 15.012131 45.908195, 15.012581 45.908153, 15.013226 45.907981, 15.01359 45.907852, 15.014277 45.907573, 15.014985 45.907531, 15.015736 45.907531, 15.016015 45.907508, 15.016701 45.907508, 15.016959 45.907531, 15.017689 45.90755, 15.018011 45.907508, 15.018182 45.907508, 15.019577 45.907402, 15.020092 45.907379, 15.020907 45.907337, 15.021444 45.907253, 15.022023 45.9071, 15.022538 45.906993, 15.023011 45.906929, 15.023375 45.906929, 15.023675 45.906951, 15.024083 45.907035, 15.024362 45.907123, 15.024748 45.907318, 15.025113 45.907531, 15.02522 45.907615, 15.025349 45.907768, 15.0255 45.907981, 15.025692 45.90839, 15.025735 45.90871, 15.025713 45.909267, 15.025799 45.910018, 15.026101 45.91045, 15.02653 45.910751, 15.026679 45.910858, 15.027709 45.911392, 15.027903 45.911563, 15.028096 45.911781, 15.02859 45.912445, 15.028782 45.912681, 15.029061 45.913131, 15.02949 45.913688, 15.029812 45.914203, 15.029769 45.91476, 15.029706 45.915168, 15.029641 45.915576, 15.029662 45.915813, 15.029726 45.916008, 15.029919 45.91637, 15.030049 45.916587, 15.030135 45.916843, 15.030091 45.917163, 15.029941 45.917423, 15.02962 45.917724, 15.029534 45.917808, 15.029318 45.917938, 15.029211 45.918087, 15.029126 45.918365, 15.029126 45.918495, 15.029318 45.918796, 15.029448 45.919097, 15.029448 45.919223, 15.029297 45.919441, 15.029232 45.91959, 15.029254 45.919761, 15.029404 45.920169, 15.029555 45.920448, 15.029706 45.920578, 15.029941 45.920642, 15.030284 45.920707, 15.030413 45.920791, 15.030478 45.921005, 15.030499 45.921241, 15.030564 45.921348, 15.030993 45.921608, 15.031229 45.921714, 15.031443 45.921779, 15.031958 45.921863, 15.032473 45.921863, 15.032688 45.921844, 15.033546 45.921649, 15.033782 45.921627, 15.034018 45.921691, 15.03419 45.921821, 15.034382 45.921993, 15.034897 45.922615, 15.035263 45.923, 15.036356 45.924331, 15.038181 45.926456, 15.038438 45.926822, 15.038931 45.927379, 15.039039 45.927528, 15.039082 45.927852, 15.038975 45.928604, 15.03891 45.929481, 15.038953 45.92976, 15.039017 45.929889, 15.039082 45.929977, 15.039125 45.930084, 15.039425 45.930404, 15.040026 45.930896, 15.041013 45.931629, 15.041314 45.931819, 15.041786 45.932205, 15.042987 45.933109, 15.043588 45.933536, 15.043995 45.93386, 15.044275 45.934051, 15.044489 45.934265, 15.045219 45.934783, 15.04582 45.935253, 15.04657 45.935791, 15.047387 45.93637, 15.047708 45.936477, 15.047837 45.936607, 15.047837 45.936821, 15.047923 45.936992, 15.048245 45.937484, 15.048459 45.937679, 15.048802 45.937915, 15.049361 45.938152, 15.04966 45.938259, 15.050498 45.938537, 15.050992 45.938728, 15.051506 45.93888, 15.05185 45.938964, 15.052515 45.939071, 15.053416 45.939395, 15.054102 45.939697, 15.054425 45.939868, 15.05481 45.940212, 15.055026 45.940425, 15.055455 45.940982, 15.055626 45.941131, 15.056205 45.941669, 15.056399 45.941841, 15.056956 45.942443, 15.057278 45.943023, 15.057471 45.943687, 15.057557 45.944438, 15.0576 45.945232, 15.057621 45.945316, 15.057686 45.945426, 15.0579 45.945747, 15.058222 45.94609, 15.058459 45.946456, 15.058737 45.947292, 15.05893 45.947677, 15.059317 45.948085, 15.05966 45.948406, 15.059681 45.948452, 15.059832 45.948577, 15.060004 45.948684, 15.060303 45.948986, 15.060433 45.949203, 15.060626 45.949779, 15.060647 45.949932, 15.060754 45.950145, 15.061055 45.950553, 15.061549 45.951068, 15.062106 45.951087, 15.062407 45.951152, 15.062728 45.951412, 15.063008 45.951667, 15.063694 45.952205, 15.064038 45.952507, 15.064467 45.952999, 15.065132 45.95362, 15.065497 45.953945, 15.066033 45.95446, 15.06614 45.954586, 15.066419 45.955059, 15.066699 45.955425, 15.066762 45.955551, 15.06702 45.95594, 15.067492 45.95652, 15.067707 45.95671, 15.067921 45.95684, 15.068694 45.957313, 15.069166 45.957634, 15.069423 45.957763, 15.06983 45.95787, 15.070132 45.957893, 15.072405 45.957847, 15.073737 45.957805, 15.074252 45.957805, 15.074465 45.957786, 15.075238 45.957786, 15.075775 45.957786, 15.077191 45.95774, 15.077342 45.957721, 15.078242 45.957721, 15.078928 45.957698, 15.07968 45.957698, 15.080109 45.957805, 15.080946 45.95819, 15.081891 45.958686, 15.08204 45.958728, 15.082469 45.958965, 15.083113 45.959266, 15.084401 45.959865, 15.084959 45.959995, 15.085989 45.959995, 15.087813 45.960037, 15.088392 45.96006, 15.088843 45.960102, 15.089186 45.960144, 15.089444 45.960231, 15.091353 45.961132, 15.091653 45.961261, 15.09234 45.961498, 15.092919 45.961818, 15.09322 45.962013, 15.093434 45.96212, 15.093756 45.962162, 15.093949 45.962162, 15.095322 45.961883, 15.095387 45.961883, 15.095923 45.961711, 15.096352 45.961498, 15.096674 45.961368, 15.097962 45.96096, 15.100236 45.960315, 15.10088 45.960166, 15.10133 45.960166, 15.101524 45.960189, 15.102017 45.960315, 15.102511 45.960445, 15.103198 45.960575, 15.103648 45.960639, 15.103819 45.960639, 15.104871 45.960723, 15.106051 45.960853, 15.106845 45.960983, 15.107253 45.96109, 15.10764 45.961174, 15.108326 45.961219, 15.108584 45.961196, 15.108798 45.961154, 15.110301 45.96096, 15.111094 45.960876, 15.111372 45.96083, 15.111566 45.960746, 15.111931 45.960639, 15.112532 45.960552, 15.112875 45.960422, 15.11324 45.96025, 15.11354 45.960102, 15.114291 45.9598, 15.114548 45.959758, 15.115622 45.959457, 15.116673 45.959114, 15.117231 45.958965, 15.118089 45.958663, 15.118325 45.958557, 15.118518 45.958492, 15.11942 45.958148, 15.119935 45.957977, 15.120278 45.957893, 15.121115 45.958255, 15.121994 45.95845, 15.122595 45.958705, 15.122874 45.958793, 15.123153 45.958942, 15.123539 45.959114, 15.124183 45.959545, 15.124377 45.959781, 15.124548 45.959865, 15.12502 45.960208, 15.125277 45.960296, 15.125792 45.960533, 15.125964 45.960701, 15.126135 45.960811, 15.126479 45.96109, 15.126694 45.961174, 15.126866 45.961345, 15.127295 45.961582, 15.127467 45.96173, 15.127659 45.961818, 15.127874 45.96199, 15.128003 45.962139, 15.128067 45.962291, 15.128046 45.962783, 15.128046 45.962978, 15.128003 45.963298, 15.128109 45.96347, 15.128195 45.963558, 15.128303 45.963748, 15.128624 45.963966, 15.129054 45.96437, 15.129548 45.964759, 15.12974 45.96495, 15.129912 45.965057, 15.130063 45.965187, 15.130084 45.965229, 15.130299 45.965381, 15.130557 45.965507, 15.1309 45.965595, 15.131264 45.965744, 15.13238 45.966026, 15.132766 45.966152, 15.13311 45.966217, 15.134032 45.96643, 15.134783 45.966648, 15.137058 45.967033, 15.137401 45.967075, 15.138024 45.967098, 15.138453 45.967075, 15.138689 45.967056, 15.139719 45.96688, 15.13989 45.966903, 15.140276 45.967075, 15.140706 45.967224, 15.141757 45.967205, 15.142272 45.96714, 15.14326 45.96714, 15.143689 45.967163, 15.144353 45.967289, 15.144976 45.96746, 15.145148 45.967483, 15.145512 45.967483, 15.146285 45.967353, 15.146671 45.967269, 15.147143 45.967205, 15.147744 45.967075, 15.148688 45.966754, 15.149203 45.966667, 15.149525 45.966583, 15.150212 45.966476, 15.151026 45.966411, 15.151284 45.966453, 15.151563 45.966476, 15.152056 45.966537, 15.152636 45.966495, 15.152979 45.966388, 15.153666 45.966304, 15.15416 45.966304, 15.154524 45.966259, 15.155426 45.966259, 15.156863 45.966281, 15.15712 45.966259, 15.157485 45.966259, 15.157851 45.966304, 15.158258 45.96643, 15.158558 45.966583, 15.159159 45.96701, 15.159331 45.967247, 15.159438 45.96746, 15.159588 45.967826, 15.159674 45.968128, 15.159911 45.968597, 15.160532 45.969051, 15.16152 45.969543, 15.162098 45.969757, 15.162249 45.969779, 15.162378 45.969821, 15.162635 45.969844, 15.163258 45.969844, 15.163622 45.969779, 15.16388 45.969673, 15.164052 45.969543, 15.164202 45.969348, 15.164244 45.969177, 15.164889 45.969284, 15.165725 45.969459, 15.16652 45.969738, 15.167035 45.969993, 15.167699 45.970443, 15.168043 45.970745, 15.168214 45.970832, 15.168408 45.970874, 15.168429 45.970893, 15.169201 45.971111, 15.169673 45.971218, 15.169888 45.971302, 15.170125 45.971431, 15.170382 45.971603, 15.170489 45.97171, 15.170681 45.971797, 15.170853 45.971946, 15.170897 45.972011, 15.171047 45.972141, 15.171111 45.972934, 15.171133 45.973598, 15.171196 45.974155, 15.171304 45.974521, 15.171669 45.974994, 15.171797 45.975231, 15.171948 45.975402, 15.172055 45.975574, 15.172356 45.975852, 15.172892 45.976303, 15.172935 45.976451, 15.172935 45.976581, 15.172871 45.976924, 15.172506 45.977588, 15.172377 45.97776, 15.172205 45.978149, 15.17227 45.978469, 15.172356 45.978664, 15.172356 45.978984, 15.172334 45.979991, 15.172356 45.980144, 15.17242 45.98027, 15.173364 45.980552, 15.173536 45.980659, 15.173815 45.980766, 15.174008 45.980896, 15.174159 45.981021, 15.174372 45.981151, 15.174544 45.981346, 15.174737 45.981475, 15.174931 45.981666, 15.175295 45.981925, 15.175532 45.982051, 15.175767 45.982116, 15.17639 45.982353, 15.176518 45.982524, 15.176476 45.982524, 15.17742 45.982955, 15.178149 45.983039, 15.17935 45.982891, 15.179415 45.982868, 15.181303 45.982784, 15.183192 45.982566, 15.184329 45.982891, 15.184994 45.983104, 15.185209 45.983211, 15.185338 45.98336, 15.185574 45.983554, 15.186131 45.983898, 15.186496 45.984092, 15.186604 45.984222, 15.186861 45.984222, 15.187269 45.984264, 15.187526 45.98439, 15.187741 45.984455, 15.188277 45.984565, 15.18875 45.984565, 15.189093 45.984519, 15.189265 45.984455, 15.189951 45.984455, 15.190917 45.984909, 15.191861 45.985614, 15.191861 45.985637, 15.192482 45.986129, 15.192612 45.986217, 15.19274 45.986389, 15.193213 45.986515, 15.193406 45.986496, 15.193663 45.986496, 15.195187 45.987117, 15.197031 45.988147, 15.197247 45.988231, 15.197354 45.988319, 15.197632 45.988403, 15.197869 45.988426, 15.198083 45.988403, 15.198384 45.988403, 15.19862 45.988319, 15.199156 45.988189, 15.199908 45.98804, 15.200315 45.987998, 15.200594 45.988018, 15.201409 45.988254, 15.201666 45.988277, 15.202181 45.988231, 15.202397 45.988189, 15.20274 45.988166, 15.202954 45.988189, 15.203684 45.988403, 15.204864 45.988941, 15.205143 45.989089, 15.20525 45.989112, 15.205551 45.989219, 15.205894 45.989242, 15.20628 45.989307, 15.206688 45.989349, 15.207053 45.989349, 15.20716 45.989372, 15.208469 45.989433, 15.208898 45.989391, 15.209563 45.989242, 15.209842 45.989154, 15.210529 45.989006, 15.210743 45.988941, 15.211044 45.988899, 15.211473 45.98862, 15.211645 45.988491, 15.21173 45.988384, 15.212138 45.988296, 15.212331 45.988277, 15.212996 45.988124, 15.213597 45.98804, 15.214026 45.98804, 15.214262 45.98806, 15.214477 45.988124, 15.214906 45.988491, 15.215485 45.98907, 15.215593 45.989261, 15.215701 45.989498, 15.215807 45.989864, 15.215828 45.990745, 15.216022 45.991344, 15.216387 45.991924, 15.217395 45.993103, 15.217545 45.993232, 15.217675 45.993469, 15.217888 45.993724, 15.218082 45.993919, 15.218747 45.994476, 15.219262 45.994819, 15.219433 45.994972, 15.219691 45.995143, 15.21997 45.995269, 15.220335 45.995529, 15.220593 45.9957, 15.220657 45.995784, 15.220679 45.995956, 15.220635 45.996494, 15.220657 45.99686, 15.220892 45.997673, 15.220957 45.998146, 15.221 45.998897, 15.221129 45.999328, 15.221236 45.999519, 15.221966 45.999992, 15.22203 46.000034, 15.222138 46.000183, 15.222224 46.000358, 15.222224 46.000637, 15.222159 46.000915, 15.22218 46.001388, 15.22218 46.001537, 15.221966 46.002288, 15.221966 46.002651, 15.221901 46.003574, 15.221837 46.003852, 15.221858 46.004047, 15.221837 46.004455, 15.221858 46.004947, 15.221922 46.005378, 15.22218 46.006107, 15.222395 46.005851, 15.222523 46.005508, 15.222739 46.005142, 15.222846 46.0051, 15.222996 46.004989, 15.223168 46.004928, 15.223446 46.004905, 15.223876 46.004905, 15.224112 46.004928, 15.224476 46.004989, 15.224841 46.005184, 15.225184 46.005397, 15.225441 46.005508, 15.225807 46.005699, 15.226408 46.005741, 15.227309 46.005935, 15.227501 46.005958, 15.227867 46.006042, 15.228833 46.006343, 15.22939 46.00658, 15.229819 46.006793, 15.230076 46.006988, 15.230162 46.0069, 15.230248 46.006706, 15.230334 46.006599, 15.230442 46.006515, 15.230699 46.006645, 15.230935 46.006706, 15.231106 46.006729, 15.231278 46.006706, 15.231408 46.006664, 15.2316 46.006557, 15.231707 46.00645, 15.231751 46.006256, 15.231751 46.00613, 15.231686 46.005912, 15.231686 46.005764, 15.231729 46.005615, 15.231901 46.005378, 15.232029 46.005271, 15.232115 46.005142, 15.232072 46.004821, 15.232158 46.004539, 15.232244 46.004367, 15.232373 46.004283, 15.232673 46.004024, 15.232931 46.003898, 15.233166 46.003746, 15.233296 46.003704, 15.23336 46.003574, 15.233403 46.003383, 15.233489 46.003189, 15.23351 46.002952, 15.233746 46.003017, 15.233983 46.003147, 15.234282 46.003253, 15.235013 46.003337, 15.235291 46.003448, 15.235591 46.003532, 15.235892 46.003555, 15.236364 46.003639, 15.236536 46.003616, 15.236815 46.003555, 15.237114 46.00349, 15.237909 46.003189, 15.23808 46.003166, 15.238488 46.003059, 15.238767 46.00291, 15.239068 46.00278, 15.239239 46.002674, 15.239562 46.002544, 15.239754 46.002502, 15.240118 46.002479, 15.24042 46.002479, 15.240633 46.002502, 15.241857 46.002716, 15.243144 46.00291, 15.243488 46.00291, 15.243745 46.002887, 15.244153 46.002803, 15.244496 46.002822, 15.245182 46.002975, 15.245827 46.003082, 15.246449 46.003124, 15.246964 46.003231, 15.247565 46.003425, 15.247929 46.003532, 15.248574 46.003898, 15.248873 46.004047, 15.249474 46.004135, 15.249753 46.004154, 15.250548 46.004325, 15.250933 46.004432, 15.25117 46.004539, 15.251599 46.004947, 15.25177 46.005054, 15.252243 46.005226, 15.252586 46.005271, 15.25323 46.005462, 15.253573 46.005527, 15.254109 46.005657, 15.254366 46.005764, 15.254538 46.005805, 15.254754 46.005935, 15.254946 46.006019, 15.255161 46.006107, 15.255375 46.006195, 15.255698 46.006237, 15.256041 46.006237, 15.25647 46.006195, 15.256706 46.00613, 15.257564 46.005634, 15.257885 46.005527, 15.258143 46.005462, 15.258508 46.00542, 15.259195 46.005313, 15.259732 46.005271, 15.26001 46.005226, 15.260546 46.00529, 15.260848 46.005378, 15.261191 46.005397, 15.261363 46.005378, 15.26162 46.005313, 15.261856 46.005226, 15.262113 46.0051, 15.262285 46.00497, 15.262478 46.004928, 15.262778 46.00497, 15.263057 46.005054, 15.263979 46.005119, 15.264345 46.005119, 15.264602 46.005142, 15.264988 46.005142, 15.265524 46.005249, 15.265696 46.005313, 15.265761 46.005355, 15.266104 46.005569, 15.266255 46.005699, 15.266812 46.006622, 15.267155 46.007007, 15.26767 46.00748, 15.267821 46.007545, 15.267992 46.007652, 15.268614 46.007823, 15.269194 46.007865, 15.269537 46.007865, 15.270417 46.00793, 15.270976 46.008018, 15.271618 46.008079, 15.27237 46.00819, 15.272713 46.00819, 15.272906 46.008167, 15.273379 46.008018, 15.273722 46.007781, 15.273915 46.007415, 15.274065 46.007244, 15.274151 46.007095, 15.274409 46.007007, 15.27458 46.006923, 15.274666 46.006858, 15.274666 46.006729, 15.274622 46.006515, 15.274687 46.006362, 15.274794 46.00632, 15.274945 46.006301, 15.275116 46.006343, 15.275309 46.006492, 15.27664 46.007137, 15.277197 46.007331, 15.27752 46.007415, 15.278227 46.007415, 15.278656 46.007331, 15.278958 46.007225, 15.279044 46.007286, 15.279 46.00735, 15.278485 46.007694, 15.278035 46.008079, 15.277756 46.008445, 15.277005 46.009174, 15.276898 46.009346, 15.276854 46.00954, 15.277005 46.009735, 15.277327 46.009754, 15.278249 46.009628, 15.278464 46.009563, 15.278764 46.009517, 15.279644 46.009735, 15.27988 46.00999, 15.28018 46.010334, 15.280438 46.010482, 15.280653 46.010658, 15.281189 46.010978, 15.28179 46.011386, 15.28194 46.011512, 15.282048 46.011665, 15.282134 46.011985, 15.282519 46.011943, 15.282691 46.011878, 15.283892 46.011577, 15.284172 46.011451, 15.284537 46.011299, 15.285073 46.011192, 15.285674 46.011192, 15.285952 46.011108, 15.286232 46.010978, 15.28694 46.010742, 15.287047 46.010807, 15.287133 46.010913, 15.287648 46.011299, 15.28784 46.01147, 15.288098 46.011577, 15.288206 46.011814, 15.288613 46.012435, 15.288721 46.012672, 15.288892 46.012931, 15.289086 46.013187, 15.289364 46.013359, 15.289687 46.013572, 15.291102 46.014476, 15.291338 46.014602, 15.291896 46.014732, 15.292475 46.014667, 15.293248 46.014434, 15.293763 46.014217, 15.294236 46.014091, 15.294471 46.01411, 15.295072 46.014434, 15.295479 46.014434, 15.29666 46.013595, 15.29711 46.013252, 15.297518 46.013103, 15.298184 46.012825, 15.29872 46.013275, 15.298934 46.013404, 15.299256 46.013511, 15.299492 46.013553, 15.299643 46.013702, 15.299707 46.013874, 15.299685 46.014133, 15.299664 46.014778, 15.299685 46.014945, 15.299771 46.01514, 15.300028 46.015506, 15.302024 46.016838, 15.302218 46.017005, 15.302368 46.017242, 15.302475 46.01765, 15.302454 46.017868, 15.30239 46.018146, 15.302132 46.018531, 15.301939 46.018745, 15.301531 46.019241, 15.300715 46.020462, 15.299986 46.021644, 15.299815 46.021987, 15.299557 46.022609, 15.299342 46.023338, 15.299321 46.023361, 15.299235 46.023616, 15.299128 46.02383, 15.298956 46.024131, 15.298763 46.02441, 15.298483 46.024734, 15.297669 46.02544, 15.29711 46.025764, 15.296725 46.026107, 15.296574 46.026321, 15.296423 46.02705, 15.296381 46.027565, 15.296445 46.027736, 15.296553 46.027843, 15.297154 46.028701, 15.297175 46.028896, 15.297132 46.029151, 15.29711 46.029518, 15.297218 46.029773, 15.297583 46.030269, 15.298012 46.031021, 15.298033 46.031276, 15.297968 46.031425, 15.296639 46.033206, 15.296296 46.033573, 15.296188 46.033767, 15.29608 46.034004, 15.296016 46.034259, 15.295973 46.034709, 15.295952 46.035697, 15.295866 46.03617, 15.295737 46.036384, 15.295523 46.036575, 15.29518 46.036792, 15.295008 46.036899, 15.2949 46.036941, 15.294665 46.037113, 15.294363 46.037498, 15.294236 46.037864, 15.2943 46.038486, 15.2943 46.038722, 15.294363 46.038936, 15.294385 46.039108, 15.294643 46.039474, 15.294793 46.039604, 15.295008 46.03984, 15.295072 46.039966, 15.295115 46.040161, 15.295094 46.040416, 15.295008 46.040588, 15.294857 46.041168, 15.294772 46.041404, 15.294535 46.041942, 15.294322 46.042285, 15.294171 46.042457, 15.293999 46.042587, 15.293763 46.042713, 15.293313 46.042778, 15.293184 46.0428, 15.292883 46.0428, 15.292691 46.042819, 15.292475 46.042972, 15.292347 46.043121, 15.292283 46.043273, 15.292262 46.043659, 15.292262 46.044021, 15.292304 46.044364, 15.292347 46.044559, 15.292454 46.044773, 15.292669 46.045009, 15.293162 46.045459, 15.293721 46.045909, 15.293913 46.04602, 15.294471 46.046188, 15.294815 46.046318, 15.296639 46.047435, 15.29711 46.047714, 15.297453 46.04795, 15.297819 46.048141, 15.298892 46.048809, 15.299278 46.048934, 15.299599 46.049064, 15.301016 46.049537, 15.301316 46.049667, 15.302002 46.049903, 15.302239 46.05001, 15.302754 46.050395, 15.302926 46.05059, 15.303162 46.050781, 15.303547 46.051059, 15.303784 46.051189, 15.304148 46.051319, 15.304471 46.051403, 15.304965 46.051467, 15.305995 46.051662, 15.306402 46.051856, 15.306745 46.051963, 15.307088 46.051982, 15.308869 46.051876, 15.309212 46.051898, 15.309535 46.051982, 15.310221 46.052368, 15.310929 46.052906, 15.311402 46.053398, 15.311659 46.053592, 15.312045 46.053806, 15.312645 46.054084, 15.312925 46.054172, 15.313654 46.054237, 15.314963 46.054107, 15.315242 46.054149, 15.315586 46.05426, 15.315865 46.054428, 15.316058 46.054687, 15.316144 46.055053, 15.316144 46.055675, 15.316101 46.05606, 15.316164 46.056297, 15.31638 46.05677, 15.316487 46.056896, 15.31653 46.057048, 15.316638 46.05722, 15.316895 46.057518, 15.317023 46.057628, 15.317067 46.05767, 15.317452 46.057861, 15.317989 46.057971, 15.318246 46.058055, 15.31844 46.058143, 15.318697 46.058334, 15.318847 46.058528, 15.319062 46.058742, 15.319362 46.058891, 15.319748 46.05902, 15.320092 46.059066, 15.320993 46.059127, 15.321765 46.059257, 15.32243 46.059429, 15.323396 46.05986, 15.323718 46.059967, 15.324276 46.060073, 15.324791 46.060222, 15.325005 46.060329, 15.325156 46.060375, 15.326766 46.061275, 15.327195 46.061424, 15.327473 46.061489, 15.327688 46.061511, 15.328053 46.06147, 15.328225 46.061405, 15.328997 46.061016, 15.32934 46.060974, 15.329619 46.060997, 15.329876 46.061061, 15.329962 46.061103, 15.330069 46.061187, 15.330542 46.061832, 15.330649 46.062046, 15.331293 46.062648, 15.331486 46.062755, 15.331851 46.062885, 15.332129 46.062927, 15.332472 46.062904, 15.333159 46.06282, 15.333395 46.062797, 15.333782 46.062797, 15.334017 46.062862, 15.335069 46.063205, 15.335349 46.063228, 15.336099 46.063228, 15.337043 46.06314, 15.337129 46.063121, 15.339039 46.062969, 15.340004 46.062885, 15.340777 46.062713, 15.342473 46.062263, 15.342794 46.062156, 15.343137 46.062004, 15.343287 46.061897, 15.343395 46.061855, 15.343846 46.061489, 15.344574 46.060867, 15.345068 46.060672, 15.345498 46.060588, 15.345734 46.060588, 15.345992 46.06063, 15.346141 46.06063, 15.347343 46.060997, 15.348115 46.061126, 15.348481 46.061145, 15.348996 46.061126, 15.349339 46.06108, 15.349746 46.060974, 15.350153 46.060825, 15.351677 46.060375, 15.355239 46.059429, 15.355669 46.059322, 15.356055 46.059234, 15.356441 46.059192, 15.356699 46.059173, 15.357256 46.059173, 15.358008 46.059345, 15.358822 46.059577, 15.35921 46.059665, 15.360239 46.059688, 15.360647 46.059707, 15.36099 46.059753, 15.361226 46.059837, 15.361419 46.059879, 15.36187 46.060157, 15.361912 46.060203, 15.361998 46.060264, 15.362192 46.060482, 15.362513 46.060932, 15.363222 46.061813, 15.363501 46.062091, 15.364874 46.062992, 15.365603 46.063377, 15.36629 46.063697, 15.366848 46.064128, 15.367256 46.064514, 15.368243 46.066059, 15.368629 46.066619, 15.368737 46.066833, 15.369122 46.06739, 15.369252 46.067649, 15.369509 46.067928, 15.370002 46.068206, 15.370368 46.068443, 15.37056 46.068679, 15.370775 46.069042, 15.370989 46.069385, 15.371483 46.070095, 15.371869 46.070522, 15.37204 46.070693, 15.372706 46.071189, 15.37365 46.071746, 15.37468 46.072456, 15.37498 46.072582, 15.375088 46.072582, 15.37526 46.072624, 15.375517 46.072647, 15.376482 46.072624, 15.376933 46.072669, 15.377384 46.072669, 15.377835 46.072669, 15.378972 46.072883, 15.379208 46.07299, 15.379401 46.073184, 15.379487 46.073356, 15.379723 46.073547, 15.380002 46.073719, 15.380496 46.073871, 15.381032 46.073913, 15.381976 46.07389, 15.382233 46.073871, 15.382512 46.073741, 15.383006 46.073421, 15.383328 46.07331, 15.383757 46.073398, 15.3841 46.073463, 15.384422 46.07344, 15.384873 46.073291, 15.38513 46.073097, 15.385302 46.073013, 15.385623 46.072967, 15.385817 46.073013, 15.386138 46.073204, 15.386374 46.073398, 15.38659 46.073677, 15.386782 46.073806, 15.386825 46.073848, 15.386911 46.074085, 15.386975 46.074451, 15.386954 46.074684, 15.386653 46.074985, 15.38659 46.075157, 15.38659 46.07537, 15.386696 46.0755, 15.386954 46.075565, 15.387083 46.075672, 15.387254 46.075996, 15.387362 46.07608, 15.38749 46.076103, 15.388198 46.076122, 15.388241 46.076145, 15.388284 46.07621, 15.388155 46.076423, 15.388113 46.076576, 15.388091 46.076789, 15.388113 46.077045, 15.38807 46.077198, 15.387941 46.077411, 15.387984 46.077518, 15.388134 46.077667, 15.388542 46.077667, 15.388691 46.077713, 15.388821 46.077949, 15.389057 46.078056, 15.3894 46.07801, 15.3897 46.078056, 15.390065 46.078228, 15.390322 46.078418, 15.391009 46.079021, 15.39131 46.079429, 15.39176 46.079536, 15.391846 46.079833, 15.392104 46.080009, 15.392361 46.080223, 15.392683 46.080329, 15.393134 46.08052, 15.393434 46.080566, 15.393799 46.080566, 15.394121 46.080631, 15.394829 46.081661, 15.395087 46.081958, 15.395236 46.082069, 15.395516 46.082153, 15.395751 46.082195, 15.396073 46.082302, 15.396224 46.082412, 15.396374 46.082603, 15.396653 46.083461, 15.39661 46.083675, 15.396674 46.083869, 15.397532 46.084899, 15.397532 46.085178, 15.397554 46.085285, 15.397704 46.085479, 15.398369 46.085823, 15.398692 46.086013, 15.399163 46.086231, 15.399528 46.086357, 15.399979 46.086486, 15.400322 46.086639, 15.400601 46.086681, 15.401352 46.086532, 15.401931 46.086574, 15.402253 46.086486, 15.402532 46.086509, 15.402618 46.086532, 15.403004 46.086532, 15.403347 46.086574, 15.403841 46.086723, 15.40412 46.086894, 15.404335 46.087066, 15.404528 46.087131, 15.404871 46.087154, 15.40721 46.087604, 15.40854 46.087776, 15.409055 46.087882, 15.409249 46.087905, 15.409527 46.087966, 15.409892 46.088138, 15.410128 46.088138, 15.410471 46.088031, 15.410643 46.087882, 15.410858 46.087795, 15.411309 46.087841, 15.411544 46.087795, 15.411759 46.087604, 15.411995 46.087432, 15.412253 46.087387, 15.412596 46.087451, 15.413003 46.087497, 15.413668 46.087432, 15.41472 46.087387, 15.414956 46.087409, 15.415385 46.087497, 15.416329 46.088054, 15.417252 46.088653, 15.417552 46.088825, 15.417832 46.088935, 15.41796 46.089019, 15.418046 46.089126, 15.418196 46.089214, 15.419075 46.089511, 15.419312 46.089664, 15.419463 46.089813, 15.419549 46.090026, 15.419549 46.090198, 15.419463 46.090328, 15.419269 46.090457, 15.418926 46.090629, 15.418668 46.090736, 15.418433 46.09095, 15.41811 46.091316, 15.417681 46.091724, 15.417338 46.091938, 15.416437 46.092239, 15.41605 46.092517, 15.416265 46.093353, 15.416373 46.093589, 15.416608 46.093803, 15.417016 46.094062, 15.417467 46.094253, 15.417552 46.094383, 15.417531 46.09449, 15.417381 46.094642, 15.417101 46.094749, 15.416952 46.094921, 15.416952 46.095115, 15.417037 46.095306, 15.41708 46.09552, 15.416952 46.095649, 15.416651 46.095737, 15.416995 46.095993, 15.417123 46.096336, 15.417252 46.096488, 15.41751 46.096679, 15.41781 46.096851, 15.418089 46.096832, 15.418604 46.096488, 15.418818 46.096229, 15.41914 46.09608, 15.41972 46.095993, 15.420213 46.095886, 15.4209 46.095672, 15.421415 46.095478, 15.421822 46.095283, 15.422187 46.095157, 15.422488 46.095092, 15.422982 46.095134, 15.423389 46.095199, 15.42399 46.095283, 15.424762 46.095329, 15.425084 46.095413, 15.425385 46.095542, 15.425642 46.09608, 15.425856 46.096401, 15.426136 46.096637, 15.426371 46.096786, 15.426736 46.096916, 15.427294 46.097023, 15.42811 46.097045, 15.428453 46.09711, 15.428818 46.097259, 15.429118 46.097282, 15.429612 46.097282, 15.430192 46.097217, 15.430685 46.097259, 15.431607 46.097473, 15.432187 46.097579, 15.432744 46.097625, 15.433174 46.097732, 15.433732 46.097988, 15.43386 46.09803, 15.434182 46.098205, 15.434826 46.098697, 15.435513 46.099361, 15.436371 46.100028, 15.43665 46.1002, 15.437058 46.100372, 15.437401 46.100433, 15.437637 46.100563, 15.437787 46.100734, 15.438066 46.100994, 15.438517 46.101272, 15.438817 46.101528, 15.439417 46.101982, 15.440276 46.102687, 15.440383 46.103012, 15.440426 46.103137, 15.440855 46.104209, 15.440941 46.104553, 15.44092 46.105007, 15.440598 46.105564, 15.440361 46.105819, 15.440361 46.106037, 15.440405 46.106185, 15.440555 46.106487, 15.440727 46.106742, 15.440941 46.106807, 15.441156 46.106849, 15.441563 46.107002, 15.441906 46.107151, 15.4424 46.107475, 15.442873 46.107879, 15.443044 46.108203, 15.443152 46.108352, 15.443366 46.108867, 15.443388 46.109039, 15.443537 46.109233, 15.443945 46.109447, 15.445126 46.109855, 15.445597 46.110046, 15.445791 46.110221, 15.445834 46.110477, 15.445834 46.110733, 15.445727 46.110992, 15.445319 46.111465, 15.445233 46.111656, 15.445276 46.11185, 15.445447 46.112171, 15.445769 46.11243, 15.446005 46.112667, 15.446306 46.113094, 15.446349 46.113159, 15.44652 46.113502, 15.446585 46.113975, 15.446692 46.114341, 15.446821 46.114532, 15.447186 46.115325, 15.447422 46.115734, 15.448258 46.116722, 15.448601 46.117065, 15.44903 46.117366, 15.44946 46.117622, 15.450146 46.117881, 15.450661 46.118007, 15.450726 46.11803, 15.45122 46.118072, 15.452314 46.118095, 15.45315 46.11835, 15.453579 46.118461, 15.454073 46.118522, 15.454396 46.11848, 15.454717 46.118396, 15.455189 46.118373, 15.455426 46.11848, 15.45609 46.118865, 15.456498 46.118995, 15.457228 46.11906, 15.458128 46.11906, 15.458729 46.119125, 15.458901 46.119148, 15.459546 46.119361, 15.459996 46.11964, 15.460425 46.120002, 15.460682 46.120178, 15.461005 46.120346, 15.461262 46.120456, 15.461733 46.120582, 15.462248 46.120628, 15.462763 46.120628, 15.463086 46.120498, 15.463386 46.120326, 15.463601 46.120113, 15.463772 46.119789, 15.463837 46.11951, 15.463815 46.119338, 15.463686 46.11906, 15.463515 46.118888, 15.463236 46.11874, 15.463192 46.118694, 15.461047 46.118053, 15.46049 46.117858, 15.460188 46.117622, 15.460039 46.117408, 15.459953 46.117149, 15.460124 46.116699, 15.46036 46.116336, 15.460576 46.116058, 15.461005 46.115798, 15.461455 46.115585, 15.462099 46.115196, 15.462378 46.114898, 15.462893 46.114402, 15.463214 46.114059, 15.463536 46.11378, 15.463772 46.113697, 15.464095 46.11359, 15.464588 46.113567, 15.465017 46.113609, 15.465554 46.113632, 15.465961 46.11359, 15.466519 46.113418, 15.466862 46.113182, 15.467248 46.112838, 15.467506 46.11256, 15.467549 46.11256, 15.467871 46.11243, 15.468321 46.112281, 15.468815 46.112171, 15.470467 46.111957, 15.471175 46.111873, 15.473042 46.111595, 15.473664 46.111358, 15.474372 46.111034, 15.476389 46.110027, 15.47699 46.109683, 15.477484 46.109447, 15.477891 46.10934, 15.478384 46.109275, 15.479265 46.109252, 15.479414 46.109233, 15.479844 46.109233, 15.480338 46.10921, 15.481818 46.10921, 15.482205 46.109169, 15.482419 46.109081, 15.482655 46.108955, 15.483127 46.108631, 15.483878 46.10844, 15.485273 46.108139, 15.485659 46.108032, 15.486045 46.107879, 15.48656 46.107643, 15.487247 46.107387, 15.487504 46.107322, 15.48759 46.10728, 15.487977 46.107257, 15.488642 46.107364, 15.489307 46.107494, 15.489329 46.107517, 15.489693 46.107643, 15.490079 46.107879, 15.490874 46.108459, 15.491602 46.108695, 15.492826 46.10889, 15.493363 46.109062, 15.493748 46.10921, 15.494071 46.109405, 15.494543 46.109535, 15.495143 46.109642, 15.495744 46.109619, 15.496968 46.109619, 15.497761 46.109642, 15.497804 46.109661, 15.498276 46.109725, 15.499049 46.10992, 15.499972 46.110134, 15.500122 46.110157, 15.50231 46.110713, 15.502461 46.110733, 15.502503 46.110778, 15.502825 46.110626, 15.503297 46.1105, 15.503748 46.110454, 15.504928 46.110477, 15.505208 46.110454, 15.505507 46.11037, 15.505658 46.110282, 15.505936 46.109985, 15.50613 46.109489, 15.506365 46.109127, 15.506645 46.108867, 15.507096 46.108631, 15.507374 46.108417, 15.507546 46.108222, 15.508212 46.107643, 15.508425 46.10741, 15.508533 46.107192, 15.508555 46.106979, 15.508833 46.106143, 15.509091 46.105606, 15.509413 46.105262, 15.510464 46.104385, 15.511302 46.103717, 15.511709 46.103462, 15.512116 46.103244, 15.512438 46.103096, 15.513039 46.102882, 15.51334 46.102794, 15.514777 46.102516, 15.518704 46.101764, 15.519348 46.101657, 15.519627 46.102088, 15.519906 46.102451, 15.52027 46.102966, 15.520699 46.103481, 15.521107 46.10374, 15.521557 46.103977, 15.521986 46.104232, 15.522202 46.104362, 15.522673 46.104618, 15.522953 46.104854, 15.523081 46.104984, 15.523146 46.105178, 15.523124 46.105262, 15.523016 46.105499, 15.522738 46.105865, 15.522631 46.106101, 15.522545 46.106422, 15.522781 46.107818, 15.522889 46.108245, 15.522974 46.108737, 15.522931 46.109039, 15.522094 46.109897, 15.521794 46.110157, 15.520978 46.110607, 15.520527 46.110969, 15.520292 46.111251, 15.519927 46.111785, 15.519648 46.112087, 15.519497 46.112342, 15.519455 46.112579, 15.519562 46.11301, 15.519713 46.113437, 15.519713 46.113632, 15.519627 46.113803, 15.519455 46.114059, 15.519348 46.114166, 15.519133 46.114383, 15.51894 46.114532, 15.518833 46.114662, 15.518855 46.114852, 15.519004 46.115089, 15.519262 46.115283, 15.519669 46.11565, 15.520184 46.116058, 15.5204 46.116271, 15.520635 46.116657, 15.520785 46.117237, 15.520571 46.117923, 15.520571 46.118331, 15.520527 46.118652, 15.520549 46.118911, 15.520936 46.11906, 15.521687 46.119209, 15.521901 46.119361, 15.522094 46.119617, 15.522244 46.119834, 15.522395 46.119941, 15.522845 46.119918, 15.523339 46.119941, 15.523725 46.11996, 15.524154 46.120025, 15.524498 46.120132, 15.524884 46.120368, 15.525313 46.12054, 15.52557 46.12067, 15.52615 46.120841, 15.52645 46.12099, 15.526944 46.121658, 15.527266 46.121978, 15.527565 46.122299, 15.527565 46.122516, 15.527502 46.122749, 15.527502 46.122966, 15.52763 46.123115, 15.528274 46.123394, 15.528595 46.12363, 15.528681 46.123825, 15.528468 46.124168, 15.528595 46.124359, 15.528918 46.124641, 15.529282 46.125068, 15.529304 46.125068, 15.529476 46.125217, 15.529625 46.125434, 15.529819 46.125606, 15.529969 46.125648, 15.530613 46.125411, 15.531579 46.125026, 15.532073 46.124855, 15.532523 46.124725, 15.532909 46.124553, 15.533059 46.124122, 15.533209 46.123996, 15.533446 46.123973, 15.534047 46.12408, 15.534089 46.124103, 15.534518 46.124061, 15.535312 46.123954, 15.535741 46.123909, 15.537051 46.123867, 15.538682 46.124145, 15.539303 46.124275, 15.53969 46.124423, 15.540076 46.12466, 15.54029 46.125026, 15.540268 46.125282, 15.540162 46.125583, 15.540119 46.125949, 15.540141 46.126014, 15.540205 46.126033, 15.540333 46.126163, 15.540612 46.126247, 15.540934 46.12627, 15.541363 46.126228, 15.541835 46.12614, 15.542093 46.126075, 15.542801 46.125885, 15.543101 46.125713, 15.543358 46.125434, 15.54353 46.125198, 15.543724 46.125045, 15.543981 46.124919, 15.544302 46.12479, 15.544496 46.124767, 15.544775 46.124683, 15.546728 46.124187, 15.547243 46.124038, 15.548187 46.123802, 15.548594 46.123695, 15.549195 46.123458, 15.549732 46.123287, 15.550204 46.123222, 15.550247 46.12318, 15.550333 46.123138, 15.551277 46.122642, 15.55147 46.122623, 15.551814 46.122516, 15.552071 46.122535, 15.552693 46.123031, 15.553229 46.123352, 15.554281 46.123802, 15.555203 46.124298, 15.555439 46.124488, 15.55559 46.12466, 15.55559 46.12511, 15.555439 46.125778, 15.555203 46.126228, 15.555139 46.126399, 15.555053 46.126678, 15.55486 46.127128, 15.554389 46.127815, 15.554259 46.128223, 15.554217 46.128459, 15.554324 46.128696, 15.554624 46.129081, 15.554796 46.12923, 15.555139 46.129337, 15.555783 46.129467, 15.556405 46.129615, 15.557307 46.129833, 15.557414 46.12981, 15.557692 46.129745, 15.55795 46.129573, 15.558272 46.129272, 15.558465 46.129188, 15.558787 46.129165, 15.559474 46.129253, 15.560246 46.129211, 15.560568 46.129165, 15.561039 46.129188, 15.561448 46.129318, 15.561705 46.129337, 15.564366 46.12936, 15.565073 46.129383, 15.565375 46.129337, 15.565289 46.129165, 15.565159 46.129081, 15.564881 46.128993, 15.564602 46.128845, 15.564666 46.128715, 15.564902 46.128715, 15.565095 46.128738, 15.56561 46.128696, 15.566447 46.128395, 15.566876 46.12833, 15.567648 46.128543, 15.568465 46.128803, 15.568486 46.128803, 15.569044 46.128822, 15.569623 46.128822, 15.571383 46.128887, 15.57179 46.128887, 15.572219 46.128845, 15.572606 46.12878, 15.572906 46.128696, 15.573979 46.128353, 15.574473 46.128242, 15.577261 46.128307, 15.577691 46.128242, 15.577841 46.128501, 15.577841 46.128696, 15.577734 46.128887, 15.577541 46.128952, 15.577305 46.12891, 15.577134 46.128974, 15.577433 46.12923, 15.577862 46.129726, 15.577884 46.129917, 15.577798 46.130069, 15.577691 46.130176, 15.577433 46.130302, 15.577176 46.130413, 15.576746 46.130645, 15.576425 46.130863, 15.576125 46.131118, 15.57591 46.131443, 15.575717 46.131786, 15.575675 46.132106, 15.575675 46.132362, 15.57576 46.132492, 15.575974 46.132621, 15.576275 46.132686, 15.576533 46.132816, 15.576683 46.133007, 15.576768 46.133178, 15.576683 46.133392, 15.576554 46.133567, 15.576361 46.133693, 15.576104 46.133823, 15.575545 46.134037, 15.574966 46.134315, 15.574515 46.134468, 15.57413 46.134552, 15.574687 46.13496, 15.575824 46.135711, 15.576146 46.135906, 15.576554 46.13599, 15.577069 46.136055, 15.577433 46.136119, 15.57782 46.136226, 15.578335 46.13644, 15.578721 46.136463, 15.579815 46.136356, 15.580094 46.136249, 15.580224 46.136138, 15.580437 46.136119, 15.581017 46.136138, 15.581511 46.136184, 15.581875 46.136184, 15.581896 46.13644, 15.581832 46.136978, 15.581725 46.137149, 15.58136 46.13747, 15.581254 46.137664, 15.581317 46.137943, 15.581317 46.138072, 15.581103 46.138179, 15.580845 46.138198, 15.580245 46.138198, 15.580159 46.138286, 15.580116 46.138416, 15.580008 46.138565, 15.579601 46.138694, 15.579407 46.138717, 15.579236 46.138759, 15.579172 46.139144, 15.579172 46.139293, 15.579108 46.139511, 15.578657 46.139854, 15.578635 46.13998, 15.5787 46.140197, 15.578936 46.140495, 15.579429 46.141288, 15.580159 46.142276, 15.580567 46.142726, 15.580717 46.142963, 15.581038 46.143394, 15.581275 46.14365, 15.581553 46.143909, 15.581661 46.143928, 15.58194 46.144123, 15.582669 46.144489, 15.582713 46.144596, 15.582605 46.145153, 15.582583 46.145561, 15.582627 46.145751, 15.58297 46.145881, 15.584343 46.146419, 15.586124 46.146804, 15.586874 46.146999, 15.587368 46.147064, 15.588292 46.14719, 15.588849 46.1473, 15.58915 46.147468, 15.589364 46.147644, 15.589514 46.147811, 15.589621 46.147987, 15.589729 46.148178, 15.590029 46.148586, 15.590222 46.148735, 15.590523 46.148841, 15.590909 46.148906, 15.591897 46.148971, 15.592776 46.149379, 15.593356 46.149703, 15.593548 46.149917, 15.593913 46.150047, 15.5943 46.150154, 15.594599 46.150215, 15.59505 46.15028, 15.595823 46.150409, 15.596338 46.150451, 15.596831 46.150539, 15.59711 46.150669, 15.597368 46.150989, 15.597603 46.151184, 15.597797 46.151268, 15.597947 46.151481, 15.597905 46.151782, 15.597775 46.152214, 15.597689 46.152641, 15.597754 46.153137, 15.597969 46.154167, 15.597991 46.154617, 15.597883 46.154872, 15.597561 46.155967, 15.597517 46.156162, 15.597432 46.156375, 15.597282 46.156547, 15.597024 46.156719, 15.595973 46.157318, 15.595715 46.157447, 15.595114 46.157684, 15.594621 46.157962, 15.594578 46.158027, 15.594599 46.158176, 15.594685 46.158329, 15.595416 46.15895, 15.595629 46.15908, 15.595909 46.159122, 15.596424 46.159164, 15.596724 46.159294, 15.596896 46.159442, 15.597024 46.159595, 15.597067 46.159721, 15.596961 46.159828, 15.595543 46.160579, 15.595179 46.160839, 15.594729 46.161354, 15.594513 46.161525, 15.594342 46.161697, 15.59417 46.161933, 15.594192 46.162319, 15.594278 46.162685, 15.5943 46.163734, 15.594278 46.163906, 15.594364 46.164184, 15.594535 46.164466, 15.594815 46.164699, 15.595007 46.16481, 15.595372 46.16513, 15.595887 46.165538, 15.596188 46.165645, 15.597647 46.165515, 15.597991 46.165515, 15.598506 46.16558, 15.598849 46.165664, 15.599213 46.165775, 15.599986 46.166072, 15.600308 46.166267, 15.600544 46.166461, 15.600758 46.166694, 15.60093 46.166847, 15.601273 46.16706, 15.601531 46.16719, 15.601723 46.16732, 15.601939 46.16751, 15.602046 46.167682, 15.602067 46.1679, 15.601809 46.168392, 15.601767 46.16867, 15.601788 46.168972, 15.601853 46.169315, 15.601616 46.1697, 15.601831 46.170001, 15.601338 46.170536, 15.601058 46.170967, 15.600844 46.171375, 15.600758 46.171695, 15.600629 46.172039, 15.600629 46.172424, 15.601016 46.172813, 15.601359 46.173198, 15.601445 46.173412, 15.601445 46.173583, 15.601359 46.173862, 15.60093 46.175086, 15.600887 46.175151, 15.600672 46.175815, 15.600458 46.176738, 15.600264 46.17781, 15.600222 46.178134, 15.600264 46.178325, 15.600329 46.179035, 15.600415 46.179592, 15.600565 46.180023, 15.600586 46.180301, 15.600565 46.180881, 15.600415 46.181652, 15.600544 46.182533, 15.600779 46.183219, 15.600823 46.183582, 15.600823 46.183818, 15.600779 46.183971, 15.600801 46.184268, 15.600865 46.184463, 15.601101 46.184764, 15.601316 46.184913, 15.601917 46.185192, 15.602025 46.185215, 15.602454 46.185428, 15.603182 46.185729, 15.603505 46.185901, 15.603655 46.186031, 15.603891 46.18605, 15.604063 46.186096, 15.604406 46.18618, 15.604749 46.186222, 15.6055 46.18618, 15.605843 46.18618, 15.606574 46.186309, 15.606809 46.186416, 15.607045 46.186588, 15.607367 46.186889, 15.607667 46.186996, 15.607968 46.187061, 15.608805 46.187126, 15.609341 46.187061, 15.610093 46.18708, 15.610478 46.187187, 15.611294 46.187358, 15.611916 46.187446, 15.612903 46.187469, 15.613739 46.187358, 15.614083 46.187404, 15.614362 46.187488, 15.614642 46.187618, 15.614963 46.187725, 15.615263 46.187725, 15.615693 46.187683, 15.615993 46.187618, 15.616508 46.187553, 15.616894 46.187576, 15.617109 46.18766, 15.617173 46.188262, 15.617302 46.188411, 15.617538 46.188583, 15.617624 46.188755, 15.617753 46.188903, 15.61801 46.189056, 15.618139 46.189098, 15.618332 46.189163, 15.618546 46.189205, 15.61874 46.189376, 15.618997 46.189483, 15.619298 46.18959, 15.619684 46.189655, 15.620005 46.189743, 15.620135 46.189826, 15.620542 46.190299, 15.620736 46.190429, 15.621315 46.190727, 15.621529 46.190902, 15.621615 46.19107, 15.621658 46.191288, 15.621658 46.191543, 15.621636 46.191589, 15.621529 46.191932, 15.621422 46.192123, 15.621615 46.192337, 15.621701 46.192596, 15.621787 46.193023, 15.622001 46.193389, 15.622109 46.193775, 15.622281 46.194034, 15.622387 46.194248, 15.623074 46.194183, 15.623396 46.194206, 15.623932 46.194313, 15.624426 46.194442, 15.625113 46.194549, 15.624404 46.19487, 15.624169 46.195064, 15.623911 46.19532, 15.623675 46.195728, 15.623632 46.196029, 15.62346 46.196502, 15.622988 46.197208, 15.622817 46.197532, 15.622796 46.197788, 15.622859 46.198089, 15.622967 46.198303, 15.623053 46.198646, 15.623031 46.198799, 15.622967 46.198989, 15.622817 46.199161, 15.622538 46.199634, 15.62243 46.199848, 15.622387 46.200019, 15.622408 46.20034, 15.622516 46.200534, 15.622752 46.200771, 15.623267 46.201179, 15.623632 46.201499, 15.623847 46.201824, 15.623975 46.202144, 15.624061 46.202445, 15.624104 46.203086, 15.624039 46.203559, 15.624039 46.203861, 15.624147 46.204441, 15.624276 46.204677, 15.624362 46.204956, 15.624662 46.205513, 15.625091 46.205986, 15.625821 46.206649, 15.626293 46.206974, 15.626658 46.207164, 15.626957 46.207252, 15.627472 46.207317, 15.627816 46.207466, 15.628181 46.207767, 15.628804 46.208431, 15.629383 46.208839, 15.629704 46.209098, 15.630112 46.209354, 15.630091 46.209548, 15.630177 46.209953, 15.630327 46.210533, 15.630498 46.212078, 15.630562 46.212295, 15.630648 46.212467, 15.630778 46.212657, 15.631056 46.212959, 15.631314 46.213153, 15.631506 46.213279, 15.631808 46.21371, 15.631872 46.213924, 15.631915 46.214138, 15.631936 46.214462, 15.631829 46.215148, 15.6317 46.215404, 15.631013 46.216178, 15.630906 46.216415, 15.630734 46.216629, 15.630627 46.216819, 15.630391 46.217056, 15.630112 46.21725, 15.629876 46.217422, 15.62949 46.217765, 15.629468 46.218132, 15.629532 46.218322, 15.62964 46.218517, 15.629812 46.218688, 15.629983 46.219116, 15.630026 46.219459, 15.630091 46.219718, 15.630241 46.220127, 15.630263 46.220642, 15.631335 46.220748, 15.631636 46.220771, 15.632129 46.220855, 15.633439 46.220985, 15.633954 46.221092, 15.634125 46.221199, 15.634211 46.221519, 15.634189 46.222206, 15.633975 46.22298, 15.633889 46.223152, 15.633932 46.223323, 15.634081 46.223453, 15.634253 46.223537, 15.634854 46.223472, 15.635176 46.223472, 15.635434 46.223518, 15.635648 46.223579, 15.636378 46.22388, 15.636893 46.224029, 15.637301 46.22414, 15.637473 46.224224, 15.637772 46.224311, 15.638115 46.224502, 15.638417 46.224803, 15.63863 46.225105, 15.638738 46.225296, 15.638953 46.225704, 15.639425 46.225639, 15.640175 46.225597, 15.640498 46.225597, 15.640862 46.225639, 15.641163 46.225402, 15.641549 46.225147, 15.641871 46.225017, 15.642257 46.224952, 15.642537 46.224998, 15.642944 46.22504, 15.643416 46.225059, 15.643931 46.225212, 15.644918 46.225555, 15.645369 46.225791, 15.645583 46.225963, 15.645884 46.2262, 15.64612 46.226432, 15.646399 46.226669, 15.647557 46.227508, 15.647793 46.227699, 15.648243 46.227981, 15.648523 46.228214, 15.648974 46.228729, 15.649252 46.228965, 15.649982 46.229461, 15.650218 46.22974, 15.650433 46.229911, 15.650733 46.230148, 15.650926 46.230297, 15.651162 46.230426, 15.65157 46.230663, 15.651934 46.23077, 15.652579 46.230918, 15.653222 46.230983, 15.653587 46.23096, 15.653695 46.230941, 15.653994 46.230896, 15.654553 46.23077, 15.654938 46.23077, 15.655303 46.230834, 15.655776 46.23096, 15.656205 46.231048, 15.656569 46.231155, 15.656891 46.231304, 15.657128 46.231475, 15.657214 46.231884, 15.657406 46.232292, 15.657599 46.232528, 15.657856 46.2327, 15.658844 46.233215, 15.659123 46.233428, 15.65938 46.233688, 15.659659 46.234031, 15.659938 46.234458, 15.660218 46.235015, 15.660389 46.235855, 15.660625 46.236797, 15.660561 46.237014, 15.660431 46.237228, 15.660946 46.23727, 15.661419 46.237228, 15.661784 46.237312, 15.66217 46.237506, 15.66247 46.23772, 15.662985 46.23817, 15.663414 46.238624, 15.663543 46.238792, 15.663779 46.239009, 15.663994 46.239135, 15.664252 46.239181, 15.664702 46.2392, 15.664938 46.239288, 15.665045 46.239479, 15.665066 46.239974, 15.665131 46.240188, 15.665217 46.240341, 15.665432 46.240489, 15.665689 46.240596, 15.666054 46.240684, 15.666676 46.240745, 15.667555 46.240745, 15.66807 46.240917, 15.66835 46.240962, 15.668671 46.241004, 15.669079 46.240917, 15.669423 46.240833, 15.669745 46.24081, 15.670195 46.240833, 15.670667 46.240898, 15.671053 46.241027, 15.671462 46.241195, 15.67174 46.241413, 15.671977 46.241733, 15.672126 46.242012, 15.672341 46.242164, 15.672598 46.24229, 15.672985 46.242527, 15.673392 46.242679, 15.673735 46.242656, 15.673972 46.242679, 15.67438 46.242805, 15.674615 46.242851, 15.674851 46.242744, 15.675452 46.242527, 15.675753 46.242507, 15.676032 46.242591, 15.676289 46.242763, 15.676504 46.24287, 15.676804 46.242935, 15.677019 46.242935, 15.677319 46.242893, 15.6785 46.242851, 15.678778 46.242958, 15.679122 46.243148, 15.679336 46.24332, 15.679594 46.243598, 15.679743 46.243709, 15.680709 46.244224, 15.681181 46.244503, 15.681611 46.244693, 15.682019 46.244911, 15.682426 46.245231, 15.682576 46.24538, 15.682984 46.245895, 15.683306 46.24641, 15.683799 46.247528, 15.684165 46.247955, 15.684529 46.248298, 15.684529 46.248493, 15.684271 46.248901, 15.683907 46.249267, 15.683628 46.249629, 15.683585 46.249717, 15.683585 46.249824, 15.683885 46.249824, 15.684165 46.249843, 15.684765 46.249973, 15.684958 46.250061, 15.685387 46.250251, 15.685837 46.250423, 15.686417 46.250724, 15.686975 46.250854, 15.68719 46.250854, 15.687426 46.250919, 15.687619 46.250938, 15.687769 46.251239, 15.688134 46.251583, 15.688284 46.251689, 15.68837 46.251838, 15.688456 46.25214, 15.688434 46.252399, 15.688456 46.252571, 15.688563 46.252677, 15.688971 46.252868, 15.689207 46.253086, 15.68925 46.253276, 15.689292 46.253944, 15.689442 46.254241, 15.689979 46.254608, 15.690194 46.254886, 15.690558 46.254585, 15.690837 46.254543, 15.691588 46.254543, 15.691975 46.254501, 15.692446 46.254417, 15.693005 46.254135, 15.693219 46.254009, 15.69339 46.253898, 15.693606 46.25373, 15.693949 46.253192, 15.69397 46.253128, 15.694121 46.252956, 15.69455 46.252655, 15.695021 46.252525, 15.695364 46.252464, 15.695858 46.252399, 15.697017 46.252098, 15.69764 46.251991, 15.698111 46.251949, 15.698712 46.251861, 15.698991 46.251838, 15.699227 46.251796, 15.699484 46.251712, 15.699635 46.251625, 15.699872 46.25156, 15.700193 46.25156, 15.700408 46.251625, 15.700622 46.25167, 15.700837 46.251819, 15.701051 46.251838, 15.701073 46.252761, 15.701159 46.253192, 15.701309 46.253494, 15.702081 46.255016, 15.702189 46.255271, 15.702317 46.256176, 15.702446 46.257331, 15.70251 46.25772, 15.702532 46.258255, 15.702467 46.258556, 15.702361 46.258769, 15.702145 46.259071, 15.702059 46.259265, 15.701995 46.259502, 15.701995 46.260078, 15.701995 46.260166, 15.70191 46.260509, 15.701888 46.260746, 15.701995 46.261283, 15.702167 46.26139, 15.702403 46.261581, 15.703003 46.26197, 15.703518 46.262249, 15.70427 46.262741, 15.705536 46.263214, 15.70575 46.263385, 15.70575 46.263534, 15.705794 46.263706, 15.705901 46.263965, 15.705901 46.264156, 15.7053 46.265251, 15.705235 46.265487, 15.705149 46.265617, 15.704957 46.265808, 15.704678 46.266109, 15.704806 46.266174, 15.704978 46.266151, 15.705514 46.265831, 15.705664 46.265701, 15.705858 46.265445, 15.705987 46.265228, 15.706179 46.26506, 15.706652 46.264843, 15.706888 46.264759, 15.707081 46.264717, 15.707403 46.264717, 15.707638 46.264778, 15.707832 46.264865, 15.708004 46.264949, 15.708067 46.265014, 15.708111 46.265102, 15.708153 46.265338, 15.708218 46.26551, 15.708369 46.265682, 15.708647 46.265766, 15.709033 46.265766, 15.709205 46.265853, 15.709463 46.266132, 15.709892 46.266517, 15.710213 46.266712, 15.710557 46.266819, 15.710879 46.266883, 15.711115 46.266967, 15.711888 46.267181, 15.712252 46.267246, 15.712424 46.267311, 15.712531 46.26744, 15.712574 46.267463, 15.71266 46.267612, 15.712724 46.267868, 15.712681 46.268428, 15.712746 46.269264, 15.712702 46.269565, 15.712403 46.270057, 15.712295 46.270294, 15.712252 46.270465, 15.71208 46.271022, 15.712016 46.27143, 15.712166 46.27246, 15.712145 46.273254, 15.712016 46.27375, 15.711651 46.274478, 15.711566 46.274692, 15.711501 46.274929, 15.711459 46.275444, 15.711351 46.27568, 15.711179 46.275917, 15.710965 46.27613, 15.710686 46.276279, 15.710235 46.276451, 15.709935 46.276603, 15.709849 46.27671, 15.709806 46.276859, 15.70987 46.277267, 15.70987 46.277545, 15.709677 46.278274, 15.709698 46.278682, 15.709806 46.279327, 15.70987 46.279479, 15.71 46.279647, 15.710106 46.279842, 15.710343 46.280334, 15.710493 46.280616, 15.71103 46.281303, 15.711437 46.28173, 15.711974 46.282333, 15.712403 46.28276, 15.71266 46.283126, 15.713282 46.284133, 15.713561 46.284542, 15.713754 46.284843, 15.714655 46.2863, 15.714934 46.287075, 15.715106 46.287719, 15.715256 46.288169, 15.715514 46.288639, 15.715706 46.288856, 15.715986 46.289028, 15.716866 46.289497, 15.71708 46.289627, 15.717295 46.289798, 15.717402 46.289928, 15.717488 46.290123, 15.717639 46.290592, 15.717766 46.290958, 15.717852 46.291194, 15.717938 46.29158, 15.71811 46.292053, 15.718153 46.292182, 15.718346 46.292331, 15.718947 46.292526, 15.719183 46.292633, 15.719311 46.292739, 15.719569 46.292976, 15.720128 46.293556, 15.720578 46.294067, 15.72077 46.294349, 15.720964 46.294605, 15.721221 46.294883, 15.721264 46.295055, 15.721264 46.295207, 15.721136 46.295463, 15.720942 46.295764, 15.720835 46.29602, 15.720792 46.296257, 15.720728 46.296428, 15.720599 46.296966, 15.720363 46.297267, 15.720191 46.297416, 15.719848 46.297588, 15.719333 46.297801, 15.718732 46.298103, 15.718625 46.29821, 15.718539 46.298316, 15.718497 46.298446, 15.718432 46.298748, 15.718389 46.29911, 15.718281 46.299926, 15.718068 46.300357, 15.717982 46.300655, 15.717938 46.300914, 15.717982 46.3013, 15.718046 46.301731, 15.718195 46.302352, 15.718195 46.303512, 15.718389 46.304306, 15.718367 46.304775, 15.718325 46.304927, 15.718003 46.305419, 15.717831 46.30585, 15.717467 46.306407, 15.717273 46.306644, 15.717209 46.306793, 15.717187 46.306945, 15.717273 46.30735, 15.717316 46.307437, 15.717702 46.307975, 15.71811 46.308662, 15.71826 46.308982, 15.718497 46.309753, 15.718604 46.309989, 15.718796 46.310184, 15.719033 46.310485, 15.719119 46.310741, 15.719269 46.311599, 15.719269 46.311729, 15.719054 46.312606, 15.719012 46.313125, 15.719012 46.313423, 15.718947 46.313594, 15.718775 46.313961, 15.718647 46.314109, 15.718497 46.314369, 15.718411 46.314666, 15.718432 46.315441, 15.718411 46.315696, 15.718432 46.315807, 15.718453 46.316535, 15.718497 46.317008, 15.718518 46.31705, 15.718518 46.317543, 15.718583 46.318336, 15.718539 46.318466, 15.717874 46.319068, 15.717724 46.319259, 15.717595 46.319538, 15.717445 46.320205, 15.717359 46.320377, 15.717209 46.320568, 15.717209 46.32072, 15.717251 46.320934, 15.717359 46.321212, 15.717488 46.321426, 15.718389 46.322521, 15.718775 46.323101, 15.718926 46.323249, 15.719333 46.323509, 15.719634 46.323745, 15.719826 46.324001, 15.719956 46.32426, 15.720084 46.324604, 15.720149 46.324924, 15.720149 46.32516, 15.720191 46.325332, 15.720255 46.325462, 15.720385 46.325611, 15.720557 46.32574, 15.720814 46.325889, 15.720964 46.326019, 15.720986 46.326126, 15.721479 46.326515, 15.722251 46.327026, 15.722487 46.327156, 15.722595 46.32735, 15.722744 46.328037, 15.72283 46.328231, 15.723024 46.328487, 15.723689 46.328937, 15.724118 46.329174, 15.724698 46.32941, 15.725234 46.329559, 15.725642 46.329753, 15.726307 46.330032, 15.728388 46.33076, 15.729719 46.331211, 15.730277 46.331447, 15.730792 46.331707, 15.731543 46.332027, 15.732401 46.332435, 15.732787 46.332672, 15.733044 46.332885, 15.733238 46.333099, 15.733495 46.333465, 15.73371 46.333679, 15.734826 46.33443, 15.735234 46.334774, 15.735749 46.335266, 15.735899 46.33546, 15.736134 46.335891, 15.736328 46.336189, 15.736456 46.336383, 15.736757 46.336875, 15.7368 46.33707, 15.736821 46.338165, 15.736757 46.339496, 15.736714 46.339752, 15.736628 46.339946, 15.736371 46.340202, 15.735705 46.340633, 15.735491 46.340869, 15.735341 46.341102, 15.735276 46.341403, 15.735255 46.341789, 15.735298 46.342113, 15.735362 46.342285, 15.735512 46.342651, 15.735663 46.343078, 15.735899 46.343593, 15.736092 46.343936, 15.736285 46.34415, 15.736628 46.344409, 15.736907 46.344688, 15.73738 46.345333, 15.738709 46.347412, 15.738688 46.347625, 15.738473 46.347843, 15.737787 46.348377, 15.737294 46.348678, 15.736757 46.348957, 15.735941 46.349323, 15.735619 46.349517, 15.734954 46.349861, 15.734053 46.350288, 15.734547 46.351017, 15.735018 46.351619, 15.735384 46.351833, 15.736199 46.352199, 15.736714 46.35239, 15.736843 46.352455, 15.7371 46.352649, 15.73725 46.352821, 15.73738 46.353229, 15.737465 46.353313, 15.737572 46.353378, 15.737723 46.353378, 15.737851 46.353294, 15.737895 46.353164, 15.737851 46.353057, 15.737572 46.352756, 15.737507 46.352626, 15.737507 46.352478, 15.737787 46.35239, 15.737916 46.352455, 15.738022 46.352561, 15.738108 46.352863, 15.738238 46.353118, 15.738581 46.353508, 15.738925 46.353851, 15.739869 46.354728, 15.740168 46.354881, 15.74049 46.355011, 15.740791 46.355072, 15.741112 46.355117, 15.741735 46.355159, 15.742014 46.355159, 15.742593 46.355201, 15.743388 46.355224, 15.743859 46.355266, 15.744504 46.355289, 15.745683 46.355289, 15.74667 46.355266, 15.747206 46.355373, 15.748473 46.355781, 15.749053 46.355846, 15.750318 46.355865, 15.750855 46.35593, 15.751305 46.355995, 15.751906 46.356189, 15.752464 46.356319, 15.752936 46.356384, 15.75315 46.356384, 15.753344 46.356403, 15.754524 46.356403, 15.754845 46.356384, 15.755061 46.356319, 15.755854 46.356018, 15.756348 46.355976, 15.756648 46.35606, 15.756991 46.356189, 15.757464 46.356426, 15.757678 46.356552, 15.757721 46.356662, 15.757678 46.356788, 15.757528 46.356853, 15.757356 46.356876, 15.757163 46.356788, 15.756991 46.356662, 15.756863 46.356533, 15.756733 46.356468, 15.756455 46.356491, 15.756304 46.356575, 15.756283 46.356727, 15.756455 46.357025, 15.75652 46.357219, 15.756541 46.357414, 15.756498 46.358055, 15.756584 46.358333, 15.756733 46.358593, 15.756819 46.358871, 15.756712 46.359043, 15.756434 46.359172, 15.756283 46.359214, 15.756197 46.359214, 15.756026 46.359279, 15.755811 46.359298, 15.755511 46.359451, 15.755404 46.359622, 15.755511 46.359859, 15.755725 46.359966, 15.757056 46.360115, 15.758494 46.360439, 15.759181 46.360652, 15.759352 46.360694, 15.759737 46.360889, 15.760146 46.361145, 15.760382 46.361297, 15.760424 46.361358, 15.760532 46.361446, 15.760853 46.361789, 15.761047 46.36211, 15.761433 46.362949, 15.761648 46.363269, 15.761991 46.363613, 15.762463 46.363937, 15.763321 46.364341, 15.763686 46.364536, 15.764029 46.364772, 15.764266 46.364986, 15.764588 46.365394, 15.764866 46.365844, 15.765402 46.366809, 15.765574 46.367088, 15.765618 46.367195, 15.767462 46.370372, 15.767785 46.370845, 15.768171 46.371231, 15.7686 46.371509, 15.769201 46.371768, 15.769995 46.371959, 15.770832 46.372131, 15.77272 46.372474, 15.773407 46.372627, 15.773686 46.372669, 15.774329 46.372818, 15.774973 46.372989, 15.776625 46.373504, 15.777205 46.373657, 15.777483 46.373741, 15.778191 46.373847, 15.778814 46.373912, 15.779135 46.373977, 15.779608 46.374126, 15.779286 46.374492, 15.779265 46.374687, 15.7792 46.374923, 15.779179 46.375694, 15.779135 46.376232, 15.779135 46.376701, 15.780594 46.376789, 15.781325 46.376895, 15.782419 46.377025, 15.783814 46.377262, 15.784522 46.377326, 15.784951 46.377452, 15.786968 46.378719, 15.787333 46.378955, 15.788255 46.379512, 15.78877 46.379791, 15.790015 46.380393, 15.79098 46.380908, 15.791259 46.381038, 15.792161 46.381553, 15.792525 46.38179, 15.793877 46.382667, 15.795058 46.383506, 15.795529 46.383934, 15.79583 46.384147, 15.796624 46.384815, 15.797246 46.385284, 15.798405 46.385993, 15.798577 46.386123, 15.798577 46.386852, 15.798619 46.38726, 15.798577 46.38919, 15.798598 46.390178, 15.798577 46.390777, 15.798619 46.39151, 15.798598 46.392303, 15.798641 46.392581, 15.798641 46.39318, 15.798619 46.393505, 15.798619 46.394725, 15.798619 46.395412, 15.798663 46.39569, 15.798641 46.396293, 15.798641 46.39653, 15.798577 46.396808, 15.798641 46.397281, 15.798619 46.397945, 15.798663 46.39949, 15.798641 46.400283, 15.798555 46.400585, 15.798619 46.401077, 15.798641 46.403076, 15.798684 46.403289, 15.798684 46.403846, 15.798727 46.404083, 15.798727 46.404319, 15.798705 46.404533, 15.798749 46.40464, 15.79879 46.404834, 15.798898 46.405006, 15.79892 46.405262, 15.79907 46.405971, 15.799199 46.406291, 15.799391 46.406936, 15.799477 46.407196, 15.799563 46.407646, 15.799799 46.408332, 15.799906 46.40876, 15.800036 46.409103, 15.800186 46.409725, 15.80025 46.410156, 15.80025 46.410327, 15.800379 46.411376, 15.800379 46.411849, 15.801774 46.411914, 15.804091 46.412021, 15.805293 46.412109, 15.806944 46.41217, 15.807782 46.412216, 15.808575 46.412216, 15.811644 46.412281, 15.812202 46.412323, 15.81482 46.412494, 15.817803 46.412624, 15.818661 46.412689, 15.820914 46.412837, 15.822651 46.412837, 15.823575 46.412773, 15.82424 46.41275, 15.825549 46.41275, 15.825957 46.412796, 15.826815 46.412879, 15.827544 46.412967, 15.827995 46.412986, 15.83042 46.413051, 15.832608 46.413093, 15.832865 46.413116, 15.833638 46.413116, 15.834389 46.413158, 15.835055 46.413158, 15.837651 46.413223, 15.837951 46.413246, 15.838231 46.413265, 15.838724 46.413311, 15.839732 46.413459, 15.840033 46.413482, 15.840483 46.413566, 15.841362 46.413696, 15.841642 46.413719, 15.841963 46.41378, 15.842157 46.413845, 15.843295 46.414405, 15.844388 46.414833, 15.845376 46.415111, 15.845784 46.415153, 15.846299 46.415176, 15.847006 46.415176, 15.847564 46.415176, 15.847542 46.415348, 15.847564 46.415584, 15.848637 46.415626, 15.848551 46.415927, 15.848572 46.416183, 15.848423 46.416572, 15.84883 46.416698, 15.849001 46.416721, 15.849688 46.416934, 15.850311 46.417087, 15.850482 46.417152, 15.850869 46.417301, 15.851169 46.417385, 15.851169 46.417472, 15.851384 46.417728, 15.851662 46.417793, 15.852049 46.417839, 15.852242 46.417881, 15.852736 46.417922, 15.853143 46.418006, 15.853444 46.418094, 15.854259 46.418308, 15.854624 46.41835, 15.855418 46.418479, 15.855847 46.418415, 15.856104 46.418308, 15.856898 46.417922, 15.857349 46.417793, 15.858078 46.417644, 15.858357 46.417556, 15.858572 46.41743, 15.858765 46.417259, 15.859044 46.417106, 15.859688 46.417171, 15.85986 46.417041, 15.860032 46.416786, 15.860632 46.416828, 15.860997 46.416744, 15.86179 46.416378, 15.862521 46.416076, 15.863786 46.415691, 15.864066 46.415561, 15.864193 46.41539, 15.864301 46.415153, 15.864344 46.415111, 15.864473 46.415027, 15.86473 46.414962, 15.866297 46.415004, 15.866833 46.415092, 15.867241 46.415218, 15.867563 46.415348, 15.867906 46.415542, 15.868292 46.415798, 15.868378 46.415885, 15.870266 46.417343, 15.870567 46.417514, 15.87091 46.417602, 15.871554 46.417663, 15.871747 46.417663, 15.871747 46.417449, 15.871918 46.417301, 15.87222 46.417259, 15.872735 46.417449, 15.873206 46.417816, 15.874108 46.418567, 15.874493 46.418846, 15.874665 46.419017, 15.875052 46.419338, 15.876339 46.420326, 15.877284 46.421314, 15.877476 46.421569, 15.879322 46.423717, 15.880415 46.424854, 15.881081 46.425476, 15.882519 46.426765, 15.884042 46.428394, 15.885501 46.429103, 15.885759 46.42921, 15.886166 46.429336, 15.886317 46.429359, 15.886639 46.429382, 15.887132 46.429382, 15.887326 46.429554, 15.887841 46.429702, 15.888119 46.429725, 15.888463 46.429874, 15.888591 46.429981, 15.888871 46.430133, 15.889386 46.430603, 15.890008 46.431633, 15.890416 46.431785, 15.890931 46.432022, 15.890372 46.433887, 15.890308 46.434101, 15.889965 46.435325, 15.889879 46.43571, 15.889879 46.435752, 15.889922 46.436439, 15.889922 46.436912, 15.889922 46.437107, 15.889943 46.437858, 15.889922 46.438373, 15.890158 46.439273, 15.890416 46.44009, 15.890458 46.440345, 15.890416 46.440948, 15.890308 46.441291, 15.890179 46.44157, 15.890179 46.441612, 15.889857 46.442298, 15.889621 46.442901, 15.889535 46.44318, 15.889493 46.443523, 15.88945 46.444423, 15.889472 46.444831, 15.889599 46.445365, 15.890372 46.447063, 15.890673 46.44749, 15.890909 46.447727, 15.891381 46.448112, 15.89181 46.448413, 15.89211 46.448673, 15.892304 46.448951, 15.892476 46.449295, 15.892582 46.449703, 15.892711 46.45232, 15.892754 46.453693, 15.892775 46.454055, 15.892711 46.454528, 15.892668 46.454723, 15.892346 46.455303, 15.889578 46.458648, 15.889042 46.459251, 15.887927 46.460346, 15.887625 46.460666, 15.887368 46.461032, 15.886639 46.462383, 15.886509 46.462638, 15.886231 46.463092, 15.88608 46.463413, 15.885994 46.463668, 15.885952 46.464508, 15.885823 46.465, 15.885308 46.465709, 15.885266 46.465793, 15.884257 46.467082, 15.883935 46.467556, 15.883184 46.468521, 15.882863 46.468994, 15.882819 46.469055, 15.882733 46.469337, 15.882712 46.469593, 15.882777 46.469894, 15.882948 46.470279, 15.883206 46.470623, 15.883677 46.471397, 15.884064 46.471996, 15.884192 46.472145, 15.884923 46.473304, 15.885072 46.473541, 15.88518 46.473861, 15.885222 46.474311, 15.885201 46.47528, 15.885201 46.475494, 15.885158 46.476371, 15.885158 46.476932, 15.885222 46.477233, 15.885308 46.477447, 15.885544 46.477855, 15.886038 46.478282, 15.88739 46.479164, 15.888054 46.479572, 15.888935 46.480148, 15.889278 46.480384, 15.889493 46.480579, 15.889664 46.480793, 15.889793 46.481029, 15.889901 46.481502, 15.889815 46.481822, 15.889664 46.482273, 15.889278 46.483024, 15.889128 46.483413, 15.88902 46.483924, 15.888999 46.48442, 15.889063 46.484741, 15.889535 46.486095, 15.889621 46.486373, 15.889707 46.486801, 15.88975 46.487167, 15.88975 46.487701, 15.889815 46.488174, 15.890028 46.488967, 15.890114 46.489547, 15.890028 46.490322, 15.889857 46.491245, 15.889879 46.491416, 15.889943 46.491542, 15.890093 46.491867, 15.890372 46.492187, 15.890629 46.492401, 15.890952 46.492553, 15.891424 46.492725, 15.892368 46.492939, 15.893655 46.493259, 15.894707 46.493495, 15.895308 46.493667, 15.896122 46.493991, 15.896831 46.494442, 15.897861 46.495277, 15.898204 46.495471, 15.899085 46.495815, 15.900586 46.496349, 15.901251 46.496627, 15.902302 46.497165, 15.903761 46.498195, 15.903912 46.498344, 15.904019 46.498497, 15.904041 46.498798, 15.903869 46.499141, 15.903376 46.499935, 15.902604 46.501049, 15.901745 46.502403, 15.901337 46.502895, 15.901272 46.503025, 15.901101 46.503559, 15.900779 46.504032, 15.900372 46.504505, 15.900221 46.504848, 15.900221 46.504978, 15.900115 46.505363, 15.8996 46.506156, 15.898869 46.507293, 15.897711 46.508754, 15.897389 46.509052, 15.896788 46.509506, 15.896187 46.509975, 15.896016 46.510211, 15.896101 46.510597, 15.896273 46.510879, 15.896294 46.511047, 15.896445 46.51152, 15.896552 46.512077, 15.897518 46.514354, 15.897646 46.514545, 15.898011 46.515213, 15.898419 46.515598, 15.898591 46.515834, 15.898591 46.516716, 15.898548 46.517036, 15.89857 46.517959, 15.898484 46.51815, 15.89844 46.518367, 15.898333 46.518516, 15.898182 46.518623, 15.897839 46.519073, 15.897732 46.519332, 15.897646 46.520339, 15.897582 46.520813, 15.897496 46.521778, 15.897518 46.522014, 15.897667 46.522506, 15.897883 46.522979, 15.897883 46.523174, 15.897925 46.523365, 15.898161 46.523643, 15.89844 46.523838, 15.898827 46.52433, 15.899085 46.524612, 15.899727 46.525039, 15.900115 46.525211, 15.900479 46.525341, 15.900822 46.525489, 15.90123 46.52581, 15.901509 46.526004, 15.901787 46.526218, 15.902216 46.526496, 15.902604 46.526649, 15.902925 46.526798, 15.903311 46.52695, 15.90359 46.527122, 15.903933 46.527359, 15.904234 46.527614, 15.904642 46.527893, 15.905328 46.528194, 15.905779 46.52843, 15.906702 46.529052, 15.907302 46.529567, 15.907646 46.529739, 15.908482 46.530105, 15.90889 46.530254, 15.90934 46.530384, 15.909941 46.530616, 15.910328 46.530811, 15.910993 46.531005, 15.911465 46.531047, 15.912774 46.531112, 15.913268 46.531177, 15.913676 46.531242, 15.914126 46.531368, 15.914919 46.531562, 15.915736 46.531627, 15.918074 46.531692, 15.918911 46.531646, 15.919211 46.531711, 15.920091 46.531948, 15.920928 46.5321, 15.921336 46.532142, 15.922173 46.532291, 15.922431 46.532356, 15.922902 46.532485, 15.923331 46.532634, 15.923718 46.532741, 15.924189 46.532806, 15.925305 46.533107, 15.925606 46.533195, 15.925842 46.533302, 15.925971 46.533386, 15.926249 46.533493, 15.926571 46.533515, 15.926894 46.533473, 15.927237 46.533321, 15.927558 46.533084, 15.928181 46.532485, 15.928545 46.532272, 15.928846 46.53199, 15.929747 46.531368, 15.930262 46.531089, 15.930563 46.530983, 15.930949 46.530899, 15.931292 46.530899, 15.931678 46.530941, 15.932923 46.531154, 15.933417 46.531219, 15.934361 46.531261, 15.934876 46.531326, 15.937365 46.531497, 15.937451 46.531562, 15.937515 46.531646, 15.937665 46.531776, 15.937836 46.532012, 15.938244 46.532741, 15.938416 46.533172, 15.938674 46.534114, 15.938716 46.534439, 15.938674 46.534912, 15.938523 46.535812, 15.938373 46.536304, 15.938309 46.536712, 15.938437 46.537315, 15.938416 46.537784, 15.938437 46.537914, 15.938502 46.538085, 15.938653 46.53828, 15.938824 46.538578, 15.93891 46.538837, 15.938952 46.539115, 15.938845 46.539524, 15.938481 46.539997, 15.938244 46.54021, 15.938116 46.540405, 15.937901 46.540855, 15.937858 46.541152, 15.93788 46.541324, 15.937987 46.541648, 15.938459 46.542377, 15.938588 46.542633, 15.938759 46.543384, 15.938824 46.543621, 15.93891 46.543815, 15.938931 46.543899, 15.939274 46.544288, 15.939597 46.544502, 15.940455 46.54523, 15.940712 46.54549, 15.940819 46.545661, 15.940905 46.545833, 15.94097 46.546089, 15.941206 46.547721, 15.941313 46.548, 15.941549 46.54845, 15.94187 46.549072, 15.942 46.549243, 15.942235 46.549373, 15.942493 46.549392, 15.94275 46.549499, 15.9429 46.549629, 15.942986 46.549781, 15.942986 46.549953, 15.9429 46.550102, 15.942729 46.550251, 15.942407 46.550468, 15.942128 46.550724, 15.941784 46.551368, 15.941678 46.551498, 15.941248 46.551841, 15.940733 46.552204, 15.940519 46.552421, 15.94039 46.55257, 15.940325 46.552677, 15.940176 46.553192, 15.940133 46.553493, 15.940133 46.553985, 15.940068 46.554714, 15.94009 46.555038, 15.940026 46.555511, 15.940004 46.555702, 15.940004 46.556346, 15.940026 46.556755, 15.94009 46.556991, 15.940176 46.557548, 15.940261 46.557785, 15.940562 46.558235, 15.940691 46.558471, 15.940819 46.558792, 15.940862 46.559093, 15.941012 46.559501, 15.941163 46.559799, 15.94142 46.560165, 15.941592 46.560272, 15.942235 46.560485, 15.942364 46.56055, 15.943137 46.560916, 15.944252 46.561496, 15.944767 46.56184, 15.944939 46.562034, 15.945433 46.562721, 15.945519 46.562999, 15.945475 46.563232, 15.945368 46.563575, 15.945282 46.563728, 15.945004 46.563919, 15.944767 46.564155, 15.94406 46.565059, 15.943265 46.565788, 15.942879 46.566238, 15.942965 46.566452, 15.942965 46.567054, 15.943137 46.567333, 15.943351 46.567611, 15.943738 46.56789, 15.944403 46.568275, 15.944725 46.568428, 15.945046 46.568511, 15.946076 46.568599, 15.947042 46.568725, 15.947643 46.568901, 15.948136 46.569007, 15.948544 46.569007, 15.949124 46.568985, 15.949145 46.569133, 15.949252 46.569263, 15.949595 46.569412, 15.950068 46.569713, 15.950325 46.56995, 15.950539 46.570209, 15.950797 46.570465, 15.951098 46.570701, 15.951397 46.570873, 15.951741 46.571022, 15.952192 46.571109, 15.952535 46.571128, 15.9529 46.571086, 15.953222 46.570961, 15.953372 46.570808, 15.953372 46.570617, 15.953222 46.570163, 15.953351 46.570037, 15.953651 46.569885, 15.953951 46.569778, 15.95423 46.569736, 15.954509 46.56982, 15.954767 46.569973, 15.955132 46.570121, 15.955946 46.570423, 15.956269 46.570594, 15.956633 46.570831, 15.956784 46.570938, 15.956934 46.571128, 15.956998 46.571346, 15.956998 46.571601, 15.956913 46.571815, 15.956376 46.572246, 15.956183 46.57246, 15.956054 46.572696, 15.956032 46.572933, 15.956097 46.573188, 15.956461 46.573791, 15.956461 46.57394, 15.95687 46.574218, 15.957148 46.574371, 15.957491 46.574649, 15.957857 46.575058, 15.95805 46.575229, 15.958136 46.575271, 15.958329 46.575424, 15.95893 46.575767, 15.959445 46.57611, 15.95953 46.576129, 15.959874 46.576278, 15.960152 46.576343, 15.960495 46.576343, 15.961118 46.576278, 15.961504 46.576301, 15.961869 46.576366, 15.962148 46.576473, 15.962428 46.576667, 15.96262 46.576881, 15.962813 46.577159, 15.963156 46.577888, 15.963286 46.578319, 15.9632 46.578857, 15.963049 46.579177, 15.96247 46.579929, 15.962212 46.58023, 15.962212 46.580398, 15.962342 46.580551, 15.96335 46.580894, 15.963908 46.581043, 15.964787 46.581321, 15.965346 46.581665, 15.965539 46.581836, 15.96571 46.582115, 15.965925 46.582309, 15.966246 46.582611, 15.966955 46.583061, 15.967276 46.58332, 15.968886 46.584392, 15.969573 46.5848, 15.970109 46.585079, 15.970881 46.585441, 15.971675 46.585872, 15.971955 46.586067, 15.972254 46.586235, 15.97247 46.586322, 15.973113 46.586517, 15.973607 46.586624, 15.973843 46.586708, 15.973971 46.586814, 15.974015 46.587051, 15.973971 46.587696, 15.974015 46.587951, 15.974186 46.588104, 15.974508 46.588233, 15.976053 46.588684, 15.976482 46.588855, 15.976889 46.589069, 15.977105 46.589263, 15.977362 46.589454, 15.977984 46.589885, 15.978284 46.590141, 15.978542 46.590293, 15.978842 46.59053, 15.9794 46.590934, 15.980109 46.59143, 15.980452 46.591728, 15.980731 46.592159, 15.980923 46.592609, 15.981053 46.592994, 15.98146 46.592975, 15.981867 46.592697, 15.982275 46.592544, 15.983391 46.592525, 15.983971 46.592544, 15.984572 46.592544, 15.985215 46.592502, 15.985901 46.592479, 15.98646 46.59246, 15.986889 46.592414, 15.987382 46.592414, 15.988026 46.592308, 15.989464 46.592071, 15.990386 46.591903, 15.991009 46.591728, 15.991545 46.591537, 15.992489 46.591129, 15.993004 46.590957, 15.994227 46.590442, 15.994978 46.590164, 15.995644 46.58982, 15.996201 46.589561, 15.996974 46.589111, 15.997768 46.588703, 15.998025 46.588512, 15.998798 46.588188, 15.999227 46.588039, 15.999421 46.588016, 16 46.58789, 16.000299 46.587825, 16.000835 46.587783, 16.001607 46.58776, 16.002016 46.587802, 16.002487 46.587951, 16.003883 46.588684, 16.004333 46.588874, 16.004611 46.588962, 16.005084 46.589004, 16.005493 46.589069, 16.006071 46.589199, 16.006629 46.589241, 16.006973 46.589992, 16.007122 46.590229, 16.007165 46.590549, 16.00708 46.590827, 16.007038 46.591152, 16.00708 46.5913, 16.007465 46.591838, 16.007659 46.592182, 16.007831 46.592353, 16.008281 46.592567, 16.008666 46.592632, 16.008968 46.592716, 16.009181 46.592823, 16.009418 46.592975, 16.00959 46.593147, 16.009805 46.593444, 16.010084 46.594047, 16.010255 46.594348, 16.01062 46.594882, 16.010835 46.595142, 16.01107 46.595184, 16.011522 46.595119, 16.011993 46.595012, 16.012144 46.594993, 16.01238 46.594993, 16.01268 46.595035, 16.013473 46.595336, 16.013816 46.59542, 16.014331 46.595462, 16.015127 46.595378, 16.015727 46.595356, 16.015792 46.595699, 16.015964 46.595977, 16.016391 46.596492, 16.016778 46.59703, 16.017036 46.597267, 16.017166 46.597415, 16.017444 46.598083, 16.017593 46.598251, 16.017723 46.598362, 16.017936 46.598468, 16.018838 46.598854, 16.019117 46.599067, 16.019161 46.599239, 16.019138 46.599433, 16.019031 46.59967, 16.018945 46.599948, 16.018903 46.600269, 16.018966 46.600612, 16.019031 46.600807, 16.019117 46.600955, 16.019309 46.601173, 16.019632 46.60128, 16.020256 46.601322, 16.020963 46.601341, 16.021671 46.601749, 16.021907 46.601814, 16.022357 46.601795, 16.023237 46.601387, 16.023559 46.601299, 16.024032 46.60128, 16.024417 46.601341, 16.024826 46.601535, 16.025039 46.601684, 16.025234 46.60186, 16.025705 46.602546, 16.025726 46.60289, 16.025812 46.60308, 16.026134 46.603401, 16.026456 46.603683, 16.026542 46.603809, 16.02667 46.604263, 16.026821 46.604583, 16.027034 46.604885, 16.027442 46.605228, 16.028537 46.606086, 16.028839 46.606384, 16.029031 46.606666, 16.029182 46.607158, 16.029396 46.607608, 16.029502 46.608383, 16.029502 46.60868, 16.029525 46.608894, 16.029632 46.60913, 16.029674 46.609302, 16.029525 46.609947, 16.029567 46.610118, 16.029653 46.61029, 16.030061 46.610893, 16.030275 46.61145, 16.030233 46.612545, 16.030168 46.612888, 16.030017 46.613189, 16.030082 46.614688, 16.030147 46.615032, 16.03034 46.615718, 16.03049 46.616085, 16.030683 46.616428, 16.030834 46.616642, 16.031198 46.617351, 16.031326 46.617671, 16.031757 46.619258, 16.031929 46.61943, 16.032527 46.619796, 16.033 46.62014, 16.033365 46.620483, 16.033323 46.620548, 16.03285 46.621234, 16.032722 46.621597, 16.032764 46.621898, 16.032829 46.622158, 16.032894 46.622306, 16.033237 46.622585, 16.033752 46.622821, 16.034051 46.622909, 16.035554 46.623489, 16.035877 46.623596, 16.036584 46.623744, 16.036884 46.623786, 16.037893 46.624111, 16.038558 46.624473, 16.038879 46.624626, 16.03931 46.624797, 16.039825 46.624946, 16.040962 46.625461, 16.041219 46.62574, 16.041477 46.626106, 16.041906 46.626579, 16.042119 46.626857, 16.042184 46.627029, 16.04227 46.627498, 16.042356 46.628166, 16.042377 46.628681, 16.042291 46.629043, 16.041969 46.629753, 16.04169 46.630054, 16.04124 46.630439, 16.041069 46.630634, 16.040983 46.630847, 16.040897 46.631191, 16.040683 46.632091, 16.040618 46.632564, 16.040468 46.63327, 16.04034 46.633659, 16.040231 46.634086, 16.040189 46.634323, 16.040081 46.634624, 16.039953 46.635162, 16.039932 46.635547, 16.039953 46.635784, 16.04006 46.635974, 16.040596 46.636577, 16.041927 46.637626, 16.042442 46.638099, 16.042593 46.638484, 16.042743 46.638938, 16.042915 46.639282, 16.04317 46.639709, 16.043664 46.640117, 16.044052 46.640354, 16.044265 46.64046, 16.044179 46.640655, 16.044029 46.640869, 16.043579 46.641212, 16.043128 46.641403, 16.042572 46.641597, 16.042057 46.641704, 16.041027 46.641769, 16.040683 46.641811, 16.039953 46.641853, 16.039909 46.641941, 16.039802 46.642005, 16.039867 46.642433, 16.039888 46.642799, 16.039888 46.643379, 16.039932 46.643569, 16.040018 46.643745, 16.040252 46.643913, 16.040468 46.643978, 16.041477 46.644195, 16.04182 46.644321, 16.042228 46.644516, 16.042871 46.644924, 16.043493 46.645286, 16.04388 46.645504, 16.044523 46.645889, 16.045724 46.646492, 16.046562 46.646854, 16.046669 46.646919, 16.046775 46.646984, 16.047399 46.64722, 16.048492 46.647689, 16.048879 46.647926, 16.049243 46.648185, 16.049545 46.648464, 16.049781 46.648784, 16.050552 46.649902, 16.050724 46.650157, 16.050983 46.650547, 16.051389 46.650909, 16.05199 46.651275, 16.052398 46.651531, 16.053171 46.651939, 16.05347 46.652069, 16.054115 46.652217, 16.054372 46.652263, 16.054759 46.65237, 16.056669 46.653057, 16.056926 46.653183, 16.057075 46.653312, 16.057163 46.653419, 16.057483 46.654045, 16.05759 46.654193, 16.05772 46.654277, 16.057956 46.654258, 16.058321 46.654045, 16.059114 46.653636, 16.059543 46.653377, 16.059865 46.653228, 16.060144 46.653057, 16.06302 46.651512, 16.063491 46.651275, 16.06553 46.65018, 16.066045 46.649879, 16.067247 46.64928, 16.067806 46.64915, 16.068233 46.649086, 16.069866 46.648765, 16.070293 46.648765, 16.07068 46.648807, 16.070959 46.648895, 16.071239 46.649021, 16.071645 46.649299, 16.071903 46.649429, 16.073169 46.649772, 16.073577 46.649944, 16.074048 46.650115, 16.074628 46.650287, 16.075315 46.650375, 16.075679 46.650394, 16.07656 46.650394, 16.077159 46.650459, 16.077503 46.650566, 16.078041 46.650779, 16.078556 46.650955, 16.079671 46.651187, 16.080186 46.65121, 16.080787 46.65121, 16.081539 46.651145, 16.08259 46.650997, 16.082761 46.650974, 16.082975 46.650974, 16.083168 46.650997, 16.083448 46.651123, 16.08362 46.651298, 16.083984 46.651767, 16.084457 46.652477, 16.084735 46.65282, 16.085014 46.653015, 16.085378 46.653228, 16.085914 46.653678, 16.08613 46.653892, 16.086408 46.654258, 16.086517 46.654365, 16.086666 46.654472, 16.08686 46.654537, 16.087032 46.654537, 16.087245 46.654495, 16.087867 46.6543, 16.088275 46.654151, 16.088897 46.654064, 16.089885 46.654064, 16.090679 46.654129, 16.091516 46.654151, 16.092546 46.654064, 16.092781 46.654087, 16.093339 46.654212, 16.093639 46.654212, 16.094005 46.654151, 16.094305 46.654045, 16.09482 46.653808, 16.095829 46.653419, 16.097352 46.652797, 16.098445 46.652412, 16.099025 46.652111, 16.09967 46.651767, 16.100334 46.651405, 16.100786 46.651233, 16.101535 46.651016, 16.101837 46.65089, 16.10205 46.65076, 16.102331 46.65063, 16.10321 46.650093, 16.103918 46.649688, 16.104261 46.649517, 16.104391 46.649879, 16.104583 46.650268, 16.104797 46.650867, 16.10527 46.651702, 16.105377 46.651939, 16.105421 46.652111, 16.105571 46.65237, 16.105743 46.652584, 16.106258 46.653163, 16.107051 46.653999, 16.107416 46.654342, 16.108253 46.655242, 16.108402 46.655502, 16.108724 46.65591, 16.108854 46.656124, 16.109153 46.65681, 16.109239 46.657218, 16.109582 46.65797, 16.109754 46.658462, 16.109819 46.658569, 16.10984 46.658763, 16.109926 46.658893, 16.110334 46.659835, 16.11042 46.660114, 16.110485 46.66035, 16.110656 46.660675, 16.110784 46.660888, 16.111042 46.661468, 16.111557 46.662841, 16.111686 46.663032, 16.111858 46.663547, 16.112287 46.664623, 16.112415 46.665008, 16.112758 46.666038, 16.112888 46.666275, 16.11306 46.666725, 16.113296 46.667026, 16.113639 46.666938, 16.114498 46.66679, 16.114904 46.666767, 16.11647 46.666873, 16.118015 46.666873, 16.118467 46.666854, 16.11911 46.66679, 16.120246 46.666637, 16.120912 46.66653, 16.122179 46.666275, 16.123336 46.665996, 16.123874 46.665843, 16.12486 46.665672, 16.125484 46.665588, 16.125976 46.665542, 16.126363 46.665481, 16.126621 46.665458, 16.12692 46.665416, 16.127222 46.665416, 16.127929 46.665458, 16.128337 46.665458, 16.129001 46.665374, 16.130397 46.665138, 16.130697 46.66505, 16.132049 46.664966, 16.133272 46.664794, 16.134151 46.66473, 16.134861 46.664642, 16.135524 46.6646, 16.136276 46.664512, 16.138637 46.664215, 16.139987 46.663936, 16.141319 46.663612, 16.142263 46.663375, 16.143035 46.663204, 16.143121 46.663204, 16.143293 46.663139, 16.143959 46.663013, 16.14458 46.662883, 16.145053 46.662796, 16.149108 46.66196, 16.150482 46.661705, 16.151468 46.661487, 16.151855 46.662345, 16.15224 46.663227, 16.153678 46.663204, 16.155138 46.663185, 16.156661 46.663185, 16.15752 46.663139, 16.158678 46.663162, 16.159645 46.66312, 16.159879 46.663185, 16.160352 46.663593, 16.160652 46.6637, 16.163764 46.663505, 16.164857 46.663463, 16.165351 46.663635, 16.166103 46.663784, 16.16784 46.664108, 16.169021 46.664344, 16.169643 46.66447, 16.169986 46.664535, 16.171274 46.664707, 16.172197 46.664855, 16.172626 46.66492, 16.173313 46.665008, 16.174556 46.665245, 16.17505 46.665435, 16.175266 46.665588, 16.175373 46.665737, 16.175973 46.666404, 16.176723 46.66711, 16.176853 46.667198, 16.177433 46.66756, 16.178226 46.667945, 16.178504 46.668121, 16.178848 46.668441, 16.178913 46.668525, 16.179149 46.668956, 16.179664 46.670051, 16.179792 46.670242, 16.179943 46.670524, 16.180114 46.670928, 16.180114 46.671253, 16.179985 46.671615, 16.179813 46.671852, 16.178827 46.672691, 16.17859 46.672988, 16.178504 46.673206, 16.178483 46.673439, 16.178548 46.673675, 16.178848 46.674125, 16.179126 46.674579, 16.179386 46.675178, 16.179792 46.676403, 16.179836 46.676704, 16.179922 46.677215, 16.179985 46.678398, 16.180028 46.678699, 16.180244 46.679405, 16.180286 46.679729, 16.180072 46.680866, 16.179985 46.68153, 16.180049 46.682216, 16.180156 46.682731, 16.180458 46.683162, 16.180736 46.683441, 16.181037 46.683719, 16.181573 46.684146, 16.182195 46.684536, 16.183355 46.685112, 16.183612 46.685306, 16.183849 46.68552, 16.184104 46.685798, 16.184341 46.685974, 16.184556 46.6861, 16.184835 46.686187, 16.184856 46.686206, 16.18505 46.686252, 16.18535 46.686271, 16.185672 46.686271, 16.18608 46.686229, 16.186552 46.686141, 16.187067 46.685974, 16.187709 46.685798, 16.188762 46.685649, 16.189083 46.68563, 16.189641 46.685565, 16.190113 46.685478, 16.191186 46.685157, 16.191444 46.685157, 16.192131 46.685371, 16.192947 46.685672, 16.193181 46.685798, 16.193889 46.686271, 16.194126 46.686401, 16.194555 46.686573, 16.195564 46.686874, 16.195972 46.687023, 16.196357 46.687194, 16.19668 46.687366, 16.197067 46.687515, 16.197431 46.687603, 16.197837 46.687667, 16.198633 46.68769, 16.199018 46.687625, 16.199361 46.687515, 16.199619 46.687408, 16.200563 46.686893, 16.201637 46.6861, 16.202152 46.685756, 16.202924 46.68552, 16.203331 46.685436, 16.203975 46.685348, 16.20539 46.685112, 16.206787 46.684898, 16.208095 46.684707, 16.209083 46.6846, 16.210197 46.684536, 16.211593 46.684536, 16.213567 46.684661, 16.214317 46.684749, 16.215047 46.684791, 16.215971 46.684791, 16.216184 46.684791, 16.217258 46.684944, 16.218545 46.685028, 16.21936 46.685134, 16.220197 46.685455, 16.220647 46.685565, 16.221464 46.685672, 16.221786 46.685691, 16.22245 46.685672, 16.222858 46.685672, 16.222988 46.6861, 16.223115 46.686336, 16.223287 46.68711, 16.223287 46.687515, 16.223222 46.687774, 16.223051 46.688461, 16.222988 46.688846, 16.222965 46.689277, 16.222965 46.689857, 16.223009 46.690521, 16.223352 46.690715, 16.223737 46.690864, 16.224317 46.691036, 16.225475 46.691421, 16.226312 46.691787, 16.22672 46.692001, 16.227064 46.69226, 16.227579 46.692817, 16.228029 46.693332, 16.228309 46.693759, 16.228652 46.694084, 16.229103 46.694404, 16.229381 46.694683, 16.229532 46.694942, 16.22966 46.69522, 16.229831 46.696144, 16.229961 46.696487, 16.230133 46.696743, 16.230583 46.697109, 16.231056 46.697429, 16.231204 46.697601, 16.232557 46.698974, 16.233158 46.699813, 16.23335 46.70005, 16.233587 46.700477, 16.233737 46.701034, 16.233909 46.701465, 16.234037 46.701892, 16.234273 46.702388, 16.235004 46.703289, 16.235519 46.703567, 16.236097 46.703804, 16.237085 46.704124, 16.237319 46.704231, 16.237857 46.704425, 16.238286 46.70462, 16.238651 46.704898, 16.239101 46.705154, 16.239337 46.705348, 16.240066 46.70565, 16.240346 46.705692, 16.240604 46.705757, 16.240924 46.705711, 16.241247 46.705711, 16.241912 46.705669, 16.242214 46.705692, 16.242792 46.705799, 16.243135 46.705947, 16.24335 46.706054, 16.243801 46.706337, 16.244079 46.70655, 16.244274 46.706722, 16.244529 46.707, 16.244594 46.70713, 16.244573 46.707622, 16.244529 46.707901)";
    //
    // assertEquals(result, route.mergeStreets().toString());
  }

  // @Test
  public void testNullInsideOut() throws Exception {
    // Inside out - fast
    Route route=test(23226, 11636, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction());

    assertNull(route);
  }

  // @Test
  public void testNullOutsideIn() throws Exception {
    // Outside in - very slow (all combinations)
    Route route=test(11636, 23226, new LengthCostFunction(),
      new GreatCircleDistanceHeuristicFunction());

    assertNull(route);
  }

  // ###################################################

  @Test
  // @Repeat(30)
  public void testFindRoute1() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    CostFunction cost=new LengthCostFunction();
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(0);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472836 46.085456, 14.472922 46.085586, 14.473029 46.08567, 14.473265 46.085758, 14.474531 46.086101, 14.474874 46.08625, 14.475024 46.086296, 14.476848 46.084533, 14.477535 46.083847, 14.478007 46.083248, 14.478158 46.083011, 14.478972 46.081851, 14.479531 46.081039, 14.479981 46.080371, 14.480046 46.080242, 14.480774 46.079147, 14.481162 46.07859, 14.481375 46.078292, 14.48202 46.07743, 14.482621 46.076618, 14.483157 46.075996, 14.483629 46.075565, 14.484745 46.074642, 14.485367 46.07415, 14.485796 46.073699, 14.486569 46.072883, 14.487062 46.072368, 14.488607 46.070674, 14.489745 46.069473, 14.491354 46.06784, 14.491633 46.06752, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.504314 46.05458, 14.504464 46.054386, 14.504549 46.054321, 14.505064 46.054195, 14.505795 46.054042, 14.507039 46.053829, 14.508219 46.053615, 14.508005 46.0531, 14.508048 46.052906, 14.508326 46.052799, 14.508542 46.052692, 14.508691 46.052585, 14.50882 46.052391, 14.5103 46.052433, 14.5103 46.052154, 14.510516 46.051681, 14.510602 46.05149, 14.510687 46.051105, 14.510708 46.050739, 14.510643 46.050651, 14.510516 46.049945, 14.510365 46.049514, 14.510258 46.049152, 14.510344 46.048141, 14.510344 46.047199, 14.510065 46.045375, 14.509764 46.044944, 14.509699 46.044815, 14.510773 46.044258, 14.511759 46.043743, 14.513369 46.043014, 14.513476 46.042949, 14.513627 46.042884, 14.514035 46.042629, 14.514378 46.042327, 14.514763 46.041942, 14.515515 46.041061, 14.515494 46.040931, 14.515965 46.040267, 14.516502 46.039665, 14.517317 46.038959, 14.517789 46.038574, 14.518476 46.038124, 14.519463 46.037628, 14.52073 46.037071, 14.52176 46.03664, 14.523476 46.035869, 14.524334 46.035331, 14.525557 46.034538, 14.526501 46.033874, 14.530235 46.031364, 14.532166 46.030097, 14.532938 46.02956, 14.533625 46.029151, 14.535149 46.028358, 14.535321 46.028251, 14.535814 46.027908, 14.536822 46.027008, 14.538195 46.025676, 14.539118 46.024734, 14.539355 46.024452, 14.540277 46.023231, 14.541157 46.021965, 14.542143 46.020378, 14.542509 46.01971, 14.542981 46.018787, 14.543259 46.0181, 14.543453 46.017459, 14.543602 46.016727, 14.543602 46.016578, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }

  @Test
  // @Repeat(30)
  public void testFindRoute2() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    Map<Level, Double> speedMap=new HashMap<Level, Double>();
    speedMap.put(Level.UNKNOWN, 90/70.0);
    speedMap.put(Level.MAJOR, 90/110.0);
    speedMap.put(Level.MEDIUM, 90/65.0);
    speedMap.put(Level.MINOR, 90/55.0);
    CostFunction cost=new LevelLengthCostFunction(speedMap);
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(0);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472493 46.084621, 14.472449 46.08419, 14.472579 46.083526, 14.472599 46.083377, 14.472535 46.083099, 14.472535 46.082969, 14.472449 46.082817, 14.472299 46.082626, 14.472213 46.08258, 14.47187 46.082302, 14.471741 46.081958, 14.471676 46.081615, 14.471205 46.081489, 14.471269 46.080821, 14.471333 46.080631, 14.471333 46.08052, 14.471269 46.080459, 14.470604 46.080093, 14.47099 46.079555, 14.471634 46.078636, 14.473008 46.077003, 14.471655 46.07653, 14.470067 46.075973, 14.469702 46.075801, 14.469359 46.075714, 14.468608 46.075481, 14.468287 46.07537, 14.467686 46.075157, 14.46554 46.074279, 14.464081 46.07357, 14.463738 46.073375, 14.462492 46.072624, 14.461119 46.071723, 14.460497 46.071273, 14.460025 46.070888, 14.459189 46.070095, 14.458544 46.069343, 14.457816 46.068336, 14.457514 46.06784, 14.457215 46.067195, 14.456935 46.06649, 14.455691 46.063076, 14.45554 46.062583, 14.455369 46.062198, 14.455133 46.061489, 14.453888 46.058055, 14.452965 46.055545, 14.452494 46.054321, 14.451807 46.05297, 14.451293 46.052154, 14.450841 46.051513, 14.450026 46.050483, 14.449662 46.050075, 14.448052 46.048358, 14.447602 46.047737, 14.447408 46.047477, 14.447129 46.047027, 14.446743 46.046253, 14.446507 46.045246, 14.446486 46.044754, 14.446528 46.044258, 14.446614 46.043807, 14.446722 46.043334, 14.446915 46.0428, 14.447279 46.042091, 14.447688 46.041381, 14.44773 46.041339, 14.448438 46.04029, 14.448845 46.039794, 14.449189 46.039451, 14.449554 46.039173, 14.449811 46.039024, 14.452193 46.037822, 14.452687 46.037628, 14.453845 46.037307, 14.455584 46.036918, 14.459746 46.03617, 14.460841 46.035934, 14.463051 46.035396, 14.464381 46.035011, 14.465325 46.034709, 14.465798 46.034561, 14.46672 46.034217, 14.467814 46.033809, 14.468608 46.033508, 14.469273 46.033229, 14.472514 46.031833, 14.473758 46.031276, 14.474273 46.031063, 14.475882 46.030418, 14.477427 46.029819, 14.478587 46.029411, 14.479638 46.029067, 14.480668 46.028743, 14.482363 46.028251, 14.483586 46.027931, 14.484401 46.027736, 14.485581 46.027435, 14.488135 46.026943, 14.48983 46.026664, 14.493306 46.026191, 14.49586 46.025913, 14.497233 46.025764, 14.498563 46.025588, 14.499164 46.025482, 14.499894 46.025375, 14.502276 46.024967, 14.503284 46.024776, 14.505516 46.024261, 14.507254 46.023811, 14.508412 46.023468, 14.509057 46.023296, 14.511309 46.022563, 14.512789 46.022006, 14.513412 46.021793, 14.515687 46.02087, 14.51912 46.019348, 14.520043 46.018981, 14.520364 46.018833, 14.520472 46.018939, 14.520771 46.018833, 14.521652 46.018489, 14.523346 46.017887, 14.523905 46.017692, 14.525557 46.017181, 14.526309 46.016944, 14.528454 46.016365, 14.530299 46.015933, 14.531586 46.015678, 14.532444 46.015483, 14.533582 46.015289, 14.536522 46.014839, 14.539419 46.014518, 14.541714 46.014347, 14.542509 46.014259, 14.542658 46.014259, 14.543453 46.014152, 14.544075 46.01411, 14.544611 46.014133, 14.544934 46.014198, 14.545212 46.014282, 14.54547 46.014389, 14.545684 46.01456, 14.545856 46.014755, 14.545942 46.015098, 14.545878 46.015377, 14.54577 46.015548, 14.545556 46.015762, 14.545041 46.016128, 14.544826 46.016193, 14.544483 46.016212, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }

  @Test
  // @Repeat(30)
  public void testFindRoute3() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    CostFunction cost=new LengthCostFunction();
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(1.0);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472836 46.085456, 14.472922 46.085586, 14.473029 46.08567, 14.473265 46.085758, 14.474531 46.086101, 14.474874 46.08625, 14.475024 46.086296, 14.476848 46.084533, 14.477535 46.083847, 14.478007 46.083248, 14.478158 46.083011, 14.478972 46.081851, 14.479531 46.081039, 14.479981 46.080371, 14.480046 46.080242, 14.480774 46.079147, 14.481162 46.07859, 14.481375 46.078292, 14.48202 46.07743, 14.482621 46.076618, 14.483157 46.075996, 14.483629 46.075565, 14.484745 46.074642, 14.485367 46.07415, 14.485796 46.073699, 14.486569 46.072883, 14.487062 46.072368, 14.488607 46.070674, 14.489745 46.069473, 14.491354 46.06784, 14.491633 46.06752, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.504314 46.05458, 14.504464 46.054386, 14.504549 46.054321, 14.505064 46.054195, 14.505795 46.054042, 14.507039 46.053829, 14.508219 46.053615, 14.508005 46.0531, 14.508048 46.052906, 14.508326 46.052799, 14.508542 46.052692, 14.508691 46.052585, 14.50882 46.052391, 14.5103 46.052433, 14.5103 46.052154, 14.510516 46.051681, 14.510602 46.05149, 14.510687 46.051105, 14.510708 46.050739, 14.510643 46.050651, 14.510516 46.049945, 14.510365 46.049514, 14.510258 46.049152, 14.510344 46.048141, 14.510344 46.047199, 14.510065 46.045375, 14.509764 46.044944, 14.509699 46.044815, 14.510773 46.044258, 14.511759 46.043743, 14.513369 46.043014, 14.513476 46.042949, 14.513627 46.042884, 14.514035 46.042629, 14.514378 46.042327, 14.514763 46.041942, 14.515515 46.041061, 14.515494 46.040931, 14.515965 46.040267, 14.516502 46.039665, 14.517317 46.038959, 14.517789 46.038574, 14.518476 46.038124, 14.519463 46.037628, 14.52073 46.037071, 14.52176 46.03664, 14.523476 46.035869, 14.524334 46.035331, 14.525557 46.034538, 14.526501 46.033874, 14.530235 46.031364, 14.532166 46.030097, 14.532938 46.02956, 14.533625 46.029151, 14.535149 46.028358, 14.535321 46.028251, 14.535814 46.027908, 14.536822 46.027008, 14.538195 46.025676, 14.539118 46.024734, 14.539355 46.024452, 14.540277 46.023231, 14.541157 46.021965, 14.542143 46.020378, 14.542509 46.01971, 14.542981 46.018787, 14.543259 46.0181, 14.543453 46.017459, 14.543602 46.016727, 14.543602 46.016578, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }

  @Test
  // @Repeat(30)
  public void testFindRoute4() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    Map<Level, Double> speedMap=new HashMap<Level, Double>();
    speedMap.put(Level.UNKNOWN, 90/70.0);
    speedMap.put(Level.MAJOR, 90/110.0);
    speedMap.put(Level.MEDIUM, 90/65.0);
    speedMap.put(Level.MINOR, 90/55.0);
    CostFunction cost=new LevelLengthCostFunction(speedMap);
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(1.0);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472493 46.084621, 14.472449 46.08419, 14.472579 46.083526, 14.472599 46.083377, 14.472535 46.083099, 14.472535 46.082969, 14.472449 46.082817, 14.472299 46.082626, 14.472213 46.08258, 14.47187 46.082302, 14.471741 46.081958, 14.471676 46.081615, 14.471205 46.081489, 14.471269 46.080821, 14.471333 46.080631, 14.471333 46.08052, 14.471269 46.080459, 14.470604 46.080093, 14.47099 46.079555, 14.471634 46.078636, 14.473008 46.077003, 14.471655 46.07653, 14.470067 46.075973, 14.469702 46.075801, 14.469359 46.075714, 14.468608 46.075481, 14.468287 46.07537, 14.467686 46.075157, 14.46554 46.074279, 14.464081 46.07357, 14.463738 46.073375, 14.462492 46.072624, 14.461119 46.071723, 14.460497 46.071273, 14.460025 46.070888, 14.459189 46.070095, 14.458544 46.069343, 14.457816 46.068336, 14.457514 46.06784, 14.457215 46.067195, 14.456935 46.06649, 14.455691 46.063076, 14.45554 46.062583, 14.455369 46.062198, 14.455133 46.061489, 14.453888 46.058055, 14.452965 46.055545, 14.452494 46.054321, 14.451807 46.05297, 14.451293 46.052154, 14.450841 46.051513, 14.450026 46.050483, 14.449662 46.050075, 14.448052 46.048358, 14.447602 46.047737, 14.447408 46.047477, 14.447129 46.047027, 14.446743 46.046253, 14.446507 46.045246, 14.446486 46.044754, 14.446528 46.044258, 14.446614 46.043807, 14.446722 46.043334, 14.446915 46.0428, 14.447279 46.042091, 14.447688 46.041381, 14.44773 46.041339, 14.448438 46.04029, 14.448845 46.039794, 14.449189 46.039451, 14.449554 46.039173, 14.449811 46.039024, 14.452193 46.037822, 14.452687 46.037628, 14.453845 46.037307, 14.455584 46.036918, 14.459746 46.03617, 14.460841 46.035934, 14.463051 46.035396, 14.464381 46.035011, 14.465325 46.034709, 14.465798 46.034561, 14.46672 46.034217, 14.467814 46.033809, 14.468608 46.033508, 14.469273 46.033229, 14.472514 46.031833, 14.473758 46.031276, 14.474273 46.031063, 14.475882 46.030418, 14.477427 46.029819, 14.478587 46.029411, 14.479638 46.029067, 14.480668 46.028743, 14.482363 46.028251, 14.483586 46.027931, 14.484401 46.027736, 14.485581 46.027435, 14.488135 46.026943, 14.48983 46.026664, 14.493306 46.026191, 14.49586 46.025913, 14.497233 46.025764, 14.498563 46.025588, 14.499164 46.025482, 14.499894 46.025375, 14.502276 46.024967, 14.503284 46.024776, 14.505516 46.024261, 14.507254 46.023811, 14.508412 46.023468, 14.509057 46.023296, 14.511309 46.022563, 14.512789 46.022006, 14.513412 46.021793, 14.515687 46.02087, 14.51912 46.019348, 14.520043 46.018981, 14.520364 46.018833, 14.520472 46.018939, 14.520771 46.018833, 14.521652 46.018489, 14.523346 46.017887, 14.523905 46.017692, 14.525557 46.017181, 14.526309 46.016944, 14.528454 46.016365, 14.530299 46.015933, 14.531586 46.015678, 14.532444 46.015483, 14.533582 46.015289, 14.536522 46.014839, 14.539419 46.014518, 14.541714 46.014347, 14.542509 46.014259, 14.542658 46.014259, 14.543453 46.014152, 14.544075 46.01411, 14.544611 46.014133, 14.544934 46.014198, 14.545212 46.014282, 14.54547 46.014389, 14.545684 46.01456, 14.545856 46.014755, 14.545942 46.015098, 14.545878 46.015377, 14.54577 46.015548, 14.545556 46.015762, 14.545041 46.016128, 14.544826 46.016193, 14.544483 46.016212, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }

  @Test
  // @Repeat(30)
  public void testFindRoute5() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    CostFunction cost=new LengthCostFunction();
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(1.2);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472836 46.085456, 14.472922 46.085586, 14.473029 46.08567, 14.473265 46.085758, 14.474531 46.086101, 14.474874 46.08625, 14.475024 46.086296, 14.476848 46.084533, 14.477535 46.083847, 14.478007 46.083248, 14.478158 46.083011, 14.478972 46.081851, 14.479531 46.081039, 14.479981 46.080371, 14.480046 46.080242, 14.480774 46.079147, 14.481162 46.07859, 14.481375 46.078292, 14.48202 46.07743, 14.482621 46.076618, 14.483157 46.075996, 14.483629 46.075565, 14.484745 46.074642, 14.485367 46.07415, 14.485796 46.073699, 14.486569 46.072883, 14.487062 46.072368, 14.488607 46.070674, 14.489745 46.069473, 14.491354 46.06784, 14.491633 46.06752, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.504314 46.05458, 14.504464 46.054386, 14.504549 46.054321, 14.505064 46.054195, 14.505795 46.054042, 14.507039 46.053829, 14.508219 46.053615, 14.508005 46.0531, 14.508048 46.052906, 14.508326 46.052799, 14.508542 46.052692, 14.508691 46.052585, 14.50882 46.052391, 14.5103 46.052433, 14.5103 46.052154, 14.510516 46.051681, 14.510602 46.05149, 14.510687 46.051105, 14.510708 46.050739, 14.510643 46.050651, 14.510516 46.049945, 14.510365 46.049514, 14.510258 46.049152, 14.510344 46.048141, 14.510344 46.047199, 14.510065 46.045375, 14.509764 46.044944, 14.509699 46.044815, 14.509657 46.044773, 14.511158 46.04396, 14.511803 46.043636, 14.513026 46.043121, 14.513369 46.042907, 14.51352 46.042819, 14.514056 46.042476, 14.514485 46.042114, 14.514828 46.04177, 14.515494 46.040931, 14.515965 46.040267, 14.516502 46.039665, 14.517317 46.038959, 14.517789 46.038574, 14.518476 46.038124, 14.519463 46.037628, 14.52073 46.037071, 14.52176 46.03664, 14.523476 46.035869, 14.524334 46.035331, 14.525557 46.034538, 14.526501 46.033874, 14.530235 46.031364, 14.532166 46.030097, 14.532938 46.02956, 14.533625 46.029151, 14.535149 46.028358, 14.535321 46.028251, 14.535814 46.027908, 14.536822 46.027008, 14.538195 46.025676, 14.539118 46.024734, 14.539355 46.024452, 14.540277 46.023231, 14.541157 46.021965, 14.542143 46.020378, 14.542509 46.01971, 14.542981 46.018787, 14.543259 46.0181, 14.543453 46.017459, 14.543602 46.016727, 14.543602 46.016578, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }

  @Test
  // @Repeat(30)
  public void testFindRoute6() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    Map<Level, Double> speedMap=new HashMap<Level, Double>();
    speedMap.put(Level.UNKNOWN, 90/70.0);
    speedMap.put(Level.MAJOR, 90/110.0);
    speedMap.put(Level.MEDIUM, 90/65.0);
    speedMap.put(Level.MINOR, 90/55.0);
    CostFunction cost=new LevelLengthCostFunction(speedMap);
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(1.2);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472493 46.084621, 14.472449 46.08419, 14.472579 46.083526, 14.472599 46.083377, 14.472535 46.083099, 14.472535 46.082969, 14.472449 46.082817, 14.472299 46.082626, 14.472213 46.08258, 14.47187 46.082302, 14.471741 46.081958, 14.471676 46.081615, 14.471205 46.081489, 14.471269 46.080821, 14.471333 46.080631, 14.471333 46.08052, 14.471269 46.080459, 14.470604 46.080093, 14.47099 46.079555, 14.471634 46.078636, 14.473008 46.077003, 14.473544 46.076488, 14.469874 46.075759, 14.468694 46.075416, 14.468394 46.075309, 14.467578 46.075008, 14.466227 46.074298, 14.464488 46.07331, 14.461205 46.071659, 14.461098 46.071594, 14.460154 46.070846, 14.459725 46.070457, 14.459059 46.069793, 14.458073 46.068462, 14.457708 46.067882, 14.457365 46.067176, 14.457043 46.06636, 14.456377 46.06443, 14.45554 46.062198, 14.454468 46.059173, 14.453738 46.057239, 14.453652 46.056941, 14.453245 46.055866, 14.453009 46.05516, 14.452601 46.054195, 14.452021 46.053054, 14.451464 46.052135, 14.451121 46.051639, 14.450198 46.05046, 14.449726 46.049964, 14.449554 46.049732, 14.44861 46.048721, 14.448073 46.048122, 14.44773 46.047691, 14.447516 46.047393, 14.4473 46.047069, 14.446893 46.046146, 14.446743 46.045589, 14.4467 46.045246, 14.446658 46.044689, 14.4467 46.044193, 14.446829 46.043487, 14.447087 46.042842, 14.447322 46.04235, 14.447837 46.041641, 14.448824 46.040718, 14.449275 46.040351, 14.449875 46.039966, 14.451121 46.039302, 14.452322 46.038764, 14.454833 46.037757, 14.456614 46.037136, 14.458073 46.036685, 14.460003 46.036231, 14.462171 46.035739, 14.462901 46.035545, 14.464231 46.035182, 14.466784 46.034366, 14.468008 46.033874, 14.468394 46.033744, 14.472428 46.032005, 14.473822 46.031383, 14.474359 46.031169, 14.474938 46.030914, 14.475947 46.030506, 14.477856 46.029796, 14.480324 46.02898, 14.48217 46.028423, 14.484293 46.027885, 14.486783 46.027328, 14.488499 46.026985, 14.490345 46.026683, 14.493735 46.026256, 14.494851 46.026126, 14.495645 46.026062, 14.498499 46.025718, 14.498907 46.025634, 14.499358 46.025569, 14.50116 46.02531, 14.501997 46.025138, 14.502533 46.025054, 14.504228 46.024688, 14.506052 46.024261, 14.50734 46.023895, 14.507919 46.023746, 14.509078 46.023403, 14.510622 46.02293, 14.511223 46.022716, 14.513541 46.021877, 14.514957 46.021301, 14.519077 46.019519, 14.520472 46.018939, 14.520771 46.018833, 14.521652 46.018489, 14.523346 46.017887, 14.523905 46.017692, 14.525557 46.017181, 14.526309 46.016944, 14.528454 46.016365, 14.530299 46.015933, 14.531586 46.015678, 14.532444 46.015483, 14.533582 46.015289, 14.536522 46.014839, 14.539419 46.014518, 14.541714 46.014347, 14.542509 46.014259, 14.542658 46.014259, 14.543453 46.014152, 14.544075 46.01411, 14.544611 46.014133, 14.544934 46.014198, 14.545212 46.014282, 14.54547 46.014389, 14.545684 46.01456, 14.545856 46.014755, 14.545942 46.015098, 14.545878 46.015377, 14.54577 46.015548, 14.545556 46.015762, 14.545041 46.016128, 14.544826 46.016193, 14.544483 46.016212, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }

  @Test
  // @Repeat(30)
  public void testFindRoute7() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    CostFunction cost=new LengthCostFunction();
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(3.0);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472493 46.084621, 14.472449 46.08419, 14.472579 46.083526, 14.474702 46.083911, 14.476119 46.084213, 14.476376 46.084297, 14.476613 46.084407, 14.476848 46.084533, 14.477535 46.083847, 14.478007 46.083248, 14.478158 46.083011, 14.478972 46.081851, 14.479531 46.081039, 14.479981 46.080371, 14.480046 46.080242, 14.480774 46.079147, 14.481162 46.07859, 14.481375 46.078292, 14.48202 46.07743, 14.482621 46.076618, 14.483157 46.075996, 14.483629 46.075565, 14.484745 46.074642, 14.485367 46.07415, 14.485796 46.073699, 14.486569 46.072883, 14.487062 46.072368, 14.488607 46.070674, 14.489745 46.069473, 14.491354 46.06784, 14.491633 46.06752, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.504314 46.05458, 14.504464 46.054386, 14.504549 46.054321, 14.504378 46.054, 14.50412 46.05355, 14.503949 46.053184, 14.503756 46.052822, 14.50352 46.052307, 14.503262 46.051727, 14.503048 46.051212, 14.502919 46.050739, 14.502791 46.050395, 14.502683 46.050029, 14.502533 46.049751, 14.50206 46.049045, 14.501524 46.048122, 14.501396 46.047863, 14.501332 46.047412, 14.501288 46.047306, 14.501224 46.047157, 14.50103 46.046318, 14.501009 46.046253, 14.502232 46.046081, 14.502511 46.046062, 14.503928 46.04589, 14.504657 46.045825, 14.505194 46.045825, 14.505344 46.045825, 14.505816 46.045845, 14.505967 46.045845, 14.506588 46.045783, 14.506825 46.045761, 14.507597 46.045631, 14.507897 46.045612, 14.508219 46.045501, 14.508348 46.04544, 14.508563 46.045352, 14.509335 46.044925, 14.509657 46.044773, 14.511158 46.04396, 14.511803 46.043636, 14.513026 46.043121, 14.513369 46.042907, 14.51352 46.042819, 14.514056 46.042476, 14.514485 46.042114, 14.514828 46.04177, 14.515494 46.040931, 14.515965 46.040267, 14.516502 46.039665, 14.517317 46.038959, 14.517789 46.038574, 14.518476 46.038124, 14.519463 46.037628, 14.52073 46.037071, 14.52176 46.03664, 14.523476 46.035869, 14.524334 46.035331, 14.525557 46.034538, 14.526501 46.033874, 14.530235 46.031364, 14.532166 46.030097, 14.532938 46.02956, 14.533625 46.029151, 14.535149 46.028358, 14.535321 46.028251, 14.535814 46.027908, 14.536822 46.027008, 14.538195 46.025676, 14.539118 46.024734, 14.539355 46.024452, 14.540277 46.023231, 14.541157 46.021965, 14.542143 46.020378, 14.542509 46.01971, 14.542981 46.018787, 14.543259 46.0181, 14.543453 46.017459, 14.543602 46.016727, 14.543602 46.016578, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }

  @Test
  // @Repeat(30)
  public void testFindRoute8() throws Exception {
    long a=75812; // Škofljica
    long b=64411; // Lj Dravlje
    Map<Level, Double> speedMap=new HashMap<Level, Double>();
    speedMap.put(Level.UNKNOWN, 90/70.0);
    speedMap.put(Level.MAJOR, 90/110.0);
    speedMap.put(Level.MEDIUM, 90/65.0);
    speedMap.put(Level.MINOR, 90/55.0);
    CostFunction cost=new LevelLengthCostFunction(speedMap);
    HeuristicFunction heuristic=new GreatCircleDistanceHeuristicFunction(3.0);
    Route route=test(a, b, cost, heuristic);

    logger.info("Found route: "+route);
    String result="LINESTRING (14.472685 46.085029, 14.472836 46.085456, 14.472922 46.085586, 14.473029 46.08567, 14.473265 46.085758, 14.474531 46.086101, 14.474874 46.08625, 14.475024 46.086296, 14.476848 46.084533, 14.477535 46.083847, 14.478007 46.083248, 14.478158 46.083011, 14.478972 46.081851, 14.479531 46.081039, 14.479981 46.080371, 14.480046 46.080242, 14.480774 46.079147, 14.481162 46.07859, 14.481375 46.078292, 14.48202 46.07743, 14.482621 46.076618, 14.483157 46.075996, 14.483629 46.075565, 14.484745 46.074642, 14.485367 46.07415, 14.485796 46.073699, 14.486569 46.072883, 14.487062 46.072368, 14.488607 46.070674, 14.489745 46.069473, 14.491354 46.06784, 14.491633 46.06752, 14.49174 46.067413, 14.492512 46.066661, 14.493671 46.065414, 14.493928 46.065093, 14.49425 46.064472, 14.494529 46.063762, 14.494701 46.0634, 14.495537 46.062282, 14.495945 46.06166, 14.496182 46.061359, 14.49719 46.060482, 14.498134 46.05973, 14.499379 46.058528, 14.500022 46.057861, 14.500537 46.057411, 14.501288 46.056575, 14.501374 46.056468, 14.50206 46.055912, 14.502575 46.05561, 14.504314 46.05458, 14.504464 46.054386, 14.504549 46.054321, 14.504378 46.054, 14.50412 46.05355, 14.503949 46.053184, 14.503756 46.052822, 14.50352 46.052307, 14.503262 46.051727, 14.503048 46.051212, 14.502919 46.050739, 14.502791 46.050395, 14.502683 46.050029, 14.502533 46.049751, 14.50206 46.049045, 14.501524 46.048122, 14.501396 46.047863, 14.501332 46.047412, 14.501288 46.047306, 14.501224 46.047157, 14.50103 46.046318, 14.501009 46.046253, 14.502232 46.046081, 14.502511 46.046062, 14.503928 46.04589, 14.504657 46.045825, 14.505194 46.045825, 14.505344 46.045825, 14.505816 46.045845, 14.505967 46.045845, 14.506588 46.045783, 14.506825 46.045761, 14.507597 46.045631, 14.507897 46.045612, 14.508219 46.045501, 14.508348 46.04544, 14.508563 46.045352, 14.509335 46.044925, 14.509657 46.044773, 14.511158 46.04396, 14.511803 46.043636, 14.513026 46.043121, 14.513369 46.042907, 14.51352 46.042819, 14.514056 46.042476, 14.514485 46.042114, 14.514828 46.04177, 14.515494 46.040931, 14.515965 46.040267, 14.516502 46.039665, 14.517317 46.038959, 14.517789 46.038574, 14.518476 46.038124, 14.519463 46.037628, 14.52073 46.037071, 14.52176 46.03664, 14.523476 46.035869, 14.524334 46.035331, 14.525557 46.034538, 14.526501 46.033874, 14.530235 46.031364, 14.532166 46.030097, 14.532938 46.02956, 14.533625 46.029151, 14.535149 46.028358, 14.535321 46.028251, 14.535814 46.027908, 14.536822 46.027008, 14.538195 46.025676, 14.539118 46.024734, 14.539355 46.024452, 14.540277 46.023231, 14.541157 46.021965, 14.542143 46.020378, 14.542509 46.01971, 14.542981 46.018787, 14.543259 46.0181, 14.543453 46.017459, 14.543602 46.016727, 14.543602 46.016578, 14.543688 46.016212, 14.543667 46.015441, 14.543646 46.015289, 14.543561 46.014862, 14.543389 46.014305, 14.542852 46.013229, 14.542766 46.012973, 14.542723 46.012607, 14.542723 46.011966, 14.542809 46.011558, 14.54296 46.011127, 14.543367 46.010227, 14.543539 46.009925, 14.543688 46.009735, 14.544182 46.009132, 14.544955 46.008361, 14.545684 46.007717, 14.546199 46.007392, 14.548281 46.006622, 14.548924 46.006343, 14.549826 46.00587, 14.551327 46.00484, 14.552336 46.004135, 14.55298 46.003555, 14.553517 46.002887, 14.55373 46.002544, 14.554289 46.001731, 14.554568 46.001277, 14.554697 46.001087, 14.555555 45.999519, 14.555705 45.999218, 14.555876 45.998874, 14.556134 45.998447, 14.556499 45.997974, 14.55695 45.997589, 14.557164 45.997459, 14.558259 45.996902, 14.559182 45.996452, 14.559739 45.996151, 14.560233 45.995807, 14.560705 45.995376, 14.561134 45.994926, 14.561563 45.994392, 14.562378 45.993148, 14.562722 45.992652, 14.563022 45.992286, 14.563494 45.991859, 14.564009 45.991451, 14.564481 45.991172, 14.565232 45.990764, 14.565276 45.990722, 14.565362 45.990657, 14.565533 45.990551, 14.566112 45.989994, 14.566519 45.989391, 14.566948 45.988746, 14.567013 45.988685, 14.56725 45.988342, 14.567485 45.988082, 14.568086 45.987655, 14.568623 45.987373, 14.568709 45.987354, 14.569803 45.987159, 14.570597 45.986881, 14.571198 45.98656, 14.571627 45.987201, 14.571497 45.987289, 14.571068 45.987503, 14.570897 45.987632, 14.570769 45.987697, 14.570318 45.987846, 14.569911 45.987846, 14.569653 45.987869)";
    assertEquals(result, route.mergeStreets().toString());
  }
}
